<?php
// $Id: EditHelpdeskBucketResponse.php 8147 2006-07-27 15:30:59Z bsmith $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action takes the information provided by the EditHelpdeskBucket action, and applies it to the database, 
 * thus editing a helpdesk bucket in the system. If the provided helpdesk bucket ID does not exist, an error is 
 * thrown. Provided no errors occur, once the helpdesk bucket is edited, this action redirects to the 
 * ShowHelpdeskBucketDetails action, where the new changes can be viewed.
 *
 * @author  Vic Fryzel
 */
class EditHelpdeskBucketResponse extends MB_Action {
	/**
	 * The constructor for an EditHelpdeskBucketResponse action object. This calls the parent constructor, 
	 * sets the string representation of this object and sets up $this->params.
	 */
	function EditHelpdeskBucketResponse() {
		parent::MB_Action();
		$this->stringRep = "EditHelpdeskBucketResponse";
		$this->params = array("hdBucketID", "hdBucketName");
	}

	/**
	 * The main method of this class that executes the EditHelpdeskBucketResponse action, and returns 
	 * success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$bucketID = ((int) $_REQUEST["hdBucketID"]);
		if(!$bucketID) {
			$this->addError(TRANS_ACTIONERROR, "No helpdesk bucket ID given.");
		} else {
			$bucket = $this->getMBAPIData("GetHelpdeskBuckets", "helpdeskBuckets", "helpdeskBucket",
				array("hdBucketID" => $bucketID, "getAdminData" => 1, "getHelpdeskDepartmentData" => 1));
			if($bucket === false || !count($bucket)) {
				$this->addError(TRANS_ACTIONERROR, "Helpdesk bucket not found.");
			} else {
				$bucket = $bucket[0][0];
				$params = array();
				
				/**
				 * 	all right we're going to take two sets of array's, create an index of the values
				 *	in both the current record, and the ones posted. Array_diff that, if there is a
				 *	diff then we should run them throught the getAddIDXML and getRemoveIDXML functions
				 */
				
				$admins = array();
				foreach( (array)$bucket["adminIDs"][0]["adminID"] as $v ) {
					if (isset($v["adminID"]) && is_array($v["adminID"]))
						$admins[] = $v["adminID"][0];
				}
				
				if (array_diff( $admins, $_REQUEST["adminIDs"]) || !notNull($admins)) {
				
					$params["admins"]["admin"][] = getAddIDXML("adminID", $admins, $_REQUEST["adminIDs"]);
					$params["removeAdminIDs"] = getRemoveIDXML("adminID", $admins, $_REQUEST["adminIDs"]);
					
				}
				unset( $admins );


				$depts = array();
				foreach( (array)$bucket["hdDepartmentIDs"][0]["hdDepartmentID"] as $v ) {
					$depts[] = $v["hdDepartmentID"][0];
				}

				if (array_diff( $depts, $_REQUEST["hdDepartmentIDs"] ) || !notNull($depts)) {
					$params["hdDepartmentIDs"]
						= getAddIDXML("hdDepartmentID", $depts, $_REQUEST["hdDepartmentIDs"]);
					
					$params["removeHelpdeskDepartmentIDs"]
						= getRemoveIDXML("hdDepartmentID", $depts, $_REQUEST["hdDepartmentIDs"]);
				}
				unset( $depts, $v );

				
				
				$query = $this->getMBAPIQuery("SetHelpdeskBuckets", "update", $params );
// echo $query;exit;
				$data = dispatchMBAPI($query, 1);
				if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					
					redirect("helpdesk.php", "ShowHelpdeskBuckets", array("hdBucketID" => $bucketID));
				}
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // EditHelpdeskBucketResponse
?>