<?php
// $Id: ShowHelpdesk.php 8143 2006-07-26 21:30:13Z bsmith $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action displays information on all helpdesk departments, buckets, and tickets associated with the currently
 * logged in admin. All helpdesk departments, buckets, and tickets are NOT shown by this action. This action is
 * meant to serve the helpdesk "homepage" for any admin, giving them the *most* relevant information at once, that
 * will be used most frequently.
 *
 * This is one of the most complicated actions in the system, it maps to almost every table in the helpdesk, mining
 * for related tickets.
 *
 * @author  Vic Fryzel
 */
class ShowHelpdesk extends MB_Action {
	/**
	 * The constructor for a ShowHelpdesk action object. This calls the parent constructor,
	 * sets the string representation of this object, and sets up $this->sortColumns for use in
	 * $this->getMBAPIQuery().
	 */
	function ShowHelpdesk() {
		parent::MB_Action();
		$this->stringRep = "ShowHelpdesk";
	}

	/**
	 * The main method of this class that executes the ShowHelpdesk action, and returns success
	 * or failure.
	 *
	 * @return  int  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		// Lots of stuff to do, here is a list of what we need to accomplish:
		//  1. A set/tree of departments somehow associated with the currently logged in admin
		//  2. For each element in the set of departments, we need a set of buckets that the admin should be
		//     seeing in the department
		//  3. For each element in BOTH the set of departments AND each element in each set of buckets, we
		//     need a set of tickets
		//
		// We need to build this:
		// $data[ $deptID]["hdDepartmentID"]
		//                ["hdDepartmentName"]
		//                ["hdDepartmentDescription"]
		//                ["numPendingTickets"]
		//                ["numOpenTickets"]
		//                ["buckets"][ $bucketID]["hdBucketID"]
		//                                      ["hdBucketName"]
		//                                      ["numPendingTickets"]
		//                                      ["numOpenTickets"]
		//                                      ["tickets"]["hdTicketID"]
		//                                                 ["..."]
		//                                      ["..."]
		//                           ["..."]
		//                ["tickets"][ $ticketID]["hdTicketID"]
		//                                      ["..."]
		//                           ["..."]
		//      ["..."]
		//
		// Remember that we key the arrays on ID because it's a set, and that's the only simple way to keep elements
		// unique.

		// Instantiate our set
		$data = array();

		// Let's start out by getting a list of all the departments directly related to the admin
		$query = $this->getMBAPIQuery("GetHelpdeskDepartments", "", array("adminIDs" => "<adminID>"
								.$CONF["user"]["id"]."</adminID>", "getHelpdeskBucketData" => 3, "getTicketData" => 1));
		$result = dispatchMBAPI($query, 1);
		
		// We've got the data, now let's put it into our set
		if($this->addMBAPIErrors($result) === 0 && !$this->getNumErrors()) {
			
			$result = $result["mbapi"][0]["results"][0];

			if(is_array($result["helpdeskDepartments"][0]["helpdeskDepartment"]) && count($result["helpdeskDepartments"][0]["helpdeskDepartment"])) {

				$tickets = NULL;

				// This is used later in some loops, lets call it once instead of n times
				$pendingID = getAPIStatusID("helpdesk", "pending");

				$dept2Buckets = array();
				if (is_array($result["helpdeskDepartments"][0]["helpdeskDepartment"]))
				foreach( $result["helpdeskDepartments"][0]["helpdeskDepartment"] as $dept ) {
					if (notZero( $dept["hdDepartmentID"][0] )) 
					foreach( (array)$dept["helpdeskBuckets"][0]["helpdeskBucket"] as $bucket ) {
						if (is_array( $bucket["hdBucketID"] ) && notZero( $bucket["hdBucketID"][0] ))
							$dept2Buckets[ (int)$dept["hdDepartmentID"][0] ][] = $bucket["hdBucketID"][0];
					}
					
				}
				unset( $dept, $bucket );

				foreach($result["helpdeskDepartments"][0]["helpdeskDepartment"] as $dept) {

					// Add this department to the set/tree
					$data[ (int)$dept["hdDepartmentID"][0] ] = collapseArrayElements($dept);
					$data[ (int)$dept["hdDepartmentID"][0] ]["buckets"] = array();
					$data[ (int)$dept["hdDepartmentID"][0] ]["tickets"] = array();
					
					$data[ (int)$dept["hdDepartmentID"][0] ]["numPendingTickets"] = 0;
					$data[ (int)$dept["hdDepartmentID"][0] ]["numOpenTickets"] = 0;
					$data[ (int)$dept["hdDepartmentID"][0] ]["numClosedTickets"] = 0;


					// Add the buckets
					if(is_array($dept["helpdeskBuckets"][0]["helpdeskBucket"]) && count($dept["helpdeskBuckets"][0]["helpdeskBucket"])) {
						
						foreach($dept["helpdeskBuckets"][0]["helpdeskBucket"] as $bucket) {

							// The following if statement is to resolve a *rare* array offset error
							// that occurs when this element apparently has no value, even though testing for
							// this above doesn't work. This only occurs WITHIN the foreach loop.

							if($bucket) {
								
								$bucketData = array();
								$bucketData["hdBucketID"] = (int)$bucket["hdBucketID"][0];
								$bucketData["hdBucketName"] = $bucket["hdBucketName"][0];
								$bucketData["tickets"] = array();

								if(is_array($bucket["helpdeskTickets"][0]["helpdeskTicket"]) && count($bucket["helpdeskTickets"][0]["helpdeskTicket"])) {
									if(count($bucket["helpdeskTickets"][0]["helpdeskTicket"]) > 1) {
										$bucketData["tickets"] =
											collapseArrayElements($bucket["helpdeskTickets"][0]["helpdeskTicket"]);
									} else {
										$bucketData["tickets"] =
											array(0 => collapseArrayElements($bucket["helpdeskTickets"][0]["helpdeskTicket"]));
									}
								}
								
								$data[ (int)$dept["hdDepartmentID"][0] ]["buckets"][ (int)$bucket["hdBucketID"][0] ] = $bucketData;
								
							}
						}
						unset($data[ (int)$dept["hdDepartmentID"][0] ]["helpdeskBuckets"]);
					}

					// Add the tickets
					if(is_array($dept["helpdeskTickets"][0]["helpdeskTicket"]) && count($dept["helpdeskTickets"][0]["helpdeskTicket"])) {
						
						foreach($dept["helpdeskTickets"][0]["helpdeskTicket"] as $ticket) {
							if (!in_array( (int)$ticket["hdBucketID"][0], $dept2Buckets[ (int)$dept["hdDepartmentID"][0] ] )) {
								$ticket["hdBucketID"][0] = 0;
							}
							
							$data[ (int)$dept["hdDepartmentID"][0] ]["buckets"][ (int)$ticket["hdBucketID"][0] ]["stats"][ (int)$ticket["hdTicketStatusID"][0] ]["count"]++;
							
							if ($ticket["hdTicketDateModified"][0] >= $data[ $dept["hdDepartmentID"][0] ]["bucket"][ (int)$ticket["hdBucketID"][0] ]["stats"][ $ticket["hdTicketStatusID"][0] ]["lastActivity"]) {
								$data[ (int)$dept["hdDepartmentID"][0] ]["buckets"][ (int)$ticket["hdBucketID"][0] ]["stats"][ (int)$ticket["hdTicketStatusID"][0] ]["lastActivity"] = $ticket["hdTicketDateModified"][0];
								$data[ (int)$dept["hdDepartmentID"][0] ]["buckets"][ (int)$ticket["hdBucketID"][0] ]["stats"][ (int)$ticket["hdTicketStatusID"][0] ]["lastReplied"]["firstName"] = $ticket["last_resp_user_first_name"][0];
								$data[ (int)$dept["hdDepartmentID"][0] ]["buckets"][ (int)$ticket["hdBucketID"][0] ]["stats"][ (int)$ticket["hdTicketStatusID"][0] ]["lastReplied"]["lastName"] = $ticket["last_resp_user_last_name"][0];
							}
							
							//Seems that tickets are being duplicated and don't need to be
							//added here if it is already in a bucket
							if(!$ticket["hdBucketID"][0]) {
								$data[ (int)$dept["hdDepartmentID"][0] ]["tickets"][ (int)$ticket["hdTicketID"][0] ]
									= collapseArrayElements($ticket);

// 								unset($data[ $dept["hdDepartmentID"][0] ]["helpdeskTickets"]);
							}
						}
					}
				}
			}
		}
/*
		// Now let's add on to the set/tree all directly related buckets and thus indirectly related departments
		// At this point we cannot completely overwrite elements of the array, we must always append
		// The following query can produce a gigantic result; I need to experiment with performance.
		// O(n^2) != cool in large result sets.
		$query = $this->getMBAPIQuery("GetHelpdeskBuckets", "", array(
								"adminIDs" => "<adminID>".$CONF["user"]["id"]."</adminID>",
								"getHelpdeskDepartmentData" => 2,
								"getTicketData" => 3));
		$result = dispatchMBAPI($query, 1);

		// We've got the data, now let's put it into our set
		if($this->addMBAPIErrors($result) === 0 && !$this->getNumErrors()) {
			
			$result = $result["mbapi"][0]["results"][0];

			if(is_array($result["helpdeskBuckets"][0]["helpdeskBucket"]) && count($result["helpdeskBuckets"][0]["helpdeskBucket"])) {

				// Save memory by declaring the variables in this scope...
				$departments = $bucketData = $ticketData = $ticketCountData = $tickets = NULL;

				foreach($result["helpdeskBuckets"][0]["helpdeskBucket"] as $bucket) {
					
					// Structure the ticket data once, so that we don't get O(n^3)
					$tickets = $bucket["helpdeskTickets"][0]["helpdeskTicket"];
					$ticketData = NULL;

					$ticketCountData["numPendingTickets"] = $ticketCountData["numOpenTickets"] = $ticketCountData["numClosedTickets"] = 0;

					if(is_array($tickets) && count($tickets)) {
					
						foreach($tickets as $ticket) {
							$ticketData[ $ticket["hdTicketID"][0] ] = collapseArrayElements($ticket);
							
							switch( $ticketData[ $ticket["hdTicketID"][0] ]["hdTicketStatusID"] ) {
							case getAPIStatusID("helpdesk", "pending"):
								$ticketCountData["numPendingTickets"]++;
							break;
							case getAPIStatusID("helpdesk", "open"):
								$ticketCountData["numOpenTickets"]++;
							break;
							default:
								$ticketCountData["numClosedTickets"]++;
							break;
							}
						
						}
						
					}
					
					// We don't allow orphan buckets, so we can safely loop through each bucket's parent departments
					$departments = $bucket["hdDepartments"][0]["hdDepartment"];

					if(is_array($departments) && count($departments)) {
						
						foreach($departments as $dept) {
							
							// Add this department to the set/tree if it does not already exist
							if(!$data[ $dept["hdDepartmentID"][0] ]) {
								
								$data[ $dept["hdDepartmentID"][0] ] = collapseArrayElements($dept);

								if(!@isset($data[ $dept["hdDepartmentID"][0] ]["buckets"])) {
									$data[ $dept["hdDepartmentID"][0] ]["buckets"] = array();
									$data[ $dept["hdDepartmentID"][0] ]["tickets"] = array();
									$data[ $dept["hdDepartmentID"][0] ]["numPendingTickets"] = 0;
									$data[ $dept["hdDepartmentID"][0] ]["numOpenTickets"] = 0;
									$data[ $dept["hdDepartmentID"][0] ]["numClosedTickets"] = 0;
								}
							}
							
							// Add this bucket to the set/tree under the above dept.
							$bucketData = array();
							$bucketData["hdBucketID"] = $bucket["hdBucketID"][0];
							$bucketData["hdBucketName"] = $bucket["hdBucketName"][0];
							$data[ $dept["hdDepartmentID"][0] ]["buckets"][ $bucket["hdBucketID"][0] ] = $bucketData;

							// Increment DEPARTMENT ticket counts
							$data[ $dept["hdDepartmentID"][0] ]["numPendingTickets"] += $ticketCountData["numPendingTickets"];
							$data[ $dept["hdDepartmentID"][0] ]["numOpenTickets"] += $ticketCountData["numOpenTickets"];
							$data[ $dept["hdDepartmentID"][0] ]["numClosedTickets"] += $ticketCountData["numClosedTickets"];

							// Add the tickets
							$data[ $dept["hdDepartmentID"][0] ]["buckets"][ $bucket["hdBucketID"][0] ]["tickets"] = $ticketData;
							
							// find last replied && last activity
							$lastReplied = $data[ $dept["hdDepartmentID"][0] ]["lastReplied"];
							$lastActivity = $data[ $dept["hdDepartmentID"][0] ]["lastActivity"];
							
							if(is_array( $ticketData ))
							foreach( $ticketData as $v ) {
								if ($v["hdTicketDateModified"] > $lastActivity) {
									$lastActivity = $v["hdTicketDateModified"];
									$lastReplied = $v["userID"];
								}
							}
							
							if (notZero( $lastReplied )) {
								$getAdminQuery =& new MBQuery("GetAdmins", "", array("adminID" => $lastReplied));
								$data[ $dept["hdDepartmentID"][0] ]["stats"][ $ticketData["hdDepartmentID"][0] ]["lastReplied"]
										= correctArray(dispatchMBAPI( $getAdminQuery, 1 ));
							} else {
								
							}
							
							$data[ $dept["hdDepartmentID"][0] ]["lastActivity"] = (int)$lastActivity;
							unset( $lastActivity, $lastReplied, $v );
							
							// Set the ticket counts for this bucket
							// Note that the following counts are for the whole bucket, so no += operator
							$data[ $dept["hdDepartmentID"][0] ]["buckets"][ $bucket["hdBucketID"][0] ]["numPendingTickets"]
											= $ticketCountData["numPendingTickets"];
							$data[ $dept["hdDepartmentID"][0] ]["buckets"][ $bucket["hdBucketID"][0] ]["numOpenTickets"]
											= $ticketCountData["numOpenTickets"];
							$data[ $dept["hdDepartmentID"][0] ]["buckets"][ $bucket["hdBucketID"][0] ]["numClosedTickets"]
											= $ticketCountData["numClosedTickets"];
						}
					}
				}
			}
		}
*/
		dbg("################### helpdeskDepartments #################");
		dbg($data);
// 		print_r($data);
		if(!$this->getNumErrors()) {
			$this->smartyVars["helpdeskDepartments"] =& $data;
			$this->smartyVars["helpdeskStatuses"] = getStatuses("helpdesk");
			$this->addTemplate("ShowHelpdesk");
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowHelpdesk
?>