<?php
// $Id: ShowHelpdeskTicketDetails.php 10442 2007-01-08 23:59:44Z bsmith $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action displays information about a single helpdesk ticket. It displays ticket details, user information,
 * response information, notes, and information on related departments and buckets.
 *
 * @author  Vic Fryzel
 */
class ShowHelpdeskTicketDetails extends MB_Action {
	/**
	 * The constructor for a ShowHelpdeskTicketDetails action object. This calls the parent constructor,
	 * sets the string representation of this object, and sets up $this->params for use in
	 * $this->getMBAPIQuery().
	 */
	function ShowHelpdeskTicketDetails() {
		parent::MB_Action();
		$this->stringRep = "ShowHelpdeskTicketDetails";
		$this->params = array("hdTicketID");
	}

	/**
	 * The main method of this class that executes the ShowHelpdeskTicketDetails action, and returns success
	 * or failure.
	 *
	 * @return  int  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$departments = correctArray($this->getMBAPIData("GetHelpdeskDepartments", "helpdeskDepartments", "helpdeskDepartment"));

		/// format data for buckets
		$tmp = correctArray($this->getMBAPIData("GetHelpdeskBuckets", "helpdeskBuckets", "helpdeskBucket"));
		$buckets=array();
		foreach( $tmp as $v ) {
			$buckets[ $v["hdBucketID"] ] = $v["hdBucketName"];
		}
		unset( $tmp, $v );
		
		$respTpls = correctArray($this->getMBAPIData("GetHelpdeskResponseTemplates", "helpdeskResponseTemplates", "helpdeskResponseTempate"));
		
		$query = $this->getMBAPIQuery("GetHelpdeskTickets", "", array(
								"getHelpdeskDepartmentData" => 1,
								"getHelpdeskBucketData" => 1,
								"getUserData" => 1,
								"getHelpdeskTicketResponseData" => 1,
								"getHelpdeskTicketNoteData" => 1,
								"hdTicketID" => (int)$_REQUEST["hdTicketID"]), 1, 1, 1, 1);
		$data = dispatchMBAPI($query, 1);
		
		if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
			$results = grabSimpleMBAPIResults($data);
			$results = $results[0];
			
			dbg("######### Simple Results from GetHelpdeskTickets ############");
			dbg($results);
			
			// Index departments and buckets on ID, for easy use in template
			$this->smartyVars["helpdeskDepartments"] = array();
			if(is_array($departments))
			foreach($departments as $dept) {
				$this->smartyVars["helpdeskDepartments"][ $dept["hdDepartmentID" ] ] = $dept;
			}
			unset( $dept );
			
			$this->smartyVars["helpdeskBuckets"] = array();
			if(is_array($buckets[0][0])) {
				$buckets = $buckets[0];
			}
			
			if (is_array($buckets))
			foreach($buckets as $bucketID => $bucket) {
				$this->smartyVars["helpdeskBuckets"][ $bucketID ] = $bucket;
			}
			unset( $buckets, $bucket, $bucketID );
			
			// Assign response template data
			$this->smartyVars["helpdeskResponseTemplates"] = array();
			if (is_array($respTpls) && count($respTpls)) {
				if(is_array($respTpls["helpdeskResponseTemplate"]) && count($respTpls["helpdeskResponseTemplate"])) {
					$this->smartyVars["helpdeskResponseTemplates"] = $respTpls["helpdeskResponseTemplate"];
				} else {
					$this->smartyVars["helpdeskResponseTemplates"] = $respTpls;
				}
			}
			
			// Assign ticket data, this will be collapsed later
			$this->smartyVars["helpdeskTicket"] =& $results;
			
			// Assign specific department and bucket IDs to this ticket
			if (is_array($this->smartyVars["helpdeskTicket"]["hdDepartmentIDs"][0])
					&& notZero($this->smartyVars["helpdeskTicket"]["hdDepartmentIDs"][0]["hdDepartmentID"])) {
				$deptRels = $this->smartyVars["helpdeskTicket"]["hdDepartmentIDs"][0]["hdDepartmentID"];
				$this->smartyVars["helpdeskTicket"]["hdDepartmentIDs"] = $deptRels;
			} else {
				$this->smartyVars["helpdeskTicket"]["hdDepartmentIDs"] = 0;
			}

			if (is_array($this->smartyVars["helpdeskTicket"]["hdBucketIDs"][0])
					&& notZero($this->smartyVars["helpdeskTicket"]["hdBucketIDs"][0]["hdBucketID"])) {
				$bucketRels = $this->smartyVars["helpdeskTicket"]["hdBucketIDs"][0]["hdBucketID"];
				$this->smartyVars["helpdeskTicket"]["hdBucketIDs"] = $bucketRels;
			} else {
				$this->smartyVars["helpdeskTicket"]["hdBucketIDs"] = 0;
			}
			
			// Generate responses
			if(is_array($this->smartyVars["helpdeskTicket"]["helpdeskTicketResponses"][0])
					&& is_array($this->smartyVars["helpdeskTicket"]["helpdeskTicketResponses"][0]["helpdeskTicketResponse"])
					&& count($this->smartyVars["helpdeskTicket"]["helpdeskTicketResponses"][0]["helpdeskTicketResponse"])) {
				$this->smartyVars["helpdeskTicket"]["helpdeskTicketResponses"]
					= $this->smartyVars["helpdeskTicket"]["helpdeskTicketResponses"][0]["helpdeskTicketResponse"];

			} else {
				$this->smartyVars["helpdeskTicket"]["helpdeskTicketResponses"] = array();
			}
			
			// Generate notes
			if(isset($this->smartyVars["helpdeskTicket"]["helpdeskTicketNotes"][0])
					&& is_array($this->smartyVars["helpdeskTicket"]["helpdeskTicketNotes"][0]["helpdeskTicketNote"])
					&& count($this->smartyVars["helpdeskTicket"]["helpdeskTicketNotes"][0]["helpdeskTicketNote"])) {
				$this->smartyVars["helpdeskTicket"]["helpdeskTicketNotes"]
					= $this->smartyVars["helpdeskTicket"]["helpdeskTicketNotes"][0]["helpdeskTicketNote"];
			} else {
				$this->smartyVars["helpdeskTicket"]["helpdeskTicketNotes"] = array();
			}

			// Parse response templates, and apply data to them
			$t =& $this->smartyVars["helpdeskTicket"];
			$fields = array();
			$fields["hdTicketID"] = $t["hdTicketID"];
			$fields["hdTicketKey"] = $t["hdTicketKey"];
			$fields["hdTicketSubject"] = $t["hdTicketSubject"];
			$fields["hdTicketBody"] = $t["hdTicketBody"];
			$fields["hdTicketPriority"] = getPriorityName($t["hdTicketPriorityID"]);
			$fields["hdTicketStatus"] = getStatusName("helpdesk", $t["hdTicketStatusID"]);
			$fields["userID"] = $t["userID"][0];
			
			if($t["userType"] == 2) {
				$fields["userFirstName"] = $t["admins"][0]["admin"][0]["adminFirstName"];
				$fields["userLastName"] = $t["admins"][0]["admin"][0]["adminLastName"];
				$fields["userEmail"] = $t["admins"][0]["admin"][0]["adminEmail"];
				$fields["username"] = $t["admins"][0]["admin"][0]["adminUsername"];
			} else {
				$fields["userFirstName"] = $t["clients"][0]["client"][0]["contactFirstName"];
				$fields["userLastName"] = $t["clients"][0]["client"][0]["contactLastName"];
				$fields["userEmail"] = $t["clients"][0]["client"][0]["contactEmail"];
				$fields["username"] = $t["clients"][0]["client"][0]["contactUsername"];
			}
			
			$search = array_keys($fields);
			$replace = array_values($fields);

			foreach($search as $key => $fieldName) {
				$search[$key] = "{".$fieldName."}";
			}
			$newBody = NULL;
			foreach($this->smartyVars["helpdeskResponseTemplates"] as $key => $respTpl) {
				$newBody = str_replace(array("\\{", "\\}"), array("[[[OPEN_CURLY_BRACE]]]", "[[[CLOSE_CURLY_BRACE]]]"), unsecureInput($respTpl["hdRespTplBody"]));
				$newBody = str_replace($search, $replace, $newBody);
				$newBody = str_replace(array("[[[OPEN_CURLY_BRACE]]]", "[[[CLOSE_CURLY_BRACE]]]"), array("\\{", "\\}"),	$newBody);
				$this->smartyVars["helpdeskResponseTemplates"][$key]["hdRespTplParsedBody"] = addslashes(str_replace("\n", "\\n", $newBody));
			}

			
			
			// meta array's
			$this->smartyVars["helpdeskStatuses"] = getStatuses("helpdesk");

			$this->smartyVars["helpdeskDepartmentSelect"] = array();
			
			if (is_array( $departments ))
			foreach( $departments as $v ) {
				$this->smartyVars["helpdeskDepartmentSelect"][ $v["hdDepartmentID"] ] = $v["hdDepartmentName"];
			}
			unset( $departments, $v );

			// grabbing client history
			if (isset($this->smartyVars["helpdeskTicket"]["clients"][0]["client"][0]["clientID"])) {
				$query =& new MBQuery("GetHelpdeskTickets", "", array("userID" => (int)$this->smartyVars["helpdeskTicket"]["clients"][0]["client"][0]["clientID"]));
				$clientResults = $query->getResult();

				$this->smartyVars["clientTickets"] = grabSimpleMBAPIResults($clientResults->getStructure());

				unset( $query, $clientResults );
			}

			if (!notNull( $this->smartyVars["helpdeskTicket"]["clients"] )) {
				// check if the contact exists in the ticket contacts list
				$query =& new MBQuery("GetHelpdeskTicketContact", "", array("hdTicketID" => $this->smartyVars["helpdeskTicket"]["hdTicketID"]));
				$contactResults = $query->getResult();

				if (!$contactResults->getErrorCount()) {
					// we did have contact info in thar!
					$contactResults = grabSimpleMBAPIResults( $contactResults->getStructure() );
					$this->smartyVars["helpdeskTicket"]["contact"] = $contactResults[0];

					// see if we can find a probable client contact
					$query =& new MBQuery("GetClients", "", array("clientContactEmail" => $contactResults[0]["hdContactEmail"], "getContactData" => 5));
					$clientResults = $query->getResult();

					if (!$clientResults->getErrorCount()) {
						// we may have one
						$clientResults = grabSimpleMBAPIResults( $clientResults->getStructure() );

						if (notNull( $clientResults)) {
							// and yes we do!
							$this->smartyVars["possibleClients"]["clients"] =& $clientResults;
							
							if (is_array($clientResults))
							foreach( $clientResults as $client ) {
								if (is_array( $client["contacts"][0] ) && notNull( $client["contacts"][0] )) {
									$this->smartyVars["possibleClients"]["bestGuess"] =& $client;
									break;
								}
							}
						}
					}
				}
			}
			
			$this->smartyVars["paginationLinks"] = $data["mbapi"][0]["results"][0]["paginationLinks"][0];
			
			dbg("############ Sent to Smarty #############");
			dbg($this->smartyVars);

			$this->addTemplate("ShowHelpdeskTicketDetails");
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowHelpdeskTicketDetails
?>