<?php
// $Id: ShowBalanceSheet.php 10474 2007-01-11 23:58:14Z kbailey $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

// Remove the time limit b/c this report can take more than 30 seconds to generate.
// A new way to generate this report may be needed soon.
set_time_limit(0);

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-modernbill/include/accounting/Accounting.php");

/**
 * This class is a child class of MB_Action. It displays the Balance Sheet
 *
 * @author  Kris Bailey
 */
class ShowBalanceSheet extends MB_Action {

	function ShowBalanceSheet() {
		parent::MB_Action();

		$this->stringRep = "ShowBalanceSheet";

		list($m, $d, $y) = explode("/", $_REQUEST["dateRange"]);
		$this->endTime = @mktime(0,0,0,$m,$d+1,$y)-1;
		unset($_REQUEST["dateRange"]);

	}

	function execute() {
		global $CONF;
		parent::execute();

	    $this->smartyVars["nowDate"] = date("m/d/Y");
	    $this->smartyVars["nowDateParts"] = explode("/", date("m/d/Y"));
	    $this->smartyVars["accountTypes"] = $CONF["ledgerCategories"];
	    $this->smartyVars["endDate"] = $this->endTime;

		// --------------------------------------------------------------
		// Will return all ledger accounts defined...
		// Example Record:
	    // 		[ledgerAccountID] => 1
	    // 		[parentLedgerAccountID] => 0
	    // 		[ledgerAccountNumber] => 10000
	    // 		[ledgerAccountType] => 1
	    // 		[ledgerAccountName] => cash
	    // 		[ledgerAccountTransKey] => TRANS_CASH
		// --------------------------------------------------------------
	    $accountsXML = "
	                <mbapi>
	                   <command>GetAccountingLedgerAccounts</command>
	                   <showXMLHeader>0</showXMLHeader>
	                   <params>
	                     <sortDir>ASC</sortDir>
	                     <sortColumn>ledgerAccountNumber</sortColumn>
	                     <recordOffset>0</recordOffset>
	                     <numRecords>-1</numRecords>
	                   </params>
	                </mbapi>
	                ";
	    $accounts = dispatchMBAPI($accountsXML, 1);
	    $this->smartyVars["ledgerAccounts"] = array();
	    if (isset($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"])){
	        foreach ($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"] as $ledgerAccount){
	            $this->smartyVars["ledgerAccounts"][$ledgerAccount["ledgerAccountID"][0]] = collapseArrayElements($ledgerAccount);
	        }
	    }

		// --------------------------------------------------------------
		// Will return all ledger entries grouped by type and summed...
		// Example Record:
		// 		[ledgerAccountID] => 4
		// 		[currencyID] => USD
		// 		[ledgerAccountType] => 1
		// 		[ledgerAccountName] => accounts_receivable
		// 		[ledgerAccountTransKey] => TRANS_ACCOUNTS_RECEIVABLE
		// 		[generalLedgerDebitAmount] => 15881990.70
		// 		[generalLedgerCreditAmount] => 15769338.54
		// --------------------------------------------------------------
	    $ledgerEntriesXML = "
	                <mbapi>
	                  <command>GetAccountingGeneralLedger</command>
	                  <showXMLHeader>0</showXMLHeader>
	                  <params>
	                    <generalLedgerDateCreatedGreatest>".$this->endTime."</generalLedgerDateCreatedGreatest>
	                    <groupByAndSum>1</groupByAndSum>
	                    <sortDir>ASC</sortDir>
	                    <sortColumn>generalLedgerID</sortColumn>
	                    <recordOffset>0</recordOffset>
	                    <numRecords>-1</numRecords>
	                  </params>
	                </mbapi>
	                ";
	    $ledgerEntries = dispatchMBAPI($ledgerEntriesXML, 1);
	    $this->smartyVars["accountTotals"] = array();
	    if (isset($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"])){
	        foreach ($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"] as $entry){
	            $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["accountInfo"] = collapseArrayElements($entry);
	            $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["credits"] += $entry["generalLedgerCreditAmount"][0];
	            $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["debits"] += $entry["generalLedgerDebitAmount"][0];
	        }
	    }

	    $this->smartyVars["ledgerTypeTotals"] = array();
	    $this->smartyVars["ledgerAccountsTotals"] = array();
	    foreach ($this->smartyVars["accountTotals"] as $currencyID => $accountTotals){
	        $this->smartyVars["ledgerTypeTotals"][$currencyID] = array();
	        foreach ($accountTotals as $accountID => $totalInfo){
	            $accountType = $CONF["ledgerCategories"][$totalInfo["accountInfo"]["ledgerAccountType"]][0];

	            switch ($accountType){

	                case "asset":
	                    $totalInfo["total"] = $totalInfo["debits"]-$totalInfo["credits"];
	                    $this->smartyVars["ledgerAccountsTotals"][$currencyID]["asset"][] = $totalInfo;
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["asset"] -= $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["asset"] += $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["debits"];
	                break;

	                case "liability":
	                    $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
	                    $this->smartyVars["ledgerAccountsTotals"][$currencyID]["liability"][] = $totalInfo;
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["liability"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["liability"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["debits"];
	                break;

	                case "equity":
	                    $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
	                    $this->smartyVars["ledgerAccountsTotals"][$currencyID]["equity"][] = $totalInfo;
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
	                break;

	                case "revenue":
	                case "otherRev":
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                break;

	                case "cogs":
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                break;

	                case "expense":
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                break;

	            }

	        }
	    }

	    ksort($this->smartyVars["ledgerTypeTotals"]);

		$this->addTemplate("accounting/ShowBalanceSheet");

	} // execute
} // MB_Action
?>
