<?php
// $Id: ShowGeneralLedger.php 10474 2007-01-11 23:58:14Z kbailey $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-modernbill/include/accounting/Accounting.php");

/**
 * This Action object collects information on all receipt journal records that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Michael Fountain
 */
class ShowGeneralLedger extends MB_Action {
	/**
	 * The constructor for a ShowGeneralLedger action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowGeneralLedger() {
		parent::MB_Action();
		$this->stringRep = "ShowGeneralLedger";

		$this->params = array
		(
			"generalLedgerDateCreated",
            "generalLedgerDateCreatedLeast",
            "generalLedgerDateCreatedGreatest"
		);

		$_REQUEST["generalLedgerDateCreatedLeast"] = $this->handleDateSelector("generalLedgerBegin", 0);
		$_REQUEST["generalLedgerDateCreatedGreatest"] = $this->handleDateSelector("generalLedgerEnd", 0);
		if ((int)$_REQUEST["generalLedgerDateCreatedGreatest"])$_REQUEST["generalLedgerDateCreatedGreatest"] += (24*60*60);
		
		// if we are searching for any possible match, set VALID params now
		if ($_REQUEST["dateQuery"][0]){
			if ((int)$_REQUEST["dateQuery"][0]>100000){
				$_REQUEST["dateQuery"][0] = date("Y/m/d", $_REQUEST["dateQuery"][0]);
			}
			list($dY,$dM,$dD) = explode("/",$_REQUEST["dateQuery"][0]);
			$_REQUEST["dateQuery"][0] = $_REQUEST["generalLedgerDateCreatedLeast"] = mktime(0,0,0,$dM,$dD,$dY);
			$_REQUEST["generalLedgerDateCreatedGreatest"] = mktime(0,0,0,$dM,$dD+1,$dY)-1;
		}
		
		$this->sortColumns = ($_REQUEST["sortColumn"])?array($_REQUEST["sortColumn"]):array("generalLedgerDateCreated");
	}

	/**
	 * The main method of this class that executes the ShowGeneralLedger action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

        $this->smartyVars = array();

		foreach (array(
			"generalLedgerBeginMonth",
			"generalLedgerBeginDay",
			"generalLedgerBeginYear",
			"generalLedgerEndMonth",
			"generalLedgerEndDay",
			"generalLedgerEndYear",
			"dateQuery") as $field){
			$this->smartyVars[$field] = $_REQUEST[$field];
		}
		$this->smartyVars["recordOffset"] = (int)$_REQUEST["recordOffset"];


		$xmlArgs = array
		(
			"generalLedgerType" => getGeneralLedgerTypeID("general"),
			"getInvoiceData" => 1
		);
		$query = $this->getMBAPIQuery("GetAccountingGeneralLedger","",$xmlArgs,1,1,1,1);
		$data = dispatchMBAPI($query, 1);

        $this->smartyVars["paginationLinks"] = collapseArrayElements($data["mbapi"][0]["results"][0]["paginationLinks"][0]);
		$results =& $data["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"];

        $this->smartyVars["currencies"] = array();
		$ledgerEntryIDs = array();
		$invoiceIDs = array();
        for ($i = 0; $i < count($results); $i++){
        	$ledgerEntryIDs[] = $results[$i]["generalLedgerID"][0];
        	$invoiceIDs[] = $results[$i]["invoiceID"][0];
            $this->smartyVars["currencies"][$results[$i]["currencyID"][0]][] = collapseArrayElements($results[$i]);
        }

////////////////// getting package names if we can for these entries
		$invoiceData = correctArray(grabResultsFromMBAPIArray2(array(
			"mbapi" => array(
				"command" => "GetInvoices",
				"params" => array(
					"invoiceID" => $invoiceIDs,
					"getInvoiceLineItemsData" => 1,
				),
			),
		)));
		$this->smartyVars["keyedInvoices"] = array();
		$this->smartyVars["lineItemsToPackages"] = array();
		if (is_array($invoiceData)){
			foreach ($invoiceData as $invoice){
				foreach ((array)$invoice["lineItems"] as $li){
					$this->smartyVars["lineItemNames"][$li["lineItemID"]] = array(
						"name" => "(".$li["packageID"].") ".$li["lineItemDescription"],
						"packageID" => $li["packageID"],
						"lineItemID" => $li["lineItemID"],
					);
				}
				$this->smartyVars["keyedInvoices"][$invoice["invoiceID"]] = $invoice;
			}
		}
		// ok, we have successfully gotten the accounting data
		
		// lets get the packages for those ledger entry ids
		$controlData = correctArray(grabResultsFromMBAPIArray2(array(
			"mbapi" => array(
				"command" => "GetAccountingControlRel",
				"params" => array(
					"groupingType" => 1,
					"controlType" => getAPITypeID("controls", "lineitems"),
					"generalLedgerID" => $ledgerEntryIDs,
				),
			),
		)));
		dbg($controlData);
		$this->smartyVars["controlData"] = $controlData;
		$this->smartyVars["ledgerPackageNames"] = array();
		foreach ((array)$controlData as $data){
			$thisLineItem = $data["controlID"];
			foreach ((array)$data["generalLedgerIDs"] as $glid){
				$this->smartyVars["ledgerPackageNames"][$glid] = $this->smartyVars["lineItemNames"][$thisLineItem];
			}
		}

////////////////// end

        $this->addTemplate("accounting/ShowGeneralLedger", $this->smartyVars);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowGeneralLedger
?>
