<?php
// $Id: ShowLedgerAccountDetails.php 10474 2007-01-11 23:58:14Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-modernbill/include/accounting/Accounting.php");

/**
 * This Action object collects information on a ledger account, and
 * generates the display of that information.
 *
 * @author  Michael Fountain
 */
class ShowLedgerAccountDetails extends MB_Action {
	/**
	 * The constructor for a ShowLedgerAccountDetails action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowLedgerAccountDetails() {
		parent::MB_Action();
		$this->stringRep = "ShowLedgerAccountDetails";
		$this->sortColumns = array("ledgerAccountNumber");

        $this->params = array
        (
            "ledgerAccountID",
            "invoiceID",
            "invoiceNumber",
            "clientID",

            "clientLastName",
            "clientFirstName",
            "generalLedgerDescription",

            "generalLedgerDateCreated",
            "generalLedgerDateCreatedGreaterThan",
            "generalLedgerDateCreatedLessThan",
            "generalLedgerDateCreatedGreatest",
            "generalLedgerDateCreatedLeast",

            "generalLedgerDebitAmount",
            "generalLedgerDebitAmountGreaterThan",
            "generalLedgerDebitAmountLessThan",
            "generalLedgerDebitAmountGreatest",
            "generalLedgerDebitAmountLeast",

            "generalLedgerCreditAmount",
            "generalLedgerCreditAmountGreaterThan",
            "generalLedgerCreditAmountLessThan",
            "generalLedgerCreditAmountGreatest",
            "generalLedgerCreditAmountLeast",

            "currencyID"

        );

        // handle date searches and correct mismatch of amount labels
        $this->setupDateRequestArgs();


        // search in a specific column
        if ($_REQUEST["queryColumn"]!="") {
            if ($_REQUEST["queryColumn"]=="invoiceID"){
                $_REQUEST["queryColumn"] = "invoiceNumber";
            }
            if ($_REQUEST["queryGreatest"]!="") {
                $_REQUEST[$_REQUEST["queryColumn"]."Greatest"] = $_REQUEST["queryGreatest"];
            }
            if ($_REQUEST["queryLeast"]!="") {
                $_REQUEST[$_REQUEST["queryColumn"]."Least"] = $_REQUEST["queryLeast"];
            }
            if ($_REQUEST["query"]!="") {
                $_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
            }
        } else if ($_REQUEST["query"]!="") {
            foreach($this->params as $value) {
                $_REQUEST[$value] = $_REQUEST["query"];
            }
        }

        // handle amount searches and correct mismatch of amount labels
        $this->setupAmountRequestArgs();



	}

	/**
	 * The main method of this class that executes the ShowLedgerAccounts action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

        $this->smartyVars = array();
        $this->smartyVars["accountTypes"] = $CONF["ledgerCategories"];

		$query = $this->getMBAPIQuery("GetAccountingLedgerAccounts", array("ledgerAccountID"=>(int)$_REQUEST["ledgerAccountID"]));
		$CONF["debug"]->add("Action Command (ShowLedgerAccountDetails:GetAccountingLedgerAccounts)", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result (ShowLedgerAccountDetails:GetAccountingLedgerAccounts)", $data);

		$results =& $data["mbapi"][0]["results"][0];
        $this->smartyVars["ledgerAccounts"] = collapseArrayElements($results["ledger_accounts"][0]["account"][0]);


        $xmlParams = array
        (
            "getAvailableClientData" => 1,
            "getAvailableInvoiceData" => 1
        );
        $ledgerDataXML = $this->getMBAPIQuery("GetAccountingGeneralLedger", "",  $xmlParams, 1, 1, 1, 1);
        $CONF["debug"]->add("Action Command (ShowLedgerAccountDetails:GetAccountingGeneralLedger)", htmlentities($ledgerDataXML));
        $ledgerDataReturn = dispatchMBAPI($ledgerDataXML, 1);
        $CONF["debug"]->add("Action Result (ShowLedgerAccountDetails:GetAccountingLedgerAccounts)", $ledgerDataReturn);

        $ledgerData = correctArray($ledgerDataReturn["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"]);

        if ($ledgerData["clientID"]){
            $ledgerData = array($ledgerData);
        }
        $this->smartyVars["ledgerData"] = $ledgerData;
        $this->smartyVars["paginationLinks"] = collapseArrayElements($ledgerDataReturn["mbapi"][0]["results"][0]["paginationLinks"][0]);

        $this->addTemplate("accounting/ShowLedgerAccountDetails", $this->smartyVars);


		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowLedgerAccountDetails
?>