<?php
// $Id: ShowReceiptsJournal.php 10474 2007-01-11 23:58:14Z kbailey $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-modernbill/include/accounting/Accounting.php");

/**
 * This Action object collects information on all receipt journal records that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Michael Fountain
 */
class ShowReceiptsJournal extends MB_Action {
	/**
	 * The constructor for a ShowReceiptsJournal action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowReceiptsJournal() {
		parent::MB_Action();
		$this->stringRep = "ShowReceiptsJournal";

		$this->params = array
		(
			"receiptDateCreated",
			"receiptDateCreatedLeast",
			"receiptDateCreatedGreatest"
		);

		$_REQUEST["receiptDateCreatedLeast"] = $this->handleDateSelector("salesJournalBegin", 0);
		$_REQUEST["receiptDateCreatedGreatest"] = $this->handleDateSelector("salesJournalEnd", 0);
		if ((int)$_REQUEST["receiptDateCreatedGreatest"])$_REQUEST["receiptDateCreatedGreatest"] += (24*60*60);
		
		// if we are searching for any possible match, set VALID params now
		if ($_REQUEST["dateQuery"][0]) {
			if ((int)$_REQUEST["dateQuery"][0]>100000){
				$_REQUEST["dateQuery"][0] = date("Y/m/d", $_REQUEST["dateQuery"][0]);
			}
			list($dY,$dM,$dD) = explode("/",$_REQUEST["dateQuery"][0]);
			$_REQUEST["dateQuery"][0] = $_REQUEST["receiptDateCreatedLeast"] = mktime(0,0,0,$dM,$dD,$dY);
			$_REQUEST["receiptDateCreatedGreatest"] = mktime(23,59,59,$dM,$dD,$dY);
		}
		

		$this->sortColumns = ($_REQUEST["sortColumn"])?array($_REQUEST["sortColumn"]):array("receiptDateCreated");
	}

	/**
	 * The main method of this class that executes the ShowReceiptsJournal action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

        $this->smartyVars = array();
		
		foreach (array(
			"salesJournalBeginMonth",
			"salesJournalBeginDay",
			"salesJournalBeginYear",
			"salesJournalEndMonth",
			"salesJournalEndDay",
			"salesJournalEndYear") as $field){
			$this->smartyVars[$field] = $_REQUEST[$field];
		}
		$this->smartyVars["recordOffset"] = (int)$_REQUEST["recordOffset"];

        $this->smartyVars["currencies"] = array();
        $this->smartyVars["currencies_totals"] = array();

		$query = $this->getMBAPIQuery("GetAccountingReceiptsJournal", "", array(), 1, 1, 1, 1);
		$data = dispatchMBAPI($query, 1);

        $this->smartyVars["paginationLinks"] = collapseArrayElements($data["mbapi"][0]["results"][0]["paginationLinks"][0]);

		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {

            $results = collapseArrayElements($results["accounting_receipts"][0]["receipt"]);
			if($data["mbapi"][0]["header"][0]["numResults"][0] == 1) {
				$results = array(0 => collapseArrayElements($results["accounting_receipts"][0]["receipt"]));
			}

			$numRows = $data["mbapi"][0]["header"][0]["numResults"][0];
            for ($i = 0; $i < $numRows; $i++){

                $this->smartyVars["currencies"][$results[$i]["currencyID"]][] = $results[$i];
                $r =& $results[$i];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptCashDebitAmount"] += $r["receiptCashDebitAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptCashCreditAmount"] += $r["receiptCashCreditAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptAccRecDebitAmount"] += $r["receiptAccRecDebitAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptAccRecCreditAmount"] += $r["receiptAccRecCreditAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptSalesDebitAmount"] += $r["receiptSalesDebitAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptSalesCreditAmount"] += $r["receiptSalesCreditAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptTaxPayDebitAmount"] += $r["receiptTaxPayDebitAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptTaxPayCreditAmount"] += $r["receiptTaxPayCreditAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptCouponDebitAmount"] += $r["receiptCouponDebitAmount"];
                $this->smartyVars["currencies_totals"][$results[$i]["currencyID"]]["receiptCouponCreditAmount"] += $r["receiptCouponCreditAmount"];

            }

            ksort($this->smartyVars["currencies"]);

			$this->addTemplate("accounting/ShowReceiptsJournal", $this->smartyVars);

		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowReceiptsJournal
?>
