<?php
// $Id: ShowClients.php 10474 2007-01-11 23:58:14Z kbailey $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

/**
 * This class is a child class of MB_Action. It lists all clients that match any of the input params.
 *
 * @author  Michael Fountain
 */
class ShowClients extends MB_Action {
	/**
	 * The constructor for a ShowClients action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowClients() {
		parent::MB_Action();
		$this->stringRep = "ShowClients";

		// set all possible xml params here, $_GET is already cleaned
		$this->params = array
		(
			"clientID",
			"clientCompany",
			"contactFirstName",
			"contactLastName",
			"contactEmail",
			"contactAddress1",
			"contactAddress2",
			"contactCity",
			"contactZip",
			"contactUsername",
			"billingAccountNumLastFour"
		);

		// if we are searching for any possible match, set VALID params now
		if ($_REQUEST["query"]!="") {
			// test if the searching for a specific column
			if ($_REQUEST["queryColumn"]!="") {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			} else {
				// test if query is an int, if yes, assume we are searching for the ID
				if (((int) $_REQUEST["query"])) {
					$_REQUEST["clientID"] = $_REQUEST["query"];
				} else {
					// if not an int, then search all other non-ID columns
					foreach($this->params as $value) {
						if ($value!="clientID") {
							$_REQUEST[$value] = $_REQUEST["query"];
						}
					}
				}
			}
		}

		// add filtered columns to $this->params LAST, they are NOT searchable
		if(isset($_REQUEST["clientStatus"]) && is_numeric($_REQUEST["clientStatus"])) $this->params[] = "clientStatus";
		if(isset($_REQUEST["clientActive"]) && is_numeric($_REQUEST["clientActive"])) $this->params[] = "clientActive";

		$this->sortColumns = array
		(
			"clientID",
			"contactFirstName",
			"contactLastName",
			"clientCompany",
			"contactEmail",
			"countriesISO2"
		);

	}

	/**
	 * The main method of this class that executes the ShowClients action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

 		$getMoreData = array
		(
			"mergePrimaryContact" => 1,
			//"getAccountingData" => 1,
			//"getContactData" => 3, // (0=None, 1=All Data, 2=Sum by Count, 3=Single Record by ContactID)
			//"getPackageData" => 1, // (0=None, 1=All Data, 2=Sum by Status, 3=Sum by Billing Cycle, 4=Sum by Product ID)
			//"getDomainData" => 1, // (0=None, 1=All Data, 2=Sum by Status, 3=Sum by TLD)
			//"getInvoiceData" => 1,
			//"getCertData" => 1,
			//"getCreditData" => 1,
			//"getClientGroupData" => 1,
			//"getBillingData" => 1, // (0=None, 1=All Data, 2=Sum by Count, 3=Single record by billingAccountID)
			//"getBookmarkData" => 1,
			//"getShareData" => 1,
			//"getCompanyData" => 1,
			"getClientLastAccess" => 1,
			//"getTicketData" => 3, // (0=None, 1=All Data, 2=Last 5, 3=Last)
			"getNoteData" => 3, // (0=None, 1=All Data, 2=Last 5, 3=Last)
		);
		
		$query = $this->getMBAPIQuery("GetClients", "", $getMoreData, 1, 1, 1, 1);
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];

		// redirect to a single client record if only one record is returned
		if(is_array($results) && $data["mbapi"][0]["header"][0]["numResults"][0] == 1){
			redirect("clients.php","ShowClientDetails", array( "clientID" => $results["clients"][0]["client"][0]["clientID"][0]));
		}

		if($this->addMBAPIErrors($data) === 0) {

			// set the sorting args for display
			$this->smartyVars["displaySortColumnName"] = $data["mbapi"][0]["header"][0]["sortColumnName"][0];
			$this->smartyVars["displaySortColumn"] = $data["mbapi"][0]["header"][0]["sortColumn"][0];
			$this->smartyVars["displaySortDir"] = $data["mbapi"][0]["header"][0]["sortDir"][0];

			// set the client data for display
			if(isset($results["clients"][0]["client"])) {
				$clients = correctArray($results["clients"][0]["client"]);
			} else {
				$clients = array();
			}
			$this->smartyVars["clients"] = $clients;
			
			$noteList = array();
			$c = count($clients);
			for($i = 0; $i < $c; $i++) {
				// leave this uncollapsed so that the template code can work with both singular and plural results; see the comment in the template for details
				$noteData =& $results["clients"][0]["client"][$i]["notes"][0]["note"];
				$noteList[$results["clients"][0]["client"][$i]["clientID"][0]] = collapseArrayElements(protectArray($noteData));

/* 
// This data should be gathered from the GetClients command itself, not here.
// Doing it here will run this MBAPI command 20+ times.
// -MF
				$getHdTickets =& new MBQuery("GetHelpdeskTickets", "", array(
										"userType" => 1,
										"userID" => $results["clients"][0]["client"][ $i ]["clientID"][0]),
											array(
										"sortColumn" => "hdTicketDateModified",
										"sortDir" => "desc",
										"numRecords" => 1));
				$hdTicket = $getHdTickets->getResult();
				$hdTicket = grabSimpleMBAPIResults( $hdTicket->getStructure() );
				$this->smartyVars["clients"][ $i ]["helpdeskTicket"] = $hdTicket[0];

				unset($hdTicket, $getHdTickets);
*/

			}

			$this->smartyVars["noteList"] = $noteList;

			// assign filtered columns
			if (isset($_REQUEST["clientStatus"]) && is_numeric($_REQUEST["clientStatus"])) {
				$this->smartyVars["clientStatus"] = ((int) $_REQUEST["clientStatus"]);
			}
			if (isset($_REQUEST["clientActive"]) && is_numeric($_REQUEST["clientActive"])) {
				$this->smartyVars["clientActive"] = ((int) $_REQUEST["clientActive"]);
			}

			$this->smartyVars["clientStatuses"] = getStatuses("client");
			$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"][0]);
			$this->addTemplate("clients/ShowClients", $this->smartyVars);

		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	} // execute
} // MB_Action
?>
