<?php
// $Id: EditDomainContactDataResponse.php 11227 2007-05-31 23:28:25Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This Action object takes information provided by the EditDomainContactData action, applies that information to 
 * records in the database, and redirects to the ShowDomainContactDetails action for the given domain.
 *
 * @author  Michael Fountain
 */
class EditDomainContactDataResponse extends MB_Action {
	/**
	 * The constructor for an EditDomainContactDataResponse action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function EditDomainContactDataResponse() {
		parent::MB_Action();
		$this->stringRep = "EditDomainContactDataResponse";
		$this->params = array
		(
			"domainID",

			"registrantAddress1", "registrantAddress2", "registrantCity", "registrantCountry",
			"registrantEmailAddress", "registrantFax", "registrantFirstName", "registrantJobTitle",
			"registrantLastName", "registrantOrganizationName", "registrantPhone", "registrantPhoneExt",
			"registrantPostalCode", "registrantStateProvince", "registrantStateProvinceChoice",

			"auxAddress1", "auxAddress2", "auxCity", "auxCountry", "auxEmailAddress", "auxFax",
			"auxFirstName", "auxJobTitle", "auxLastName", "auxOrganizationName", "auxPhone",
			"auxPhoneExt", "auxPostalCode", "auxStateProvince", "auxStateProvinceChoice",

			"techAddress1", "techAddress2", "techCity", "techCountry", "techEmailAddress",
			"techFax", "techFirstName", "techJobTitle", "techLastName", "techOrganizationName",
			"techPhone", "techPhoneExt", "techPostalCode", "techStateProvince",
			"techStateProvinceChoice",

			"adminAddress1", "adminAddress2", "adminCity", "adminCountry", "adminEmailAddress",
			"adminFax", "adminFirstName", "adminJobTitle", "adminLastName", "adminOrganizationName",
			"adminPhone", "adminPhoneExt", "adminPostalCode", "adminStateProvince",
			"adminStateProvinceChoice",

			"billingPartyID", "billingAddress1", "billingAddress2", "billingCity",
			"billingCountry", "billingEmailAddress", "billingFax", "billingFirstName",
			"billingJobTitle", "billingLastName", "billingOrganizationName", "billingPhone",
			"billingPhoneExt", "billingPostalCode", "billingStateProvince", "billingStateProvinceChoice",
			"billingFullCountry", "useBelowAuxInfo"
		);
	}

	/**
	 * The main method of this class that executes the EditDomainResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$domainID = ((int) $_REQUEST["domainID"]);
		if(!$domainID) {
			$this->addError(TRANS_ACTIONERROR, "No domain ID given.");
		} else {

		 	// fetch this domains information
			$query = $this->getMBAPIQuery("GetDomains", "", array("packageType" => getTypeID("products",TRANS_DOMAIN_PRODUCT)));
			$mbapiData = dispatchMBAPI($query, 1);

			// is the domainID valid?
			if ($mbapiData["mbapi"][0]["header"][0]["numResults"][0]!=1) {
				$this->addError(TRANS_ACTIONERROR, "Invalid domain ID.");
			} else {

				// fetching domain data from the registrar
				$domainSLD = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainSLD"][0];
				$domainTLD = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainTLD"][0];
				$domainID = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainID"][0];

				$contactXML = "\n";
				$contactData = array("registrant","admin","tech","auxBilling");

				foreach($contactData as $contactType) {

					// clean up state/province data
					if (eregi("S",$_REQUEST[$contactType]["StateProvinceChoice"])) {
						$_REQUEST[$contactType]["StateProvince"] = $_REQUEST[$contactType]["State"];
					} else if (eregi("P",$_REQUEST[$contactType]["StateProvinceChoice"])) {
						$_REQUEST[$contactType]["StateProvince"] = $_REQUEST[$contactType]["Province"];
					} else {
						$_REQUEST[$contactType]["StateProvince"] = "";
					}

					// remove variables not needed by registrars
					unset($_REQUEST[$contactType]["State"],$_REQUEST[$contactType]["Province"]);

					// build XML command
					if (is_array($_REQUEST[$contactType])) {
					 	$contactXML .= "                              <$contactType>\n";
						foreach($_REQUEST[$contactType] as $key => $value) {
							$contactXML .= "                                <$contactType$key>$value</$contactType$key>\n";
						}
						$contactXML .=	"                              </$contactType>\n";
					}
				}
				$contactXML .= "\n";

				$xml = "
				<mbapi>
				  <command>ProcessRegistrar</command>
				  <showXMLHeader>0</showXMLHeader>
				  <params>
					<subCommand>setDomainContactData</subCommand>
					<configGroupName>0</configGroupName>
					<configID>0</configID>
					<domainID>".$domainID."</domainID>
					<domainSLD>".$domainSLD."</domainSLD>
					<domainTLD>".$domainTLD."</domainTLD>
					<domainContactData>$contactXML
					</domainContactData>
				</params>
				</mbapi>";
				$data = dispatchMBAPI($xml,1);

				if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					redirect("domains.php",getActionID("ShowDomainDetails"), array("domainID"=>$domainID));
				}

			} // if ($mbapiData["mbapi"][0]["header"][0]["numResults"][0]!=1)

		} // if(!$domainID)

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	} // execute

} // EditDomainResponse
?>