<?php
// $Id: MigrateProcessCommon.php 12712 2008-02-28 23:22:49Z bwoolley $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

set_time_limit(0);
ignore_user_abort();

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-tk/include/io/hash.php");

class MigrateProcessCommon extends SourceAction {
	var $cycleIDCache;
	
	function MigrateProcessCommon() {
		$this->stringRep = "MigrateProcessCommon";
		$this->cycleIDCache =& new PersistentHash("cycles");

		$this->oneTimeCycleMonthMap = array(
			100 => 1,
			103 => 3,
			106 => 6,
			112 => 12,
		);
		$this->domainCycleYearMap = array(
			111 => 1,
			124 => 2,
			136 => 3,
			148 => 4,
			160 => 5,
			172 => 6,
			184 => 7,
			196 => 8,
			1108 => 9,
			1120 => 10, 
		);
		$this->domainYearCycleMap = array_flip($this->domainCycleYearMap);
		$this->registrarNameMap = array(
			2 => "mod_opensrs",
			3 => "mod_emailregister",
			4 => "mod_enom",
			9 => "mod_nominet",
			11 => "mod_onlinenic",
			12 => "mod_srsplus",
			14 => "mod_registercom",
		);
		$this->registrarIDMap = array_flip($this->registrarNameMap);

	}

	function getCycleType($cycle) {
		if($this->isDomainCycle($cycle)) {
			return "domain";
		} else if($this->isOneTimeCycle($cycle)) {
			return "onetime";
		} else {
			return "recurring";
		}
	}

	function isOneTimeCycle($cycle) {
		return isset($this->oneTimeCycleMonthMap[$cycle]);
	}

	function getOneTimeCycleMonth($cycle) {
		return $this->oneTimeCycleMonthMap[$cycle];
	}

	function getOneTimeCycleList() {
		return "(" . implode(", ", array_keys($this->oneTimeCycleMonthMap)) . ")";
	}

	function isDomainCycle($cycle) {
		return isset($this->domainCycleYearMap[$cycle]);
	}

	function getDomainCycleYear($cycle) {
		return $this->domainCycleYearMap[$cycle];
	}
	
	function getDomainCycleList() {
		return "(" . implode(", ", array_keys($this->domainCycleYearMap)) . ")";
	}

	function splitPrice($price, $type, $cycle) {
		show("Splitting price: $price");
		$price_array = explode("|", $price);
		switch($type) {
			case 'price': // PRICE
				switch($cycle) { // MM.MM|QQ.QQ|SS.SS|YY.YY
					case 1:  $this_price = $price_array[0]; break;
					case 3:  $this_price = $price_array[1]; break;
					case 6:  $this_price = $price_array[2]; break;
					case 12: $this_price = $price_array[3]; break;
					case 24: $this_price = $price_array[4]; break;
					case 36: $this_price = $price_array[5]; break;
					default: $this_price = $price_array[0]; break;
				}
				if ($this_price == 0.001) {
					show("Found: Free");
    				return 0;
				} else {
					show("Found: " . (($this_price) ? $this_price : $price_array[0]));
    				return ($this_price) ? $this_price : $price_array[0] ;
				}
			break;

			case 'setup': // SETUP
    			switch($cycle) { // MM.MM|QQ.QQ|SS.SS|YY.YY
					case 1:  $this_price = $price_array[0]; break;
					case 3:  $this_price = $price_array[1]; break;
					case 6:  $this_price = $price_array[2]; break;
					case 12: $this_price = $price_array[3]; break;
					case 24: $this_price = $price_array[4]; break;
					case 36: $this_price = $price_array[5]; break;
					default: $this_price = $price_array[0]; break;
    			}
				show("Found: $this_price");
    			return $this_price;
			break;

			case 'domain': // DOMAIN
    			switch($cycle) { // 1Y|2Y|3Y|4Y|5Y|6Y|7Y|8Y|9Y|10Y
					case 1:  $this_price = $price_array[0]; break;
					case 2:  $this_price = $price_array[1]; break;
					case 3:  $this_price = $price_array[2]; break;
					case 4:  $this_price = $price_array[3]; break;
					case 5:  $this_price = $price_array[4]; break;
					case 6:  $this_price = $price_array[5]; break;
					case 7:  $this_price = $price_array[6]; break;
					case 8:  $this_price = $price_array[7]; break;
					case 9:  $this_price = $price_array[8]; break;
					case 10: $this_price = $price_array[9]; break;
					default: $this_price = $price_array[0]; break;
    			}
    			if ($this_price==0.001) {
					show("Found: Free");
        			return 0;
    			} else {
					show("Found: " . (($this_price) ? $this_price : $price_array[0]));
        			return ($this_price) ? $this_price : $price_array[0] ;
    			}
			break;

			default:
				show("Found: " . $price_array[0]);
    			return $price_array[0];
			break;
		}
	}

	function getPrice($price, $cycleID, $type = NULL) {
		
		if($type === NULL) $type = "general";

		$type = getAPITypeID("price", $type);
		$price = (float) $price;
		if($price == 0.001 || $price == 0.0001) $price = 0.0;
		return array(
			"productVariantPriceType" => $type,
			"cycleID" => $cycleID,
			"currencyID" => $this->currencyID,
			"productVariantPriceAmount" => $price,
		);
	}

	function getYearsFromDomainCycle($extCycleCode) {
		switch($extCycleCode) {
			case 111:	$extCycleYears = 1;	break;
			case 124:	$extCycleYears = 2;	break;
			case 136:	$extCycleYears = 3;	break;
			case 148:	$extCycleYears = 4;	break;
			case 160:	$extCycleYears = 5;	break;
			case 172:	$extCycleYears = 6;	break;
			case 184:	$extCycleYears = 7;	break;
			case 196:	$extCycleYears = 8;	break;
			case 1108:	$extCycleYears = 9;	break;
			case 1120:	$extCycleYears = 10;	break;
			default:	$extCycleYears = 1;	break;
		}
		return $extCycleYears;
	}

	function getCycleID($extCycleInfo, $productType = NULL, $getMonth = false) {
		show("Getting cycle for $extCycleInfo.");
		$isDomain = $this->isDomainCycle($extCycleInfo);
		$key = $extCycleInfo . "|" . $isDomain;
		
		if($this->cycleIDCache->exists($key)) {
			$c = explode(":", $this->cycleIDCache->get($key));
			show("found in cache: " . $c[0]);
			if($getMonth) {
				return $c;
			} else {
				return $c[0];
			}
		}
		
		if($isDomain) {
			$extCycleCode = $extCycleInfo;	
			$extCycleYears = $this->getYearsFromDomainCycle($extCycleCode);

			if(!$intCycle = oneOf(
				"cycles",
				array(
					"cycleNumSeconds" => 0,
					"cycleNumMinutes" => 0,
					"cycleNumHours" => 0,
					"cycleNumDays" => 0,
					"cycleNumMonths" => 0,
					"cycleNumYears" => $extCycleYears,
				)
			)) {
				$intCycle = oneOf(
					"cycles",
					array(
						"cycleNumSeconds" => 0,
						"cycleNumMinutes" => 0,
						"cycleNumHours" => 0,
						"cycleNumDays" => 0,
						"cycleNumMonths" => $extCycleYears * 12,
						"cycleNumYears" => 0,
					)
				);
			}
		} else {
			switch($extCycleMonths = $extCycleInfo) {
				case 103:
					$oneTime = true;
					$extCycleDays = 0;
					$extCycleMonths = 3;
					$extCycleYears = 0;
					break;
				case 106:
					$oneTime = true;
					$extCycleDays = 0;
					$extCycleMonths = 6;
					$extCycleYears = 0;
					break;
				case 112:
					$oneTime = true;
					$extCycleDays = 0;
					$extCycleMonths = 0;
					$extCycleYears = 1;
					break;
				case 100:
					$oneTime = true;
					$extCycleDays = 1;
					$extCycleMonths = 0;
					$extCycleYears = 0;
					break;
				default:
					$oneTime = false;
					$extCycleDays = 0;
					$extCycleYears = 0;
					break;
			}

			if($extCycleMonths) {
				if(!$intCycle = oneOf(
					"cycles",
					array(
						"cycleNumSeconds" => 0,
						"cycleNumMinutes" => 0,
						"cycleNumHours" => 0,
						"cycleNumDays" => 0,
						"cycleNumMonths" => $extCycleMonths,
						"cycleNumYears" => 0,
					)
				)) {
					$intCycle = oneOf(
						"cycles",
						array(
							"cycleNumSeconds" => 0,
							"cycleNumMinutes" => 0,
							"cycleNumHours" => 0,
							"cycleNumDays" => 0,
							"cycleNumMonths" => 0,
							"cycleNumYears" => $extCycleMonths / 12,
						)
					);
				}
			} else if($extCycleYears) {
				if(!$intCycle = oneOf(
					"cycles",
					array(
						"cycleNumSeconds" => 0,
						"cycleNumMinutes" => 0,
						"cycleNumHours" => 0,
						"cycleNumDays" => 0,
						"cycleNumMonths" => 0,
						"cycleNumYears" => $extCycleYears,
					)
				)) {
					$intCycle = oneOf(
						"cycles",
						array(
							"cycleNumSeconds" => 0,
							"cycleNumMinutes" => 0,
							"cycleNumHours" => 0,
							"cycleNumDays" => 0,
							"cycleNumMonths" => $extCycleYears * 12,
							"cycleNumYears" => 0,
						)
					);
				}
			} else if($extCycleDays) {
				$intCycle = oneOf(
					"cycles",
					array(
						"cycleNumSeconds" => 0,
						"cycleNumMinutes" => 0,
						"cycleNumHours" => 0,
						"cycleNumDays" => $extCycleDays,
						"cycleNumMonths" => 0,
						"cycleNumYears" => 0,
					)
				);
			}
		}
		$a = array($intCycle["cycleID"], $extCycleInfo);
		$this->cycleIDCache->set($key, $r = implode(":", $a));
		show("Found in database: " . $r);
		if($getMonth) {
			return $a;
		} else {
			return $a[0];
		}
		
	}

}
