<?php
// $Id: ShowMBRequirementsTest.php 10891 2007-03-15 21:56:59Z bsmith $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This Action object displays the modules menu.
 */
class ShowMBRequirementsTest extends MB_Action {
	/**
	 * The constructor for a ShowMBRequirementsTestMenu action object. This method calls the parent constructor,
	 * and sets the string representation of this object.
	 */
	function ShowMBRequirementsTest() {
		parent::MB_Action();
		$this->stringRep = "ShowMBRequirementsTest";
	}

	/**
	 * The main method of this class that executes the ShowMBRequirementsTestMenu action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$this->smartyVars['requirements'] =& system_requirements();
		$this->addTemplate("modules/ShowMBRequirementsTest");

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowMBRequirementsTest



/**
 * THIS IS THE REQUIREMENTS TEST SCRIPT BELOW.
 * ANY REVISIONS  TO THE REQUIREMENTS SCRIPT
 * SHOULD BE REFLECTED HERE.
 */

define("REQUIREMENT_MUST", 1);
define("REQUIREMENT_SHOULD", 2);
define("REQUIREMENT_MAY", 3);

function cURLpath() {
	global $CONF;
	if(isset($CONF["cliCurlPath"])) {
		return $CONF["cliCurlPath"];
	} else {
		return "/usr/bin/env curl";
	}
}

class Condition {
	var $description;
	var $evaluated;
	var $satisfied;

	function Condition($description) {
		$this->description = $description;
		$this->evaluated = false;
		$this->satisfied = NULL;
	}

	function getName($checkbox) {
		return $this->description . ($checkbox ? (" " . $this->checkbox()) : "");
	}

	function satisfy() {
		return $this->satisifed = true;
	}

	function checkbox() {
		$satisfied = $this->getSatisfaction();
		if($satisfied === NULL) {
			$determination = "unknown";
		} else if($satisfied) {
			$determination = "you do";
		} else {
			$determination = "you do <em>not</em>";
		}
		return "[${determination}]";
	}

	function getSatisfaction() {
		if(!$this->evaluated) {
			$this->satisified = $this->satisfy();
			$this->evaluated = true;
		}
		return $this->satisified;
	}
}

class XMLWhiteSpaceCondition extends Condition {
	function XMLWhiteSpaceCondition() {
		Condition::Condition("an XML parser that preserves whitespace");
	}

	function satisfy() {
		if(!function_exists("xml_parser_create")) {
			return false;
		}

		$val = "My\ntest\nvalue";
		$xml = "<testNode>${val}</testNode>";

		$parser = xml_parser_create("UTF-8");
		xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
		xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);

		$elements = array();
		xml_parse_into_struct($parser, $xml, $elements);

		if(isset($elements[0]["value"])) {
			$outVal = $elements[0]["value"];
			if($outVal === $val) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
}

class LicenseServerCondition extends Condition {
	function LicenseServerCondition() {
		Condition::Condition("the ability to connect to the license server (www.mbapi.com port 80)");
	}

	function satisfy() {
		$sock = fsockopen("www.mbapi.com", 80, $errno = 0, $errstr = "", 20);
		if(!$sock) return false;
		$msg = "GET / HTTP/1.1\r\nHost: www.mbapi.com\r\nConnection: close\r\n\r\n";
                while ($msg) {
                        $sent = fwrite($sock, $msg);
                        if($sent === false) return false;
                        $msg = substr($msg, $sent);
                }

                $buffer = "";
                while (!feof($sock)) {
			$s = fread($sock, 8192);
			if($s === false) return false;
                        $buffer .= $s;
                }

                fclose($sock);

		return true;
	}
}

class WritableTempCondition extends Condition {
	function WritableTempCondition() {
		Condition::Condition("a writable temporary folder");
	}

	function satisfy() {
		$windows = strpos(strtolower(PHP_OS), "win") === 0;

		if($windows) {
			if(isset($CONF["tempDirectory"])) {
				$tempDir = $CONF["tempDirectory"];
			} else {
				$tempDir = ($TMP = getenv("TMP"))
					? $TMP
					: "c:\\windows\\temp";
			}
			$tempFile = $tempDir . "\\test123.txt";
		} else {
			if(isset($CONF["tempDirectory"])) {
				$tempDir = $CONF["tempDirectory"];
			} else {
				$tempDir = ($tempFile = tempnam("","inst"))
					? dirname($tempFile)
					: "/tmp";
			}
			$tempFile = $tempDir . "/test123.txt";
		}

		if(!$f = fopen($tempFile, "w")) return false;
		if(fwrite($f, "test") === false) return false;
		if(fflush($f) === false) return false;
		if(fclose($f) === false) return false;
		if(!$f = fopen($tempFile, "r")) return false;
		if(fread($f, 4) === false) return false;
		if(fclose($f) === false) return false;
		if(unlink($tempFile) === false) return false;

		return true;

	}
}

class CommandCondition extends Condition {
	var $command;
	var $retval;

	function CommandCondition($description, $command, $retval = 0) {
		Condition::Condition("the command \"${command}\" for ${description}");
		$this->command = $command;
		$this->retval = $retval;
	}

	function satisfy() {
		@exec($this->command, $data = NULL, $retval = 1);
		return $retval == $this->retval;
	}
}

class CLICurlCondition extends Condition {
	function CLICurlCondition($description) {
		Condition::Condition("SSL-enabled CLI cURL for ${description}");
	}

	function satisfy() {
		if(strpos(strtolower(ini_get("safe_mode")), "on") !== false) {
			return NULL;
		} else {
			$command = cURLpath() . " --version --stderr -";
			$line = exec($command);
			if(strpos(strtolower($line), "ssl") !== false) {
				return true;
			} else {
				return false;
			}
		}
	}
}

class PHPCurlCondition extends Condition {
	function PHPCurlCondition($description) {
		Condition::Condition("SSL-enabled PHP cURL for ${description}");
	}

	function satisfy() {
		if(function_exists("curl_version")) {
			$info = curl_version();
			if(is_array($info) && in_array("https", $info["protocols"]) || strpos(strtolower($info), "ssl") !== false) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
}

class PHPCondition extends Condition {
	var $version;
	var $op;

	function PHPCondition($description, $op, $version) {
		Condition::Condition("PHP version ${op} ${version} for ${description}");
		$this->version = $version;
		$this->op = $op;
	}

	function satisfy() {
		return version_compare(phpversion(), $this->version, $this->op);
	}
}

class INICondition extends Condition {
	var $condition_description;

	function INICondition($description, $setting, $condition_description) {
		Condition::Condition("the php.ini setting \"${setting}\" ${condition_description} for ${description}");
		$this->setting = $setting;
		$this->condition_description = $condition_description;
	}

	function conditional($setting) {
		return true;
	}

	function satisfy() {
		return $this->conditional(ini_get($this->setting));
	}
}

class INIPatternCondition extends INICondition {
	var $pattern;
	var $match;

	function INIPatternCondition($description, $setting, $condition_description, $pattern, $match = true) {
		$qualifier = $match ? "" : "not ";
		INICondition::INICondition($description, $setting, $condition_description . " (does ${qualifier}match pattern \"${pattern}\")");
		$this->pattern = $pattern;
		$this->match = $match;
	}

	function conditional($setting) {
		if(function_exists("preg_match")) {
			return ((bool) preg_match($this->pattern, $setting)) === (bool) $this->match;
		} else {
			return NULL;
		}
	}
}

class SafeModeCondition extends INICondition {
	var $enabled;

	function SafeModeCondition($description, $enabled = true) {
		INICondition::INICondition($description, "safe_mode", $enabled ? "enabled" : "disabled");
		$this->enabled = $enabled;
	}

	function conditional($setting) {
		return (strpos(strtolower($setting), "on") !== false) === $this->enabled;
	}
}

class INIUnsetCondition extends INICondition {
	function INIUnsetCondition($description, $setting) {
		INICondition::INICondition($description, $setting, "unset");
	}

	function conditional($setting) {
		return $setting == "";
	}
}

class ININumericCondition extends INICondition {
	var $op;
	var $value;

	function ININumericCondition($description, $setting, $op, $value) {
		INICondition::INICondition($description, $setting, $op . " " . $value);
		$this->op = $op;
		$this->value = $value;
	}

	function conditional($setting) {
		return $this->compare_bytes($setting, $this->op, $this->value);
	}

	function return_bytes($val) {
		$val = trim($val);
		$last = strtolower($val{strlen($val)-1});
		switch($last) {
			// The 'G' modifier is available since PHP 5.1.0
			case 'g':
				$val *= 1024;
			case 'm':
				$val *= 1024;
			case 'k':
				$val *= 1024;
		}

		return $val;
	}

	function compare_bytes($b1, $op, $b2) {
		switch($op) {
			case "<";
			case "lt";
				return $this->return_bytes($b1) < $this->return_bytes($b2);
			case "<=";
			case "le";
				return $this->return_bytes($b1) <= $this->return_bytes($b2);
			case ">";
			case "gt";
				return $this->return_bytes($b1) > $this->return_bytes($b2);
			case ">=";
			case "ge";
				return $this->return_bytes($b1) >= $this->return_bytes($b2);
			case "==";
			case "=";
			case "eq";
				return $this->return_bytes($b1) == $this->return_bytes($b2);
			case "!=";
			case "<>";
			case "ne";
				return $this->return_bytes($b1) != $this->return_bytes($b2);
		}
	}
}

class ModuleCondition extends Condition {
	var $module;
	var $version;
	var $op;

	function ModuleCondition($description, $module, $op = NULL, $version = NULL) {
		$info = "the \"${module}\" extension";
		if($version) $info .= " ${op} ${version}";
		$info .= " loaded";
		Condition::Condition("${info} for ${description}");
		$this->module = $module;
		$this->version = $version;
		$this->op = $op;
	}

	function satisfy() {
		return extension_loaded($this->module) && ($this->version === NULL || version_compare(phpversion($this->module), $this->version, $this->op));
	}

}

class FunctionCondition extends Condition {
	var $function;

	function FunctionCondition($description, $function) {
		Condition::Condition("the function \"${function}\" available for ${description}");
		$this->function = $function;
	}

	function satisfy() {
		return function_exists($this->function);
	}
}

class ClassCondition extends Condition {
	var $function;

	function ClassCondition($description, $class) {
		Condition::Condition("the class \"${class}\" available for ${description}");
		$this->class = $class;
	}

	function satisfy() {
		return class_exists($this->class);
	}
}

class BooleanCondition extends Condition {
	var $op;

	function BooleanCondition($op) {
		Condition::Condition("${op} boolean operator");
		$this->op = $op;
	}
}

class SingularBooleanCondition extends BooleanCondition {
	var $condition;

	function getName($checkbox) {
		$op = $this->op;
		$box = $checkbox ?  " " . $this->checkbox() : "";
		return "${op} (" . $this->condition->getName($checkbox) . ")${box}";
	}

	function setCondition(&$condition) {
		$this->condition =& $condition;
	}
}

class PluralBooleanCondition extends BooleanCondition {
	var $conditions;

	function getName($checkbox) {
		$op_delim = ", " . $this->op . " ";
		$names = array();
		foreach($this->conditions as $i => $blah) {
			$names[] = $this->conditions[$i]->getName($checkbox);
		}
		$box = $checkbox ? " " . $this->checkbox() : "";
		return "(" . implode($op_delim, $names) . ")${box}";
	}

	function addCondition(&$condition) {
		$this->conditions[] =& $condition;
	}
}

class OrCondition extends PluralBooleanCondition {
	function OrCondition() {
		BooleanCondition::BooleanCondition("or");
	}

	function satisfy() {
		foreach($this->conditions as $i => $blah) {
			$condition =& $this->conditions[$i]; //  to protect the reference
			if($condition->getSatisfaction()) return true;
		}
		return false;
	}
}

class AndCondition extends PluralBooleanCondition {
	function AndCondition() {
		BooleanCondition::BooleanCondition("and");
	}

	function satisfy() {
		foreach($this->conditions as $i => $blah) {
			$condition =& $this->conditions[$i]; //  to protect the reference
			if(!$condition->getSatisfaction()) return false;
		}
		return true;
	}
}

class NotCondition extends SingularBooleanCondition {
	function NotCondition() {
		BooleanCondition::BooleanCondition("not");
	}

	function satisfy() {
		return !$this->condition->getSatisfaction();
	}
}

class Requirement {
	var $level;
	var $condition;

	function Requirement($level, &$condition) {
		$this->level = $level;
		$this->condition =& $condition;
	}

	function isFatal() {
		if($this->level == REQUIREMENT_MUST && !$this->condition->getSatisfaction()) {
			return true;
		} else {
			return false;
		}
	}

	function isNoteworthy($level = REQUIREMENT_MAY) {
		return $level >= $this->level;
	}

	function getDescription($checkbox) {
		switch($this->level) {
			case REQUIREMENT_MUST:
				$qualifier = "MUST";
				$colorMap = array(
					true => array("black", "white"),
					false => array("white", "red"),
				);
				break;
			case REQUIREMENT_SHOULD:
				$qualifier = "SHOULD";
				$colorMap = array(
					true => array("black", "white"),
					false => array("black", "yellow"),
				);
				break;
			case REQUIREMENT_MAY:
				$qualifier = "MAY";
				$colorMap = array(
					true => array("black", "white"),
					false => array("black", "white"),
				);
				break;
		}
		$satisfaction = (bool) $this->condition->getSatisfaction();
		list($fg, $bg) = $colorMap[$satisfaction];

		$description = "\t<li><span style=\"color: ${fg}; background: ${bg};\">You ${qualifier} have " . $this->condition->getName($checkbox) . ".</span></li>";
		return $description;
	}
}

class Requirements {
	var $requirements;

	function Requirements() {
		$this->requirements = array();
	}

	function addRequirement(&$requirement) {
		$this->requirements[] =& $requirement;
	}

	function areNotSatisfied() {
		foreach($this->requirements as $i => $blah) {
			$requirement =& $this->requirements[$i];
			if($requirement->isFatal()) return true;
		}
		return false;
	}

	function getDescription($all = true, $checkbox = true, $display = REQUIREMENT_MAY) {
		$description = "<ul>\n";
		foreach($this->requirements as $i => $blah) {
			$requirement =& $this->requirements[$i];
			if($requirement->isNoteworthy($display) && ($all || !$requirement->condition->getSatisfaction())) {
				$description .= $requirement->getDescription($checkbox) . "\n";
			}
		}
		$description .= "</ul>\n";
		return $description;
	}

	function evaluate() {
	}

}

function &system_requirements() {

$requirements =& new Requirements();

$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new PHPCondition("minimal operation of ModernBill", ">=", "4.3.0")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new PHPCondition("recommended operation of ModernBill", ">=", "4.4.0")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new PHPCondition("more efficient operation of ModernBill", ">=", "5.0.0")));

$decoder_condition =& new OrCondition();
$decoder_condition->addCondition(new ModuleCondition("decoding encoded code", "Zend Optimizer"));
$decoder_condition->addCondition(new ModuleCondition("decoding encoded code", "ionCube Loader"));
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, $decoder_condition));

$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new LicenseServerCondition()));

$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new WritableTempCondition()));

$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new SafeModeCondition("access to remote API queries", false)));
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new INIUnsetCondition("access to system-level API libraries", "open_basedir")));

$memory_condition =& new OrCondition();
$memory_condition->addCondition(new ININumericCondition("proper operation", "memory_limit", ">=", "64M"));
$memory_condition->addCondition(new INIUnsetCondition("proper operation", "memory_limit"));

$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, $memory_condition));

$error_condition =& new OrCondition();
$error_condition->addCondition(new INIPatternCondition("suppressing the display of errors inline", "display_errors", "set to \"Off\"", "/on/i", false));
$error_condition->addCondition(new ININumericCondition("suppressing E_NOTICE errors", "error_reporting", "==", E_ALL & ~E_NOTICE));
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, $error_condition));

$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new FunctionCondition("generic text parsing", "ereg")));
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new ModuleCondition("generic text parsing", "pcre")));

$database_condition =& new OrCondition();
$database_condition->addCondition(new ModuleCondition("MySQL support built into PHP", "mysql"));
$database_condition->addCondition(new ModuleCondition("PostgreSQL support built into PHP", "pgsql"));
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, $database_condition));

$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new ModuleCondition("XML parsing of internal and remote API queries", "xml")));
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, new XMLWhiteSpaceCondition()));

$curl_condition =& new OrCondition();
$curl_module_condition =& new PHPCurlCondition("faster secure remote API queries");
$curl_cli_condition =& new CLICurlCondition("secure remote API queries");
$curl_condition->addCondition($curl_module_condition);
$curl_condition->addCondition($curl_cli_condition);
$requirements->addRequirement(new Requirement(REQUIREMENT_MUST, $curl_condition));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, $curl_module_condition));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new ModuleCondition("asynchronous secure remote API queries", "openssl")));

$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new INIUnsetCondition("proper operation", "disable_functions")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new INIUnsetCondition("proper operation", "disable_classes")));

$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new ModuleCondition("report graphs", "gd")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new ModuleCondition("automatic config file uploading at installation", "ftp")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new ModuleCondition("gzip compression", "zlib")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new ModuleCondition("v4 imports from databases not in UTF-8", "iconv")));
$requirements->addRequirement(new Requirement(REQUIREMENT_SHOULD, new ModuleCondition("support for particular locales", "mbstring")));

$requirements->addRequirement(new Requirement(REQUIREMENT_MAY, new ModuleCondition("Unix performance optimizations", "sysvshm")));
$requirements->addRequirement(new Requirement(REQUIREMENT_MAY, new ModuleCondition("Unix performance optimizations", "sysvsem")));

return $requirements;

}

/*

either Zend Optimizer 3.2 if PHP >= 5.1 or ioncube loaders

*/

?>