<?php
// $Id: DowngradePackage.php 12256 2007-11-19 17:42:33Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-tk/include/xml/xml.php");
require_once mb_path("/lib-tk/toolkit.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");

/**
 * This Action object collects information on a package so that that the package may be modified in the
 * database.
 *
 * @author  Kris Bailey
 */
class DowngradePackage extends MB_Action {
	/**
	 * The constructor for a DowngradePackage action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object.
	 */
	function DowngradePackage() {
		parent::MB_Action();
		$this->stringRep = "DowngradePackage";
		$this->params = array("packageID");
	}

	/**
	 * The main method of this class that executes the DowngradePackage action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$this->smartyVars["feePriceType"] = getAPITypeID("price", "downgradefee");
		
		$packageInfo = grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetPackages",
				"params" => array(
					"packageID" => (int)$_REQUEST["packageID"],
					"getPackageAttributeData" => 1,
					"getPackageAddonData" => 1,
				),
			)
		));
		
		$selectedAddonData = array();
		foreach ((array)$packageInfo["packageAddons"][0]["packageAddon"] as $addon){
			$selectedAddonData[$addon["packageAddonProductID"]] = $addon;
		}
		$this->smartyVars["selectedAddons"] =& $selectedAddonData;
		
		dbg($packageInfo);
		
		if (!$packageInfo["packageID"]){
			$this->addError("TRANS_ACTIONERROR", "Invalid package id.");
		} else {

			$this->smartyVars['packageInfo'] = $packageInfo;
			$this->smartyVars['packageStatuses'] = getStatuses("package");
			$this->smartyVars['types'] = getTypes("products");

			$now = time()-(time()%(24*60*60))+(24*60*60);
			
			$priorFeeData = oneOf(":packageFeeRels", array(
				"packageID" => (int)$packageInfo["packageID"],
			));
			if ($priorFeeData){
				$packageIDs = array((int)$packageInfo["packageID"], $priorFeeData["packageFeeID"]);
			} else {
				$packageIDs = $packageInfo["packageID"];
			}
			
			$lineItems = arrayFor("lineItems", array(
				"packageID" => $packageIDs,
				"sortColumn" => "lineItemDateCurrentRenewal",
				"sortDir" => "ASC",
				"lineItemDateNextRenewalGreaterThan" => $now-1,
			));

			$this->smartyVars["cango"] = count($lineItems);


			$this->smartyVars["downgradeProducts"] = arrayFor(
				"productRelations", 
				array(
					"subjectProductID" => $packageInfo["productID"], 
					"productRelationType" => getAPITypeID("productrelations", "downgrade")
				)
			);
						
			$productIDs = array($packageInfo["productID"]);
			foreach ((array)$this->smartyVars["downgradeProducts"] as $relation){
				$productIDs[] = $relation["relatedProductID"];
			}

			dbg($productIDs);

			// at this point we have our package info, and we have our product ids.
			// we need to get all of our product info for each of those products
			// AND we need to get the available addons for those products.  once we
			// do that, we need to filter those addons so that they don't appear
			// if they don't have prices in the currency and cycle we are in.
			// this is complicated schtuff, so edit at your own peril...
			
			// first, get all of the products we need: (products has a datasource so we can use arrayFor)
			$rawProductsInfo = arrayFor("products", array(
				"productID" => $productIDs,
				"productActive" => 1,
				"getProductVariantData" => 1,
				"getProductVariantPriceData" => 1,
				"getCycleData" => 1));
			
			dbg($rawProductsInfo);


			foreach ((array)$rawProductsInfo as $k=>$product){
			
				$rawProductsInfo[$k]["addonInfo"] = arrayFor("products", array(
					"parentProductID" => $product["productID"],
					"productActive" => 1,
					"getProductVariantData" => 1,
					"getProductVariantPriceData" => 1,
					"getCycleData" => 1));

				// get the addon groups for these addons AND this product
				$groups = array();
				
				foreach ((array)$rawProductsInfo[$k]["addonInfo"] as $kx=>$addon){
					$groups[$addon["productAddonGroupID"]] = 1;
				}
				
				$groups = array_keys($groups);			
				$groups = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetProductAddonGroups",
						"params" => array(
							"getGroupsByProductID" => $product["productID"],
						)
					)
				)));

				$rawProductsInfo[$k]["addonGroups"] = array();
				foreach ((array)$groups as $group){
					$rawProductsInfo[$k]["addonGroups"][$group["productAddonGroupID"]] = $group;
				}



			}


			$productsInfo = array(); // lets construct the product info, one row for EACH variant
			foreach ((array)$rawProductsInfo as $k=>$product){
				dbg("adding in for product: ".$product["productName"]);
				foreach ((array)$rawProductsInfo[$k]["productVariants"] as $variant){
					if ((int)$variant["productVariantActive"]){
						$newProduct = $product;
						$newProduct["productVariantID"] = $variant["productVariantID"];
						if (count($rawProductsInfo[$k]["productVariants"])>1){
							$newProduct["productVariantName"] = $variant["productVariantDisplayName"];
						}
						$productsInfo[] = $newProduct;
					}
				}
			}


			// this code loops through all of our products, AND their addons, and eliminates all prices
			// that don't match our cycle and currency.
			foreach ((array)$productsInfo as $k=>$product){
				$prices = array();
		 		if (isset($productsInfo[$k]["productVariantPrices"])){
					foreach ($productsInfo[$k]["productVariantPrices"] as $price){
						dbg("checking price: ");
						dbg($price);
						dbg("against: ".$productsInfo[$k]["productVariantID"]);
						if ($price["productVariantID"]==$productsInfo[$k]["productVariantID"] && $price["currencyID"]==$packageInfo["paymentGroupCurrencyID"] && $price["cycleID"]==$packageInfo["cycleID"]){
							$prices[$price["productVariantPriceType"]] = $price;
							dbg("got in!");
						}
					}
				} else {
					dbg("variant price not set");
				}
				dbg("the prices we got for this prod are:");
				dbg($prices);
				$productsInfo[$k]["prices"] = $prices;
				if (count($productsInfo[$k]["prices"])){
					unset($productsInfo[$k]["productVariantPrices"]);
				} else {
					dbg("just unset for: ".$productsInfo[$k]["productName"]);
			 		unset($productsInfo[$k]);
				}

				if (is_array($productsInfo[$k]["addonInfo"])){
					foreach ($productsInfo[$k]["addonInfo"] as $ak=>$addon){
						$prices = array();
		 				if (isset($productsInfo[$k]["addonInfo"][$ak]["productVariantPrices"])){
							foreach ($productsInfo[$k]["addonInfo"][$ak]["productVariantPrices"] as $price){
								if ($price["currencyID"]==$packageInfo["paymentGroupCurrencyID"] && $price["cycleID"]==$packageInfo["cycleID"]){
									$prices[$price["productVariantPriceType"]] = $price;
								}
							}
						} else {
							dbg("variant price not set for addon");
						}
						$productsInfo[$k]["addonInfo"][$ak]["prices"] = $prices;
						if (count($productsInfo[$k]["addonInfo"][$ak]["prices"])){
							unset($productsInfo[$k]["addonInfo"][$ak]["productVariantPrices"]);
						} else {
			 				unset($productsInfo[$k]["addonInfo"][$ak]);
						}
					}
					
					$productsInfo[$k]["addons"] = array();
					foreach ($productsInfo[$k]["addonInfo"] as $ak=>$addon){
						$productsInfo[$k]["addons"][$addon["productAddonGroupID"]][] =& $productsInfo[$k]["addonInfo"][$ak];
					}
					
				}
			}
			
			
			$this->smartyVars["downgradeProducts"] =& $productsInfo;


			$this->addTemplate("packages/DowngradePackage", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // EditPackage
?>
