<?php
// $Id: ShowPackageDetails.php 12872 2008-04-22 16:20:58Z btyra $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");

/**
 * This class will gather all information about a package and display it in an intuitive
 * manner for.
 *
 * @author  Kris Bailey
 */
class ShowPackageDetails extends MB_Action {
	/**
	 * The constructor for a ShowPackageDetails object. This merely sets up our string represenation, calls 
	 * the parent constructor, and sets up $this->params.
	 */
	function ShowPackageDetails() {
		parent::MB_Action();
		$this->stringRep = "ShowPackageDetails";
		$this->params = array("packageID");
	}

	/**
	 * Execute this action object, and display any output generated to the screen.
	 *
	 * @return  1 if no errors occurred during execution, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$packageID = ((int) $_REQUEST["packageID"]);
		if(!$packageID) {
			$this->addError(TRANS_ACTIONERROR, "No Package ID given.");
		} else {
			
			if ($_REQUEST["reprovisionServerGroupID"]){
			
				$packageInfo = oneByID(":packages", $packageID);
				$attributes = arrayFor(":packageAttributes", array(
					"packageID" => $packageID,
					"serverGroupID" => $_REQUEST["reprovisionServerGroupID"],
				));
				$keyedAttributes = array();
				foreach ($attributes as $attr){
					$keyedAttributes[$attr["packageAttributeName"]] = $attr;
				}
				$productAttributes = arrayFor(":productAttributes", array(
					"productID" => $packageInfo["productID"],
					"serverGroupID" => $_REQUEST["reprovisionServerGroupID"],
				));
				
				foreach ($productAttributes as $attr){
					if (!$keyedAttributes[$attr["productAttributeName"]]){
						insertFor(":packageAttributes", array(
							"packageID" => $packageID,
							"packageAttributeName" => $attr["productAttributeName"],
							"packageAttributeValue" => $attr["productAttributeValue"],
							"serverGroupID" => $attr["serverGroupID"],
							"packageAttributeDateCreated" => time(),
						));
					} else {
						updateForKey(":packageAttributes", $keyedAttributes[$attr["productAttributeName"]]["packageAttributeID"], array(
							"packageAttributeValue" => $attr["productAttributeValue"],
						));
					}
				}
				
				$ProcessPanelXML = "<mbapi>
					<command>ProcessPanel</command>
					<subCommand>panelAdd</subCommand>
					<params>
						<disableEmail>1</disableEmail>
						<subCommand>panelAdd</subCommand>
						<packageID>".$packageID."</packageID>
						<serverGroupID>".(int)$_REQUEST["reprovisionServerGroupID"]."</serverGroupID>
					</params>
				</mbapi>";

				$queueID = $this->_setQueueHelper(0, "ProcessPanel", $ProcessPanelXML, $packageID, (int)$_REQUEST["reprovisionServerGroupID"]);
				$results = dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "ProcessSystemQueue",
						"params" => array(
							"systemQueueID" => $queueID,
						),
					),
				)), 1);
			}

			if ((int)$_REQUEST["resendPackageInfoEmail"] && (int)$_REQUEST["configGroupID"]){
				require_once mb_path('/lib-mbapi/include/mail/emailTemplates/packageInfoEmail.php');
				$email =& new PackageInfoEmail($packageID, (int)$_REQUEST["configGroupID"]); // need package ID and configGroupID
				if ($email->execute()){
					$this->smartyVars["resentPackageInfoEmail"] = "TRANS_PACKAGE_INFO_EMAIL_RESENT_SUCCESSFULLY";
				} else {
					$this->smartyVars["resentPackageInfoEmail"] = "TRANS_PACKAGE_INFO_EMAIL_RESENT_UNSUCCESSFULLY";
				}
			}

			// lets get the package, addon, and child package info here:
			$this->smartyVars["packageInfo"] = correctArray(grabFirstResultFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackages",
					"params" => array(
						"packageID" => $packageID,
						"getPackageAttributeData" => 1,
						"getDomainData" => 1
					),
				),
			)));
			$this->smartyVars["packageInfo"]["statusAPIName"] = getAPIStatusName("package", $this->smartyVars["packageInfo"]["packageStatus"]);
			$packageInfo =& $this->smartyVars["packageInfo"];
			$this->smartyVars["packageInfo"]["packageComments"] = utf8_html_entity_decode($this->smartyVars["packageInfo"]["packageComments"]);
			$this->smartyVars["packageAddonInfo"] = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackageAddons",
					"params" => array(
						"packageID" => $packageID,
						"getPackageAddonAttributes" => 1,
						"packageAddonActive" => 1,
					),
				),
			)));
			$this->smartyVars["totalAddonPrice"] = 0;
			$packageAddonInfo =& $this->smartyVars["packageAddonInfo"];
			$this->smartyVars["childPackageInfo"] = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackages",
					"params" => array(
						"parentPackageID" => $packageID,
						"getPackageAttributeData" => 1,
						"packageActive" => 1,
					),
				),
			)));
			$childPackageInfo =& $this->smartyVars["childPackageInfo"];
			// ok we have the info here in:
			// $packageInfo
			// $packageAddonInfo
			// $childPackageInfo
			
			$packageInfo += (array)$this->getStopDateInfo($packageInfo);
			
			dbg("===========================================================");
			dbg($packageInfo);
			dbg($packageAddonInfo);
			dbg($childPackageInfo);
			
			$serverGroupStatuses = array();
			
			$attributes["package"] = array();
			foreach ((array)$packageInfo["packageAttributes"] as $attribute){
				if ($attribute["packageAttributeName"]=="panel_status" && $attribute["serverGroupID"]){
					$serverGroupStatuses[$attribute["serverGroupID"]] = $attribute;
				}

/*
################################################### interject brians sweet config group server code for client side stuffs here
				$tmpConfigGroupID = (int)$attribute["configGroupID"];
				
				if (notZero( $attribute["configGroupID"] )) {
					if (!is_array($configGroup[ $tmpConfigGroupID ] )) {
						$configGroup[ $tmpConfigGroupID ] = array();

						$getConfig =& new MBQuery("GetConfigGroups", "", array("configGroupID" => $tmpConfigGroupID));
						$cg = $getConfig->getResult();
							
						if (!$cg->getErrorCount()) {
							$cg = grabSimpleMBAPIResults( $cg->getStructure() );
							$configGroup[ $tmpConfigGroupID ] = $cg[0];
							
							// get the panel, only if user is primary
							if (notZero( $configGroup[ $tmpConfigGroupID ]["parentConfigGroupID"] ) && $contactPrimary) {
								$getConfig =& new MBQuery("GetConfigGroups", "", array("configGroupID" => (int)$configGroup[ $tmpConfigGroupID ]["parentConfigGroupID"]));
								$configGroup[ $tmpConfigGroupID ]["parent"] = $getConfig->getResult();
								
								$configGroup[ $tmpConfigGroupID ]["parent"] = grabSimpleMBAPIResults( $configGroup[ $tmpConfigGroupID ]["parent"]->getStructure() );
								$configGroup[ $tmpConfigGroupID ]["parent"] = $configGroup[ $tmpConfigGroupID ]["parent"][0];

								$panelName = (string)$configGroup[ $tmpConfigGroupID ]["parent"]["configGroupName"];
		
								// see if we can grab the panel
								if (file_exists(mb_path("/lib-mbapi/include/modules/panel/$panelName.php"))) {
									require_once mb_path("/lib-mbapi/include/modules/panel/$panelName.php");
									
									if (class_exists( $panelName )) {
										$params = array("subCommand" => "panelDummy", "configGroupID" => $tmpConfigGroupID);
										$panelObj =& new $panelName( $params );

										if (is_array( $CONF["server"]["serverGroups"]["panel"][ $panelName ] ))
										foreach( $CONF["server"]["serverGroups"]["panel"][ $panelName ] as $k => $panel ) {
											if ($panel["configGroupID"] == $tmpConfigGroupID) {
												$configGroup[ $tmpConfigGroupID ]["panelConfig"] =& $CONF["server"]["serverGroups"]["panel"][ $panelName ][ $k ];
											}
											unset( $panel, $k );
										}
										$configGroup[ $tmpConfigGroupID ]["panelCapabilities"] = $panelObj->capabilities;
										unset($panelObj);
									}
								}
							}
						}
						unset($getConfig, $cg);
					}
				}
				
				switch( $attribute["packageAttributeName"] ) {
				case ("packageUsername"):
					$configGroup[ $tmpConfigGroupID ]["panelUsername"] = $attribute["packageAttributeValue"];
					break;
					
				case ("packagePassword"):
					$configGroup[ $tmpConfigGroupID ]["panelPassword"] = $attribute["packageAttributeValue"];
					break;
				default:
					if (!is_array( $configGroup[ $tmpConfigGroupID ]["serverGroup"] ) && notZero($attribute["serverGroupID"])) {
						$configGroup[ $tmpConfigGroupID ]["serverGroup"] = arrayFor("serverGroups", array("serverGroupID" => $attribute["serverGroupID"]));
					}
					break;
				}

################################################### not going to use it yet! */

				if(strpos($attribute["packageAttributeName"],"Password")) {
					$attribute["packageAttributeValue"] = getStuff($attribute["packageAttributeValue"]);
				}

				if($attribute["packageAttributeName"] == "ip") {
					$ip = $attribute["packageAttributeValue"];
					if(long2ip($ip) != 0) {
						$attribute["packageAttributeValue"] = long2ip($ip);
					} else {
						$attribute["packageAttributeValue"] = long2ip(ip2long($ip));
					}
					$panels[$attribute["configGroupID"]] = $attribute["configGroupID"];
				}

				$attributes["package"][$attribute["serverGroupID"]][$attribute["packageAttributeName"]] = $attribute;
	
				//Get the servers for the reassign link
				$configGroupID[$attribute["serverGroupID"]] = $attribute["configGroupID"];

				//Usage	
				foreach((array)$panels as $panel) {
	
					$tmp = collapsePluralsAndSingulars(grabResultsFromMBAPIArray(
						array(
							"mbapi" => array(
								"command" => "GetPackageUsageLog",
								"params" => array(
									"packageID" => $packageID,
									"configGroupID" => $panel,
									"usageLogDateBilled" => -1,
									"getTotals" => 1,
	
								)
							)
						)
					));
					$this->smartyVars["packageUsage"][$packageID] = $tmp[0];
					unset($tmp);
				}
	
				//End Usage

			}
			
			
			// lets get the list of server groups and servers
			$serverGroupData = arrayFor(":serverGroups");
			$serverGroupNames = array();
			foreach ($serverGroupData as $sgd) $serverGroupNames[$sgd["serverGroupID"]] = $sgd["serverGroupName"];
			$serverRelData = arrayFor(":serverGroupRels");
			$serversToGroups = array();
			foreach ($serverRelData as $srd){
				$serversToGroups[$srd["serverID"]] = array($srd["serverGroupID"], $serverGroupNames[$srd["serverGroupID"]]);
			}
			// done getting groups keyed by serverID, now lets get servers and make a list
			$panels = $this->getPanelOptions();
			unset($panels[""]);
			$this->smartyVars["serverGroupServersList"] = array();
			foreach ($panels as $serverID=>$serverName){
				$this->smartyVars["serverGroupServersList"][$serversToGroups[$serverID][0]."|".$serverID] = $serversToGroups[$serverID][1]." >> ".$serverName;
			}

			$this->smartyVars["configGroupID"] = $configGroupID;
			$this->smartyVars["panels"] = $panels;

			$sgids = array_keys($attributes["package"]);
			foreach ($sgids as $id){
				if (!$serverGroupStatuses[$id])$serverGroupStatuses[$id] = array("absent"=>"1");
			}
			$this->smartyVars["serverGroupStatuses"] = $serverGroupStatuses;

			unset( $configGroup[0] );
			$this->smartyVars["numConfigGroups"] = count($configGroup);
			$this->smartyVars["configGroups"] =& $configGroup;

			$attributes["addon"] = array();
			foreach ((array)$packageAddonInfo as $addon){
				$this->smartyVars["totalAddonPrice"] += $addon["packageAddonPrice"];
				foreach ((array)$addon["packageAddonAttributes"] as $attribute){
					$attributes["addon"][$attribute["serverGroupID"]][$attribute["packageAttributeName"]][] = $attribute;
				}
			}
			foreach ((array)$childPackageInfo as $k=>$package){
				$this->smartyVars["totalAddonPrice"] += $package["packagePrice"];
				$childPackageInfo[$k] += $this->getStopDateInfo($package);
				foreach ((array)$package["packageAttributes"] as $attribute){
					$attributes["addon"][$attribute["serverGroupID"]][$attribute["packageAttributeName"]][] = $attribute;
				}
			}
			
			$this->smartyVars["allAttributes"] = $attributes;
			
			/* I do not think this is needed. We can get the domain using 
			the GetPackages command passing in GetDomainData. -MF
			$dd =& new DataSource("clientDomains", array(
				"clientID" => $packageInfo["clientID"],
				"getBillingData" => 1
			));
			$tldTable =& $this->getTableFromSource($dd);
			if($type == "domain") {
				$TLDsByExt =& $tldTable->getAssociatedTablesBy("domainTLD");
				$extension = $TLD["tldExtension"];
				if(isset($TLDsByExt[$extension])) {
						$tldTable =& $TLDsByExt[$extension];
						$this->smartyVars["clientDomains"] = $tldTable->getOptions();
				} else {
					$clientDomains = array();
				}
			} else {
				$clientDomains = $tldTable->getOptions();
			}

			// load the domains into the packageInfo array
			foreach((array)$clientDomains as $domainID => $domainName) {
				$packageInfo["domains"][] = array(
					"domainID" => $domainID,
					"domainName" => $domainName);
			}
			$this->smartyVars["domains"] = $clientDomains;
			*/

			$this->smartyVars["taxZoneGroups"] = grabHTMLOpsFromArray(
				grabSimpleMBAPIResults(dispatchMBAPI("<mbapi><command>GetTaxZoneGroups</command></mbapi>", 1)), 
				"taxZoneGroupID",
				"taxZoneGroupName");
			$this->smartyVars["packageStatuses"] = getStatuses("package");
			$this->smartyVars["packageTypes"] = getTypes("products");
			$this->smartyVars["packageClientPermissions"] = getTypes("packagemanagement");
			$this->smartyVars["packageCancellationTypes"] = getTypes("cancellation");
			$this->smartyVars["packagePendingCancellations"] = getTypes("pendingcancellation");

			$this->smartyVars["lineItem"] = correctArray(grabFirstResultFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetLineItems",
					"params" => array(
						"packageID" => $packageID,
						"packageAddonID" => 0,
						"sortDir" => "DESC",
						"sortColumn" => "lineItemDateCurrentRenewal",
						"numRecords" => 1,
					),
				),
			)));
			if ($this->smartyVars["lineItem"]["invoiceID"]){
				$this->smartyVars["invoiceInfo"] = correctArray(grabFirstResultFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetInvoices",
						"params" => array(
							"invoiceID" => $this->smartyVars["lineItem"]["invoiceID"],
							"getInvoiceLineItemsData" => 1,
							"packageID" => array(
								$packageID,
							),
							"numRecords" => 1,
						),
					),
				)));
				$this->smartyVars["addonTotals"] = array();
				$lastMainLineItem = 0;
				foreach ($this->smartyVars["invoiceInfo"]["lineItems"] as $k=>$v){
					if ($v["packageAddonID"]){
					} else {
						$lastMainLineItem = $v["lineItemID"];
					}
					$this->smartyVars["addonTotals"]["total"][$lastMainLineItem] += $v["lineItemTotalAmount"];
					$this->smartyVars["addonTotals"]["setup"][$lastMainLineItem] += $v["lineItemSetupAmount"];
					$this->smartyVars["addonTotals"]["tax"][$lastMainLineItem] += $v["lineItemTaxAmount"];
					
					$this->smartyVars["invoiceInfo"]["lineItems"][$k]["packageComments"] = utf8_html_entity_decode($this->smartyVars["invoiceInfo"]["lineItems"][$k]["packageComments"]);
					$this->smartyVars["invoiceInfo"]["lineItems"][$k]["lineItemAmountDue"] = $v["lineItemTotalAmount"]-$v["lineItemPaymentAmount"];
				}
				$this->smartyVars["lineItems"] =& $this->smartyVars["invoiceInfo"]["lineItems"];
			}
			
			$shareData = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetBillingAccountShares",
					"params" => array(
						"packageID" => $packageID,
						"verifyBillingAccount" => 1,
					),
				),
			)));

			//Get Queue Items for this package
			$queueStatuses = getStatuses("queue");
			$queueItem = array();

			$packagesTemp = correctArray($results["packages"][0]["package"]);

			if ((int)$packageID){

				//$queueStatus = array(getAPIStatusID("queue", "new"),
				//					getAPIStatusID("queue", "error"),
				//					getAPIStatusID("queue", "rule"),
				//					getAPIStatusID("queue", "force"),
				//					getAPIStatusID("queue", "cancelled"),
				//					);

				$results = arrayFor(":systemQueue", array(
					"packageID" => $packageID,
					"sortColumn" => "systemQueueID",
					"sortDir" => "DESC",
				));

				if ($results){
					foreach($results as $v) {
						preg_match("/<subCommand>([^<]*)<\/subCommand>/s", utf8_html_entity_decode($v["systemQueueCommandInput"]), $matches);
						if (!$queueItems[$v["configGroupID"]]){
							$queueItem = array(
										"systemQueueID" => $v["systemQueueID"],
										"configGroupID" => $v["configGroupID"],
										"serverGroupID" => $v["serverGroupID"],
										"systemQueueStatus" => getAPIStatusName("queue", $v["systemQueueStatus"]),
										"subCommand" => str_replace("panel","",$matches[0]),
											);
							$queueItems[$v["configGroupID"]] = $queueItem;
						}
					}
					$this->smartyVars["queueItems"] = $queueItems;
				}
			}

			loadConfigs("payment", "paymentQueue");
			$this->smartyVars["paymentQueueSettings"] =& $CONF["payment"]["paymentQueue"];

			$keyedShares = array();
			$billingAccountIDs = array();
			$this->smartyVars["keyedShares"] = array();
			foreach ($shareData as $share){
				$billingAccountIDs[$share["billingAccountID"]] = 1;
				$keyedShares[$share["billingAccountShareGroup"]][] = $share;
				$this->smartyVars["keyedShares"][$share["billingAccountID"]] = $share;
			}
			$billingAccountIDs = array_keys($billingAccountIDs);
			$clientContactIDs = IDsOF("clientContacts", array("clientID" => $this->smartyVars["packageInfo"]["clientID"]));
			$billingAccounts = arrayFor("billingAccounts", array("clientContactID" => $clientContactIDs));
			$this->smartyVars["billingAccounts"] = array();
			foreach ($billingAccounts as $ba){
				$this->smartyVars["billingAccounts"][$ba["billingAccountID"]] = $ba;
				$this->smartyVars["billingAccounts"][$ba["billingAccountID"]]["typeName"] = getTypeName("billingAccount", $ba["billingAccountType"]);
			}
			
			
			$sharePercentages = array();
			foreach ($keyedShares as $group=>$shares){
				$sharePercentages[$group] = $this->getSharePercentages($shares);
			}
			$this->smartyVars["shares"] = $keyedShares;
			$this->smartyVars["sharePercentages"] = $sharePercentages;
			
			$_REQUEST["clientID"] = $packageInfo["clientID"];
			$this->mergePrimaryContactData();
			$this->smartyVars['types'] = getTypes("products");
			$this->mergeCycleData();
			$this->mergeServerGroups();
			$this->smartyVars["serverGroupList"][0] = TRANS_GENERAL_ATTRIBUTES;
			
			$this->addTemplate("packages/ShowPackageDetails", $this->smartyVars);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
	
	function getSharePercentages($shares){
		$totalShares = 0;
		foreach ($shares as $share){
			$totalShares += $share["billingAccountShareValue"];
		}
		$totalShares = ($totalShares)?$totalShares:1;
		$sharePercentages = array();
		foreach ($shares as $share){
			$sharePercentages[$share["billingAccountShareID"]] = sprintf("%01.2f", round(($share["billingAccountShareValue"]/$totalShares)*100, 2));
		}
		return $sharePercentages;
	}
	
	function getStopDateInfo($package){
		if ($package["packageDateStopRenewal"]>(25*60*60)){
			$numCycles = 0;
			$nextRenewal = $package["packageDateNextRenewal"];
			while ($nextRenewal<$package["packageDateStopRenewal"]){
				$numCycles++;
				$nextRenewal = getNextBillingTimestamp($nextRenewal, $package["cycleID"]);
			}
			return array(
				"moreCycles" => $numCycles,
				"stopDate" => $nextRenewal);
		} else {
			return array(
				"moreCycles" => 0,
				"stopDate" => 0);
		}
	}
	
	function _setQueueHelper($dateToRun, $command, $xml, $packageID, $serverGroupID, $parentQueueID = NULL, $configGroupID = NULL){

		if ($dateToRun==0){
			$dateToRun = time();
		}

		$timeData = explode(";", date("i;G;j;n;Y", $dateToRun));
		$cmdArgs = array(
			"systemQueueCommand" => $command,
			"systemQueueCommandInput" => utf8_xml_entity_encode($xml),
			"systemQueueMinute" => $timeData[0],
			"systemQueueHour" => $timeData[1],
			"systemQueueDay" => $timeData[2],
			"systemQueueMonth" => $timeData[3],
			"systemQueueYear" => $timeData[4],
			"systemQueueIsRecurring" => 0,
			"systemQueueDateCreated" => time(),
			"systemQueueDateToRun" => $dateToRun,
			"systemQueueDateStarted" => 0,
			"systemQueueDateFinished" => 0,
			"packageID" => $packageID,
			"systemQueueStatus" => getAPIStatusID("queue", "force"),
		);

		if($parentQueueID){
			$cmdArgs["systemQueueParentID"] = $parentQueueID;
		}

		if($serverGroupID){
			$cmdArgs["serverGroupID"] = $serverGroupID;
		}

		if (notZero( $configGroupID )) {
			$cmdArgs["configGroupID"] = $configGroupID;
		}
		
//		$query = getMBAPIQuery("SetSystemQueue", "insert", $cmdArgs);
		$queuedata = dispatchMBAPI(array(
			"mbapi" => array(
				"command" => "SetSystemQueue",
				"subCommand" => "insert",
				"params" => $cmdArgs,
			),
		), 1);
		dbg($queuedata);

		return $queuedata["mbapi"][0]["header"][0]["lastInsertID"][0];
	}
	
} // ShowPackageDetails
?>
