<?php
// $Id: AddPriceResponse.php 10474 2007-01-11 23:58:14Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

/**
 * This Action object takes information provided by the AddPrice action, applies that information to
 * records in the database, and redirects to the ShowProductDetails action for the given product.
 *
 * @author  getActionFiles.php (Automatically Generated)
 */
class AddPriceResponse extends MB_Action {
	/**
	 * The constructor for an AddPriceResponse action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function AddPriceResponse() {
		parent::MB_Action();
		$this->stringRep = "AddPriceResponse";
		$this->params = array(
			"price",
			"cogs",
			"currencyID",
			"tldID",
			"cycleID",
			"productID",
			"productAddonID",
			"priceType",
		);
	}

	/**
	 * The main method of this class that executes the AddPriceResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
				
		$this->setReferralActionID(getActionID("AddPrice"));
		$this->setReferralActionFile("products.php");

		$currencyID = getTextParam('currencyID');
		$productID = getParam('productID');
		$productAddonID = getParam('productAddonID');
		$tldID = getParam('tldID');
		$cycleID = getParam('cycleID');
		$priceType = getParam('priceType');

		$price = (float) getTextParam('price');
		$cogs = (float) getTextParam('cogs');

		$restrictions = array(
			"price" => INPUT_RESTRICTION_PRICE|INPUT_RESTRICTION_REQUIRED,
			"cogs" => INPUT_RESTRICTION_PRICE|INPUT_RESTRICTION_REQUIRED,
		);

		if(!is_numeric($priceType)) {
			$this->addError(TRANS_ACTIONERROR, "No price type given.");
		} else {
			$this->smartyVars["priceType"] = $priceType;
			if(!$currencyID) {
				$this->addError(TRANS_ACTIONERROR, "No currency ID given.");
			} else if (preg_match('/^[A-Z]{3}$/', $currencyID)) {

				$restrictions["currencyID"] = INPUT_RESTRICTION_REQUIRED;
				if($cycleID) {
					$restrictions["cycleID"] = INPUT_RESTRICTION_UNSIGNED_INT;
				}
				
				if($tldID) {
					$restrictions["tldID"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;

					if(!$productID) {
						$this->addError(TRANS_ACTIONERROR, "No product ID given.");
					} else {
						$restrictions["productID"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;
					}
				} else {

					if(!$productID && !$productAddonID) {
						$this->addError(TRANS_ACTIONERROR, "No product ID or Product Addon ID given.");
					} else {
						if($productID) {
							$restrictions["productID"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;
						} else {
							$restrictions["productAddonID"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;
						}
					}
				}
			} else {
				$this->addError(TRANS_ACTIONERROR, "Invalid currency ID given.");
			}
		}

		if(!$this->getNumErrors()) {

			$success = $this->validateMultiInputs(
				$restrictions,
				array(),
				array(),
				1
			);
			if($success) {
				if($productID) {
					$productPrice = array(
						'productID' => $productID,
						'productPriceType' => $priceType,
						'productPrice' => $price,
						'productCogs' => $cogs,
						'currencyID' => $currencyID,
					);
					if($cycleID) {
						$productPrice['cycleID'] = $cycleID;
					}
					if($tldID) {
						$productPrice['tldID'] = $tldID;
					}
					
					$this->getRootFromMBAPI("SetPrices", "insert", $productPrice);
				} else {
					$productAddonPrice = array(
						'productAddonID' => $productAddonID,
						'productAddonPriceType' => $priceType,
						'productAddonPrice' => $price,
						'productAddonCogs' => $cogs,
						'currencyID' => $currencyID,
					);
					if($cycleID) {
						$productAddonPrice['cycleID'] = $cycleID;
					}
										
					$this->getRootFromMBAPI("SetPrices", "insert", $productAddonPrice);
				}
						
				if(!$this->getNumErrors()) {
					if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
						redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
					} else {
						redirect("products.php", getActionID("ShowProductStats"), array());
					}
				}

			}
		
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // AddPriceResponse
?>