<?php
// $Id: AddProduct.php 10474 2007-01-11 23:58:14Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/include/wizard/Wizard.php");


/**
 * This Action object collects information on a product so that that product may be added to the
 * database.
 *
 * @author  Ben Woolley
 */
class AddProduct extends MB_Action {
	/**
	 * The constructor for a AddProduct action object. This merely calls the parent constructor,
	 * and sets the string representation of this object.
	 */
	function AddProduct() {
		parent::MB_Action();
		$this->stringRep = "AddProduct";
	}

	/**
	 * The main method of this class that executes the AddProduct action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	 function execute() {
		global $CONF;
		parent::execute();


		$w =& new Wizard(intval($_REQUEST["wizardID"]), TRANS_ADD_PRODUCT, TRANS_ADD_PRODUCT_MESG);

		// extract the main data from the wizard into a prettified form
		$wizardData = $w->getData();
		$cursor = 0;
		$product = array(
			'name' =>         $wizardData[++$cursor],
			'type' =>         $wizardData[++$cursor],
			'comment' =>      $wizardData[++$cursor],
			'billingPlan' => $wizardData[++$cursor],
			'proratable' => $wizardData[++$cursor],
			'genericAttributeCount' =>	$wizardData[++$cursor],
			'signupEmailActionGroup' =>    $wizardData[++$cursor],
			'welcomeEmailActionGroup' =>   $wizardData[++$cursor],
			'children' =>     $wizardData[++$cursor],
			'addons' =>       $wizardData[++$cursor],
			'cycles' =>       $wizardData[++$cursor],
			'orderForms' =>   $wizardData[++$cursor],
			'coupons' =>      $wizardData[++$cursor],
			'serverGroups' => $wizardData[++$cursor],
		);
		if((int) $product['type'] === getTypeID("products", TRANS_DOMAIN_PRODUCT)) {
			$product += array(
				'TLDs' =>	$wizardData[++$cursor],
			);
		}
		if($product['billingPlan'] === "3") {
			$product += array(
				'numCycles' =>	$wizardData[++$cursor],
			);
		}
		if($product['billingPlan'] === "1") {
			$product['stopNumCycles'] = 0;
		} else if($product['billingPlan'] === "2") {
			$product['stopNumCycles'] = 1;
		} else {
			$product['stopNumCycles'] = $product['numCycles'];		
		}
		
		$gac = $product['genericAttributeCount'];
		$product['genericAttributes'] = array();
		for($gai = 0; $gai < $gac; $gai++) {
			$attrVar = $wizardData[++$cursor];
			$attrVal = $wizardData[++$cursor];
			$product['genericAttributes'][$attrVar] = $attrVal;
		}
		
		if($product['orderForms']) {
			if($product['TLDs']) {
				$prices = array();
				foreach($product['TLDs'] as $TLD) {
					$TLDprices = array();
					// pick up the number of currencies
					$c = (int) $wizardData[++$cursor];
					// pick up whether a transfer price is included
					$t = $wizardData[++$cursor] ? true : false;
					
					// pick up the currencies (transfer will be last, if it is included)
					for($i = 0; $i < $c; $i++) {
						$allPrices = $wizardData[++$cursor];
						$allCOGS = $wizardData[++$cursor];
						$TLDprices[$i] = array();
						$TLDcogs[$i] = array();
						if($t) {
							// pop transfer prices off the end
							$TLDprices[$i]['transfer'] = array_pop($allPrices);
							$TLDcogs[$i]['transfer'] = array_pop($allCOGS);
						}
						// setup prices are per currency
						// shift setup prices off the beginning
						$TLDprices[$i]['setup'] = array_shift($allPrices);
						$TLDcogs[$i]['setup'] = array_shift($allCOGS);
						
						$TLDprices[$i] += array_combine($product['cycles'], $allPrices);
						$TLDcogs[$i] += array_combine($product['cycles'], $allCOGS);
					}
					
					$prices[$TLD] = array(
						'prices' => $TLDprices,
						'cogs' => $TLDcogs,
					);
				}
				$product['prices'] = $prices;
			} else {
				$prices = array();
				$setups = array();
				$cogs = array();
				$c = (int) $wizardData[++$cursor];
				for($i = 0; $i < $c; $i++) {
					// setup prices are per cycle
					$prices[$i] = array_combine($product['cycles'], $wizardData[++$cursor]);
					$cogs[$i] = array_combine($product['cycles'], $wizardData[++$cursor]);
					$setupPrices[$i] = array_combine($product['cycles'], $wizardData[++$cursor]);
					$setupCogs[$i] = array_combine($product['cycles'], $wizardData[++$cursor]);
				}
				$product['prices'] = array(
					'prices' => $prices,
					'cogs' => $cogs,
					'setupPrices' => $setupPrices,
					'setupCogs' => $setupCogs,
				);
			}
		}
				
		if($product['serverGroups']) {
			$product['serverGroupAttributes'] = array();
			$sgc = count($product['serverGroups']);
			
			for($sgi = 0; $sgi < $sgc; $sgi++) {
				// init structure for this server group
				$serverGroup = $product['serverGroups'][$sgi];
				$product['serverGroupAttributes'][$serverGroup] = array();
				$serverGroupAttrs =& $product['serverGroupAttributes'][$serverGroup];

				// init common attrs for this server group
				$serverGroupAttrs['attrs'] = array();
				$attrs =& $serverGroupAttrs['attrs'];
				
				// add the attributes
				$c = (int) $wizardData[++$cursor];
				for($i = 0; $i < $c; $i++) {
					$attrs[] = $wizardData[++$cursor];
				}
			}
			
			/*
			for($sgi = 0; $sgi < $sgc; $sgi++) {
				// init structure for this server group
				$serverGroup = $product['serverGroups'][$sgi];
				$product['serverGroupAttributes'][$serverGroup] = array();
				$serverGroupAttrs =& $product['serverGroupAttributes'][$serverGroup];

				// init common attrs for this server group
				$serverGroupAttrs['commonAttrs'] = array();
				$attrs =& $serverGroupAttrs['commonAttrs'];
				
				// add the attributes
				$c = (int) $wizardData[++$cursor];
				for($i = 0; $i < $c; $i++) {
					$attrs[] = $wizardData[++$cursor];
				}
		
				// figure out if we are basic or advanced
				$attrAdvanced = (int) $wizardData[++$cursor][0];
				if($attrAdvanced) {
					$attrType = 'advancedAttrs';
				} else {
					$attrType = 'basicAttrs';
				}
		
				// init the rest of the attrs for this server group
				$serverGroupAttrs[$attrType] = array();
				$attrs =& $serverGroupAttrs[$attrType];

				// add the attributes
				$c = (int) $wizardData[++$cursor];
				for($i = 0; $i < $c; $i++) {
					$attrs[] = $wizardData[++$cursor];
				}
			}
			*/
		}
		
		// start displaying
		$this->addTemplate("products/AddProduct", $this->smartyVars);
		
		$wizardStepCount = count($w->getSteps());

		$stepStart = 1;
		
		// Only reload the data if ANY of the steps don't exist. Remember, a back might have deleted a single step.
		if($w->isComplete() && $wizardStepCount < $stepStart++) {
			// force the "finished" button not to display yet.
			$w->setNotFinished(1);

			// obtain the last step ID and question ID because the static IDs became useless after serialization.
			$stepID = 0;
			$questionID = 0;
			
			// 1 - Basic product data
			
			// name
			$s =& new WizardStep(TRANS_CONFIGURE_BASE_PRODUCT, TRANS_CONFIGURE_BASE_PRODUCT_MESG);
			$s->setID(++$stepID); // static variables do not persist across serialization, so we must force IDs from now on.
			
			$q =& new WizardQuestion(QUESTION_INPUT_TEXT, TRANS_NAME);
			$q->setID(++$questionID);
			$q->setSize(50);
			$q->setMaxLength(100);
			$q->setRestrictions(INPUT_RESTRICTION_REQUIRED);
			$s->addQuestion($q);
			
			// type
			$q =& new WizardQuestion(QUESTION_INPUT_SELECT, TRANS_TYPE);
			$q->setID(++$questionID);
			$types = $CONF["types"]["products"];
			foreach($types as $key => $value) {
				if($key == getTypeID("products", TRANS_REGULAR)) {
					$q->addAnswer($value, $key, 1);
				} else {
					$q->addAnswer($value, $key);
				}
				$q->setRestrictions(INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_UNSIGNED_INT);
				$s->addQuestion($q);
			}

			// comment
			$q =& new WizardQuestion(QUESTION_INPUT_TEXTAREA, TRANS_COMMENT);
			$q->setID(++$questionID);
			$q->setSize(10);
			$q->setMaxLength(40);
			$q->setRestrictions(INPUT_RESTRICTION_NONE);
			$s->addQuestion($q);

			// billing plan
			$q =& new WizardQuestion(QUESTION_INPUT_RADIO, TRANS_BILLING_PLAN);
			$q->setID(++$questionID);
			$q->addAnswer(TRANS_FOREVER, "1", "1");
			$q->addAnswer(TRANS_ONE_TIME_ONLY, "2");
			$q->addAnswer(TRANS_FIXED_DURATION, "3");
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);

			// proration
			$q =& new WizardQuestion(QUESTION_INPUT_CHECKBOX, TRANS_PRORATABLE);
			$q->setID(++$questionID);
			$q->addAnswer(TRANS_PRORATABLE, "1", "1");
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);		
			$s->addQuestion($q);

			// number of generic attributes
			$q =& new WizardQuestion(QUESTION_INPUT_TEXT, TRANS_GENERIC_ATTRIBUTE_COUNT);
			$q->setID(++$questionID);
			$q->setSize(50);
			$q->setMaxLength(100);
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);

			// email action groups
			require_once mb_path("/lib-modernbill/include/config/internal/emailactions.php");
			$q =& new WizardQuestion(QUESTION_INPUT_SELECT, TRANS_SIGNUP_EMAIL);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetEmailActionGroups</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
		<emailActionID>" . getTypeID("emailAction", TRANS_SIGNUP) . "</emailActionID>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$signupEmailActionGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["emailActionGroups"][0]["emailActionGroup"]));
			$c = count($signupEmailActionGroups);
			for($i = 0; $i < $c; $i++) {
				$signupEmailActionGroup =& $signupEmailActionGroups[$i];
				$q->addAnswer($signupEmailActionGroup['emailActionGroupName'], $signupEmailActionGroup['emailActionGroupID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);

			$q =& new WizardQuestion(QUESTION_INPUT_SELECT, TRANS_WELCOME_EMAIL);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetEmailActionGroups</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
		<emailActionID>" . getTypeID("emailAction", TRANS_WELCOME) . "</emailActionID>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$welcomeEmailActionGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["emailActionGroups"][0]["emailActionGroup"]));
			$c = count($welcomeEmailActionGroups);
			for($i = 0; $i < $c; $i++) {
				$welcomeEmailActionGroup =& $welcomeEmailActionGroups[$i];
				$q->addAnswer($welcomeEmailActionGroup['emailActionGroupName'], $welcomeEmailActionGroup['emailActionGroupID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			
			// product children
			
			$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_PRODUCT_CHILDREN);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetProducts</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$productChildren = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["products"][0]["product"]));
			$c = count($productChildren);
			for($i = 0; $i < $c; $i++) {
				$productChild =& $productChildren[$i];
				$q->addAnswer($productChild['productName'], $productChild['productID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			
			// product addons
			$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_PRODUCT_ADDONS);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetProductAddons</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$productAddons = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["productAddons"][0]["productAddon"]));
			$c = count($productAddons);
			for($i = 0; $i < $c; $i++) {
				$productAddon =& $productAddons[$i];
				$q->addAnswer($productAddon['productAddonName'], $productAddon['productAddonID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			
			// cycles
			$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_TIME_CYCLES);
			$q->setID(++$questionID);
			
			$query = "<mbapi>
	<command>GetCycles</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$cycles = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["cycles"][0]["cycle"]));
			$c = count($cycles);
			for($i = 0; $i < $c; $i++) {
				$cycle =& $cycles[$i];
				$q->addAnswer($cycle['cycleDisplayName'], $cycle['cycleID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			
			// order forms
			$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_ORDER_FORMS);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetOrderForms</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$orderForms = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"]));
			$c = count($orderForms);
			for($i = 0; $i < $c; $i++) {
				$orderForm =& $orderForms[$i];
				$q->addAnswer($orderForm['orderFormName'], $orderForm['orderFormID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			
			// coupons
			$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_COUPONS);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetCoupons</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$coupons = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["coupons"][0]["coupon"]));
			$c = count($coupons);
			for($i = 0; $i < $c; $i++) {
				$coupon =& $coupons[$i];
				$q->addAnswer($coupon['couponCode'], $coupon['couponID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			
			// server groups
			$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_SERVER_GROUPS);
			$q->setID(++$questionID);
			$query = "<mbapi>
	<command>GetServerGroups</command>
	<showXMLHeader>0</showXMLHeader>
	<params>
	</params>
</mbapi>";
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			$serverGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["serverGroups"][0]["serverGroup"]));
			$c = count($serverGroups);
			for($i = 0; $i < $c; $i++) {
				$serverGroup =& $serverGroups[$i];
				$q->addAnswer($serverGroup['serverGroupName'], $serverGroup['serverGroupID']);
			}
			$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
			$s->addQuestion($q);
			$w->addSteps($s);
		}
		
		$goDomain = (int) $product['type'] === getTypeID("products", TRANS_DOMAIN_PRODUCT);
		$goBillingCycles = $product['billingPlan'] === "3";
		$goGenericAttrs = (int) $product['genericAttributeCount'];
		if($w->isComplete() && ($goDomain || $goBillingCycles || $goGenericAttrs) && $wizardStepCount < $stepStart++) {
			// force the "finished" button not to display yet.
			$w->setNotFinished(1);

			// obtain the last step ID and question ID because the static IDs became useless after serialization.
			$stepID = $w->getLastStepID();
			$questionID = $w->getLastQuestionID();
			
			$s =& new WizardStep(TRANS_SELECT_PRODUCT_SPECIFICS, TRANS_SELECT_PRODUCT_SPECIFICS);
			$s->setID(++$stepID);
			
			if($goDomain) {
				// TLDs
				$q =& new WizardQuestion(QUESTION_INPUT_MULTISELECT, TRANS_TLDS);
				$q->setID(++$questionID);
				$query = "<mbapi>
		<command>GetTLDs</command>
		<showXMLHeader>0</showXMLHeader>
		<params>
		</params>
	</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$TLDs = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["TLDs"][0]["TLD"]));
				$c = count($TLDs);
				for($i = 0; $i < $c; $i++) {
					$TLD =& $TLDs[$i];
					$q->addAnswer($TLD['tldName'], $TLD['tldID']);
				}
				$q->setRestrictions(INPUT_RESTRICTION_UNSIGNED_INT);
				$s->addQuestion($q);
			}
			
			if($goBillingCycles) {
				// cycles for which to bill
				$q =& new WizardQuestion(QUESTION_INPUT_TEXT, TRANS_CYCLES_TO_BILL_FOR);
				$q->setID(++$questionID);
				$q->setSize(50);
				$q->setMaxLength(100);
				$q->setRestrictions(INPUT_RESTRICTION_POSITIVE_INT);
				$s->addQuestion($q);
			}

			if($goGenericAttrs) {
				// generic attributes
				$c = (int) $product['genericAttributeCount'];
				for($i = 0; $i < $c; $i++) {
					$q =& new WizardQuestion(QUESTION_INPUT_TEXT, TRANS_ATTRIBUTE_VARNAME);
					$q->setID(++$questionID);
					$q->setSize(50);
					$q->setRestrictions(INPUT_RESTRICTION_REQUIRED);
					$s->addQuestion($q);
					$q =& new WizardQuestion(QUESTION_INPUT_TEXT, TRANS_ATTRIBUTE_VALUE);
					$q->setID(++$questionID);
					$q->setSize(50);
					$q->setRestrictions(INPUT_RESTRICTION_REQUIRED);
					$s->addQuestion($q);
				}
			}
			
			$w->addSteps($s);
		}
		if($w->isComplete() && $product['orderForms'] && $wizardStepCount < $stepStart++) {
				// price selector
			
				// force the "finished" button not to display yet.
				$w->setNotFinished(1);
				
				// obtain the last step ID and question ID because the static IDs became useless after serialization.
				$stepID = $w->getLastStepID();
				$questionID = $w->getLastQuestionID();

				$s =& new WizardStep(TRANS_SELECT_PRICES, TRANS_SELECT_PRICES_MESG);
				$s->setID(++$stepID);
				
				$query = "<mbapi>
		<command>GetOrderForms</command>
		<showXMLHeader>0</showXMLHeader>
		<params>
			" . (
				$product['orderForms']
				? "<orderFormID>" . implode("</orderFormID><orderFormID>", $product['orderForms']) . "</orderFormID>"
				: ""
			) . "
			<getPaymentGroupData>1</getPaymentGroupData>
		</params>
	</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$results2 =& $data["mbapi"][0]["results"][0];
				$orderForms = collapseArrayElements(protectArray($results2["orderForms"][0]["orderForm"]));
				$currencyIDs = array();
				if (isset($orderForms)){
				    foreach($orderForms as $orderForm) {
					    $currencyIDs[$orderForm['paymentGroups']['paymentGroup']['paymentGroupCurrencyID']] = true;
				    }
				}
				$currencyIDs = array_keys($currencyIDs);

				if($product['cycles']) {
					$query = "<mbapi>
		<command>GetCycles</command>
		<showXMLHeader>0</showXMLHeader>
		<params>
					" . (
						$product['cycles']
						? "<cycleID>" . implode("</cycleID><cycleID>", $product['cycles']) . "</cycleID>"
						: ""
					) . "
		</params>
	</mbapi>";
					$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
					$data = dispatchMBAPI($query, 1);
					$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
					$cycleResults = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["cycles"][0]["cycle"]));
					$cycles = associateArrayRowByID($cycleResults, "cycleID");
				
					//echo '<pre>';
					//print_r($cycleResults);
					//print_r($cycles);
					//echo '</pre>';
					
					if($product['TLDs']) {
						$query = "<mbapi>
			<command>GetTLDs</command>
			<showXMLHeader>0</showXMLHeader>
			<params>
						" . (
							$product['TLDs']
							? "<tldID>" . implode("</tldID><tldID>", $product['TLDs']) . "</tldID>"
							: ""
						) . "
			</params>
		</mbapi>";
						$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
						$data = dispatchMBAPI($query, 1);
						$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
						$tldResults = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["TLDs"][0]["TLD"]));
						$TLDsByID = associateArrayRowByID($tldResults, "tldID");
						foreach($product['TLDs'] as $TLD) {
							$transfer = (int) $TLDsByID[$TLD]['tldIsTransferable'] ? 1 : 0;
							
							// pass along number of currencies
							$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, count($currencyIDs));
							$q->setID(++$questionID);
							$s->addQuestion($q);

							// pass along whether a transfer price is included
							$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, $transfer);
							$q->setID(++$questionID);
							$s->addQuestion($q);

							foreach($currencyIDs as $currencyID) {
								$q =& new WizardQuestion(QUESTION_INPUT_PRICES, TRANS_PRICES . ' - ' . $TLDsByID[$TLD]['tldName'] . ' - ' . $currencyID);
								$q->setID(++$questionID);
								
								$q->addAnswer(TRANS_SETUP, '0.00');
								foreach($product['cycles'] as $cycle) {
									$q->addAnswer($cycles[$cycle]['cycleDisplayName'], '0.00');
								}
								if($transfer) {
									$q->addAnswer(TRANS_TRANSFER, '0.00');
								}
								
								$q->setRestrictions(INPUT_RESTRICTION_DOUBLE);
								$s->addQuestion($q);

								$q =& new WizardQuestion(QUESTION_INPUT_PRICES, TRANS_COGS . ' - ' . $TLDsByID[$TLD]['tldName'] . ' - ' . $currencyID);
								$q->setID(++$questionID);
								
								$q->addAnswer(TRANS_SETUP, '0.00');
								foreach($product['cycles'] as $cycle) {
									$q->addAnswer($cycles[$cycle]['cycleDisplayName'], '0.00');
								}
								if($transfer) {
									$q->addAnswer(TRANS_TRANSFER, '0.00');
								}
								
								$q->setRestrictions(INPUT_RESTRICTION_DOUBLE);
								$s->addQuestion($q);
							}
						}
					} else {
						$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, count($currencyIDs));
						$q->setID(++$questionID);
						$s->addQuestion($q);

						foreach($currencyIDs as $currencyID) {
							$q =& new WizardQuestion(QUESTION_INPUT_PRICES, TRANS_PRICES . ' - ' . $currencyID);
							$q->setID(++$questionID);
							foreach($product['cycles'] as $cycle) {
								$q->addAnswer($cycles[$cycle]['cycleDisplayName'], '0.00');
							}
							$q->setRestrictions(INPUT_RESTRICTION_DOUBLE);
							$s->addQuestion($q);

							$q =& new WizardQuestion(QUESTION_INPUT_PRICES, TRANS_COGS . ' - ' . $currencyID);
							$q->setID(++$questionID);
							foreach($product['cycles'] as $cycle) {
								$q->addAnswer($cycles[$cycle]['cycleDisplayName'], '0.00');
							}
							$q->setRestrictions(INPUT_RESTRICTION_DOUBLE);
							$s->addQuestion($q);

							$q =& new WizardQuestion(QUESTION_INPUT_PRICES, TRANS_SETUP_PRICES . ' - ' . $currencyID);
							$q->setID(++$questionID);
							foreach($product['cycles'] as $cycle) {
								$q->addAnswer($cycles[$cycle]['cycleDisplayName'], '0.00');
							}
							$q->setRestrictions(INPUT_RESTRICTION_DOUBLE);
							$s->addQuestion($q);

							$q =& new WizardQuestion(QUESTION_INPUT_PRICES, TRANS_SETUP_COGS . ' - ' . $currencyID);
							$q->setID(++$questionID);
							foreach($product['cycles'] as $cycle) {
								$q->addAnswer($cycles[$cycle]['cycleDisplayName'], '0.00');
							}
							$q->setRestrictions(INPUT_RESTRICTION_DOUBLE);
							$s->addQuestion($q);

						}
					}
				}
				
				$w->addSteps($s);

				
		}

		// cycle through the server groups until we find out which step needs to be added.
		$serverGroupCount = count($product['serverGroups']);
		$serverGroupCursor = 0;
		
		if(is_array($product['serverGroups'])) {
			for($i = $stepStart; $i < ($stepStart + $serverGroupCount); $i++) {
				$serverGroupID = $product['serverGroups'][$serverGroupCursor++];
				if ($w->isComplete() && $wizardStepCount == $i - 1) {
					// force the "finished" button not to display yet.
					$w->setNotFinished(1);

					// obtain the last step ID and question ID because the static IDs became useless after serialization.
					$stepID = $w->getLastStepID();
					$questionID = $w->getLastQuestionID();

					// 6 - common product attributes
					if(success($attributes = $this->getAttributesForServerGroupID($serverGroupID))) {
						$s =& new WizardStep(TRANS_CONFIGURE_ATTRIBUTES, TRANS_CONFIGURE_ATTRIBUTES_MESG);
						$s->setID(++$stepID); // static variables do not persist across serialization, so we must force IDs from now on.
				
						$attribute_count = count($attributes);
						$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, $attribute_count);
						$q->setID(++$questionID);
						$s->addQuestion($q);

						foreach($attributes as $attrName => $attrData) {
							$q =& $attrData['question'];
							$q->setID(++$questionID);
							$s->addQuestion($q);
						}
					
					} else {
						$s =& new WizardStep(TRANS_NO_PANEL_FOR_SERVER_GROUP, TRANS_NO_PANEL_FOR_SERVER_GROUP_MESG);
						$s->setID(++$stepID); // static variables do not persist across serialization, so we must force IDs from now on.

						$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, 0);
						$q->setID(++$questionID);
						$s->addQuestion($q);
					}

					$w->addSteps($s);

					// override the cache and store these new questions in the database
					$w->store();
				}
			}
		}

		/*
		// cycle through the server groups until we find out which step needs to be added.
		$stepStep = 2;
		$serverGroupCount = count($product['serverGroups']);
		$serverGroupCursor = 0;
		
		for($i = $stepStart; $i < ($stepStart + ($serverGroupCount * $stepStep)); $i += $stepStep) {		
			$serverGroupID = $product['serverGroups'][$serverGroupCursor++];
			if ($w->isComplete() && $wizardStepCount == $i - 1) {
				// force the "finished" button not to display yet.
				$w->setNotFinished(1);
	
				// obtain the last step ID and question ID because the static IDs became useless after serialization.
				$stepID = $w->getLastStepID();
				$questionID = $w->getLastQuestionID();
				
				$attributes = $this->getAttributesForServerGroupID($serverGroupID);
	
				// 6 - common product attributes
				$s =& new WizardStep(TRANS_CONFIGURE_ATTRIBUTES, TRANS_CONFIGURE_ATTRIBUTES_MESG);
				$s->setID(++$stepID); // static variables do not persist across serialization, so we must force IDs from now on.
				
				$attribute_count = 0;
				foreach($attributes as $attrName => $attrData) {
					if($attrData['type'] == getTypeID("productAttribute", "both")) {
						$attribute_count++;
					}
				}
				$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, $attribute_count);
				$q->setID(++$questionID);
				$s->addQuestion($q);
				
				foreach($attributes as $attrName => $attrData) {
					if($attrData['type'] == getTypeID("productAttribute", "both")) {
						$q =& $attrData['question'];
					 	$q->setID(++$questionID);
						$s->addQuestion($q);
					}
				} 
	
				$q =& new WizardQuestion(QUESTION_INPUT_CHECKBOX, TRANS_ADVANCED);
				$q->setID(++$questionID);
				$q->addAnswer(TRANS_ADVANCED, "1");
				$s->addQuestion($q);			
	
				$w->addSteps($s);
				
				// override the cache and store these new questions in the database
				$w->store();
			} else if ($w->isComplete() && $wizardStepCount == $i) {
				// allow the "finished" button to display
				if($stepCount - 1 == $wizardStepCount) {
					$w->setNotFinished(0);
				} else {
					$w->setNotFinished(1);
				}
	
				// obtain the last step ID and question ID because the static IDs became useless after serialization.
				$stepID = $w->getLastStepID();
				$questionID = $w->getLastQuestionID();
	
				// obtain the last question, for it contains whether or not we go into advanced mode.
				$steps =& $w->getSteps();
				$lastStep =& $steps[$stepID];
				$answers =& $lastStep->getData();
				$advanced = $answers[$questionID][0];
				if((int) $advanced) {
					$attrType = "advanced";
				} else {
					$attrType = "basic";
				}
				
				$attributes = $this->getAttributesForServerGroupID($serverGroupID);
	
				// 7 - extended product attributes
				if($attrType == "advanced") {
					$s =& new WizardStep(TRANS_CONFIGURE_ADVANCED_ATTRIBUTES, TRANS_CONFIGURE_ADVANCED_ATTRIBUTES_MESG);
				} else {
					$s =& new WizardStep(TRANS_CONFIGURE_BASIC_ATTRIBUTES, TRANS_CONFIGURE_BASIC_ATTRIBUTES_MESG);
				}
				$s->setID(++$stepID); // static variables do not persist across serialization, so we must force IDs from now on.
				
				$attribute_count = 0;
				foreach($attributes as $attrName => $attrData) {
					if($attrData['type'] == getTypeID("productAttribute", $attrType)) {
						$attribute_count++;
					}
				}
				$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, $attribute_count);
				$q->setID(++$questionID);
				$s->addQuestion($q);
				
				foreach($attributes as $attrName => $attrData) {
					if($attrData['type'] == getTypeID("productAttribute", $attrType)) {
						$q =& $attrData['question'];
					 	$q->setID(++$questionID);
						$s->addQuestion($q);
					}
				} 
	
				$w->addSteps($s);
				
				// override the cache and store these new questions in the database
				$w->store();
			}
		}
		*/


////////////////////////////////////////////////// CONFIRMATION PAGE ////////////////////////////////////////



		if(!$w->isComplete()) { 
			$w->displayNextStep();
			//$w->display(-1);
		} else {
			$summary = array(
				array(TRANS_PRODUCT_NAME, $product['name']),
				array(TRANS_PRODUCT_TYPE, getTypeName("products", $product['type'])),
				array(TRANS_PRODUCT_COMMENT, $product['comment']),
			);

			if((int) $product['type'] === getTypeID("products", TRANS_DOMAIN_PRODUCT) && $product['TLDs']) {
				$query = "<mbapi>
	<command>GetTLDs</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['TLDs']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<tldID>' . $product['TLDs'][$i] . '</tldID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$TLDs = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["TLDs"][0]["TLD"]));
				$tldNames = array();
				$c = count($TLDs);
				for($i = 0; $i < $c; $i++) {
					$tldNames[] = $TLDs[$i]['tldName'];
				}
				$summary[] = array(TRANS_TLDS, implode(' - ', $tldNames));
			}

			if($product['children']) {
				$query = "<mbapi>
	<command>GetProducts</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['children']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<productID>' . $product['children'][$i] . '</productID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$productChildren = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["products"][0]["product"]));
				$childrenNames = array();
				$c = count($productChildren);
				for($i = 0; $i < $c; $i++) {
					$childrenNames[] = $productChildren[$i]['productName'];
				}
				$summary[] = array(TRANS_PRODUCT_CHILDREN, implode(' - ', $childrenNames));
			}

			if($product['addons']) {
				$query = "<mbapi>
	<command>GetProductAddons</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['addons']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<productAddonID>' . $product['addons'][$i] . '</productAddonID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$productAddons = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["productAddons"][0]["productAddon"]));
				$addonNames = array();
				$c = count($productAddons);
				for($i = 0; $i < $c; $i++) {
					$addonNames[] = $productAddons[$i]['productAddonName'];
				}
				$summary[] = array(TRANS_PRODUCT_ADDONS, implode('; ', $addonNames));
			}

			if($product['cycleGroups']) {
				$query = "<mbapi>
	<command>GetCycleGroups</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['cycleGroups']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<cycleGroupID>' . $product['cycleGroups'][$i] . '</cycleGroupID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$cycleGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["cycleGroups"][0]["cycleGroup"]));
				$cycleGroupNames = array();
				$c = count($cycleGroups);
				for($i = 0; $i < $c; $i++) {
					$cycleGroupNames[] = $cycleGroups[$i]['cycleGroupDisplayName'];
				}
				$summary[] = array(TRANS_TIME_CYCLE_GROUPS, implode(' - ', $cycleGroupNames));
			}

			if($product['cycles']) {
				$query = "<mbapi>
	<command>GetCycles</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['cycles']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<cycleID>' . $product['cycles'][$i] . '</cycleID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$cycles = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["cycles"][0]["cycle"]));
				$cycleNames = array();
				$c = count($cycles);
				for($i = 0; $i < $c; $i++) {
					$cycleNames[] = $cycles[$i]['cycleDisplayName'];
				}
				$summary[] = array(TRANS_TIME_CYCLES, implode(' - ', $cycleNames));
			}

			if($product['orderForms']) {
				$query = "<mbapi>
	<command>GetOrderForms</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['orderForms']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<orderFormIDs>' . $product['orderForms'][$i] . '</orderFormIDs>';
				}
				$query .= "
		<getPaymentGroupData>1</getPaymentGroupData>
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$orderForms = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"]));
				$orderFormNames = array();
				$c = count($orderForms);
				if($orderForms) {
					for($i = 0; $i < $c; $i++) {
						$orderFormNames[] = $orderForms[$i]['orderFormName'];
					}
					$summary[] = array(TRANS_ORDER_FORMS, implode(' - ', $orderFormNames));

					$currencyIDs = array();
					foreach($orderForms as $orderForm) {
						$currencyIDs[$orderForm['paymentGroups']['paymentGroup']['paymentGroupCurrencyID']] = true;
					}
					$currencyIDs = array_keys($currencyIDs);
					$summary[] = array(TRANS_CURRENCY_IDS, implode(' - ', $currencyIDs));					
				} else {
					$summary[] = array(TRANS_ORDER_FORMS, TRANS_NONE);
				}


			}

			if($product['signupEmailActionGroups']) {
				$query = $this->getMBAPIQuery("GetEmailActionGroups", "", array("emailActionGroupID" => $product['signupEmailActionGroups']));
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$signupEmailActionGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["emailActionGroups"][0]["emailActionGroup"]));
				if(isset($signupEmailActionGroups[0])) {
					$signupEmailActionGroupName = $signupEmailActionGroups[0]['emailActionGroupName'];
					$summary[] = array(TRANS_SIGNUP, $signupEmailActionGroupName);
				} else {
					$summary[] = array(TRANS_SIGNUP, TRANS_NONE);
				}
			}

			if($product['welcomeEmailActionGroups']) {
				$query = $this->getMBAPIQuery("GetEmailActionGroups", "", array("emailActionGroupID" => $product['welcomeEmailActionGroups']));
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$welcomeEmailActionGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["emailActionGroups"][0]["emailActionGroup"]));
				if(isset($welcomeEmailActionGroups[0])) {
					$welcomeEmailActionGroupName = $welcomeEmailActionGroups[0]['emailActionGroupName'];
					$summary[] = array(TRANS_WELCOME, $welcomeEmailActionGroupName);
				} else {
					$summary[] = array(TRANS_WELCOME, TRANS_NONE);
				}
			}

			if($product['serverGroups']) {
				$query = "<mbapi>
	<command>GetServerGroups</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['serverGroups']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<serverGroupID>' . $product['serverGroups'][$i] . '</serverGroupID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$serverGroups = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["serverGroups"][0]["serverGroup"]));
				$serverGroupNames = array();
				$c = count($serverGroups);
				for($i = 0; $i < $c; $i++) {
					$serverGroupNames[] = $serverGroups[$i]['serverGroupName'];
				}
				$summary[] = array(TRANS_SERVER_GROUPS, implode(' - ', $serverGroupNames));
			}

			if($product['coupons']) {
				$query = "<mbapi>
	<command>GetCoupons</command>
	<showXMLHeader>0</showXMLHeader>
	<params>";
				$c = count($product['coupons']);
				for($i = 0; $i < $c; $i++) {
					$query .= '<couponID>' . $product['coupons'][$i] . '</couponID>';
				}
				$query .= "
	</params>
</mbapi>";
				$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
				$data = dispatchMBAPI($query, 1);
				$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
				$coupons = collapseArrayElements(protectArray($data["mbapi"][0]["results"][0]["coupons"][0]["coupon"]));
				$couponCodes = array();
				$c = count($coupons);
				for($i = 0; $i < $c; $i++) {
					$couponCodes[] = $coupons[$i]['couponCode'];
				}
				$summary[] = array(TRANS_COUPONS, implode(' - ', $couponCodes));
			}

			if($product['serverGroups']) {
				$sgc = count($product['serverGroups']);
				for($sgi = 0; $sgi < $sgc; $sgi++) {
					// init structure for this server group
					$serverGroup = $product['serverGroups'][$sgi];
					$serverGroupID = $serverGroups[$sgi]['serverGroupID'];
					$serverGroupName = $serverGroups[$sgi]['serverGroupName'];
					$serverGroupAttrs =& $product['serverGroupAttributes'][$serverGroup];
					
					$summary[] = array(TRANS_SERVER_GROUP_NAME, $serverGroupName);

					$attributes = $this->getAttributesForServerGroupID($serverGroupID);
					
					if($attributes) {
						$cursor = 0;
						foreach($attributes as $attrName => $attrData) {
							$q =& $attrData['question'];
							$answers = $serverGroupAttrs['attrs'][$cursor++];
							if(isset($attrData['valueMap'])) {
								$possibles = $q->getAnswers();
								$selected = array();
								$c = count($possibles);
								for($i = 0; $i < $c; $i++) {
									if(@in_array($possibles[$i]['value'], $answers)) {
										$selected[] = $possibles[$i]['answer'];
									}
								}
								$summary[] = array($q->getQuestion(), implode(' - ', $selected));
							} else {
								$summary[] = array($q->getQuestion(), $answers);
							}
						}

					/*
					$cursor = 0;
					foreach($attributes as $attrName => $attrData) {
						if($attrData['type'] == getTypeID("productAttribute", "both")) {
							$q =& $attrData['question'];
							$answers = $serverGroupAttrs['commonAttrs'][$cursor++];
							if(isset($attrData['valueMap'])) {
								$possibles = $q->getAnswers();
								$selected = array();
								$c = count($possibles);
								for($i = 0; $i < $c; $i++) {
									if(in_array($possibles[$i]['value'], $answers)) {
										$selected[] = $possibles[$i]['answer'];
									}
								}
								$summary[] = array($q->getQuestion(), implode(' - ', $selected));
							} else {
								$summary[] = array($q->getQuestion(), $answers);
							}
						}
					}
		
					if($serverGroupAttrs['basicAttrs']) {
						$attrType = 'basic';
					} else {
						$attrType = 'advanced';
					}
		
					$cursor = 0;
					foreach($attributes as $attrName => $attrData) {
						if($attrData['type'] == getTypeID("productAttribute", $attrType)) {
							$q =& $attrData['question'];
							$answers = $serverGroupAttrs[$attrType . 'Attrs'][$cursor++];
							if(isset($attrData['valueMap'])) {
								$possibles = $q->getAnswers();
								$selected = array();
								$c = count($possibles);
								for($i = 0; $i < $c; $i++) {
									if(in_array($possibles[$i]['value'], $answers)) {
										$selected[] = $possibles[$i]['answer'];
									}
								}
								$summary[] = array($q->getQuestion(), implode(' - ', $selected));
							} else {
								$summary[] = array($q->getQuestion(), $answers);
							}
						}
					}
					*/
					}
				}
			}
			if($product['orderForms'] && (int) $product['type'] === getTypeID("products", TRANS_DOMAIN_PRODUCT) && $product['TLDs']) {
				$priceTableHead = array(TRANS_TLD_NAME, TRANS_CURRENCY_ID, TRANS_CYCLE_NAME, TRANS_PRICE, TRANS_COGS);
				$priceTableBody = array();
				// depends on $currencyIDs, $TLDs, $cycles being set above
				$TLDsByID = associateArrayRowByID($TLDs, "tldID");
				$cyclesByID = associateArrayRowByID($cycles, "cycleID");
				foreach($product['prices'] as $tldID => $currencyCosts) {
					$tldName = $TLDsByID[$tldID]['tldName'];
					$currencyPrices = $currencyCosts['prices'];
					$currencyCogs = $currencyCosts['cogs'];
					$pricesByCurrencyID = array();
					$cogsByCurrencyID = array();
					$tableRows = array();
					$c = count($currencyIDs);
					for($i = 0; $i < $c; $i++) {
						$pricesByCurrencyID[$currencyIDs[$i]] = $currencyPrices[$i];
						$cogsByCurrencyID[$currencyIDs[$i]] = $currencyCogs[$i];
					}
					foreach($pricesByCurrencyID as $currencyID => $prices) {
						foreach($prices as $cycleID => $price) {
							if($cycleID === 'transfer') {
								$cycleName = TRANS_TRANSFER;
							} else if($cycleID === 'setup') {
								$cycleName = TRANS_SETUP;
							} else {
								$cycleName = $cyclesByID[$cycleID]['cycleDisplayName'];
							}
							$tableRows[] = array($tldName, $currencyID, $cycleName, $price);
						}
					}
					// count is 1 setup + N cycles + 1 optional transfer
					$c = 1 + count($cycles) + ($prices['transfer'] ? 1 : 0);
					$i = 0;
					foreach($cogsByCurrencyID as $currencyID => $cogses) {
						$j = 0;
						foreach($cogses as $cycleID => $cogs) {
							$tableRows[$i * $c + $j][] = $cogs;
							$priceTableBody[] = $tableRows[$i * $c + $j];
							$j++;
						}
						$i++;
					}
				}
			} else {
				$priceTableHead = array(TRANS_CURRENCY_ID, TRANS_CYCLE_NAME, TRANS_PRICE, TRANS_COGS, TRANS_SETUP_PRICE, TRANS_SETUP_COGS);
				$priceTableBody = array();
				// depends on $currencyIDs, $cycles being set above
				$cyclesByID = associateArrayRowByID($cycles, "cycleID");
				$currencyCosts = $product['prices'];
				$currencyPrices = $currencyCosts['prices'];
				$currencyCogs = $currencyCosts['cogs'];
				$currencySetupPrices = $currencyCosts['setupPrices'];
				$currencySetupCogs = $currencyCosts['setupCogs'];
				$pricesByCurrencyID = array();
				$cogsByCurrencyID = array();
				$tableRows = array();
				$c = count($currencyIDs);
				for($i = 0; $i < $c; $i++) {
					$pricesByCurrencyID[$currencyIDs[$i]] = $currencyPrices[$i];
					$cogsByCurrencyID[$currencyIDs[$i]] = $currencyCogs[$i];
					$setupPricesByCurrencyID[$currencyIDs[$i]] = $currencySetupPrices[$i];
					$setupCogsByCurrencyID[$currencyIDs[$i]] = $currencySetupCogs[$i];
				}
				foreach($pricesByCurrencyID as $currencyID => $prices) {
					foreach($prices as $cycleID => $price) {
						$cycleName = $cyclesByID[$cycleID]['cycleDisplayName'];
						$tableRows[] = array($currencyID, $cycleName, $price);
					}
				}
				$c = count($cycles);
				$i = 0;
				foreach($cogsByCurrencyID as $currencyID => $cogses) {
					$j = 0;
					foreach($cogses as $cycleID => $cogs) {
						$rowNum = $i * $c + $j;
						///echo $i . ' * ' . $c . ' + ' . $j . ' = ' . ($i * $c + $j) . "; \n";
						$setupPrice = $setupPricesByCurrencyID[$currencyID][$cycleID];
						$setupCogs = $setupCogsByCurrencyID[$currencyID][$cycleID];
						$tableRows[$rowNum][] = $cogs;
						$tableRows[$rowNum][] = $setupPrice;
						$tableRows[$rowNum][] = $setupCogs;
						$priceTableBody[] = $tableRows[$rowNum];
						$j++;
					}
					$i++;
				}
			}

			if($product['genericAttributes']) {
				foreach($product['genericAttributes'] as $attrVar => $attrVal) {
					$summary[] = array($attrVar, $attrVal);
				}
			}
			
			$summary[] = array(TRANS_BILL_FOR_CYCLES, $product['stopNumCycles'] ? $product['stopNumCycles'] : TRANS_FOREVER);
			
			//echo '<pre>';
			//print_r($priceTableHead);
			//print_r($priceTableBody);
			//echo '</pre>';
			
			if($priceTableBody) {
				$priceSummary = array(
					'priceTableHead' => $priceTableHead,
					'priceTableBody' => $priceTableBody,
					'doPrices' => 1,
				);
			} else {
				$priceSummary = array(
					'doPrices' => 0,
				);
			}

			$this->addTemplate(
				"products/AddProduct2",
				array(
					'summary' => $summary,
					'wizardID' => $w->getID(),
				) + $priceSummary
			);

			//echo '<pre>';
			//print_r($summary);
			//echo '</pre>';
			
			// we must store the stuff from the last step, which is added into the wizard on this page.
			$w->store();
		}
		
		//echo '<pre>';
		//print_r($_REQUEST);
		//print_r($product);
		//print_r($w);
		//echo '</pre>';

		$this->addTemplate("footer", array());

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

} // AddProduct

?>