<?php
// $Id: EditPrice.php 10474 2007-01-11 23:58:14Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

/**
 * Generic action for editing a single price for anything kind of price.
 *
 * @author  Ben Woolley
 */
class EditPrice extends MB_Action {

	function EditPrice() {
		parent::MB_Action();
		$this->stringRep = "EditPrice";
		$this->params = array(
			"productPriceID",
			"productAddonPriceID",
			"currencyID",
			"productID",
			"productAddonID",
			"tldID",
			"cycleID",
			"priceType",
		);
	}

	function execute() {
		global $CONF;
		parent::execute();

		$productPriceID = getParam('productPriceID');
		$productAddonPriceID = getParam('productAddonPriceID');
		if($productPriceID) {
			$query = array("productPriceIDs" => array('productPriceID' => $productPriceID));
			$path = array("mbapi", "results", "productPrices", "productPrice", "0");
			$selectedPrice = collapseArrayElements($this->getPathFromMBAPI("GetPrices", "", $query, $path));
		} else if($productAddonPriceID) {
			$query = array("productAddonPriceIDs" => array('productAddonPriceID' => $productAddonPriceID));
			$path = array("mbapi", "results", "productAddonPrices", "productAddonPrice", "0");
			$selectedPrice = collapseArrayElements($this->getPathFromMBAPI("GetPrices", "", $query, $path));
		} else {
			$this->addError(TRANS_ACTIONERROR, "No price given.");			
		}
		$valid = false;
		if($productPriceID && $selectedPrice) {
			$valid = true;
			$productID = $selectedPrice["productID"];
			$currencyID = $selectedPrice["currencyID"];
			$tldID = $selectedPrice["tldID"];
			$cycleID = $selectedPrice["cycleID"];
			$priceType = $selectedPrice["productPriceType"];
			$formDefaults = array(
				'price' => $selectedPrice["productPrice"],
				'cogs' => $selectedPrice["productCogs"],
			);
			
		} else if($productAddonPriceID && $selectedPrice) {
			$valid = true;
			$productAddonID = $selectedPrice["productAddonID"];
			$currencyID = $selectedPrice["currencyID"];
			$tldID = $selectedPrice["tldID"];
			$cycleID = $selectedPrice["cycleID"];
			$priceType = $selectedPrice["productAddonPriceType"];
			$formDefaults = array(
				'price' => $selectedPrice["productAddonPrice"],
				'cogs' => $selectedPrice["productAddonCogs"],
			);
		}
		if($valid) {
			$this->smartyVars["priceType"] = $priceType;
			$this->smartyVars["currencyID"] = $currencyID;
			$this->smartyVars["tldID"] = $tldID;
			$this->smartyVars["cycleID"] = $cycleID;
			$this->smartyVars["productID"] = $productID;
			$this->smartyVars["productPriceID"] = $productPriceID;
			$this->smartyVars["productAddonID"] = $productAddonID;
			$this->smartyVars["productAddonPriceID"] = $productAddonPriceID;

			$this->smartyVars["priceTypes"] = $CONF["types"]["price"];
			$this->smartyVars["productTypes"] = $CONF["types"]["products"];

			if(!is_numeric($priceType)) {
				$this->addError(TRANS_ACTIONERROR, "No price type given.");
			} else {
				$this->smartyVars["priceType"] = $priceType;
				if(!$currencyID) {
					$this->addError(TRANS_ACTIONERROR, "No currency ID given.");
				} else if (preg_match('/^[A-Z]{3}$/', $currencyID)) {

					$this->smartyVars["currencyID"] = $currencyID;
					if($cycleID) {
						$this->assignPathFromMBAPI("cycle", "GetCycles", "", array("cycleID" => $cycleID), array("mbapi", "results", "cycles", "cycle", "0"));					
					}

					if($tldID) {
						$this->assignPathFromMBAPI("TLD", "GetTLDs", "", array("tldID" => $tldID), array("mbapi", "results", "TLDs", "TLD", "0"));

						if(!$productID) {
							$this->addError(TRANS_ACTIONERROR, "No product ID given.");
						} else {
							$this->assignPathFromMBAPI("product", "GetProducts", "", array("productID" => $productID), array("mbapi", "results", "products", "product", "0"));
						}
					} else {

						if(!$productID && !$productAddonID) {
							$this->addError(TRANS_ACTIONERROR, "No product ID or Product Addon ID given.");
						} else {
							if($productID) {
								$this->assignPathFromMBAPI("product", "GetProducts", "", array("productID" => $productID), array("mbapi", "results", "products", "product", "0"));
							} else {
								$this->assignPathFromMBAPI("productAddon", "GetProductAddons", "", array("productAddonID" => $productAddonID), array("mbapi", "results", "productAddons", "productAddon", "0"));
							}
						}
					}
				} else {
					$this->addError(TRANS_ACTIONERROR, "Invalid currency ID given.");
				}
			}

			if(!$this->getNumErrors()) {

				$this->mergeFormState($formDefaults);

				$this->addTemplate("products/EditPrice", $this->smartyVars);
			}

		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // MB_Action
?>