<?php
// $Id: EditProductAttributes.php 12754 2008-03-11 15:07:13Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/app-modernbill-admin/include/lib-action/products/ShowProductDetailsAction.php");
require_once mb_path("/lib-modernbill/include/config/internal/emailactions.php");
require_once mb_path("/lib-tk/toolkit.php");
require_once mb_path("/lib-action/include/wizard/Wizard.php");

/**
 * product <-> order form relation editor
 *
 * @author  Ben Woolley
 */
class EditProductAttributes extends MB_ProductAction {
	/**
	 * The constructor for a EditProductAttributes action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object.
	 */
	function EditProductAttributes() {
		parent::MB_ProductAction("attributes");
		$this->stringRep = "EditProductAttributes";
		$this->params = array("productID");
	}

	/**
	 * The main method of this class that executes the EditProductAttributes action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$productID = getParam("productID");
		if(!$productID) {
			$this->addError(TRANS_ACTIONERROR, "No product ID given.");
		} else {
			$q =& new MBQuery("GetProducts", "", array("productID" => $productID, "getProductAttributeData" => 1, "getServerGroupGhostData" => 1));
			$r =& $q->getResult();
			if($r->getSuccess()) {
				//echo '<pre>';
				$gs =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", "ghostServerGroups", "ghostServerGroup"),
					"serverGroupID", "serverGroupName"
				);
				$serverGroups = $gs->getAssociated();
				$s =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", "productAttributes", "productAttribute"),
					"productAttributeID", "productAttributeName"
				);
				$ps =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product"),
					"productID", "productName"
				);
				$pt =& $ps->getTable();
				$product = $pt->getOne();
				$product["productAddonGroupID"] = (int)$product["productAddonGroupID"];

				$this->smartyVars["productVariants"] = arrayFor(":productVariants", array(
					"productID" => $productID,
				));
				
				$this->setupTypeButtons($product["productType"]);
				$this->smartyVars["productButtons"] = $this->productButtons->getButtons();

				$s->setKey("serverGroupID");
				$attrsTableByServerGroup = $s->getAssociatedSets();
				$genericAttributes = $attrsTableByServerGroup[NULL];
				$attrsByServerGroup = $s->getAssociated();
				
				$wantedServerGroups = $gs->getKeys();
				$wantedServerGroups[] = 0;
				$serverGroupAttributes = array();
				$multiSelects = array();

				foreach($wantedServerGroups as $serverGroup) {
					$panelAttributes = $this->getAttributesForServerGroupID($serverGroup, $product, false, true);
					// here we test if what we got back from the panel object
					// is a numerically indexed array, meaning we are going to
					// use a custom attributes template provided by the panel,
					// or a keyed array of wizard questions
					if ($panelAttributes && !is_numeric(key($panelAttributes))){
						$panelStep =& new WizardStep($serverGroups[$serverGroup]['serverGroupName'], TRANS_ATTRIBUTES);
						$panelStep->setID($serverGroup);
						
						if(isset($attrsTableByServerGroup[$serverGroup])) {
							$panelAttributeValues = associateArrayRowByID($attrsTableByServerGroup[$serverGroup], "productAttributeName");
						} else {
							$panelAttributeValues = array();
						}
						
						$reverseValMap = array();
						if (is_array( $panelAttributes ))
						foreach($panelAttributes as $attrName => $attrData) {
							$attr =& $panelAttributes[$attrName];

							if(isset($attr['valueMap'])) {
								$reverseValMap[$attrName] = array();
								foreach($attr['valueMap'] as $value => $map) {
									foreach($map as $name => $bool) {
										list($trueVal, $falseVal) = $bool;
										
										if(isset($panelAttributeValues[$name])) {
											if($panelAttributeValues[$name]["productAttributeValue"] === $trueVal) {
												$reverseValMap[$attrName][$value] = true;
											} else if(
													$panelAttributeValues[$name]["productAttributeValue"] === $falseVal
												&&	isset($reverseValMap[$attrName][$value])
											) {
												unset($reverseValMap[$attrName][$value]);
											}
										}
									}
								}
							}
						}
						//show($reverseValMap);
						
						$i = 0;
						if (is_array( $panelAttributes ))
						foreach($panelAttributes as $attrName => $attrData) {
							if (isset($panelAttributes[$attrName])){
								$attr =& $panelAttributes[$attrName];
								if(isset($panelAttributeValues[$attrName])) {
									$attrSettings = $panelAttributeValues[$attrName];
								} else {
									$attrSettings = false;
								}

								$panelQuestion =& $attr['question'];
								$panelQuestion->setID(++$i);

								$questionType = $panelQuestion->getType();
								if($questionType === QUESTION_INPUT_MULTISELECT) {
									$multiSelects[$attrName] = true;
								}
								$isMulti = $panelQuestion->hasOptions();

								$userVal = $this->getUserDataFor('question_' . $serverGroup . '_' . $i);
								if($userVal === NULL) {
									if($isMulti) {
										if(isset($attr['valueMap'])) {
											$options = array_keys($reverseValMap[$attrName]);
											$panelQuestion->setSelected($options);
										} else if($attrSettings) {
											$panelQuestion->setSelected($attrSettings["productAttributeValue"]);
										}
									} else {
										if(isset($attr['valueMap'])) {
											$options = array_keys($reverseValMap[$attrName]);
											$panelQuestion->setAnswers(array());
											$panelQuestion->addAnswer($options[0]);
										} else if($attrSettings) {
											$panelQuestion->setAnswers(array());
											$panelQuestion->addAnswer($attrSettings["productAttributeValue"]);
										}
									}
								} else {
									if($isMulti) {
										$panelQuestion->setSelected($userVal);
									} else if($attrSettings) {
										$panelQuestion->setAnswers(array($userVal));
									}

									$errors = $panelQuestion->validateFor($userVal);
									$panelStep->erroredQuestions[$i] = $errors;
									$panelStep->invalidQuestions[$i] = errorMesgsFor($errors);
								}


								$panelStep->addQuestion($panelQuestion);
							} else {
								dbg("got an attribute that wasn't there...");
								dbg($attrName);
								dbg($panelAttributes);
							}

						}
						//echo '<pre>';
						//print_r($panelStep);
						//echo '</pre>';
						$serverGroupAttributes[] = array(
							"type" => "content", 
							"data" => $panelStep->display(false, true),
						);
					} elseif ($panelAttributes){
						$attributes = arrayFor(":productAttributes", array(
							"productID" => $productID,
							"serverGroupID" => $serverGroup,
						));
						$keyedAttributes = array();
						foreach ($attributes as $a){
							$keyedAttributes[$a["productAttributeName"]][] = $a;
						}
						$serverGroupAttributes[] = array(
							"type" => "include",
							"file" => $panelAttributes[0],
							"serverGroupID" => $serverGroup,
							"serverGroupName" => $serverGroups[$serverGroup]['serverGroupName'],
							"attributes" => $keyedAttributes,
							"panelVariables" => $panelAttributes[1],
						);
					}
				}

				$javascript = "";
				foreach($multiSelects as $multiSelect => $blah) {
					$javascript .= "selectAllOptions(document.EditProductAttributeResponse.elements['" . $multiSelect . "[]']);";
				}

				$this->smartyVars['productID'] = $productID;
				$this->smartyVars['product'] = $product;
				$this->smartyVars['javascript'] = $javascript;
				$this->smartyVars['productAttributes'] = $productAttributes;
				$this->smartyVars['genericAttributes'] = $genericAttributes;
				$this->smartyVars['serverGroupAttributes'] = $serverGroupAttributes;
				$this->smartyVars['cycles'] = getCyclesArray();
				
				//print_r($serverGroups);
				//print_r($genericAttrs);
				//print_r($attrsByServerGroup);
				//print_r($attrsTableByServerGroup);
				//print_r($s);
				//print_r($productAttributes);
				//print_r($wantedServerGroups);
				//print_r($gs);
				//print_r($r);
				
				//echo htmlentities($serverGroupAttributes);
				//echo '</pre>';
				//$formDefaults = $product;
				//$this->mergeFormState($formDefaults);
				$this->addTemplate("widgets/products/EditProductAttributes", $this->smartyVars);
				
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

}
?>
