<?php
// $Id: EditProductPricesResponse.php 10474 2007-01-11 23:58:14Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-tk/toolkit.php");
require_once mb_path("/lib-modernbill/include/accounting/Accounting.php");

/**
 * This Action object takes information provided by the EditProductPrices action, applies that information to
 * records in the database, and redirects to the ShowProductDetails action for the given product.
 *
 * @author  Ben Woolley
 */
class EditProductPricesResponse extends MB_Action {
	/**
	 * The constructor for an EditProductPricesResponse action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function EditProductPricesResponse() {
		parent::MB_Action();
		$this->stringRep = "EditProductPricesResponse";
		$this->params = array(
			"productID",
		);
	}

	/**
	 * The main method of this class that executes the EditProductPricesResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		//echo '<pre>';
		//print_r($_REQUEST);
		//echo '</pre>';
		//exit;

		do {

			$this->setReferralActionID(getActionID("ShowProductDetails"));
			$this->setReferralActionFile("products.php");

			if(
					($productID = $this->validateParam("productID", INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED)) !== false
				&&	($productType = $this->validateParam("productType", INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED)) !== false
			) {
				//echo '<pre>';
				//print_r($_REQUEST);

				$productVariantMap = array();
				$productVariantPrices = array();
				$productLedgerEntries = array();
				
				$ghostProductVariantSet =& new Set();
				$ghostProductVariantMap = array();
				$ghostProductVariantPriceMap = array();
				$ghostProductLedgerEntryMap = array();
				$ghostProductVariantCursor = 0;
				
				if($productType == getTypeID("products", TRANS_DOMAIN_PRODUCT)) {
					$domain = true;
				} else {
					$domain = false;
				}

				if($domain) {
					foreach($_REQUEST as $arg => $val) {
						$matches = array();
						if(preg_match("/^(price|cogs|id|gid|vid)_([0-9]+)_([A-Z]{3})_([0-9]+|setup|transfer)$/", $arg, $matches)) {
							//print_r($matches);
							list($entirety, $field, $tldID, $currencyID, $cycleIDorType) = $matches;
							if(!isset($ghostProductVariantPriceMap[$tldID])) {
								$ghostProductVariantPriceMap[$tldID] = array();
							}
							if(!isset($ghostProductLedgerEntryMap[$tldID])) {
								$ghostProductLedgerEntryMap[$tldID] = array();
							}
							if($field == "vid") {
								$productVariant = array(
									"productID" => $productID,
									"tldID" => $tldID,
								);
								if((int) $val > 0) {
									$productVariantMap[serialize($productVariant)] = (int) $val;
								}
							} 
						}
					}
					$byPrice = array();
					$byCogs = array();
					foreach($_REQUEST as $arg => $val) {
						$matches = array();
						if(preg_match("/^(price|cogs|id|gid|vid)_([0-9]+)_([A-Z]{3})_([0-9]+|setup|transfer)$/", $arg, $matches)) {
							$_REQUEST[$arg] = $this->getNumberFromInput($val);
							//print_r($matches);
							list($entirety, $field, $tldID, $currencyID, $cycleIDorType) = $matches;

							$this->validateParam($arg, $field == "id" || $field == "gid" || $field == "vid" ? INPUT_RESTRICTION_UNSIGNED_INT : INPUT_RESTRICTION_PRICE|INPUT_RESTRICTION_REQUIRED);

							$priceKey = serialize(array($tldID, $currencyID, $cycleIDorType));

							if($field == "price" || $field == "id") {
								if(!isset($byPrice[$priceKey])) $byPrice[$priceKey] = array();
								$byPrice[$priceKey][$field] = $_REQUEST[$arg];
							} else if(!($field == "vid")) {
								if(!isset($byCogs[$priceKey])) $byCogs[$priceKey] = array();
								$byCogs[$priceKey][$field] = $_REQUEST[$arg];
							}
						}
					}

					foreach($byPrice as $priceKey => $price) {
						list($tldID, $currencyID, $cycleIDorType) = unserialize($priceKey);
						
						$productVariant = array(
							"productID" => $productID,
							"tldID" => $tldID,
						);
						if(isset($productVariantMap[serialize($productVariant)])) {
							$productVariantID = $productVariantMap[serialize($productVariant)];
							$ghostProductVariantID = false;
						} else {
							if($ghostProductVariantSet->has($productVariant)) {
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)];
							} else {
								$ghostProductVariantSet->addElement($productVariant);
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)] = ++$ghostProductVariantCursor;
							}
						}
						
						$productVariantPrice = array(
							'productVariantPriceAmount' => (float) $price['price'],
							'currencyID' => $currencyID,
							//'tldID' => $tldID,
						);
						if($cycleIDorType === "setup") {
							$productVariantPrice['productVariantPriceType'] = getTypeID("price", TRANS_SETUP);
						} else if($cycleIDorType === "transfer") {
							$productVariantPrice['productVariantPriceType'] = getTypeID("price", TRANS_TRANSFER);
						} else {
							$productVariantPrice['productVariantPriceType'] = getTypeID("price", TRANS_GENERAL);
							$productVariantPrice['cycleID'] = $cycleIDorType;
						}
						if($price['id']) {
							$productVariantPrice['productVariantPriceID'] = (int) $price['id'];
						}
						if($productVariantID !== false) {
							$productVariantPrice['productVariantID'] = $productVariantID;
						} else {
							$ghostProductVariantPriceMap[$tldID][serialize($productVariantPrice)] = $ghostProductVariantID;
						}
						$productVariantPrices[] = $productVariantPrice;
					}
					foreach($byCogs as $cogsKey => $cogs) {
						list($tldID, $currencyID, $cycleIDorType) = unserialize($cogsKey);
						
						$productVariant = array(
							"productID" => $productID,
							"tldID" => $tldID,
						);
						if(isset($productVariantMap[serialize($productVariant)])) {
							$productVariantID = $productVariantMap[serialize($productVariant)];
							$ghostProductVariantID = false;
						} else {
							if($ghostProductVariantSet->has($productVariant)) {
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)];
							} else {
								$ghostProductVariantSet->addElement($productVariant);
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)] = ++$ghostProductVariantCursor;
							}
						}

						$productLedgerEntry = array(
                            'generalLedgerType' => 1,
                            'journalType' => 'general',
                            'ledgerAccountID' => getGeneralLedgerAccountID('domain_cogs'),
							'productLedgerDebit' => (float) $cogs['cogs'],
                            'productLedgerCredit' => 0,
							'currencyID' => $currencyID,
							'tldID' => $tldID,
						);
						if($cycleIDorType === "setup") {
							$productLedgerEntry['productLedgerComment'] = getTypeID("price", TRANS_SETUP);
                            $productLedgerEntry['ledgerAccountID'] = getGeneralLedgerAccountID('setup_cogs');
                        } else if($cycleIDorType === "transfer") {
							$productLedgerEntry['productLedgerComment'] = getTypeID("price", TRANS_TRANSFER);
						} else {
							$productLedgerEntry['productLedgerComment'] = getTypeID("price", TRANS_GENERAL);
							$productLedgerEntry['cycleID'] = $cycleIDorType;
						}
						if(isset($cogs["gid"])) {
							$productLedgerEntry["productLedgerID"] = (int) $cogs["gid"];
						}
						if($productVariantID !== false) {
							$productLedgerEntry['productVariantID'] = $productVariantID;
						} else {
							$ghostProductLedgerEntryMap[$tldID][serialize($productLedgerEntry)] = $ghostProductVariantID;
						}
						//print_r($cycleIDorType);
                        $productLedgerEntries[] = $productLedgerEntry;

                        // corresponding entry.
                        $productLedgerEntry["productLedgerCredit"] = $productLedgerEntry["productLedgerDebit"];
                        $productLedgerEntry["productLedgerDebit"] = 0;
                        $productLedgerEntry["ledgerAccountID"] = getGeneralLedgerAccountID('accounts_payable');
						if ($productVariantID !== false){
							$ghostProductLedgerEntryMap[$tldID][serialize($productLedgerEntry)] = $ghostProductVariantID;
						}
                        $productLedgerEntries[] = $productLedgerEntry;

                    }
				} else {
					foreach($_REQUEST as $arg => $val) {
						$matches = array();
						if(preg_match("/^(price|cogs|id|gid|vid)_([A-Z]{3})_([0-9]+)_([0-9]+)$/", $arg, $matches)) {
							//print_r($matches);
							list($entirety, $field, $currencyID, $cycleID, $priceType) = $matches;
							if($field == "vid") {
								$productVariant = array(
									"productID" => $productID,
								);
								if((int) $val > 0) {
									$productVariantMap[serialize($productVariant)] = (int) $val;
								}
							} 
						}
					}
					$byPrice = array();
					$byCogs = array();
					foreach($_REQUEST as $arg => $val) {
						$matches = array();
						if(preg_match("/^(price|cogs|id|gid|vid)_([A-Z]{3})_([0-9]+)_([0-9]+)$/", $arg, $matches)) {
							$_REQUEST[$arg] = $this->getNumberFromInput($val);
							//print_r($matches);
							list($entirety, $field, $currencyID, $cycleID, $priceType) = $matches;

							$this->validateParam($arg, $field == "id" || $field == "gid" || $field == "vid" ? INPUT_RESTRICTION_UNSIGNED_INT : INPUT_RESTRICTION_PRICE|INPUT_RESTRICTION_REQUIRED);

							$priceKey = serialize(array($currencyID, $cycleID, $priceType));

							if($field == "price" || $field == "id") {
								if(!isset($byPrice[$priceKey])) $byPrice[$priceKey] = array();
								$byPrice[$priceKey][$field] = $_REQUEST[$arg];
							} else if(!($field == "vid")) {
								if(!isset($byCogs[$priceKey])) $byCogs[$priceKey] = array();
								$byCogs[$priceKey][$field] = $_REQUEST[$arg];
							}
						}
					}
					
					foreach($byPrice as $priceKey => $price) {
						list($currencyID, $cycleID, $priceType) = unserialize($priceKey);
						
						$productVariant = array(
							"productID" => $productID,
						);
						if(isset($productVariantMap[serialize($productVariant)])) {
							$productVariantID = $productVariantMap[serialize($productVariant)];
							$ghostProductVariantID = false;
						} else {
							if($ghostProductVariantSet->has($productVariant)) {
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)];
							} else {
								$ghostProductVariantSet->addElement($productVariant);
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)] = ++$ghostProductVariantCursor;
							}
						}

						$productVariantPrice = array(
							'productVariantPriceAmount' => (float) $price['price'],
							'currencyID' => $currencyID,
							'cycleID' => $cycleID,
						);
						if((int) $priceType === (int) getTypeID("price", TRANS_SETUP)) {
							$productVariantPrice['productVariantPriceType'] = getTypeID("price", TRANS_SETUP);
						} else {
							$productVariantPrice['productVariantPriceType'] = getTypeID("price", TRANS_GENERAL);
						}
						if($price['id']) {
							$productVariantPrice['productVariantPriceID'] = (int) $price['id'];
						}
						if($productVariantID !== false) {
							$productVariantPrice['productVariantID'] = $productVariantID;
						} else {
							$ghostProductVariantPriceMap[serialize($productVariantPrice)] = $ghostProductVariantID;
						}
						$productVariantPrices[] = $productVariantPrice;
					}
					foreach($byCogs as $cogsKey => $cogs) {
						list($currencyID, $cycleID, $priceType) = unserialize($cogsKey);
												
						$productVariant = array(
							"productID" => $productID,
						);
						if(isset($productVariantMap[serialize($productVariant)])) {
							$productVariantID = $productVariantMap[serialize($productVariant)];
							$ghostProductVariantID = false;
						} else {
							if($ghostProductVariantSet->has($productVariant)) {
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)];
							} else {
								$ghostProductVariantSet->addElement($productVariant);
								$productVariantID = false;
								$ghostProductVariantID = $ghostProductVariantMap[serialize($productVariant)] = ++$ghostProductVariantCursor;
							}
						}
						//echo '[[[' . (int) $productVariantID . ']]]';

						$productLedgerEntry = array(
                            'generalLedgerType' => 1,
                            'journalType' => 'general',
                            'ledgerAccountID' => getGeneralLedgerAccountID('cogs'),
							'productLedgerDebit' => (float) $cogs['cogs'],
                            'productLedgerCredit' => 0,
							'currencyID' => $currencyID,
							'cycleID' => $cycleID,
						);
						if((int) $priceType === (int) getTypeID("price", TRANS_SETUP)) {
							$productLedgerEntry['productLedgerComment'] = getTypeID("price", TRANS_SETUP);
                            $productLedgerEntry['ledgerAccountID'] = getGeneralLedgerAccountID('setup_cogs');
						} else {
							$productLedgerEntry['productLedgerComment'] = getTypeID("price", TRANS_GENERAL);
						}
						if(isset($cogs["gid"])) {
							$productLedgerEntry["productLedgerID"] = (int) $cogs["gid"];
						}
						if($productVariantID !== false) {
							$productLedgerEntry['productVariantID'] = $productVariantID;
						} else {
							$ghostProductLedgerEntryMap[serialize($productLedgerEntry)] = $ghostProductVariantID;
						}
						$productLedgerEntries[] = $productLedgerEntry;

                        // corresponding entry.
                        $productLedgerEntry["productLedgerCredit"] = $productLedgerEntry["productLedgerDebit"];
                        $productLedgerEntry["productLedgerDebit"] = 0;
                        $productLedgerEntry["ledgerAccountID"] = getGeneralLedgerAccountID('accounts_payable');
                        $productLedgerEntries[] = $productLedgerEntry;
					}
				}

				$productVariants = array();
				foreach($productVariantMap as $productVariant => $productVariantID) {
					$productVariant = unserialize($productVariant);
					$productVariant["productVariantID"] = $productVariantID;
					$productVariants[] = $productVariant;
				}
				$newProductVariants =& new Table($productVariants, "productVariantID");
				$newProductVariantPrices =& new Table($productVariantPrices, "productVariantPriceID");
				$newProductLedgerEntries =& new Table($productLedgerEntries, "productLedgerID");
				
				$newProductVariantsByID =& $newProductVariants->getAssociated();
				$newProductVariantPricesByID =& $newProductVariantPrices->getAssociated();
				$newProductLedgerEntriesByID =& $newProductLedgerEntries->getAssociated();

				$newProductVariantIDs = $newProductVariants->getKeys();

				$newLedgerEntriesByVariant = $newProductLedgerEntries->getAssociatedSetsBy("productVariantID");

				//print_r($byPrice);
				//echo '<pre>';
				//print_r($byCogs);
				//echo '</pre>';
				//print_r($productVariants);
				//print_r($productVariantMap);
				//print_r($productVariantPrices);
				//print_r($productLedgerEntries);
				//print_r($ghostProductVariantMap);
				//print_r($ghostProductVariantPriceMap);
				//print_r($ghostProductLedgerEntryMap);
				
				//print_r($_REQUEST);
				//print_r($validateArgs);
				//print_r($byPrice);
				//echo 'asdfs';
				//print_r($productPrices);
				//print_r($newPricesTable);
				//print_r($newPricesByID);
				//print_r($this->invalidParams);
				//echo '</pre>';

			}
			//exit;
			
			if(!$this->validationRedirect(
				array(
					"loadActionFile" => "products",
					"loadActionID" => getActionID("EditProductPrices"),
				)
			)) break;
			
			$q =& new MBQuery(
				"GetProducts", "",
				array(
					'productID' => $productID,
					'getProductVariantData' => 1,
					'getProductVariantPriceData' => 1,
				)	
			);
			if(!$this->addErrorsFromResult($priceResult =& $q->getResult())) break;

			$oldProductVariantTable =& $priceResult->getTableFrom(
				array("mbapi", "results", "products", "product", "productVariants", "productVariant"),
				"productVariantID"
			);
			$oldProductVariantsByID = $oldProductVariantTable->getAssociated();
			
			$oldProductVariantPriceTable =& $priceResult->getTableFrom(
				array("mbapi", "results", "products", "product", "productVariantPrices", "productVariantPrice"),
				"productVariantPriceID"
			);
			$oldProductVariantPricesByID =& $oldProductVariantPriceTable->getAssociated();
			$oldPriceTable =& $oldProductVariantPriceTable->getLeftJoinedWith($oldProductVariantTable);
			$oldPricesByID = $oldPriceTable->getAssociated();
						
			//echo '234523452345: ';
			//print_r($newProductVariantIDs);
			$cogsQuery =& new MBQuery(
				"GetProductLedgerEntries", "",
				array(
					"productVariantID" => $newProductVariantIDs,
					"getProductLedgerCycleIDs" => 1,
				)
			);
			if(!$this->addErrorsFromResult($cogsResult =& $cogsQuery->getResult())) break;

			$cogsSubject =& $cogsResult->getSubjectFrom(array("mbapi", "results", "productLedgerEntries", "productLedgerEntry"));

			$c = $cogsSubject->getNumRows();
			for($i = 0; $i < $c; $i++) {
				if(isset($cogsSubject->structure[$i]) && isset($cogsSubject->structure[$i]["cycles"][0]["cycle"][0]["cycleID"][0])) {
					$cogsSubject->structure[$i]["cycleID"] = $cogsSubject->structure[$i]["cycles"][0]["cycle"][0]["cycleID"][0];
				}
			}
			
			$oldLedgerEntriesTable =& $cogsSubject->getTable("productVariantID");
			$oldCogsTable =& $oldLedgerEntriesTable->getLeftJoinedWith($oldProductVariantTable);
			$oldCogsTable->setKey("productVariantID");
			$oldCogsByID = $oldCogsTable->getAssociated();
			$oldLedgerEntriesByVariant = $oldLedgerEntriesTable->getAssociatedSetsBy("productVariantID");
			
			

			//echo '<pre>';
			//print_r($priceResult);
			//print_r($oldProductVariantTable);
			//print_r($oldPriceTable);
			//print_r($oldPricesByID);
			//print_r($oldCogsTable);
			//print_r($oldCogsByID);
			//exit;
			
			$updateParams = array();
			$deleteVariants = array();
			$deleteVariantPrices = array();


			//$oldProductVariantsByID
			//$newProductVariantsByID

			//print_r($oldProductVariantsByID);
			//print_r($newProductVariantsByID);

			//print_r($oldProductVariantPricesByID);
			//print_r($newProductVariantPricesByID);
			
			$productVariants = array();
			$productLedgerEntries = array();

			$pricesForExistingVariants = array();
			
			$newPricesByVariant =& $newProductVariantPrices->getAssociatedSetsBy("productVariantID");
			foreach($newPricesByVariant as $productVariantID => $newPrices) {
				if(!$productVariantID) continue;
				foreach($newPrices as $newPrice) {
					if(isset($newPrice["productVariantPriceID"])) {
						$productVariantPriceID = $newPrice["productVariantPriceID"];
						$oldPrice = (array) $oldProductVariantPricesByID[$productVariantPriceID];
						$diffPrice = array_diff_assoc($newPrice, $oldPrice);
						if($diffPrice) {
							if(!isset($pricesForExistingVariants[$productVariantID])) {
								$pricesForExistingVariants[$productVariantID] = array();
							}
							$pricesForExistingVariants[$productVariantID][] = $diffPrice + array("productVariantPriceID" => $productVariantPriceID);
						}
					} else {
						if(!isset($pricesForExistingVariants[$productVariantID])) {
							$pricesForExistingVariants[$productVariantID] = array();
						}
						$pricesForExistingVariants[$productVariantID][] = $newPrice;
					}
				}
			}
			//echo 'test';
			//print_r($pricesForExistingVariants);
			
			//foreach($pricesForExistingVariants as $productVariantID => $prices) {
			//	$productVariant
			//}

			if($domain) {
				foreach($ghostProductVariantMap as $ghostProductVariant => $ghostID) {
					$productVariant = unserialize($ghostProductVariant);

					foreach($ghostProductVariantPriceMap as $tldID => $ghostProductVariantPriceMapForTLD) {
						foreach($ghostProductVariantPriceMapForTLD as $ghostProductVariantPrice => $potentialGhostID) {
							//echo $potentialGhostID . ':' . $ghostID . "<br />";
							if($potentialGhostID == $ghostID) {
								if(!isset($productVariant['productVariantPrices'][0]['productVariantPrice'])) {
									$productVariant['productVariantPrices'] = array(
										0 => array(
											'productVariantPrice' => array(),
										),
									);
								}
								$productVariant['productVariantPrices'][0]['productVariantPrice'][] = unserialize($ghostProductVariantPrice);
							}
						}
					}

					foreach($ghostProductLedgerEntryMap as $tldID => $ghostProductLedgerEntryMapForTLD) {
						foreach($ghostProductLedgerEntryMapForTLD as $productLedgerEntry => $potentialGhostID) {
							//echo $potentialGhostID . ':' . $ghostID . "<br />";
							if($potentialGhostID == $ghostID) {
								$productLedgerEntries[] = unserialize($ghostProductVariantPrice);
							}
						}
					}
					$productVariants[] = $productVariant;
				}
			} else {
				foreach($ghostProductVariantMap as $ghostProductVariant => $ghostID) {
					$productVariant = unserialize($ghostProductVariant);

					foreach($ghostProductVariantPriceMap as $ghostProductVariantPrice => $potentialGhostID) {
						//echo $potentialGhostID . ':' . $ghostID . "<br />";
						if($potentialGhostID == $ghostID) {
							if(!isset($productVariant['productVariantPrices'][0]['productVariantPrice'])) {
								$productVariant['productVariantPrices'] = array(
									0 => array(
										'productVariantPrice' => array(),
									),
								);
							}
							$productVariant['productVariantPrices'][0]['productVariantPrice'][] = unserialize($ghostProductVariantPrice);
						}
					}

					foreach($ghostProductLedgerEntryMap as $productLedgerEntry => $potentialGhostID) {
						//echo $potentialGhostID . ':' . $ghostID . "<br />";
						if($potentialGhostID == $ghostID) {
							$productLedgerEntries[] = unserialize($ghostProductVariantPrice);
						}
					}
					$productVariants[] = $productVariant;
				}
			}
			//print_r($productVariants);
			//print_r($productLedgerEntries);
		
		
			foreach($oldProductVariantsByID as $productVariantID => $oldProductVariant) {
				$variant = array();
				if(isset($newProductVariantsByID[$productVariantID]) && $newProductVariantsByID[$productVariantID] !== NULL) {
					$oldProductVariant["productID"] = $productID;
					$newProductVariant = $newProductVariantsByID[$productVariantID];
					$diffProductVariant = array_diff_assoc($newProductVariant, $oldProductVariant);
					if($diffProductVariant) {
						$variant = $diffProductVariant + array('productVariantID' => $newProductVariant['productVariantID']);
					}
					if(isset($pricesForExistingVariants[$productVariantID])) {
						$variant += array(
							"productVariantID" => $productVariantID,
							"productVariantPrices" => array(
								0 => array(
									"productVariantPrice" => $pricesForExistingVariants[$productVariantID],
								),
							),
						);
					}
					if($variant) $updateParams[] = $variant;
				} else {
					if(isset($diffProductVariant['productVariantID'])) {
						$deleteVariants[] = $oldProductVariant['productVariantID'];
					}
				}				
			}

			$updateParams = array_merge($updateParams, $productVariants);
			
/*			
			echo "insert:\n";
			print_r($insertProductVariants);
			echo "update:\n";
			print_r($updateParams);
			echo "delete:\n";
			print_r($deleteParams);
			echo '</pre>';
*/
			
			
			
			
			if($updateParams || $deleteVariants || $deleteVariantPrices) {
				$setParams = array(
					"productID" => $productID,
				);
				if($updateParams) {
					$setParams += array(
						"productVariants" => array(
							"productVariant" => $updateParams,
						),
					);
				}
				if($deleteVariants) {
					$setParams += array(
						"removeProductVariantIDs" => array(
							"productVariantID" => $deleteVariants,
						),
					);
				}
				if($deleteVariantPrices) {
					$setParams += array(
						"removeProductVariantPriceIDs" => array(
							"productVariantPriceID" => $deleteVariantPrices,
						),
					);
				}
			} else {
				$setParams = false;
			}
			
			if($setParams) {
				$updateQuery =& new MBQuery("SetProducts", "update", $setParams);

				//echo "<pre>";
				//echo htmlentities($updateQuery->getXMLQuery());
				//echo "</pre>";
				//exit;

				if(!$this->addErrorsFromResult($updateResult =& $updateQuery->getResult())) break;
				//print_r($updateResult);

				//echo "</pre>";
			}
			//exit;
			
			//echo '<pre>';
			//print_r($oldLedgerEntriesByVariant);
			//print_r($newLedgerEntriesByVariant);
			//print_r($oldLedgerEntriesTable);
			//print_r($newProductLedgerEntries);
			$oldLedger = $oldLedgerEntriesTable->getTable();
			$newLedger = $newProductLedgerEntries->getTable();
			$newLedgerByID = $newProductLedgerEntries->getAssociatedSetsBy("productLedgerID");
			$oldLedgerIDs = $oldLedgerEntriesTable->getKeys();
			$newLedgerIDs = $newProductLedgerEntries->getKeys();
			$deletableLedgerIDs = array_diff($oldLedgerIDs, $newLedgerIDs);
			$updatableLedgerIDs = array_diff($newLedgerIDs, $oldLedgerIDs);
			//$insertableLedgerIDs = array_diff($newLedgerIDs, $updatableLedgerIDs, $deletableLedgerIDs);
			//print_r($oldLedger);
			//print_r($newLedger);
			//print_r($updatableLedgerIDs);
			//print_r($deletableLedgerIDs);
			//print_r($productLedgerEntries);
			//exit;
			
			
			//echo '<pre>';
			if($updatableLedgerIDs) {
				foreach($updatableLedgerIDs as $productLedgerID) {
					if($productLedgerID) {
						$ledger = $newLedgerByID[$productLedgerID];
						if($ledger["cycleID"]) {
							$ledger += array(
								"cycleIDs" => array(
									"cycleID" => $ledger["cycleID"]
								)
							);
						}
						if($ledger["currencyID"]) {
							$ledger += array(
								"currencyIDs" => array(
									"currencyID" => $ledger["currencyID"]
								)
							);
						}
						$updateQuery =& new MBQuery("SetProductLedgerEntries", "update", $ledger);
						//htmlentities($updateQuery->getXMLQuery());
						if(!$this->addErrorsFromResult($updateResult =& $updateQuery->getResult())) break 2;
					} else {
						foreach($newLedgerByID[$productLedgerID] as $ledger) {
							unset($ledger["productLedgerID"]);
                            if($ledger["cycleID"]) {
                                $ledger += array(
                                    "cycleIDs" => array(
                                        "cycleID" => $ledger["cycleID"]
                                    )
                                );
                            }
                            if($ledger["currencyID"]) {
                                $ledger += array(
                                    "currencyIDs" => array(
                                        "currencyID" => $ledger["currencyID"]
                                    )
                                );
                            }
							$insertQuery =& new MBQuery("SetProductLedgerEntries", "insert", $ledger);
							//htmlentities($insertQuery->getXMLQuery());
							if(!$this->addErrorsFromResult($insertResult =& $insertQuery->getResult())) break 3;
						}
					}
				}
			}
			if($deletableLedgerIDs) {
				foreach($deletableLedgerIDs as $productLedgerID) {
					$deleteQuery =& new MBQuery(
						"SetProductLedgerEntries", "delete",
						array(
							"productLedgerID" => $productLedgerID,
						)
					);
					//htmlentities($deleteQuery->getXMLQuery());
					if(!$this->addErrorsFromResult($deleteResult =& $deleteQuery->getResult())) break 2;
				}
			}
			
			//echo '</pre>';
			//flush();
			//exit;
			
			if(!$this->goBack()) {
				redirect(
					"products.php", getActionID("ShowProductDetails"),
					array(
						"productID" => $productID,
						"loadActionFile" => "products",
						"loadActionID" => getActionID("EditProductPrices"),
					)
				);
			}
		} while (false);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
?>