<?php
// $Id: ShowTLDs.php 12266 2007-11-21 17:43:16Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

/**
 * This Action object collects information on all product TLDs that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Andy Christ
 * @author  Ben Woolley
 */
class ShowTLDs extends MB_Action {
	/**
	 * The constructor for a ShowTLDs action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowTLDs() {
		parent::MB_Action();
		$this->stringRep = "ShowTLDs";
		
		$this->params = array(
			"tldID",
			"tldName",
			"tldExtension",
			"tldActive",
			"tldWhoisServer",
			"tldWhoisResp",
			"tldAutoSearch",
			"tldTransferMinimumCycleID",
			"tldMinimumTerm",
			"tldMaximumTerm",
			"tldRenewType",
			"tldIsTransferable",
			"tldTransferType",
			"tldIDNSupport",
			"tldIDNType",
			"tldRemoteLockEnabled",
			"orderFormID"
		);
		$this->sortColumns = array("tldDisplayOrder", "ASC");
	}

	/**
	 * The main method of this class that executes the ShowTLDs action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$query = $this->getMBAPIQuery("GetTLDs", "", array("getOrderFormData" => 1, "getCycleData" => 1), 1, 1, 1, 1);
		$CONF["debug"]->add("Action Command", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result", $data);

		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {
			$this->smartyVars["TLDs"] = collapseArrayElements(protectArray($results["TLDs"][0]["TLD"]));
			$this->smartyVars["acceptableUnacceptable"] = $CONF["types"]["tld"];
			$this->smartyVars["tldRenewTypes"] = $CONF["types"]["tldRenew"];
			$this->smartyVars["enabledDisabled"] = array(TRANS_DISABLED, TRANS_ENABLED);
			$this->smartyVars["yesNo"] = array(TRANS_YES, TRANS_NO);
			$this->smartyVars["transferTypes"] = $CONF["types"]["tldTransfer"];
			$this->smartyVars["IDNTypes"] = $CONF["types"]["tldIDN"];
			$this->mergeCycleData();
												
			$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"][0]);
			$this->addTemplate("products/ShowTLDs", $this->smartyVars);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowProducts
?>