<?php
// $Id: ShowWizardSteps.php 10474 2007-01-11 23:58:14Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/include/wizard/WizardStep.php");

/**
 * This Action object collects information on all wizard steps that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Ben Woolley
 */
class ShowWizardSteps extends MB_Action {
	/**
	 * The constructor for a ShowWizardSteps action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowWizardSteps() {
		parent::MB_Action();
		$this->stringRep = "ShowWizardSteps";
		
		$this->params = array(
			"productID",
			"productAddonId",
			"productAddonName",
			"stepID",
			"serverGroupID",
			"productAddonActive",
			"productAddonUpgrade",
			"productAddonOrder",
		);
		$this->sortColumns = array("");
	}

	/**
	 * The main method of this class that executes the ShowWizardSteps action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$query = $this->getMBAPIQuery("GetWizardQuestions", "", array(), 1, 1, 1, 1);
		$CONF["debug"]->add("Action Command", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result", $data);

		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {						
			$questions = collapseArrayElements(protectArray($results["questions"][0]["question"]));
			if(is_array($questions[0])) {
				$steps = associateArrayRowsByID($questions, "stepID");
			} else {
				$steps = array();
			}

			$cmd = "
<mbapi>
	<command>GetWizardQuestionParams</command>
	<subCommand></subCommand>
	<params>
		<getParamValues>1</getParamValues>
	</params>
</mbapi>";

			$CONF["debug"]->add("Action Command", htmlentities($cmd));
			$data2 = dispatchMBAPI($cmd, 1);
			$CONF["debug"]->add("Action Result", $data2);

			$tmp = collapseArrayElements(protectArray($data2["mbapi"][0]["results"][0]["questionParams"][0]["questionParam"]));
			$questionParams =& associateArrayRowsByID($tmp, "questionID");
			//print_r($questionParams);				

			$cmd = "
<mbapi>
	<command>GetWizardSteps</command>
	<subCommand></subCommand>
	<params>
	</params>
</mbapi>";

			$CONF["debug"]->add("Action Command", htmlentities($cmd));
			$data3 = dispatchMBAPI($cmd, 1);
			$CONF["debug"]->add("Action Result", $data3);
			
			$tmp = collapseArrayElements(protectArray($data3["mbapi"][0]["results"][0]["steps"][0]["step"]));
			if(is_array($tmp)) {
				$displaySteps =& associateArrayRowByID($tmp, "stepID");
			} else {
				$displaySteps = array();
			}
			
			
			foreach($steps as $stepID => $questions) {
				$s =& new WizardStep(TRANS_STEP_PREVIEW, TRANS_STEP_PREVIEW_MESG);
				foreach($questions as $question) {
					$q =& new WizardQuestion(QUESTION_INPUT_HIDDEN, "");

					$qid = $question['questionID'];
					$params = $questionParams[$qid];

					foreach($params as $param) {
						$paramName = $param['questionParamName'];
						$paramValues = $param['questionParamValues']['questionParamValue'];
						if(!is_array($paramValues)) {
							$paramValues = array($paramValues);
						}
						//echo '<pre>';
						//print_r($paramName);
						//print_r($paramValues);
						//echo '</pre>';
						call_user_func_array(array(&$q, $paramName), array_map(array('ShowWizardSteps', 'constantize'), $paramValues));
					}

					$s->addQuestion($q);
				}
				//echo '<pre>';
				//print_r($s);
				//echo '</pre>';

				$displaySteps[$stepID]['preview'] = $s->display(true, true);
				$displaySteps[$stepID]['questionCount'] = count($questions);
			}
			
			$this->smartyVars['steps'] = array_values($displaySteps);
						
			// $smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"][0]);
			$this->addTemplate("products/ShowWizardSteps", $this->smartyVars);
			
			
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
	
	/**
	 * if a value is a constant, resolve it
	 *
	 * @param  string  value
	 * @return  string|constant  value or resolved value
	 */
	function constantize($value) {
		return defined($value) ? constant($value) : $value;
	}
	
} // ShowProducts
?>