<?php
// $Id: ShowServerGroups.php 11377 2007-06-18 15:37:21Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-mbapi/include/modules/module.php");

/**
 * This Action object collects information on all servergroups that match the given parameters, and 
 * generates the display of that information.
 *
 * @author  Andy Christ
 */
class ShowServerGroups extends MB_Action {
	/**
	 * The constructor for a ShowServerGroups action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowServerGroups() {
		parent::MB_Action();
		$this->stringRep = "ShowServerGroups";
		
		$this->params = array("serverGroupID",
			"serverGroupName");
		$this->sortColumns = array("serverGroupID",
			"serverGroupName");
	}

	/**
	 * The main method of this class that executes the ShowServerGroups action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$query = $this->getMBAPIQuery("GetServerGroups", "", array("getPanels" => 1));
		$CONF["debug"]->add("Action Command", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result", $data);

		$results =& $data["mbapi"][0]["results"][0];

		//Get the server names
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "moduleDisplayName", "getFormAttributes" => 1));
		$CONF["debug"]->add("Action Command", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result", $data);

		//$serverNames =& $data["mbapi"][0]["results"][0];

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {

			$serverNames = collapseArrayElements($data["mbapi"][0]["results"][0]);

			if(is_array($serverNames["configs"]["config"][0])) {
				foreach($serverNames["configs"]["config"] as $key => $value) {
					$temp[$value["configGroupID"]] = $value;
				}
				$serverNames = $temp;
				$temp = NULL;
			} else {
				$serverNames[$serverNames["configs"]["config"]["configGroupID"]] = $serverNames["configs"]["config"];
			}
		}

		//Get the max accounts for each panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelMaxAccounts", "getFormAttributes" => 1));
		$CONF["debug"]->add("Action Command", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result", $data);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
			foreach($maxAccounts as $value) {
				$serverNames[$value["configGroupID"]]["panelMaxAccounts"] = (int) $value["configParamValue"];
			}
		}

		//Get the current number of accounts per panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelCurrentAccounts", "getFormAttributes" => 1));
		$CONF["debug"]->add("Action Command", htmlentities($query));
		$data = dispatchMBAPI($query, 1);
		$CONF["debug"]->add("Action Result", $data);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
			foreach($maxAccounts as $value) {
				$serverNames[$value["configGroupID"]]["panelCurrentAccounts"] = (int) $value["configParamValue"];
			}
		}

		if($this->addMBAPIErrors($data) === 0) {
			$this->smartyVars = array();
			// Get list of files in modules/panel to build panel list
			// Only if a Panel isn't associated yet, otherwise, must keep the same Panel.
			$panels = getAvailableModules("panel");
			sort($panels);
			foreach ($panels as $panelFile) {
				$panelFile = substr($panelFile, 0, -4);
				require_once mb_path("/lib-mbapi/include/modules/panel/".secureInput($panelFile).".php");
				$moduleInfo = call_user_func(array($panelFile,"getModuleInfo"));
				$p =& new $panelFile(array("subCommand" => "getCapabilities"));;
				$canGetServerLoad = $p->capabilities[PANEL_GETSERVERLOAD];
				$useInternalIPQueue =  $p->capabilities[PANEL_USE_INTERNAL_IP_QUEUE];

				//The smarty var name panels will not work
				$this->smartyVars["controlPanels"][$panelFile] = $moduleInfo["displayName"];
				$this->smartyVars["canGetServerLoad"][$panelFile] = $canGetServerLoad;
				$this->smartyVars["useInternalIPQueue"][$panelFile] = $useInternalIPQueue;
			}

			$this->smartyVars["paginationLinks"] = $results["paginationLinks"][0];
			$this->smartyVars = collapseArrayElements(protectArray($this->smartyVars));
			$ips = array();
			// collapse the outer array first and properly format the sub array if 1 item or more than 1
			if(is_array($results["serverGroups"][0]["serverGroup"])) {
				foreach($results["serverGroups"][0]["serverGroup"] as $key => $value) {
					$panels = $value["panels"][0]["panel"];

					$details = correctArray($panels,"panels");

					foreach((array)$details as $panelInstance) {
						if($this->smartyVars["useInternalIPQueue"][strtolower($panelInstance["configGroupTransKey"])]) {
							//Pull the count of IPs for this panel instance
							$panelDetails = correctArray($panelInstance);
	
							$query = $this->getMBAPIQuery("GetIPLogs", "", array("configGroupID" => $panelDetails["configGroupID"]));
							$CONF["debug"]->add("Action Command", htmlentities($query));
							$data = correctArray(dispatchMBAPI($query, 1));
							$CONF["debug"]->add("Action Result", $data);
							$ips[$panelDetails["configGroupID"]] = $data["mbapi"][0]["header"][0]["numTotalRecords"];
						}
					}
					$configGroups[$key] = collapseArrayElements($value);
					if(count($panels)>1) {
						$value = collapseArrayElements(arrayParentToChild($panels,"panels"));
						$configGroups[$key]["panels"] = $value;
						$configGroups[$key]["panelType"] = $value[0]["configGroupTransKey"];
						$configGroups[$key]["canGetServerLoad"] = $this->smartyVars["canGetServerLoad"][strtolower($value[0]["configGroupTransKey"])];
						$configGroups[$key]["useInternalIPQueue"] = $this->smartyVars["useInternalIPQueue"][strtolower($value[0]["configGroupTransKey"])];

					} else {
						$panels = collapseArrayElements($panels);
						unset($configGroups[$key]["panels"]);
						if($panels) {
							$configGroups[$key]["panels"][0] = $panels;
							$configGroups[$key]["panelType"] = $panels["configGroupTransKey"];
							$configGroups[$key]["canGetServerLoad"] = $this->smartyVars["canGetServerLoad"][strtolower($panels["configGroupTransKey"])];
							$configGroups[$key]["useInternalIPQueue"] = $this->smartyVars["useInternalIPQueue"][strtolower($panels["configGroupTransKey"])];
						}
					}
				}
			}

			if(count($serverNames)>1) {
				$this->smartyVars["serverNames"] = $serverNames;
			} else {
				$this->smartyVars["serverNames"][0] = $serverNames;
			}
			$this->smartyVars["ips"] = $ips;
			$this->smartyVars["serverGroups"] = $configGroups;
			$this->mergeSystemNavData();
			$this->addTemplate("servers/ShowServerGroups", $this->smartyVars, 1);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowServerGroups
?>