#!/usr/bin/env python

"""
$Id: mbapilib.py 12474 2008-01-14 23:40:41Z bwoolley $
+----------------------------------------------------------------------+
| ModernBill [TM] .:. Client Billing System                            |
+----------------------------------------------------------------------+
| Copyright (c) 2001-2006 ModernGigabyte, LLC                          |
+----------------------------------------------------------------------+
| This source file is subject to the ModernBill End User License       |
| Agreement (EULA), that is bundled with this package in the file      |
| LICENSE, and is available at through the world-wide-web at           |
| http://www.modernbill.com/extranet/LICENSE.txt                       |
| If you did not receive a copy of the ModernBill license and are      |
| unable to obtain it through the world-wide-web, please send a note   |
| to license@modernbill.com so we can email you a copy immediately.    |
+----------------------------------------------------------------------+
| Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
| Support: http://www.modernsupport.com/modernbill/                    |
+----------------------------------------------------------------------+
| ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
+----------------------------------------------------------------------+

Python MBAPI library
"""

import re
strip_xml_re = re.compile("[\\x00-\\x08\\x0b-\\x0c\\x0e-\\x1f]")

class maps:
	bool = {
		True: "true",
		False: "false",
	}

	active = {
		True: "active",
		False: "inactive",
	}

class MBAPIError(Exception):
	pass

class ItemsOf:
	def __init__(self, container):
		self.container = container
		
	def __getattribute__(self, name):
		return self.__dict__["container"][name]

class MBAPISubject:
	def __init__(self, element, field_map = {}):
		self._init_subject(element, field_map)
	
	def _init_subject(self, element, field_map = {}):
		self.element = element
		self.field_map = field_map
		self.index = 0
		self.fields = ItemsOf(self)
		
	def getContent(self, cast = str):
		text = ""
		for node in self.element.childNodes:
			if node.nodeType == node.TEXT_NODE:
				text += node.data
		return cast(text)
	
	def __str__(self):
		return self.getContent(str)
	
	def __int__(self):
		return self.getContent(int)
	
	def __nonzero__(self):
		return self.getContent(bool)
		
	def __len__(self):
		return len(str(self))
	
	def __getitem__(self, name):
		if hasattr(name, "split"):
			subject = self.getSubjectFrom(name)
			if subject is not None:
				if name in self.field_map:
					return subject.getContent(self.field_map[name])
				else:
					return subject
			else:
				raise KeyError, name
		else:
			self.index = name
			return self

	def getSubjectFrom(self, path, field_map = {}):
		element = self.element
				
		if not hasattr(path, "append"):
			path = [path]
		
		for point in path:
			elements = element.getElementsByTagName(point)
			if elements:
				element = elements[self.index]
			else:
				raise KeyError, point
				
		return MBAPISubject(element, field_map)


class MBAPIHeader(MBAPISubject):
	def __init__(self, element):
		self._init_subject(
			element,
			{
				"lastInsertID": int,
				"success": bool,
				"numResults": int,
				"numAffectedRows": int,
				"numTotalRecords": int,
				"errorCount": int,
				"timestamp": int,
			}
		)
		
	
class MBAPIResult(MBAPISubject):
	def __init__(self, result):
		from xml.dom.minidom import parseString

		self.xml = result
		self.dom = parseString(result)
		
		elements = self.dom.getElementsByTagName("mbapi")
		self._init_subject(elements and elements[0] or None)
		
		self.header = MBAPIHeader(self.getSubjectFrom("header").element)
		self.results = None
		
		if int(self.header["errorCount"]):
			err = self.header["errors"]["error"]
			message = str(err["title"]) + ": " + str(err["message"])
			raise MBAPIError, message
			
		
	def getHeader(self):
		return self.header
		

class XMLTree:
	def __init__(self, struct = {}, isUTF8 = False):
		self._init_xml(self, struct, isUTF8)
		
	def _init_xml(self, struct = {}, isUTF8 = False):
		self.depth = 0
		self.indent = "\t"
		self.struct = struct
		self.xml = ""
		self.isUTF8 = isUTF8
	
	def escape_sql(self, string):
		return unicode(string).replace(
			"'",
			"''"
		).replace(
			"\\",
			"\\\\"
		)

	def escape_xml(self, string):
		return strip_xml_re.sub(
			"",
			unicode(string).replace(
				'&',
				'&amp;'
			).replace(
				'<',
				'&lt;'
			).replace(
				'"',
				'&quot;'
			),
		)
		
	def escape(self, string):
		if self.isUTF8:
			string = unicode(str(string), encoding = "utf_8")

		if isinstance(string, unicode):
			string = self.escape_xml(string)
			import codecs
			encoder = codecs.getencoder("ascii")
			string = encoder(string, "xmlcharrefreplace")[0]

		return self.escape_xml(string)

	def walk_xml(self, struct, parent_name = None):
		for element in struct:
			content = struct[element]
			if hasattr(content, "update"):
				# dictionary list
				self.xml += "%(prefix)s<%(element)s>\n" % {
					"prefix": self.indent * self.depth,
					"element": element,
				}
				self.depth += 1
				
				for name in content:
					self.walk_xml({name: content[name]}, name)
				
				self.depth -= 1
				self.xml += "%(prefix)s</%(element)s>\n" % {
					"prefix": self.indent * self.depth,
					"element": element,
				}
				
			elif hasattr(content, "append"):
				# ordered list
				for value in content:
					self.walk_xml({parent_name: value})
					
			else:
				#string
				if content != "":
					self.xml += "%(prefix)s<%(element)s>%(content)s</%(element)s>\n" % {
						"prefix": self.indent * self.depth,
						"element": element,
						"content": self.escape(content)
					}
				else:
					self.xml += "%(prefix)s<%(element)s/>\n" % {
						"prefix": self.indent * self.depth,
						"element": element,
					}
					

class MBAPICommand(XMLTree):
	def __init__(self, connection, command, subCommand = "", params = {}, isUTF8 = False):
		self._init_xml({})
		self.connection = connection
		self.command = command
		self.subCommand = subCommand
		self.params = params
		self.isUTF8 = isUTF8
		
	def toStruct(self):
		self.struct = {
			"mbapi": {
				"command": self.command,
				"subCommand": self.subCommand,
				"params": self.params,
			}
		}
		return self.struct
		
	def toXML(self):
		self.xml = ""
		self.walk_xml(self.toStruct())
		return self.xml

	def call(self):
		return self.connection.call(self.toXML())
		

class MBAPIConnection:
	delim = "<?mb end ?>"

	def __init__(self):
		self.initConnection()
	
	def initConnection(self):
		self.isUTF8 = True

	def call(self, command):
		#print command
		self.input.write("%s\r\n%s\r\n" % (command, self.delim))
		self.input.flush()
		
		response = ""
		#for line in self.output:
		while True:
			line = self.output.readline()
			if not line:
				print line
				import sys
				sys.exit(1)
				
			#print "LINE:", line
			
			if self.delim in line:
				return MBAPIResult(response)
			else:
				response += line

	def getCommand(self, command, subCommand = "", params = {}, isUTF8 = None):
		if isUTF8 == None:
			isUTF8 = self.isUTF8
		return MBAPICommand(self, command, subCommand, params, isUTF8)

	def insert(self, source, params = {}):
		return int(self.getCommand("SetData", "insert:" + source, params).call()["lastInsertID"])

	def select(self, source, params = {}):
		return self.getCommand("GetData", source, params).call()

	def close(self):
		del self.input
		del self.output


class MBAPISocket(MBAPIConnection):
	def __init__(self, auth, host = "localhost", port = 850):
		self.auth = auth
		self.host = host
		self.port = port

		self.initConnection()
		
	def open(self):
		import socket
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.connect((self.host, self.port))
		
		self.input = self.sock.makefile("wb")
		self.output = self.sock.makefile("rb")
		
		self.input.write("auth=%s\r\n" % (self.auth))
		self.input.flush()
				
	def close(self):
		del self.input
		del self.output
		del self.sock
		

class MBAPIPipe(MBAPIConnection):
	#def __init__(self, auth, command):
	def __init__(self, command = "./mbapi.php"):
		#self.auth = auth
		self.command = command

		self.initConnection()
		
	def open(self):
		from subprocess import Popen, PIPE
		from sys import stderr
		from os.path import dirname
		
		self.process = Popen(
			self.command,
			
			cwd=dirname(self.command),
			stdin=PIPE,
			stdout=PIPE,
			stderr=stderr,
			close_fds=True
		)
	
		self.input = self.process.stdin
		self.output = self.process.stdout
		
		#self.input.write("auth=%s\r\n" % (self.auth))
		self.input.flush()

def test():
	from sys import argv, stdout
	#proc, auth = argv[1], argv[2]
	commands = argv[1:]
	m = MBAPIPipe()
	m.open()

	for command in commands:
		#result = m.call(command)
		#stdout.write(result.getElementValueFrom(result.dom, "adminID"))
		stdout.write(m.call(command).xml)
		stdout.write(MBAPIPipe.delim + "\n")

	m.close()

if __name__ == '__main__':
        test()
