<?php
// $Id: ShowPanelClientLogin.php 12754 2008-03-11 15:07:13Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action automate, as much as we can, logging the client into
 * the selected panel's administration.
 *
 * @author  Brian Smith
 */
class ShowPanelClientLogin extends MB_Action {
	/**
	 * The constructor for a ShowPanelClientLogin action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowPanelClientLogin() {
		parent::MB_Action();
		$this->stringRep = "ShowPanelClientLogin";
		$this->params = array();
	}

	/**
	 * The main method of this class that executes the ShowPanelClientLogin action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;

		$configGroupID = (int)$_REQUEST["configGroupID"];
		$packageID = (int)$_REQUEST["packageID"];
		
		$package = oneByID(":packages", $packageID, array(
			"clientID" => $CONF["user"]["id"],
		));
		
		if (!notZero( $configGroupID )) {
			$this->addError(TRANS_ACTIONERROR, "Config Group ID is not given.");
		} elseif (!notZero( $package["packageID"] )) {
			$this->addError(TRANS_ACTIONERROR, "Invalid Package ID provided.");
		} else {
			$getConfig =& new MBQuery("GetConfigGroups", "", array("configGroupID" => $configGroupID));
			$cg = $getConfig->getResult();
			
			if (!$cg->getErrorCount()) {
				$cg = grabSimpleMBAPIResults( $cg->getStructure() );
				$configGroup = $cg[0];
				
				// get the panel
				if (notZero( $configGroup["parentConfigGroupID"] )) {
					$getConfig =& new MBQuery("GetConfigGroups", "", array("configGroupID" => (int)$configGroup["parentConfigGroupID"]));
					$configGroup["parent"] = $getConfig->getResult();

					$configGroup["parent"] = grabSimpleMBAPIResults( $configGroup["parent"]->getStructure() );
					$configGroup["parent"] = $configGroup["parent"][0];
				}
			}
			unset($getConfig, $cg);


			$params = array();

			foreach( (array)arrayFor("packageAttributes", array("packageID" => $packageID)) as $attrib ) {

				if ($attrib["configGroupID"] == $configGroupID || $attrib["serverGroupID"] == 0) {

					switch( $attrib["packageAttributeName"] ) {
						case "domainUserName":
							//Any panel that maintains a client-level account with
							//many domains under it needs the clientLogin, so don't kill it
							if(!$configGroup["panelUsername"]) {
								$configGroup["panelUsername"] = $attrib["packageAttributeValue"];
								break;
							}
						case "clientLogin":
							$configGroup["panelUsername"] = $attrib["packageAttributeValue"];
							break;
		
						case "clientPassword":
						case "domainPassword":
							$configGroup["panelPassword"] = $attrib["packageAttributeValue"];
							break;
					}
					$params[$attrib["packageAttributeName"]] = $attrib["packageAttributeValue"];
				}
			}

			$panelName = (string)$configGroup["parent"]["configGroupName"];
			
			// see if we can grab the panel
			if (file_exists(mb_path("/lib-mbapi/include/modules/panel/$panelName.php"))) {
				require_once mb_path("/lib-mbapi/include/modules/panel/$panelName.php");
				
				if (class_exists( $panelName )) {

					$params += array("subCommand" => "panelDummy", "configGroupID" => $configGroupID, "country" => $CONF["user"]["locale"]->localeArray["locale_country_iso_2"]);
					$panelObj =& new $panelName( $params );
					
					$xmlResult =& new MBXMLLite($panelObj->panelClientLogin( $configGroup["panelUsername"], $configGroup["panelPassword"] ));
					$result = correctArray($xmlResult->toArray());

					if (isset($result["module"][0]["panel"][0][ $panelName ][0]["header"][0]["errorCount"]) && notZero( $result["module"][0]["panel"][0][ $panelName ][0]["header"][0]["errorCount"] )) {
						if (is_array($result["module"][0]["panel"][0][ $panelName ][0]["header"][0]["errors"]))
						foreach( $result["module"][0]["panel"][0][ $panelName ][0]["header"][0]["errors"] as $err ) {
							if (notNull( $err["message"] )) {
								$this->addError($err["title"], $err["message"]);
							} else {	// this came from the panel, but because of inconsistent error reporting I did this
								$this->addError(TRANS_PANELERROR, $err["title"]);
							}
						}
					} else {	// SUCCESS!
						$this->hasExecutedSuccessfully = 1;
					}
				} else {
					$this->addError(TRANS_ACTIONERROR, "Failed to find panel class.");
				}
			} else {
				$this->addError(TRANS_ACTIONERROR, "Failed to find panel file.");
			}
			
		}
		
		
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowPanelClientLogin
