<?php
// $Id: ShowPanelOptions.php 11815 2007-08-27 22:22:11Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action will list all panels connected to the package,
 * those applicable for remote client login automation we will
 * supply the method to do automatically log that user into their panel.
 *
 * @author  Brian Smith
 */
class ShowPanelOptions extends MB_Action {
	/**
	 * The constructor for a ShowPanelOptions action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowPanelOptions() {
		parent::MB_Action();
		$this->stringRep = "ShowPanelOptions";
		$this->params = array();
	}

	/**
	 * The main method of this class that executes the ShowPanelOptions action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;

		$packageID = (int)$_REQUEST["packageID"];
		$package = oneByID(":packages", $packageID, array(
			"clientID" => $CONF["user"]["id"],
		));
		
		if (!notZero( $package["packageID"] )) {
			$this->addError(TRANS_ACTIONERROR, "Invalid Package ID provided.");
		} else {

			$packageAttrib = arrayFor("packageAttributes", array("packageID" => $packageID));
			
			$thePkg =& new MBQuery("GetPackages", "", array(
								"clientID" => $CONF["user"]["id"],
								"packageID" => $packageID,
								"getDomainData" => 1));
			$pkg = $thePkg->getResult();
			unset( $thePkg );
		
			if (!$pkg->getErrorCount()) {
				$this->smartyVars["package"] = grabSimpleMBAPIResults( $pkg->getStructure() );
				$this->smartyVars["package"] = $this->smartyVars["package"][0];

				// check if user is primary contact
				$user = arrayFor("clientContacts", array("clientContactID" => (int)$CONF["user"]["clientContactID"]));
				$contactPrimary = (bool)$user[0]["contactPrimary"];
				unset( $user );
				
				// try to find our configGroupID && grab our panel auth info
				$configGroup =array();

				loadconfigs("server");
				
				if (is_array($packageAttrib))
				foreach( $packageAttrib as $attrib ) {
					$tmpConfigGroupID = (int)$attrib["configGroupID"];
					
					if (notZero( $attrib["configGroupID"] )) {
						if (!is_array($configGroup[ $tmpConfigGroupID ] )) {
							$configGroup[ $tmpConfigGroupID ] = array();

							$getConfig =& new MBQuery("GetConfigGroups", "", array("configGroupID" => $tmpConfigGroupID));
							$cg = $getConfig->getResult();
								
							if (!$cg->getErrorCount()) {
								$cg = grabSimpleMBAPIResults( $cg->getStructure() );
								$configGroup[ $tmpConfigGroupID ] = $cg[0];
								
								// get the panel, only if user is primary
								if (notZero( $configGroup[ $tmpConfigGroupID ]["parentConfigGroupID"] ) && $contactPrimary) {
									$getConfig =& new MBQuery("GetConfigGroups", "", array("configGroupID" => (int)$configGroup[ $tmpConfigGroupID ]["parentConfigGroupID"]));
									$configGroup[ $tmpConfigGroupID ]["parent"] = $getConfig->getResult();
									
									$configGroup[ $tmpConfigGroupID ]["parent"] = grabSimpleMBAPIResults( $configGroup[ $tmpConfigGroupID ]["parent"]->getStructure() );
									$configGroup[ $tmpConfigGroupID ]["parent"] = $configGroup[ $tmpConfigGroupID ]["parent"][0];

									$panelName = (string)$configGroup[ $tmpConfigGroupID ]["parent"]["configGroupName"];
			
									// see if we can grab the panel
									if (file_exists(mb_path("/lib-mbapi/include/modules/panel/$panelName.php"))) {
										require_once mb_path("/lib-mbapi/include/modules/panel/$panelName.php");
										
										if (class_exists( $panelName )) {
											$params = array("subCommand" => "panelDummy", "configGroupID" => $tmpConfigGroupID);
											$panelObj =& new $panelName( $params );

											if (is_array( $CONF["server"]["serverGroups"]["panel"][ $panelName ] ))
											foreach( $CONF["server"]["serverGroups"]["panel"][ $panelName ] as $k => $panel ) {
												if ($panel["configGroupID"] == $tmpConfigGroupID) {
													$configGroup[ $tmpConfigGroupID ]["panelConfig"] =& $CONF["server"]["serverGroups"]["panel"][ $panelName ][ $k ];
												}
												unset( $panel, $k );
											}
											$configGroup[ $tmpConfigGroupID ]["panelCapabilities"] = $panelObj->capabilities;
											unset($panelObj);
										}
									}
								}
							}
							unset($getConfig, $cg);
						}
					}
					
					switch( $attrib["packageAttributeName"] ) {
					case ("packageUsername"):
						$configGroup[ $tmpConfigGroupID ]["panelUsername"] = $attrib["packageAttributeValue"];
						break;
						
					case ("packagePassword"):
						$configGroup[ $tmpConfigGroupID ]["panelPassword"] = $attrib["packageAttributeValue"];
						break;
					default:
						if (!is_array( $configGroup[ $tmpConfigGroupID ]["serverGroup"] ) && notZero($attrib["serverGroupID"])) {
							$configGroup[ $tmpConfigGroupID ]["serverGroup"] = arrayFor("serverGroups", array("serverGroupID" => $attrib["serverGroupID"]));
						}
						break;
					}
				}
				
				// remove unset config group's
				unset( $configGroup[0] );
				$this->smartyVars["numConfigGroups"] = count($configGroup);
				$this->smartyVars["configGroups"] =& $configGroup;
				$this->smartyVars["packageTypes"] = getTypes("products");
				$this->smartyVars["packageStatuses"] = getStatuses("package");
				
				dbg($this->smartyVars);
				
				$this->addTemplate("packages/ShowPanelOptions");
				$this->hasExecutedSuccessfully = 1;
			} else {
				$this->addError(TRANS_ACTIONERROR, $pkg->getLastError());
			}
			
			
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowPanelOptions
?>
