<?php
// $Id: EditClientContactResponse.php 12893 2008-04-24 16:59:21Z btyra $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This class is a child class of MB_Action. It handles the data from EditClientContact action
 * and modifies an existing client contact record if data is valid; else redirects back to EditClientContact.
 *
 * @author  Michael Fountain
 */
class EditClientContactResponse extends MB_Action {
	/**
	 *	Keep an 'cache' of the contact info
	 */
	var $_clientContact = array();
	
	
	function EditClientContactResponse() {
		parent::MB_Action();

		$this->stringRep = "EditClientContactResponse";
		loadConfigs("userSettings");
		
		// set all possible xml params here, $_REQUEST is already cleaned
		$this->params = array
		(
			"clientContactID",
			"contactFirstName",
			"contactLastName",
			"contactEmail",
			"contactUsername",
			"contactEmailFormat",
			"contactCompany",
			"contactAddress1",
			"contactAddress2",
			"contactCity",
			"contactState",
			"contactZip",
			"countriesID",
			"languageID",
			"contactPhone1",
			"contactPhone2",
			"contactFax",
			"contactPrimary",
			"emailActionIDs",
			"selectableEmailActions",
			"contactActive"
		);
		
		/**
		 *	unset illegal key values that we don't want on the back end
		 */
		$tmp = array_flip($this->params);
		$this->unsetIllegalKeys( $tmp );
		$this->params = array_flip($tmp);
	}

	function execute() {
		global $CONF;
		parent::execute();

		$currentUser = oneByID("contacts", $CONF["user"]["clientContactID"]);
		$clientContactID = (int)$_REQUEST["clientContactID"];
		
		$requestedClientContact = oneByID('contacts',$_REQUEST['clientContactID']);
		
		if (!$currentUser["contactPrimary"] && $requestedClientContact['clientContactID'] != $currentUser['clientContactID']) {
			$this->addError(TRANS_ACTIONERROR, "Only the primary contact can add/edit others' contacts.");
		} elseif (!notZero($clientContactID)) {
			$this->addError(TRANS_ACTIONERROR, "Client Contact ID is not given.");
		} elseif ($CONF['user']['id'] != $requestedClientContact['clientID']) {
			$this->addError(TRANS_ACTIONERROR, "Requested client Contact ID does not belong to logged in client.");
	    } else {
			// if are the primary ensure is primary
			if ($currentUser["contactPrimary"] == 1 && ($clientContactID == $CONF["user"]["clientContactID"])) {
				$_REQUEST["contactPrimary"] = 1;
			}
			
			$this->setReferralActionID(getActionID("EditClientContact"));
			$this->setReferralActionFile("profile.php");

			$postedValues = array(
					"clientContactID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactUsername" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactPassword" => array(INPUT_RESTRICTION_PASSWORD, array("confirm" => "contactPasswordConfirm")),
					"contactEmail" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
					"contactCompany" => INPUT_RESTRICTION_STRING,
					"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactAddress2" => INPUT_RESTRICTION_STRING,
					"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactState" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"contactPhone1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"contactPhone2" => INPUT_RESTRICTION_STRING,
					"contactFax" => INPUT_RESTRICTION_STRING,
					"contactPrimary" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
					);
			/**
			 *	unset illegal key values that we don't want on the back end
			 */
			$this->unsetIllegalKeys( $postedValues );

			$success = $this->validateMultiInputs(
					$postedValues,
					array("contactPassword","contactPasswordConfirm"),
					array(),
					1 // for edit mode
					);

			if($success) {
				unset($_REQUEST["contactPasswordConfirm"]);
				
				// make sure no other exists with the same username
				
				if ($_REQUEST["contactUsername"]) {
					if ($clientContactID != $CONF["user"]["clientContactID"]) {
						if($contactByName = oneByName(":contacts", $_REQUEST["contactUsername"])) {
							if($contactByName["clientContactID"] != $clientContactID) {
								$this->addError(TRANS_ACTIONERROR, "Another user by the same username already exists.");
							}
						}
					}
				}

				if (!$this->getNumErrors()) {
					// only update the password if set
					unset($_REQUEST["contactPasswordConfirm"]);
					if ($_REQUEST["contactPassword"]=="") {
						unset($_REQUEST["contactPassword"]);
					} else {
						$this->params[] = "contactPassword";
					}
	
					$xmlParams = "";
					foreach ($this->params as $paramName) {
						if ($paramName == "emailActionIDs"){
							$xmlParams .= "<emailActionIDs>\n";
							if(is_array($_REQUEST[$paramName])) {
								foreach($_REQUEST[$paramName] as $key=>$eaid){
									$xmlParams .= "<emailActionID>".(int)$eaid."</emailActionID>\n";
								}
							}
							$xmlParams .= "</emailActionIDs>\n";
						} else if ($paramName == "selectableEmailActions"){
							$xmlParams .= "<removeEmailActionIDs>\n";
							if(is_array($_REQUEST[$paramName])) {
								foreach($_REQUEST[$paramName] as $key=>$eaid){
									$xmlParams .= "<emailActionID>".(int)$eaid."</emailActionID>\n";
								}
							}
							$xmlParams .= "</removeEmailActionIDs>\n";
						}else{
							$xmlParams .= "<".$paramName.">".$_REQUEST[$paramName]."</".$paramName.">\n";
						}
					}
					// fetch client ID for this contact first
					$query = "
					<mbapi>
					<command>GetClients</command>
						<params>
							<clientContactID>$clientContactID</clientContactID>
						</params>
					</mbapi>
					";
					$data = dispatchMBAPI($query, 1);
	
					$clientID = $data["mbapi"][0]["results"][0]["clients"][0]["client"][0]["clientID"][0];
	
					// create sanatized xml command
					$query = "
					<mbapi>
					<command>SetClients</command>
					<subCommand>update</subCommand>
						<params>
							<clientID>$clientID</clientID>
							<clientContacts>
								<clientContact>
									<clientContactID>$clientContactID</clientContactID>
									$xmlParams
								</clientContact>
							</clientContacts>
						</params>
					</mbapi>
					";

					$data = dispatchMBAPI($query, 1);
	
					$results =& $data["mbapi"][0]["results"][0];
					if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
						if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
							redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
						} else {
							redirect("profile.php", getActionID("ShowClientContacts"), array("clientID"=>$clientID));
						}
					}
				}
			}
		}
		
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	} // execute

	function unsetIllegalKeys( &$array ) {
		global $CONF;
		
		if (!notNull( $this->_clientContact )) {
			$this->_clientContact = oneOf('clientContacts', array('clientContactID' => (int)$_REQUEST['clientContactID']));
			$this->_clientContact['clientContactCount'] = count(arrayFor('clientContacts', array('clientID' => (int)$_REQUEST['clientID'])));
		}
		
		if ($this->_clientContact['clientContactCount'] == 1) {
			unset($array['contactActive']);
		}
	#	echo '1';
	#	show($array);

		foreach( (array)$CONF["userSettings"]["display"] as $k => $v ) {
			if ($v==1)
				continue;
			
			switch( $k ) {
			case "editUserName":
				unset($array["contactUsername"]);
				break;
			case "editFirstName":
				unset($array["contactFirstName"]);
				break;
			case "editLastName":
				unset($array["contactLastName"]);
				break;
			case "editEmail":
				unset($array["contactEmail"]);
				break;
			case "editCompanyName":
				unset($array["contactCompany"]);
				break;
			case "editCountry":
				unset($array["countriesID"]);
				break;
			case "editEmailActions":
				unset($array["emailActionIDs"]);
				unset($array["selectableEmailActions"]);
				break;
			}
		}	unset( $k, $v );
	#	echo 2;
	#	show($array);
	}

} // MB_Action
?>
