<?php
// $Id: ShowTodos.php 9336 2006-10-06 20:46:55Z bsmith $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This Action object collects information on all todos that match the given parameters, and 
 * generates the display of that information.
 *
 * @author  Mark Garrett
 * @author  Brian Smith
 */
class ShowTodos extends MB_Action {
	/**
	 * The constructor for a ShowTodos action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowTodos() {
		parent::MB_Action();
		$this->stringRep = "ShowTodos";
		
		$this->params = array(
			"todoID",
			"clientID",
			"productID",
			"todoTitle",
			"todoDesc",
			"userID",
			"todoStatusID",
			"todoDateDue",
			"todoDateCreated",
			"todoAssignedAdminID",
			"todoIsPublic");
		$this->sortColumns = array(
			"todoStatusID",
			"todoTitle",
			"userID",
			"todoDateDue",
			"todoAssignedAdminID",
			"todoIsPublic");
	}

	/**
	 * The main method of this class that executes the ShowTodos action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		// past due todo items
		$query = $this->getMBAPIQuery("GetTodos", "",
				array("todoDateDueGreatest" => time(),
					"getAdminData" => 1,
					"getAssignedData" => 1));
		$data = dispatchMBAPI($query, 1);
		$this->handleGroup( "pastdue", $data );
		
		$this->smartyVars["rowCount"]["pastdue"] = $this->getMaxCount( "pastdue" );
		
		
		// get todo's that are a day old
		$query = $this->getMBAPIQuery("GetTodos", "",
				array("todoDateDueLeast" => time(),
				"todoDateDueGreatest" => time() + 86400,
				"getAdminData" => 1,
				"getAssignedData" => 1));
		$data = dispatchMBAPI($query, 1);
		$this->handleGroup( "near", $data );
		
		$this->smartyVars["rowCount"]["near"] = $this->getMaxCount( "near" );
		
		// get todo's for the next 6 days (1 week forward)
		$query = $this->getMBAPIQuery("GetTodos", "",
				array( "todoDateDueLeast" => time() + 86400,
				"todoDateDueGreatest" => time() + 604800,
				"getAdminData" => 1,
				"getAssignedData" => 1));
		$data = dispatchMBAPI($query, 1);
		$this->handleGroup("week", $data);

		$this->smartyVars["rowCount"]["week"] = $this->getMaxCount( "week" );
		
		// defs
		$this->smartyVars["todostatus"] = getStatuses("todo");
		$this->smartyVars["todostatusheaders"] = array(
						"pastdue" => TRANS_DUE,
						"near" => TRANS_24HOURS,
						"week" => TRANS_1WEEK);

		$data = arrayFor("admins", array("adminActive"=>1));
		
		foreach( (array)$data as $admin ) {
			$this->smartyVars["adminsList"][ $admin["userID"] ] = $admin["adminLastName"].", ".$admin["adminFirstName"];
		}
		
		if (!$this->_err) {
			$this->smartyVars = correctArray( $this->smartyVars );

			$this->addTemplate("todos/ShowTodos");
		}
		
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	function handleGroup($group, &$data ) {
		global $CONF;
		
		if ($this->addMBAPIErrors($data) === 0) {
			
			if($data["mbapi"][0]["header"][0]["numResults"][0] == 1) {
				$results["todos"] = array(0 => $results["todos"][0]["todo"]);
			}
			
			$results = correctArray($data["mbapi"][0]["results"][0]);
			
			if (isset($results["todos"][0]) && is_array($results["todos"][0])) {
				$this->smartyVars["todos"][$group] = array();
				
				if (is_array( $results["todos"] ))
				foreach($results["todos"] as $item) {
					$key=null;
					
					if ($item["todoAssignedAdminID"] == $CONF["user"]["id"]) {
						// my todo tasks
						$key = "mine";
					} elseif (($item["userID"] == $CONF["user"]["id"]) && ((int)$item["todoAssignedAdminID"] > 0)) {
						// tasks that I am the author of
						$key = "authored";
					} elseif (($item["todoIsPublic"] == 1) || ( ($item["userID"] == $CONF["user"]["id"]) && !$item["todoAssignedAdminID"]) ) {
						// public todo tasks
						$key = "public";
					}
					
					if (!is_null( $key )) {
						$item["assigned"] = $item["assigned"][0]["assigned"];
						$this->smartyVars["todos"][ $group ][ $key ][] = $item;
					}
					
				}
			}
		} else {
			$this->_err=1;
		}
		unset( $query, $data, $key, $item, $group );
		$this->smartyVars["paginationLinks"] = $results["paginationLinks"][0];
		
	}

	function getMaxCount( $group ) {
		$mine = count( $this->smartyVars["todos"][ $group ]["mine"] );
		$auth = count( $this->smartyVars["todos"][ $group ]["authored"] );
		$pub = count( $this->smartyVars["todos"][ $group ]["public"] );

		if ($mine > $auth && $mine > $pub)
			return $mine;

		if ($auth > $mine && $auth > $pub)
			return $auth;

		if (!notZero( $pub ))
			return 1;
		return $pub;
	}
} // ShowTodos
?>