<?php
/**
 * Smarty plugin
 * @package Smarty
 * @subpackage plugins
 */

/**
 * Smarty {html_select_date} plugin
 *
 * Type:     function<br>
 * Name:     html_select_locale_date<br>
 * Purpose:  Prints the dropdowns for date selection based on users Locale.
 *
 * @author   Michael Fountain
 * @param array
 * @param Smarty
 * @return string
 */
function smarty_function_html_select_locale_date($params, &$smarty)
{
 	global $CONF;

    require_once $smarty->_get_plugin_filepath('shared','make_timestamp');
    require_once $smarty->_get_plugin_filepath('function','html_options');
    /* Default values. */
	$setID			 = false;
    $prefix          = "Date_";
    $start_year      = strftime("%Y")-1;
    $end_year        = 2029;
    $display_days    = true;
    $display_months  = true;
    $display_years   = true;
    $display_time    = false;
    $minute_format   = "%02d";
    $minute_value_format   = "%02d";
    $hour_format     = "%02d";
    $hour_value_format     = "%02d";
    $month_format    = "%B";
    /* Write months as numbers by default  GL */
    $month_value_format = "%m";
    $day_format      = "%02d";
    /* Write day values using this format MB */
    $day_value_format = "%d";
    $year_as_text    = false;
    /* Display years in reverse order? Ie. 2000,1999,.... */
    $reverse_years   = false;
    /* Should the select boxes be part of an array when returned from PHP?
       e.g. setting it to "birthday", would create "birthday[Day]",
       "birthday[Month]" & "birthday[Year]". Can be combined with prefix */
    $field_array     = null;
    /* <select size>'s of the different <select> tags.
       If not set, uses default dropdown. */
    $day_size        = null;
    $month_size      = null;
    $year_size       = null;
    /* Unparsed attributes common to *ALL* the <select>/<input> tags.
       An example might be in the template: all_extra ='class ="foo"'. */
    $all_extra       = null;
    /* Separate attributes for the tags. */
    $day_extra       = null;
    $month_extra     = null;
    $year_extra      = null;
    /* Order in which to display the fields.
       "D" -> day, "M" -> month, "Y" -> year. */
    $fieldOrder     = 'DMY';
    /* String printed between the different fields. */
    $field_separator = "\n";
    $time = time();
    $all_empty       = null;
    $day_empty       = null;
    $month_empty     = null;
    $year_empty      = null;
    $hour_empty      = null;
    $minute_empty    = null;

    foreach ($params as $_key=>$_value) {
        switch ($_key) {
         	case 'fieldOrder':
            case 'setID':
            case 'prefix':
            case 'time':
            case 'start_year':
            case 'end_year':
            case 'month_format':
            case 'day_format':
            case 'day_value_format':
            case 'field_array':
            case 'day_size':
            case 'month_size':
            case 'year_size':
            case 'all_extra':
            case 'day_extra':
            case 'month_extra':
            case 'year_extra':
            case 'field_separator':
            case 'month_value_format':
            case 'month_empty':
            case 'day_empty':
            case 'year_empty':
            case 'hour':
            case 'minute':
                $$_key = (string)$_value;
                break;

            case 'all_empty':
                $$_key = (string)$_value;
                $day_empty = $month_empty = $year_empty = $all_empty;
                break;

            case 'display_days':
            case 'display_months':
            case 'display_years':
            case 'display_time':
            case 'year_as_text':
            case 'reverse_years':
                $$_key = (bool)$_value;
                break;

            default:
                $smarty->trigger_error("[html_select_date] unknown parameter $_key", E_USER_WARNING);

        }
    }

    if(preg_match('!^-\d+$!',$time)) {
        // negative timestamp, use date()
        $time = date('Y-m-d',$time);
    }
    // If $time is not in format yyyy-mm-dd
    if (!preg_match('/^\d{0,4}-\d{0,2}-\d{0,2}$/', $time)) {
        // use smarty_make_timestamp to get an unix timestamp and
        // strftime to make yyyy-mm-dd
        $time = strftime('%Y-%m-%d', smarty_make_timestamp($time));
    }
    // Now split this in pieces, which later can be used to set the select
    $time = explode("-", $time);

	$time[3] = (!isset($hour))?date("G"):$hour;
	$time[4] = (!isset($minute))?date("i"):$minute;

    // make syntax "+N" or "-N" work with start_year and end_year
    if (preg_match('!^(\+|\-)\s*(\d+)$!', $end_year, $match)) {
        if ($match[1] == '+') {
            $end_year = strftime('%Y') + $match[2];
        } else {
            $end_year = strftime('%Y') - $match[2];
        }
    }
    if (preg_match('!^(\+|\-)\s*(\d+)$!', $start_year, $match)) {
        if ($match[1] == '+') {
            $start_year = strftime('%Y') + $match[2];
        } else {
            $start_year = strftime('%Y') - $match[2];
        }
    }
    if (strlen($time[0]) > 0) {
        if ($start_year > $time[0] && !isset($params['start_year'])) {
            // force start year to include given date if not explicitly set
            $start_year = $time[0];
        }
        if($end_year < $time[0] && !isset($params['end_year'])) {
            // force end year to include given date if not explicitly set
            $end_year = $time[0];
        }
    }

    $fieldOrder = strtoupper($fieldOrder);

    $html_result = $month_result = $day_result = $year_result = $hour_result = $minute_result = "";

    if ($display_months) {
        $month_names = array();
        $month_values = array();
        if(isset($month_empty)) {
            $month_names[''] = $month_empty;
            $month_values[''] = '';
        }
        for ($i = 1; $i <= 12; $i++) {
            $month_names[$i] = $CONF["user"]["locale"]->localeArray['locale_month_wide_'.$i];
            $month_values[$i] = strftime($month_value_format, mktime(0, 0, 0, $i, 1, 2000));
        }

        $month_result .= '<select style="width:100px" name=';
        if (null !== $field_array){
            $month_result .= '"' . $field_array . '[' . $prefix . 'Month]"';
        } else {
            $month_result .= '"' . $prefix . 'Month"';
        }
        if (null !== $month_size){
            $month_result .= ' size="' . $month_size . '"';
        }
        if (null !== $month_extra){
            $month_result .= ' ' . $month_extra;
        }
        if (null !== $all_extra){
            $month_result .= ' ' . $all_extra;
        }
        if ($setID){
            $month_result .= ' id="'.$prefix.'Month"';
        }
        $month_result .= '>'."\n";

        $month_result .= smarty_function_html_options(array('output'     => $month_names,
                                                            'values'     => $month_values,
                                                            'selected'   => $a=$time[1] ? strftime($month_value_format, mktime(0, 0, 0, (int)$time[1], 1, 2000)) : '',
                                                            'print_result' => false),
                                                      $smarty);
        $month_result .= '</select>';
    }

    if ($display_days) {
        $days = array();
        if (isset($day_empty)) {
            $days[''] = $day_empty;
            $day_values[''] = '';
        }
        for ($i = 1; $i <= 31; $i++) {
            $days[] = sprintf($day_format, $i);
            $day_values[] = sprintf($day_value_format, $i);
        }

        $day_result .= '<select style="width:50px" name=';
        if (null !== $field_array){
            $day_result .= '"' . $field_array . '[' . $prefix . 'Day]"';
        } else {
            $day_result .= '"' . $prefix . 'Day"';
        }
        if (null !== $day_size){
            $day_result .= ' size="' . $day_size . '"';
        }
        if (null !== $all_extra){
            $day_result .= ' ' . $all_extra;
        }
        if (null !== $day_extra){
            $day_result .= ' ' . $day_extra;
        }
        if ($setID){
            $day_result .= ' id="'.$prefix.'Day"';
        }
        $day_result .= '>'."\n";
        $day_result .= smarty_function_html_options(array('output'     => $days,
                                                          'values'     => $day_values,
                                                          'selected'   => $time[2],
                                                          'print_result' => false),
                                                    $smarty);
        $day_result .= '</select>';
    }

    if ($display_years) {
        if (null !== $field_array){
            $year_name = $field_array . '[' . $prefix . 'Year]';
        } else {
            $year_name = $prefix . 'Year';
        }
        if ($year_as_text) {
            $year_result .= '<input type="text" name="' . $year_name . '" value="' . $time[0] . '" size="4" maxlength="4"';
            if (null !== $all_extra){
                $year_result .= ' ' . $all_extra;
            }
            if (null !== $year_extra){
                $year_result .= ' ' . $year_extra;
            }
	        if ($setID){
	            $year_result .= ' id="'.$prefix.'Year"';
	        }
            $year_result .= '>';
        } else {
            $years = range((int)$start_year, (int)$end_year);
            if ($reverse_years) {
                rsort($years, SORT_NUMERIC);
            } else {
                sort($years, SORT_NUMERIC);
            }
            $yearvals = $years;
            if(isset($year_empty)) {
                array_unshift($years, $year_empty);
                array_unshift($yearvals, '');
            }
            $year_result .= '<select style="width:60px" name="' . $year_name . '"';
            if (null !== $year_size){
                $year_result .= ' size="' . $year_size . '"';
            }
            if (null !== $all_extra){
                $year_result .= ' ' . $all_extra;
            }
            if (null !== $year_extra){
                $year_result .= ' ' . $year_extra;
            }
	        if ($setID){
	            $year_result .= ' id="'.$year_name.'"';
	        }
            $year_result .= '>'."\n";
            $year_result .= smarty_function_html_options(array('output' => $years,
                                                               'values' => $yearvals,
                                                               'selected'   => $time[0],
                                                               'print_result' => false),
                                                         $smarty);
            $year_result .= '</select>';
        }
    }

    if ($display_time) {
        $hours = array();
        if (isset($hour_empty)) {
            $hours[''] = $hour_empty;
            $hour_values[''] = '';
        }
        for ($i = 0; $i <= 23; $i++) {
            $hours[] = sprintf($hour_format, $i);
            $hour_values[] = sprintf($hour_value_format, $i);
        }

        $hour_result .= '<select style="width:50px" name=';
        if (null !== $field_array){
            $hour_result .= '"' . $field_array . '[' . $prefix . 'Hour]"';
        } else {
            $hour_result .= '"' . $prefix . 'Hour"';
        }
        if (null !== $hour_size){
            $hour_result .= ' size="' . $hour_size . '"';
        }
        if (null !== $all_extra){
            $hour_result .= ' ' . $all_extra;
        }
        if (null !== $hour_extra){
            $hour_result .= ' ' . $hour_extra;
        }
        if ($setID){
            $hour_result .= ' id="'.$prefix.'Hour"';
        }
        $hour_result .= '>'."\n";
        $hour_result .= smarty_function_html_options(array('output'     => $hours,
                                                          'values'     => $hour_values,
                                                          'selected'   => $time[3],
                                                          'print_result' => false),
                                                    $smarty);
        $hour_result .= '</select>';

        $minutes = array();
        if (isset($minute_empty)) {
            $minutes[''] = $minute_empty;
            $minute_values[''] = '';
        }
        for ($i = 0; $i <= 59; $i++) {
            $minutes[] = sprintf($minute_format, $i);
            $minute_values[] = sprintf($minute_value_format, $i);
        }

        $minute_result .= '<select style="width:50px" name=';
        if (null !== $field_array){
            $minute_result .= '"' . $field_array . '[' . $prefix . 'Minute]"';
        } else {
            $minute_result .= '"' . $prefix . 'Minute"';
        }
        if (null !== $minute_size){
            $minute_result .= ' size="' . $minute_size . '"';
        }
        if (null !== $all_extra){
            $minute_result .= ' ' . $all_extra;
        }
        if (null !== $minute_extra){
            $minute_result .= ' ' . $minute_extra;
        }
        if ($setID){
            $minute_result .= ' id="'.$prefix.'Minute"';
        }
        $minute_result .= '>'."\n";
        $minute_result .= smarty_function_html_options(array('output'     => $minutes,
                                                          'values'     => $minute_values,
                                                          'selected'   => $time[4],
                                                          'print_result' => false),
                                                    $smarty);
        $minute_result .= '</select>';
    }

    // Loop thru the fieldOrder field
    for ($i = 0; $i <= 2; $i++){
        $c = substr($fieldOrder, $i, 1);
        switch (TRUE){
            case eregi('D',$c):
                $html_result .= $day_result;
                break;
            case eregi('M',$c):
                $html_result .= $month_result;
                break;
            case eregi('Y',$c):
                $html_result .= $year_result;
                break;
        }
        // Add the field seperator
        if($i != 2) {
            $html_result .= $field_separator;
        }
    }

    if ($display_time) {
		$html_result .= $field_separator.$field_separator.$hour_result.":".$minute_result;
    }

    return $html_result;
}

/* vim: set expandtab: */

?>
