<?php
/**
 * Smarty plugin
 * @package Smarty
 * @subpackage plugins
 */


/**
 * Multi-byte aware Smarty truncate modifier plugin
 *
 * Type:	 modifier<br>
 * Name:	 truncate<br>
 * Purpose:	 Truncate a string to a certain length if necessary,
 *			 optionally splitting in the middle of a word, and
 *			 appending the $etc string or inserting $etc into the middle.
 * @link http://smarty.php.net/manual/en/language.modifier.truncate.php
 *			truncate (Smarty online manual)
 * @author	 Monte Ohrt <monte at ohrt dot com>
 * @author	 Mark Garrett <monte at ohrt dot com>
 * @param string
 * @param integer
 * @param string
 * @param boolean
 * @param boolean
 * @return string
 */
function smarty_modifier_mb_truncate($string, $length = 80, $etc = '...', $break_words = false, $middle = false){
	if ($length == 0){
		return '';
	}
	
	$string = unsecureInput($string);
	
	if (strlen(utf8_decode($string)) > $length) {
		$length -= strlen($etc);
		
		if(utf8_decode($string) !== $string){
			if (function_exists("mb_substr")){
				mb_internal_encoding("UTF-8");
				if (!$break_words && !$middle) {
					$string = preg_replace('/\s+?(\S+)?$/u', '', mb_substr($string, 0, $length+1));
				}
				if(!$middle) {
					$string= mb_substr($string, 0, $length).$etc;
				} else {
					$string= mb_substr($string, 0, $length/2) . $etc . mb_substr($string, -$length/2);
				}	
			}
		}else{
			if (!$break_words && !$middle) {
				$string = preg_replace('/\s+?(\S+)?$/u', '', substr($string, 0, $length+1));
			}
			if(!$middle) {
				$string = substr($string, 0, $length).$etc;
			} else {
				$string = substr($string, 0, $length/2) . $etc . substr($string, -$length/2);
			}
		}
	}
	return $string;
}

/* vim: set expandtab: */

?>
