<?php
// $Id: paypal.php 12796 2008-03-20 14:47:06Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+
require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

require_once mb_path("/lib-mbapi/include/modules/gateway/gateway.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");
require_once mb_path("/lib-modernbill/include/config/internal/emailactions.php");
require_once mb_path("/lib-mbapi/include/mail/mail.php");
require_once mb_path("/lib-modernbill/include/accounting/Payment.php");
require_once mb_path("/lib-modernbill/include/accounting/General.php");

/**
* This class is an instantion of the Gateway class, that executes a command pertaining
* to comunicating with a payment gateway, in the default case from the database. Pass in
* only the array parameters that are set.
*
*<module>
* <gateway>
*  <paypal>
*     <params>
*       <configGroupName></configGroupName>						(Required: Configuration instance of BluePay module)
*       <subCommand>charge</subCommand>       						(Required: One of charge, auth, finalize, refund)
*       <invoiceID></invoiceID>
*       <authCode></authCode>										(Required for finalize)
*       <transactionCode></transactionCode>						(Required for refund)
*       <paymentDescription></paymentDescription>					(Required)
*       <paymentAmount></paymentAmount>							(Required: 0.00)
*       <paymentMethod></paymentMethod>							(Required: CC, echeck)
*       <isRecurring></isRecurring>								(0,1)
*       <billingAccountID></billingAccountID>			(Enter either this or billing info below)
*     </params>
*   </paypal>
* </gateway>
*<module>
*
* @package modernbill
* @subpackage mbapi
*/

$className = "PayPal";
$GLOBALS["moduleInfo"][$className] = array
(
	"type" => "gateway",
	"status" => "TRANS_STABLE",
	"version" => "1.0.3",
	"gatewaytype" => "2",
	"displayName" => "PayPal",
	"author" => "ModernGigabyte, LLC",
	"capabilities" => array(GATEWAY_CHARGE)
);

class PayPal extends Gateway {

	/**
	* Returns the module display for the PayPal module. This includes version number and display name.
	* the columns in config_params. This array will be looped by SetConfigGroups.
	*
	* @return	An array of arrays containing all the display info for the module.
	*/
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["PayPal"];
	}

	/**
	* Execute the command based upon the given information, and return the result.
	*
	* @return  The result of the PayPal gateway command based upon the given information.
	*/
	function PayPal($input='') {
		$this->stringRep = "paypal";
		$this->displayInfo = $this->getModuleInfo();
		parent::Gateway($input);
		dbg("test");

		foreach ($this->displayInfo["capabilities"] as $cap){
			$this->capabilities[$cap] = 1;
		}
		$this->IPNVerifyURL = "https://www.paypal.com/cgi-bin/webscr";
		$this->subscriptionURL = "https://www.paypal.com/cgi-bin/webscr";
		$this->paymentURL = "https://www.paypal.com/cgi-bin/webscr";

		$this->maxTerms = array("d"=>90, // days, max 90
								"w"=>52, // weeks, max 52
								"m"=>24, // months, max 24
								"y"=>5); // years, max 5

	}

	/**
	* Returns the default configuration parameters for the PayPal module. The number of elements in each array must me identical to 
	* the columns in config_params. This array will be looped by SetConfigGroups.
	* 
	* @return	An array of arrays containing all the parameters the module uses. 
	*/
	function addDefaultConfigParams() {
		global $CONF;
		$defaultConfigParams = NULL;
		$urlArray = parse_url($CONF["secureURLProtocol"]."://".$CONF["secureURL"]);
		$defaultConfigParams = array(
			array(NULL, NULL, "PayPal_ID", $CONF["user"]["email"], "", "TRANS_PAYPAL_ID", "TRANS_PAYPAL_ID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Item_Name", "", "", "TRANS_PAYPAL_ITEMNAME", "TRANS_PAYPAL_ITEMNAME_DESC", "t", 1, 2, 0, NULL),
			array(NULL, NULL, "Return_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"], "", "TRANS_PAYPAL_RETURNURL","TRANS_PAYPAL_RETURNURL_DESC", "t", 1, 3, 0, NULL),
			array(NULL, NULL, "Cancel_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"], "", "TRANS_PAYPAL_CANCELURL","TRANS_PAYPAL_CANCELURL_DESC", "t", 1, 4, 0, NULL),
			array(NULL, NULL, "Display_Logo", $CONF["secureURLProtocol"]."://".$CONF["secureURL"]."/lib-themes/default/lib-modernbill/images/logos/logo.gif", "", "TRANS_PAYPAL_DISPLAYLOGO","TRANS_PAYPAL_DISPLAYLOGO_DESC", "t", 1, 5, 0, NULL),
			array(NULL, NULL, "Submit_Image", "https://www.paypal.com/en_US/i/btn/x-click-but03.gif", "", "TRANS_PAYPAL_SUBMITIMAGE","TRANS_PAYPAL_SUBMITIMAGE_DESC", "t", 1, 6, 0, NULL),
			array(NULL, NULL, "IPN_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"]."/lib-mbapi/include/modules/gateway/return/paypal.php", "", "TRANS_PAYPAL_IPNURL","TRANS_PAYPAL_IPNURL_DESC", "t", 1, 7, 0, NULL),
			array(NULL, NULL, "Enable_Subscriptions", "1", "", "TRANS_PAYPAL_SUBENABLE","TRANS_PAYPAL_SUBENABLE_DESC", "r", 0, 8, 0, array
				(
					array(NULL, NULL, "0","TRANS_NO",1),
					array(NULL, NULL, "1","TRANS_YES",2)
				)),
/*			array(NULL, NULL, "Subscriptions_Override", "0", "", "TRANS_PAYPAL_SUBOVERRIDE","TRANS_PAYPAL_SUBOVERRIDE_DESC", "r", 0, 9, 0, array
				(
					array(NULL, NULL, "0","TRANS_NO",1),
					array(NULL, NULL, "1","TRANS_YES",2)
				)),*/
			array(NULL, NULL, "Subscriptions_Image", "https://www.paypal.com/images/x-click-but20.gif", "", "TRANS_PAYPAL_SUBIMAGE","TRANS_PAYPAL_SUBIMAGE_DESC", "t", 1, 10, 0, NULL),
/*			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "r", 1, 11, 0, array
				(
					array(NULL, NULL, "0","TRANS_NO",1),
					array(NULL, NULL, "1","TRANS_YES",2)
				)),*/
			array(NULL, NULL, "Acceptable_PayPal_Payees", $CONF["user"]["email"], "", "TRANS_PAYPAL_ACCEPT_ADDRESSES", "TRANS_PAYPAL_ACCEPT_ADDRESSES_DESC", "a", 1, 11, 0, NULL),
			);
		return $defaultConfigParams;
	}

	/**
	*
	* From a child module this function must do the following:
	*   - Maintain $this->hasExecuted, $this->hasExecutedSuccessfully, $this->timestamp,
	*     $this->status, $this->accountID
	*   - Add any queries to the debug list with addDebug()
	*   - Add any errors to the error list with addError()
	*   - Return $this->returnData, or the appropriate error constant
	*
	* @return  Returns feedback from execution of this Gateway object, or an error constant defined
	*          at the top of this file.
	*/
	function execute() {
		global $db, $CONF;
		$postFields = null;

		if ($this->status === GATEWAY_PARAM_FAILURE) {
			return $this->status;
		}

		$this->gatewayInfo =& $this->config;

		if ((int)$this->params["singlePayment"][0]){
			$this->gatewayInfo["Enable_Subscriptions"] = 0;
		}

		// get the payment group info, we need the currency...
		$h =& new QueryMaster("payment_groups.*", "payment_groups_gateway_rel.*");
		$h->enableStrictMode();
		$h->addAndClause("config_group_id", "=", $this->gatewayInfo["configGroupID"], "i");
		$paymentGroupInfo = $h->getResults(-1, 0);
		$this->gatewayInfo["currencyID"] = $paymentGroupInfo[0]["payment_group_currency_id"];

		// lets get all invoice info

		$h =& new QueryMaster("invoices.*");
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceInfo = $h->getRow();

		// get the line items
		$h =& new QueryMaster("invoice_line_items.*", "packages.*");
		$h->enableStrictMode();
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceLineItems = $h->getResults(-1, 0);

		// get package and line item ids
		$lineItemIDs = array();
		$pids = array();
		$clientID = 0;
		$lineItemToPackage = array();
		foreach ($invoiceLineItems as $row){
			$pids[] = $row["package_id"];
			$clientID = $row["client_id"];
			$lineItemIDs[] = $row["line_item_id"];
			$lineItemToPackage[$row["line_item_id"]] = $row["package_id"];
			$lineItemToAddon[$row["line_item_id"]] = $row["package_addon_id"];
		}

		$clientInfo = oneByID(":clients", $clientID);

		// getting all shares for those ids
		$hx =& new QueryMaster("client_billing_account_shares.*");
		$hx->addAndClause("package_id", "=", $pids, "i");
		$bas = $hx->getResults(-1, 0);
		// making keyed array of billing accounts
		$billingAccounts = array();
		foreach ($bas as $row){
			$billingAccounts[$row["package_id"]] = $row["billing_account_id"];
		}

		$this->currentTaxZones = array();
		// if we dont have a tax exempt client:
		if ((int)$clientInfo["applyTax"]){

			// gotta get tax zones
			$h =& new QueryMaster("line_items_tax_zone_rel.*");
			$h->addColumns("tax_zones.*");
			$h->addJoin("tax_zones", "tax_zone_id", "line_items_tax_zone_rel");
			$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
			$h->addOrderByColumns("tax_zone_stackable", "DESC");
			$currentTaxZones = $h->getResults(-1, 0);
			dbg($currentTaxZones);
 
			foreach ($currentTaxZones as $row){
				if ((int)$lineItemToAddon[$row["line_item_id"]]){
					$this->currentAddonTaxZones[$lineItemToAddon[$row["line_item_id"]]][] = $row;
				} else {
					$this->currentTaxZones[$lineItemToPackage[$row["line_item_id"]]][] = $row;
				}
			}
			dbg($this->currentTaxZones);
		}

		// init addon ids we need to get
		$packageAddonIDs = array();

		// run through line items getting any addon ids we need
		foreach ($invoiceLineItems as $lineItem){
			if ($lineItem["package_addon_id"]){
				$packageAddonIDs[] = $lineItem["package_addon_id"];
			}
		}

		// query the db for addon info for the addon ids we need
		$h =& new QueryMaster("package_addons.*");
		$h->addAndClause("package_addon_id", "=", $packageAddonIDs, "i");
		$addons = $h->getResults(-1, 0);

		// init keyed addon array
		$keyedAddons = array();

		// populate the keyed addon array (not enough data to worry about passing by reference)
		foreach ($addons as $addon){
			if ($addon["package_addon_id"]){
				$keyedAddons[$addon["package_addon_id"]] = $addon;
			}
		}

		// go through line items and add addon data into line items data where needed
		foreach ($invoiceLineItems as $k=>$lineItem){
			$invoiceLineItems[$k]["numRenewals"] = $this->getNumRenewals($lineItem); // get the number of times this needs to renew
			$invoiceLineItems[$k]["cycleInfo"] = $this->translateCycleInfo($lineItem);
			if ($lineItem["package_addon_id"]){
				$invoiceLineItems[$k]["addonInfo"] = $keyedAddons[$lineItem["package_addon_id"]];
			}
		}
		dbg($invoiceLineItems);


		// initialize the array we will keep track of how many (and for what) subscriptions we will need
		$subscriptions = array();

		$unsupportedCycle = false;

		$needSubscription = 0;

		// loop through the line items of this invoice and group them into subarrays based on their cycle and
		// coupon num cycles
		$oneOffPayments = array();
		
		foreach ($invoiceLineItems as $lineItem){
			if ($lineItem["cycleInfo"]===false && (int)$this->gatewayInfo["Enable_Subscriptions"]){
				$unsupportedCycle = true;
			}
			$billingAccountID = $billingAccounts[$lineItem["package_id"]];
			if ($lineItem["package_addon_id"]){
				$cycle_id = $lineItem["cycle_id"];
				$coupon_num_cycles = $lineItem["addonInfo"]["coupon_num_cycles"];
				$num_renewals = $lineItem["numRenewals"];
			} else {
				$cycle_id = $lineItem["cycle_id"];
				$coupon_num_cycles = $lineItem["coupon_num_cycles"];
				$num_renewals = $lineItem["numRenewals"];
			}
			if ($num_renewals!=0){
				$needSubscription = 1;
			}
			if ($num_renewals===0){
				$oneOffPayments[] = $lineItem;
			} else {
				$subscriptions[$cycle_id."-".$num_renewals."-".$coupon_num_cycles."-".$billingAccountID][] = $lineItem;
			}
		}

		dbg($subscriptions);

		$returnHTML = array();


		if (!(int)$this->gatewayInfo["Enable_Subscriptions"] || $needSubscription==0){

			if (round($invoiceInfo["invoice_amount"]-$invoiceInfo["invoice_amount_paid"], 2)>0){
				$oneOffPayments = array(); // need to zero out the array if we did a one off payment for the whole invoice
				// make the one time payment stuff here
				$oneTime = array();
				$oneTime["cmd"] = "_xclick";
				$oneTime["lc"] = strtoupper($this->params["countriesISO2"][0]);
				$oneTime["image_url"] = urlencode($this->gatewayInfo["Display_Logo"]);
				$oneTime["&#110;otify_url"] = $this->gatewayInfo["IPN_URL"];
				$oneTime["business"] = $this->gatewayInfo["PayPal_ID"];
				$oneTime["item_name"] = $this->gatewayInfo["Item_Name"];
				$oneTime["item_number"] = "I".$this->params["invoiceID"][0]."P".$this->gatewayInfo["configGroupID"];
				$oneTime["amount"] = $invoiceInfo["invoice_amount"]-$invoiceInfo["invoice_amount_paid"];
				$oneTime["no_note"] = "1";
				$oneTime["invoice"] = $this->params["invoiceID"][0]."P".$this->gatewayInfo["configGroupID"];
				$oneTime["currency_code"] = $this->gatewayInfo["currencyID"];
		
				$returnHTML[] = $this->createOneTimeHTMLOutput($oneTime);
			
			}
	
		}
		// ok, added first one time payment, now doing subscriptions

		if ((int)$this->gatewayInfo["Enable_Subscriptions"] && $needSubscription==1){

			// for each subscription group we have, loop through the items and get the prices and the cycle unit
			// and the num promotional period cycles.
			foreach ($subscriptions as $items){

				$initialPeriodPrice = 0;
				$firstprice = 0;
				$regprice = 0;
				$packages = array();
				$addons = array();
				foreach ($oneOffPayments as $item){
					$initialPeriodPrice += $item["line_item_total_amount"];
				}
				$oneOffPayments = array(); // already took care of these so lets get rid of them
				foreach ($items as $item){
					$initialPeriodPrice += $item["line_item_total_amount"];
					if ($item["package_addon_id"]){
						$addons[] = $item["package_addon_id"];
						$couponcycles = $item["addonInfo"]["coupon_num_cycles"];
						if ($couponcycles>1)$firstprice = round($firstprice+$item["line_item_total_amount"], 2);
						$tp = $item["addonInfo"]["package_addon_price"]*$item["line_item_quantity"];
						$tp += $this->calculateTaxes($tp, $item["package_id"], $item["package_addon_id"]);
						$regprice = round($regprice+$tp, 2);
					} else {
						$packages[] = $item["package_id"];
						$couponcycles = $item["coupon_num_cycles"];
						if ($couponcycles>1)$firstprice = round($firstprice+$item["line_item_total_amount"], 2);
						$tp = $item["package_price"]*$item["line_item_quantity"];
						$tp += $this->calculateTaxes($tp, $item["package_id"]);
						$regprice = round($regprice+$tp, 2);
					}
				}

				$h =& new QueryMaster("insert");
				$h->addTables("subscriptions");
				$h->addParam("subscription_status", getAPIStatusID("subscription", "inactive"), "i");
				$h->addParam("client_id", $clientID, "i");
				$h->addParam("subscription_date_created", mktime(), "i");
				$h->addParam("subscription_date_modified", mktime(), "i");
				$h->addParam("config_group_id", $this->gatewayInfo["configGroupID"], "i");
				$h->addParam("billing_account_id", $billingAccounts[$item[0]["package_id"]], "i");
				$h->addParam("subscription_account_identifier", trim(rtrim($this->gatewayInfo["PayPal_ID"])), "s");
				$subresult = $h->execute();
				$subscriptionID = $h->getLastInsertID();

				dbg($packages);
				dbg($addons);

				foreach ($packages as $pid){
					$h =& new QueryMaster("insert");
					$h->addTables("subscription_package_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_id", $pid, "i");
					$h->execute();
				}

				foreach ($addons as $pid){
					$h =& new QueryMaster("insert");
					$h->addTables("subscription_package_addon_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_addon_id", $pid, "i");
					$h->execute();
				}


				//$initialPeriodPrice = (count($returnHTML)==0)?$invoiceInfo["invoice_amount"]:0;

				$periodLength = $items[0]["cycleInfo"]["cycles"];
				$periodUnit = $items[0]["cycleInfo"]["unit"];

				$subscription = array();
				$subscription["image_url"] = urlencode($this->gatewayInfo["Display_Logo"]);
				$subscription["lc"] = strtoupper($this->params["countriesISO2"][0]);
				$subscription["cmd"] = "_xclick-subscriptions";
				$subscription["notify_url"] = $this->gatewayInfo["IPN_URL"];
				$subscription["business"] = $this->gatewayInfo["PayPal_ID"];
				$subscription["item_name"] = $this->gatewayInfo["Item_Name"];
				$subscription["item_number"] = "SCR".$subscriptionID;
				$subscription["no_shipping"] = "1";
				$subscription["no_note"] = "1";
				$subscription["currency_code"] = $this->gatewayInfo["currencyID"];

				$subscription["a1"] = round($initialPeriodPrice, 2); // trial period 1 price
				$subscription["p1"] = $periodLength; // trial period 1 length
				$subscription["t1"] = $periodUnit; // trial period 1 unit

				if ($firstprice>0){
					$subscription["a2"] = $firstprice; // trial period 2 price
					$subscription["p2"] = $couponcycles; // trial period 2 length
					$subscription["t2"] = $periodUnit; // trial period 2 unit
				}

				$subscription["a3"] = $regprice; // price per cycle
				$subscription["p3"] = $periodLength; // reg cycle length
				$subscription["t3"] = $periodUnit; // reg cycle units

				$subscription["src"] = "1";
				$subscription["sra"] = "1";
				if ($items[0]["numRenewals"]>-1){
					$subscription["srt"] = $items[0]["numRenewals"]; // number of times this payment will happen
				}

				$returnHTML[] = $this->createSubscriptionHTMLOutput($subscription);

			}

			////

		}

		$this->hasExecuted = 1;

		if ($unsupportedCycle){
			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>3</status>\n";
			$retval .= "	 <unsupportedCycle>1</unsupportedCycle>\n";
			$retval .= "   </gatewayReturn>\n";
		} else {
			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>1</status>\n";
			foreach ($returnHTML as $ret){
				$retval .= "     <paymentForm>".utf8_html_entity_encode($ret["paymentForm"])."</paymentForm>\n";
				$retval .= "     <paymentLink>".utf8_html_entity_encode($ret["paymentLink"])."</paymentLink>\n";
				$retval .= "     <paymentLinkURL>".utf8_html_entity_encode($ret["paymentLinkURL"])."</paymentLinkURL>\n";
			}
			$retval .= "   </gatewayReturn>\n";
		}

		$this->xmlData = $retval;
		return $retval;
	} //execute


	/**
	*	take a fields array and create the html output fields needed
	*
	*/
	function createSubscriptionHTMLOutput($fields){
		global $CONF;

		// init return variable
		$ret = array();

		$ret["paymentForm"] = "<form action='".$this->subscriptionURL."' method='post' target='_blank'>
			<input type='image' src='".$this->gatewayInfo["Subscriptions_Image"]."' border='0' name='submit' alt='Pay with Paypal'>";

		$ret["paymentLinkURL"] = $this->subscriptionURL."?";

		foreach ($fields as $k=>$v){
			$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
			$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".(($this->params["imageLinks"]) ? "<img src=\"".$this->gatewayInfo["Submit_Image"]."\" />" : TRANS_PAY_WITH_PAYPAL)."</a>";

		return $ret;

	}

	/**
	*	take a fields array and create the html output fields needed
	*
	*/
	function createOneTimeHTMLOutput($fields){
		global $CONF;

		// init return variable
		$ret = array();

		$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
			<input type='image' src='".$this->gatewayInfo["Submit_Image"]."' border='0' name='submit' alt=\"Make payments with PayPal - it's fast, free and secure!\">";

		$ret["paymentLinkURL"] = $this->paymentURL."?";

		foreach ($fields as $k=>$v){
			$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
			$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".(($this->params["imageLinks"]) ? "<img src=\"".$this->gatewayInfo["Submit_Image"]."\" border=\"0\" />" : TRANS_PAY_WITH_PAYPAL)."</a>";

		return $ret;

	}

	function setOverrideClientID($clientID){
		$this->manualClientID = $clientID;
	}

	function enableCreateClient(){
		$this->createClientOnDemand = 1;
	}

	/**
	*	process an IPN
	*
	*/
	function processCallbackReplay($callbackLogID){
		global $CONF;

		if (!$callbackLogID){
			return false;
		}
		
		$this->callbackLogID = $callbackLogID;
		
		$h =& new QueryMaster("callback_log.*");
		$h->addAndClause("callback_log_id", "=", $callbackLogID, "i");
		$callbackData = $h->getRow();
		
		$h =& new QueryMaster("callback_log_attributes.*");
		$h->addAndClause("callback_log_id", "=", $callbackLogID, "i");
		$attrs = $h->getResults();

		$this->IPNData = array();
		foreach ($attrs as $attr){
			$this->IPNData[$attr["callback_log_attribute_name"]] = $attr["callback_log_attribute_value"];
		}
		
		if (!$this->IPNData["receiver_email"]){
			return false;
		}

		dbg("PayPal: processing ipn replay");
			
			do {
			
				list($this->callbackLogAction, $this->callbackLogActionStatus) = $this->getCallbackLogAction($this->IPNData);
				$mapData = $this->getIPNMapData($this->IPNData);

				$this->currentPaypalID = "";
				$h =& new QueryMaster("config_params.config_param_value");
				if ($mapData["configGroupID"]){ // if we have it narrow it down, if not just use one...
					$h->addAndClause("config_group_id", "=", $mapData["configGroupID"], "i");
				}
				$h->addAndClause("config_param_name", "=", array("PayPal_ID", "Acceptable_PayPal_Payees"), "s");
				$results = $h->getCol();
				foreach ($results as $r){
					$this->currentPaypalID .= " ".$r;
				}
				$this->currentPaypalID = strtolower($this->currentPaypalID);

				// check for IPN's for paypal accounts we do not recognize
				if (strtolower($this->IPNData["receiver_email"])!=strtolower($this->subscriptionInfo["subscription_account_identifier"]) &&
					strtolower($this->IPNData["business"])!=strtolower($this->subscriptionInfo["subscription_account_identifier"]) &&
					strpos($this->currentPaypalID, strtolower($this->IPNData["receiver_email"]))===false &&
					strpos($this->currentPaypalID, strtolower($this->IPNData["business"]))===false){
					dbg("current paypal id: ".$this->currentPaypalID);
					dbg("subscription stuff: ".$this->subscriptionInfo["subscription_account_identifier"]);
					dbg("trying to match these to it: ".$this->IPNData["receiver_email"]." -- ".$this->IPNData["business"]);
						$this->logIPN(TRANS_INVALID_RECEIVER_FRAUD_DETECTED);
						break; // lets get outta here!
				}

				if (!$mapData["clientID"]){
					$this->logIPN(TRANS_UNABLE_TO_FIND_APPROPRIATE_CLIENT);
					break;
				}
				
				updateForKey("callbackLogs", $callbackLogID, $mapData);

				$this->handleIPNActions($mapData, $this->IPNData, $this->callbackLogAction);

			} while (false);

			return true;
	}


	/**
	*	process an IPN
	*
	*/
	function processReturn($post){
		global $CONF;

		$this->IPNData = array();
		if ($post["V4IPN"]){
			$this->IPNData["V4IPN"] = $post["V4IPN"];
			$this->v4ipn = true;
			$post = unserialize(base64_decode($post["V4IPN"]));
			dbg("got a v4 ipn");
			dbg($post);
		}
		
		if (!$post["receiver_email"]){
			return false;
		}

		$this->allowedIPNVariables = array(
			"txn_type",
			"subscr_date",
			"subscr_effective",
			"period1",
			"period2",
			"period3",
			"amount1",
			"amount2",
			"amount3",
			"mc_amount1",
			"mc_amount2",
			"mc_amount3",
			"mc_currency",
			"recurring",
			"reattempt",
			"retry_at",
			"recur_times",
			"username",
			"password",
			"subscr_id",
			"business",
			"receiver_email",
			"receiver_id",
			"item_name",
			"item_number",
			"invoice",
			"custom",
			"option_name1",
			"option_selection1",
			"option_name2",
			"option_selection2",
			"payment_status",
			"pending_reason",
			"reason_code",
			"payment_date",
			"txn_id",
			"parent_txn_id",
			"mc_gross",
			"mc_fee",
			"mc_currency",
			"settle_amount",
			"settle_currency",
			"exchange_rate",
			"payment_gross",
			"payment_fee",
			"first_name",
			"last_name",
			"payer_business_name",
			"address_name",
			"address_street",
			"address_city",
			"address_state",
			"address_zip",
			"address_country",
			"payer_email",
			"payer_id",
			"payer_status",
			"payment_type"
		);

		dbg("PayPal: processing ipn return");
		// paypal sample code:
		// read the post from PayPal system and add 'cmd'

		$postFields = "cmd=_notify-validate&";

		foreach ($post as $k=>$v){
			//if (in_array($k, $this->allowedIPNVariables)){
				$this->IPNData[$k] = secureInput(stripslashes($v));
			//}
			$postFields .= "$k=".urlencode(stripslashes($v))."&";
		}

		dbg("getting ready to do the curl call:");
		dbg($postFields);
		$hashString = "";
		foreach ($this->IPNData as $k=>$v) $hashString .= "<$k>$v";
		$this->IPNData["MBv5Signiture"] = sha1($hashString);

		$curlcall =& new Curl($this->IPNVerifyURL, $postFields);
		$curlResult = $curlcall->toString();

		// uncomment this to fake a good result:
		// $curlResult = " VERIFIED ";
		dbg("did curl call");

		dbg($curlResult);

		if (eregi("VERIFIED", $curlResult)){
			// is a valid ipn call
			dbg("IPN is valid");

			// better than a big if statement.... Woolley style
			do {
			
				// lets test for duplicate IPN's
				// we take the hash of our stuff and check for a callback log attribute
				// for a paypal callback log record that matches our hash.
				$h =& new QueryMaster("callback_log_attributes.*");
				$h->addTables("callback_log");
				$h->addJoin("callback_log", "callback_log_id", "callback_log_attributes");
				$h->addAndClause("callback_log_attributes.callback_log_attribute_name", "=", "MBv5Signiture", "s");
				$h->addAndClause("callback_log.callback_log_type", "=", getAPITypeID("callbackLog", "paypal"), "s");
				$h->addAndClause("callback_log_attributes.callback_log_attribute_value", "=", $this->IPNData["MBv5Signiture"], "s");
				$testRow = $h->getRow();
				if ($testRow["callback_log_attribute_value"]==$this->IPNData["MBv5Signiture"]){
					dbg($testRow);
					dbg($this->IPNData);
					$this->logIPN(TRANS_DUPLICATE_IPN_POSTING);
					break; // no more processing, this was a dupe
				}

				// this little block of code inserts a callback log record AND its
				// callback log attributes (which are everything in the $this->IPNData
				// array, which includes our identity hash).
				list($this->callbackLogAction, $this->callbackLogActionStatus) = $this->getCallbackLogAction($this->IPNData);
				$callbackLogID = $this->insertCallbackLog($callbackLogData = array(
					"callbackLogType" => getAPITypeID("callbackLog", "paypal"),
					"callbackLogTransactionID" => $this->IPNData["txn_id"],
					"callbackLogParentTransactionID" => $this->IPNData["parent_txn_id"],
					"callbackLogAction" => getAPITypeID("callbackAction", $this->callbackLogAction),
					"callbackLogActionStatus" => getAPIStatusID("callbackAction", $this->callbackLogActionStatus),
					"callbackLogDateCreated" => time(),
				) + ($mapData = (array)$this->getIPNMapData($this->IPNData)), $this->IPNData);
				$this->callbackLogID = $callbackLogID;

				$this->currentPaypalID = "";
				$h =& new QueryMaster("config_params.config_param_value");
				if ($mapData["configGroupID"]){ // if we have it narrow it down, if not just use one...
					$h->addAndClause("config_group_id", "=", $mapData["configGroupID"], "i");
				}
				$h->addAndClause("config_param_name", "=", array("PayPal_ID", "Acceptable_PayPal_Payees"), "s");
				$results = $h->getCol();
				foreach ($results as $r){
					$this->currentPaypalID .= " ".$r;
				}
				$this->currentPaypalID = strtolower($this->currentPaypalID);

				// check for IPN's for paypal accounts we do not recognize
				if (strtolower($this->IPNData["receiver_email"])!=strtolower($this->subscriptionInfo["subscription_account_identifier"]) &&
					strtolower($this->IPNData["business"])!=strtolower($this->subscriptionInfo["subscription_account_identifier"]) &&
					strpos($this->currentPaypalID, strtolower($this->IPNData["receiver_email"]))===false &&
					strpos($this->currentPaypalID, strtolower($this->IPNData["business"]))===false){
					dbg("current paypal id: ".$this->currentPaypalID);
					dbg("subscription stuff: ".$this->subscriptionInfo["subscription_account_identifier"]);
					dbg("trying to match these to it: ".$this->IPNData["receiver_email"]." -- ".$this->IPNData["business"]);
						$this->logIPN(TRANS_INVALID_RECEIVER_FRAUD_DETECTED);
						break; // lets get outta here!
				}

				if (!$mapData["clientID"]){
					$this->logIPN(TRANS_UNABLE_TO_FIND_APPROPRIATE_CLIENT);
					break;
				}

				$this->handleIPNActions($mapData, $this->IPNData, $this->callbackLogAction);

			} while (false);

			return true;
		} else {
			dbg("IPN is NOT valid");
			$this->logIPN(TRANS_PAYPAL_DIDNT_VERIFY);
			return false;
		}

	}

	/**
	 * function to handle things like chargebacks, chargeback reversals, pending echecks, echeck payments
	 */
	function handleIPNActions($mapData, $data, $callbackLogAction){
		$this->isSubscriptionAction = (strlen($data["subscr_id"])>2)?1:0;
		$this->parentTransactionInformation = $this->findParentInformation($data);
		switch ($callbackLogAction){
			// non-payment related ipn's
			case "subscriptionSignup": // tested works
				$this->handleSubscriptionSignup($mapData, $data);
				break;
			case "subscriptionFailed": // uses same code (so it works)
				$this->handleSubscriptionFailed($mapData);
				break;
			case "subscriptionCancelation": // tested works
				$this->handleSubscriptionCanceled($mapData);
				break;
			case "subscriptionExpired": // uses same code (so it works)
				$this->handleSubscriptionExpired($mapData);
				break;
			case "paymentEcheckPending":
				break;
			case "newDisputeCase":
				break;
			
			// payment related ipn's
			case "subscriptionPayment": // tested works
				$this->handleSubscriptionPayment($mapData, $data);
				break;
			case "paymentEcheckFinished": // post the echeck
			case "payment": // tested works
				$this->handlePayment($mapData, $data);
				break;
			
			// repayment (treat these as new payments based on whether they are subscription related or not)
			case "paymentReversalCancelation":
			case "paymentChargebackCancelation":
			case "paymentEcheckReversalCancelation":
				if ($this->isSubscriptionAction){
					$this->handleSubscriptionPayment($mapData, $data);
				} else {
					$this->handlePayment($mapData, $data);
				}
				break;
			
			// refund related, do refund actions
			case "paymentEcheckDenied":
			case "paymentReversal":
			case "paymentChargeback":
			case "paymentEcheckReversal":
			case "paymentRefundedGateway": // tested works
				$this->handleTransactionRefund($mapData, $data);
				break;
		}
	}

	function handleTransactionRefund($mapData, $IPNData){
		if (!$this->parentTransactionInformation["callbackLog"]["callback_log_id"]){
			dbg($this->parentTransactionInformation);
			$this->logIPN("TRANS_REFUND_IPN_WITH_NO_PARENT");
			return;
		}
		
		$parentTransaction = $this->getParentTransaction($IPNData["parent_txn_id"]);
		
		if ($parentTransaction["transaction_id"]){
			$transactionID = $parentTransaction["transaction_id"];
		
			// update the transaction status to refunded
			$h =& new QueryMaster("update");
			$h->addTables("transaction_log");
			$h->addAndClause("transaction_id", "=", $transactionID, "i");
			$h->addParam("auth_return", getAPIStatusID("transactions", "refunded"), "i");
			$h->addParam("transaction_amount_refunded", abs($IPNData["mc_gross"]), "i");
			$h->execute();
					
					
			$h =& new QueryMaster("transaction_success_rel.*");
			$h->addColumns("invoices.*");
			$h->addColumns("invoice_line_items.invoice_id");
			$h->addJoin("invoices", "invoice_id", "invoice_line_items");
			$h->addJoin("invoice_line_items", "line_item_id", "transaction_success_rel");
			$h->addAndClause("transaction_id", "=", $transactionID, "i");
			$lineItems = $h->getResults();
			$invoices = array();
			$invoicesData = array();
			foreach ((array)$lineItems as $li){
				$invoicesData[$li["invoice_id"]] = $li;
				$invoices[$li["invoice_id"]] += $li["line_item_payment_amount"];
			}
			
			// delete the payment relations to line items for it
			$h =& new QueryMaster("delete");
			$h->addTables("transaction_success_rel");
			$h->addAndClause("transaction_id", "=", $transactionID, "i");
			dbg($h->getQuery());
			$h->execute();
		
			foreach ((array)$invoices as $invoiceID=>$amountRefunded){
				$invoiceInfo = $invoicesData[$invoiceID];
				$h =& new QueryMaster("update");
				$h->addTables("invoices");
				$h->addAndClause("invoice_id", "=", $invoiceID, "i");
				$h->addParam("invoice_amount_paid", $invoiceInfo["invoice_amount_paid"]-$amountRefunded, "d");
				$h->addParam("invoice_status", getAPIStatusID("invoice", "due"), "i");
				$h->execute();
			}
		
			dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
					),
				)
			)));
		
		} else {
			dbg($this->parentTransactionInformation);
			dbg($parentTransaction);
			$this->logIPN("TRANS_REFUND_IPN_WITH_MISSING_TRANSACTION");
		}
	}

	function findParentInformation($data){
		if ($data["parent_txn_id"]){
			$h =& new QueryMaster("callback_log.*");
			$h->addAndClause("callback_log.callback_log_type", "=", getAPITypeID("callbackLog", "paypal"), "i");
			$h->addAndClause("callback_log.callback_log_transaction_id", "=", $data["parent_txn_id"], "s");
			$callbackLog = $h->getRow();
			dbg($h->getQuery());
			if ($callbackLog["callback_log_id"]){
				$h =& new QueryMaster("callback_log_attributes");
				$h->addAndClause("callback_log_id", "=", $callbackLog["callback_log_id"], "i");
				$callbackLogAttributes = $h->getResults();
				return array(
					"callbackLog" => $callbackLog,
					"callbackLogAttributes" => $callbackLogAttributes
				);
			} else {
				dbg("didnt get anything from query");
			}
		}
		dbg("no parent or didnt get anything from query");
		return array();
	}
	
	function getCallbackLogAttributeByName($attributes, $name){
		foreach ((array)$attributes as $attr){
			if ($attr["callback_log_attribute_name"]==$name){
				return $attr;
			}
		}
	}

	function handleSubscriptionExpired($mapData){
		dbg("subscription end of term IPN.");
		if ($mapData["subscriptionID"]){
			$this->updateSubscriptionStatus($mapData["subscriptionID"], "expired");
		} else {
			$this->logIPN(TRANS_UNMATCHED_SUBSCRIPTION);
		}
	}

	function handleSubscriptionFailed($mapData){
		if ($mapData["subscriptionID"]){
			$this->updateSubscriptionStatus($mapData["subscriptionID"], "failed");
		} else {
			$this->logIPN(TRANS_UNMATCHED_SUBSCRIPTION);
		}
	}

	function handleSubscriptionCanceled($mapData){
		if ($mapData["subscriptionID"]){
			$this->updateSubscriptionStatus($mapData["subscriptionID"], "cancelled");
		} else {
			$this->logIPN(TRANS_UNMATCHED_SUBSCRIPTION);
		}
	}

	function updateSubscriptionStatus($subscriptionID, $statusName){
		dbg("updating subscription $subscriptionID to status $statusName");
		if ($subscriptionID){
			dbg("made it in");
			$h =& new QueryMaster("update");
			$h->addTables("subscriptions");
			$h->addParam("subscription_status", getAPIStatusID("subscription", $statusName), "i");
			$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
			dbg($h->getQuery());
			$h->execute();
		}
	}
	
	function handleSubscriptionSignup($mapData, $data){
		if (!$mapData["subscriptionID"]){
			if ($mapData["clientID"]){
			
				$h =& new QueryMaster("insert");
				$h->addTables("subscriptions");
				$h->addParam("subscription_status", getAPIStatusID("subscription", "active"), "i");
				$h->addParam("client_id", $mapData["clientID"], "i");
				$h->addParam("subscription_date_created", time(), "i");
				$h->addParam("subscription_date_modified", time(), "i");
				$h->addParam("config_group_id", $mapData["configGroupID"], "i");
				$h->addParam("subscription_account_identifier", trim(rtrim($this->gatewayInfo["PayPal_ID"])), "s");
				$subresult = $h->execute();
				$subscriptionID = $h->getLastInsertID();

				$h =& new QueryMaster("packages.package_id");
				$h->addAndClause("client_id", "=", $mapData["clientID"], "i");
				$packageIDs = $h->getCol();

				$h =& new QueryMaster("package_addons.package_addon_id");
				$h->addTables("packages");
				$h->addJoin("packages", "package_id", "package_addons");
				$h->addAndClause("client_id", "=", $mapData["clientID"], "i");
				$packageAddonIDs = $h->getCol();

				foreach ($packageIDs as $pid){
					$h =& new QueryMaster("insert");
					$h->addTables("subscription_package_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_id", $pid, "i");
					$h->execute();
				}

				foreach ($packageAddonIDs as $pid){
					$h =& new QueryMaster("insert");
					$h->addTables("subscription_package_addon_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_addon_id", $pid, "i");
					$h->execute();
				}

			}
		} else {
			$h =& new QueryMaster("update");
			$h->addTables("subscription");
			$h->addParam("subscription_status", getAPIStatusID("subscription", "active"), "i");
			$h->addAndClause("subscription_id", "=", $mapData["subscriptionID"], "i");
			$h->execute();
		}
	}


	/**
	 * function to handle a subscription payment
	 * BEGIN applying payment to packages and addons (for a subscription ipn)
	 * this will create a transaction to record the ipn, as well as creating the success ids for the
	 * line items that got paid, and changing the status of the invoice they are on accordingly as is necessary.
	 */
	function handleSubscriptionPayment($mapData, $IPNData){
	
		if ($IPNData["subscr_id"] && $this->subscriptionInfo["subscription_external_id"]==""){
			$h =& new QueryMaster("update");
			$h->addTables("subscriptions");
			$h->addParam("subscription_status", getAPIStatusID("subscription", "active"), "i");
			$h->addParam("subscription_external_id", $IPNData["subscr_id"], "s");
			$h->addAndClause("subscription_id", "=", $this->subscriptionInfo["subscription_id"], "i");
			$h->execute();
			dbg("updated subscription status");
		}

		$transactionID = $this->makeTransaction($IPNData, false, $this->subscriptionInfo["config_group_id"]);
		
		// lets set the client id in the main scope
		$clientID = $mapData["clientID"];
		$invoiceID = (int)$mapData["invoiceID"];

		// need to get all of the unpaid invoices and their line items that correspond to the package ids we have
		$allLineItems = $this->getLineItems($invoiceID, $this->getSubscriptionRelatedPackages());
		$lineItemIDs = $this->getLineItemIDsFromLineItems($allLineItems);
		$lineItemPayments = $this->getLineItemPayments($lineItemIDs);

		// the invoice id we will work with, will be the first invoice for a package we are paying for
		// that has one that isn't paid
		$invoiceID = (!$invoiceID)?$allLineItems[0]["invoice_id"]:$invoiceID;

		// now we need to apply the payment
		$this->payLineItemsAndDoAccounting(
			$mapData,
			$IPNData,
			$this->getUnpaidLineItems($allLineItems, $lineItemPayments, $invoiceID),
			$transactionID,
			$invoiceID,
			$clientID);
		
		// send subscription payment receipt email (LAST because people can mess up the template...)
		$this->sendPaymentEmail($IPNData, $clientID);

	}
		

	/**
	 * Handle a payment that is not for a subscription we have a record of
	 * this will create a transaction to record the ipn, as well as creating the success ids for the
	 * line items that got paid, and changing the status of the invoice as is necessary.
	 */
	function handlePayment($mapData, $IPNData){
	
		dbg("applying IPN paypal payment for invoice");

		$clientID = $mapData["clientID"];

		$invoiceID = $this->invoiceID;
		$configGroupID = $this->configGroupID;
		
		// if we dont have an invoice to pay lets get one
		if (!$invoiceID){
			$h =& new QueryMaster("invoices.*");
			$h->addTables("invoice_line_items");
			$h->addTables("packages");
			$h->addJoin("packages", "package_id", "invoice_line_items");
			$h->addJoin("invoice_line_items", "invoice_id", "invoices");
			$h->addAndClause("packages.client_id", "=", $mapData["clientID"], "i");
			$h->addCustomAndClause("invoices.invoice_amount_paid <> invoices.invoice_amount");
			$h->addAndClause("invoices.invoice_status", "=", array(
				getAPIStatusID("invoice", "due"),
				getAPIStatusID("invoice", "declined"),
				getAPIStatusID("invoice", "errored"),
			), "i");
			$h->addOrderByColumns("invoice_date_due", "ASC");
			$invoiceInfo = $h->getRow();
			$invoiceID = $invoiceInfo["invoice_id"];
		} else {
			$h =& new QueryMaster("invoices.*");
			$h->addAndClause("invoice_id", "=", $invoiceID, "i");
			$invoiceInfo = $h->getRow();
		}
		
		// ok, so we have an invoice, but don't have a config group for our paypal
		// gateway instance, lets get the first one for the invoice that we have
		// and use it.
		if (!$configGroupID && $invoiceID){
			$h =& new QueryMaster("config_groups.config_group_id");
			$h->addAndClause("config_group_name", "=", "paypal", "s");
			$parentConfigGroupID = $h->getOne();
			
			$h =& new QueryMaster("payment_groups_gateway_rel.config_group_id");
			$h->addTables("invoices");
			$h->addTables("config_groups");
			$h->addJoin("config_groups.parent_config_group_id", "=", $parentConfigGroupID, "i");
			$h->addJoin("invoices", "payment_group_id", "payment_groups_gateway_rel");
			$h->addAndClause("payment_groups_gateway_rel.gateway_type", "=", 0, "i");
			$h->addAndClause("invoices.invoice_id", "=", $invoiceID, "i");
			$configGroupID = $h->getOne();
		}
		
		// lets go ahead and make the transaction
		$transactionID = $this->makeTransaction($IPNData, $invoiceInfo["invoice_number"], $configGroupID);
		dbg("inserted transaction: ".$transactionID);
		
		// if we couldn't find an invoice to pay, lets just give this client a credit and be done with it.
		if (!$invoiceID){
			$this->giveClientCredit($mapData, $IPNData);
		} else {
			// get the line items data...
			$allLineItems = $this->getLineItems($invoiceID);
			$lineItemIDs = $this->getLineItemIDsFromLineItems($allLineItems);
			$lineItemPayments = $this->getLineItemPayments($lineItemIDs);
	
			// now we need to apply the payment
			$this->payLineItemsAndDoAccounting(
				$mapData,
				$IPNData,
				$this->getUnpaidLineItems($allLineItems, $lineItemPayments),
				$transactionID,
				$invoiceID,
				$clientID);
		}
		
		// send the email last because people can break the template
		$this->sendPaymentEmail($IPNData, $clientID);

	}
	
	
	/**
	 * function to give a client a credit because we can't find anything to apply the payment to
	 */
	function giveClientCredit($mapData, $IPNData){
		dispatchMBAPI(toXML(array(
			"mbapi" => array(
				"command" => "SetCreditAccounting",
				"params" => array(
					"clientID" => $mapData["clientID"],
					"ledgerAccountName" => "cash",
					"amount" => $IPNData["mc_gross"],
					"currencyID" => $IPNData["mc_currency"],
					"description" => TRANS_PAYPAL_PAYMENT_WITH_NOTHING_TO_PAY,
				),
			),
		)));
	}
	
	/**
	 * function to get a transaction based on txn_id
	 */
	function getParentTransaction($txn_id){
		$h =& new QueryMaster("transaction_log.*");
		$h->addAndClause("transaction_code", "=", "paypal".$txn_id, "s");
		dbg("got parent:");
		dbg($x = $h->getRow());
		return $x;
	}
	
	/**
	 * function to shortcut making a transaction
	 */
	function makeTransaction($IPNData, $invoiceNumber, $configGroupID=0){
		$transactionID = $this->insertTransaction(array(
			"auth_return" => GATEWAY_SUCCESS,
			"transaction_payment_method" => getAPITypeID("payments", "paypal"),
			"transaction_code" => "paypal".$IPNData["txn_id"],
			"transaction_notes" => TRANS_PAYMENT_FOR_INVOICE. (($invoiceNumber)?": ".$invoiceNumber:''),
			"transaction_date_created" => time(),
			"transaction_amount" => doubleval($IPNData["mc_gross"]),
			"currency_id" => $IPNData["mc_currency"],
			"config_group_id" => $configGroupID,
		));
		if ((int)$this->callbackLogID){
			updateForKey("callbackLogs", $this->callbackLogID, array(
				"transactionID" => $transactionID,
			));
		}
		return $transactionID;
	}
	
	/**
	 * function to set the payments for line items and make the accounting entries
	 */
	function payLineItemsAndDoAccounting($mapData, $IPNData, $unpaidLineItems, $transactionID, $invoiceID, $clientID){

		// ok, got all the unpaid line items for the oldest invoice that has packages we are paying for
		// in an array called $unpaidLineItems, now we just need to loop through them and apply the money
		// towards them.
		$invoiceInfo = $unpaidLineItems[0]; // the invoice info was gotten in the line items query

		$amountAllowcated = 0;
		foreach ($unpaidLineItems as $k=>$li){
			$available = doubleval($IPNData["mc_gross"])-$amountAllowcated;
			$amountNeeded = $li["line_item_total_amount"]-$li["paid"];
			if ($amountNeeded>$available){
				$payAmount = $available;
			} else {
				$payAmount = $amountNeeded;
			}
			$unpaidLineItems[$k]["applyAmount"] = $payAmount;
			$amountAllowcated += $payAmount;
		}
		$unallowcated = doubleval($IPNData["mc_gross"])-$amountAllowcated;
		dbg("applied payments to line items, unallowcated: ".$unallowcated);


		// ok, so we made the amounts to pay, lets pay them
		// (that means inserting the transaction_success_rel entries and updating the invoice paid amt)
		foreach ($unpaidLineItems as $k=>$li){
			if ($li["applyAmount"]){
				$h =& new QueryMaster("insert");
				$h->addTables("transaction_success_rel");
				$h->addParam("transaction_id", $transactionID, "i");
				$h->addParam("line_item_id", $li["line_item_id"], "i");
				$h->addParam("line_item_payment_amount", doubleval($li["applyAmount"]), "d");
				$h->execute();
				$invoiceInfo["amountPaidNow"] += doubleval($li["applyAmount"]);
				dbg("added payment for line item: ".$li["line_item_id"]." of ".doubleval($li["applyAmount"]));
			}
		}

		$h =& new QueryMaster("update");
		$h->addTables("invoices");
		$h->addAndClause("invoice_id", "=", $invoiceID, "i");
		$h->addParam("invoice_amount_paid", round(doubleval($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]), 2), "d");
		$h->addParam("invoice_date_paid", time(), "i");
		if (round(($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]),2)==round($invoiceInfo["invoice_amount"], 2)){
			$h->addParam("invoice_status", 1, "i");
			dbg("invoice is closed");
		}
		$h->execute();
		dbg("updated invoice info");

		if ($unallowcated){
			$credit = $unallowcated;
			dbg("theres a credit amount");
		}

		// make the accounting entry
		$receipt =& new Payment();
		$receipt->setClientID($clientID);
		$receipt->setInvoiceID($invoiceID);
		$receipt->invoiceNumber = $invoiceInfo["invoice_number"];
		$receipt->setTransactionID($transactionID);
		$receipt->setDescription(TRANS_PAYPAL_PAYMENT_FOR_INVOICE.": ".$invoiceInfo["invoice_number"]);
		$receipt->setPaymentAmount(doubleval($IPNData["mc_gross"]));
		if ($credit>0){
			$receipt->setClientCreditAmount($credit);
		}
		$receipt->setCurrencyID($IPNData["mc_currency"]);
		$receipt->execute();
		dbg("made the acocunting entry");

		// associate the invoice with the transaction
		$h =& new QueryMaster("insert");
		$h->addTables("invoice_transaction_rel");
		$h->addParam("invoice_id", $invoiceID, "i");
		$h->addParam("transaction_id", $transactionID, "i");
		$h->execute();
		dbg("set the invoice transaction rel record");

		$this->addPaypalFee($clientID, $invoiceID, $IPNData["mc_fee"]);
		// handle currency conversion if necessary
		$this->makeAccountingCurrenyConversion($IPNData["mc_gross"], $IPNData["mc_currency"], $IPNData["settle_amount"], $IPNData["settle_currency"], $clientID, $invoiceID);
	}


	/**
	 * function to get the unpaid line items from a set of line items and their payment data
	 */
	function getUnpaidLineItems($allLineItems, $lineItemPayments, $invoiceID=false){
		$unpaidLineItems = array();
		if ($invoiceID){
			foreach ($allLineItems as $k=>$li){
				$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
				if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"] && $li["invoice_id"]==$invoiceID){
					$unpaidLineItems[] = $allLineItems[$k];
				}
			}
			dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));
			return $unpaidLineItems;
		} else {
			foreach ($allLineItems as $k=>$li){
				$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
				if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"]){
					$unpaidLineItems[] = $allLineItems[$k];
				}
			}
			dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));
			return $unpaidLineItems;
		}
	}

	/**
	 * function to get the line items ids out of the line items array
	 */
	function getLineItemIDsFromLineItems($lineItems){
		$lineItemIDs = array();
		foreach ($lineItems as $li){
			$lineItemIDs[] = $li["line_item_id"];
		}
		return $lineItemIDs;
	}

	/**
	 * function to get the line items for certain parameters
	 */
	function getLineItems($invoiceID=false, $packageIDs=false){
		if ($invoiceID){
			$h =& new QueryMaster("invoices.*", "invoice_line_items.*");
			$h->addAndClause("invoices.invoice_id", "=", $invoiceID, "i");
			$allLineItems = $h->getResults(-1, 0);
			dbg("query: ".$h->getQuery());
			dbg("got all line items: ".count($allLineItems));
			return $allLineItems;
		} else {
			$h =& new QueryMaster("invoices.*", "invoice_line_items.*");
			$h->addAndClause("package_id", "=", $packageIDs, "i");
			$h->addCustomAndClause("invoice_amount_paid < invoice_amount");
			$h->addOrderByColumns("invoice_date_due", "ASC");
			$allLineItems = $h->getResults(-1, 0);
			dbg("got all line items: ".count($allLineItems));
			return $allLineItems;
		}
	}

	/**
	 * function to get the success relations for certain line items
	 */
	function getLineItemPayments($lineItemIDs){
		$h =& new QueryMaster("transaction_success_rel.*");
		$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
		$payments = $h->getResults(-1, 0);
		dbg("got transaction success rows: ".count($payments));

		// add up the payments according to line item id and organize them as well
		$lineItemPayments = array();
		foreach ($payments as $pmnt){
			$lineItemPayments[$pmnt["line_item_id"]]["paid"] += $pmnt["line_item_payment_amount"];
			$lineItemPayments[$pmnt["line_item_id"]]["payments"][] = $pmnt;
		}
		return $lineItemPayments;
	}

	/**
	 * function to send out a payment email
	 */
	function sendPaymentEmail($IPNData, $clientID){
		dbg("starting email process");
		if (!$this->emailTemplateGroupID){
			$h =& new QueryMaster("email_tpl_groups.email_tpl_group_id");
			$h->addAndClause("email_action_id", "=", getTypeID("emailAction", TRANS_PAYMENT_SECONDARY_EMAIL), "i");
			$this->emailTemplateGroupID = $h->getOne();
		}
		$email =& new MBTemplateBasedEmail($this->emailTemplateGroupID);
		$email->addTemplateVars("client", $clientID);
		$email->addVars("gatewayName", "PayPal");
		$email->addVars("transactionDateCreated", time());
		$email->addVars("transactionCode", $IPNData["txn_id"]);
		$email->addVars("transactionAmount", $IPNData["mc_gross"]);
		$email->addVars("currencyID", $IPNData["mc_currency"]);
		$email->sendEmails();
	}

	/**
	 * function to get what packages we need to get for a subscription
	 */
	function getSubscriptionRelatedPackages($subscriptionID=false, $clientID=false){
		$subscriptionID = ($subscriptionID)?$subscriptionID:$this->subscriptionInfo["subscription_id"];
		$clientID = ($clientID)?$clientID:$this->subscriptionInfo["client_id"];
		// get packages related to this subscription
		$h =& new QueryMaster("subscription_package_rel.*", "packages.*");
		$h->addJoin("subscription_package_rel", "package_id", "packages");
		$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
		$relatedPackages = $h->getResults(-1, 0);
		dbg("Got related packages:");
		dbg($relatedPackages);
		// get package addons related to this subscription
		$h =& new QueryMaster("subscription_package_addon_rel.*", "packages.*", "package_addons.*");
		$h->addJoin("subscription_package_addon_rel", "package_addon_id", "package_addons");
		$h->addJoin("package_addons", "package_id", "packages");
		$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
		$relatedPackageAddons = $h->getResults(-1, 0);
		dbg("Got related package addons:");
		dbg($relatedPackageAddons);

		// if we don't have any packages related to the subscription, lets just
		// get them for the client
		if (!count($relatedPackages)){
			// get packages related to this subscription
			$h =& new QueryMaster("packages.*");
			$h->addAndClause("packages.client_id", "=", $clientID, "i");
			$relatedPackages = $h->getResults(-1, 0);
			dbg("Got related packages:");
			dbg($relatedPackages);
			// get package addons related to this subscription
			$h =& new QueryMaster("packages.*", "package_addons.*");
			$h->addAndClause("packages.client_id", "=", $clientID, "i");
			$relatedPackageAddons = $h->getResults(-1, 0);
			dbg("Got related package addons:");
			dbg($relatedPackageAddons);
		}

		$packageIDsToGet = array();
		foreach ((array)$relatedPackages as $rp){
			$packageIDsToGet[] = $rp["package_id"];
		}
		foreach ((array)$relatedPackageAddons as $rpa){
			$packageIDsToGet[] = $rpa["package_id"];
		}
		return $packageIDsToGet;
	}

	/**
	 * function to get subscription info using a few different parameters
	 */
	function getSubscriptionInfo($subscriptionID=false, $externalSubscriptionID=false){
		if ($subscriptionID){
			$h =& new QueryMaster("subscriptions.*");
			$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
			$subscriptionInfo = $h->getRow();
		} else {
			$h =& new QueryMaster("subscriptions.*");
			$h->addAndClause("subscription_external_id", "=", $externalSubscriptionID, "s");
			$subscriptionInfo = $h->getRow();
		}
		return $subscriptionInfo;
	}

	/**
	 * function to set subscription info
	 */
	function setSubscriptionInfo($data, $subscriptionID=false, $externalSubscriptionID=false){
		if ($subscriptionID || $externalSubscriptionID){
			$h =& new QueryMaster("update");
			$h->addTables("subscriptions");
			if ($subscriptionID){
				$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
			} else {
				$h->addAndClause("subscription_external_id", "=", $externalSubscriptionID, "i");
			}
		} else {
			$h =& new QueryMaster("insert");
			$h->addTables("subscriptions");
		}
		foreach ((array)$data as $field=>$row){
			$h->addParam($field, $row["data"], $row["type"]);
		}
		return $h->execute();
	}


	/**
	 * function to log an error'd ipn
	 */
	function logIPN($message){
		dbg("logging IPN: ".$message);
		insertFor("invalidIpnLogs", array(
			"ipnStatus" => getAPIStatusID("invalidipns", "new"),
			"ipnReason" => $message,
			"ipnModule" => getAPITypeID("ipnmodules", "paypal"),
			"ipnServer" => base64_encode(serialize($_SERVER)),
			"ipnGet" => base64_encode(serialize($_GET)),
			"ipnPost" => base64_encode(serialize($_POST)),
			"ipnDateCreated" => time(),
		));
	}

	/**
	 * do what needs to be done when a subscription expires
	 */
	function runSubscriptionExpire(){

		// which is nothing at the moment.

	}

	/**
	*	get the number of times this package is going to renew
	*/
	function getNumRenewals($packagerow){
		global $CONF;
		if ((int)$packagerow["package_date_stop_renewal"]<(25*60*60)){
			return -1;
		} else {
			$num = 0;
			$runningdate = $packagerow["package_date_next_renewal"];
			if ($runningdate>(25*60*60)){
				while ($runningdate<(int)$packagerow["package_date_stop_renewal"]){
					$runningdate = getNextBillingTimestamp($runningdate, $packagerow["cycle_id"]);
					$num++;
				}
				return $num;
			} else {
				return 1;
			}
		}
	}


	/**
	* get cycle unit and number
	*/
	function translateCycleInfo($packagerow){
		global $CONF;
		dbg($packagerow);
		$cycleData = getCycleData($packagerow["cycle_id"]);
		list($sec,$min,$hour,$day,$month,$year) = $cycleData;

		dbg($cycleData);
		dbg($CONF["cycles"]);
		
		// go through our cycle data and first check if they have more than one unit filled in
		// because paypal doesn't support multiple units
		$num = 0;
		foreach ((array)$cycleData as $unit){
			if ($unit>0)$num++;
		}
		// if true this isn't a cycle supported by paypal
		if ($num>1)return false;

		// init return
		$ret = array();

		// got past that check? ok, now to see what we have here...
		if ($day>0 && $day%7>0){// have a day value that isnt issued in whole weeks
			$ret["unit"] = "D";
			$ret["cycles"] = $day;
			return $ret;
		} elseif ($day>0){     // have a week based cycle expressed in days
			$ret["unit"] = "W";
			$ret["cycles"] = $day/7;
			return $ret;
		} elseif ($month>0){  // have a month based cycle
			$ret["unit"] = "M";
			$ret["cycles"] = $month;
			return $ret;
		} elseif ($year>0){  // have a year based cycle
			$ret["unit"] = "Y";
			$ret["cycles"] = $year;
			return $ret;
		}

		// this isn't a cycle that is supported by paypal
		return false;

	}


	/**
	* Calculates the taxes for this invoice amount
	*
	* @param amount    the amount to get taxes on
	* @param packageID  the package id we are getting tax zones for
	* @return the tax amount
	*/
	function calculateTaxes($amount, $packageID, $packageAddonID=0) {

		$taxZones = $packageAddonID ? $this->currentAddonTaxZones[$packageAddonID] : $this->currentTaxZones[$packageID];
		$tax = 0;
		$taxableAmount = $amount;
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if (!(int)$taxZone["tax_zone_stackable"]) continue;
			dbg("applying tax zone:");
			dbg($taxZone);
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				$cTax = round($amount*($taxZone["line_item_tax_amount"]/100), 2);
				$taxableAmount = round($taxableAmount + $cTax, 2);
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				$tax = round($tax + $cTax, 2);
			}
		}
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if ((int)$taxZone["tax_zone_stackable"]) continue;
			dbg("applying tax zone:");
			dbg($taxZone);
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				$cTax = round($taxableAmount*($taxZone["line_item_tax_amount"]/100), 2);
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				$tax = round($tax + $cTax, 2);
			}
		}
		return $tax;
	}
	
	/**
	 * function to get the callback action and callback action status
	 */
	function getCallbackLogAction($data){
		$data["payment_status"] = str_replace("-", "_", $data["payment_status"]);
		$data["reason_code"] = str_replace("-", "_", $data["reason_code"]);
		$callbackLogAction = '';
		$callbackLogActionStatus = 'complete';
		if ($data["txn_type"]=="subscr_eot"){
			$callbackLogAction = 'subscriptionExpired';
			$callbackLogAction = 'complete';
		} elseif (strlen($data["payment_type"])){
			if ($data["payment_type"]=="echeck"){
				if (!$data["payment_status"] && $data["pending_reason"]=="echeck"){
					$callbackLogAction = 'paymentEcheckPending';
					$callbackLogAction = 'pending';
				} elseif ($data["payment_status"]=="Denied" || $data["payment_status"]=="Expired"){
					$callbackLogAction = 'paymentEcheckDenied';
				} elseif ($data["payment_status"]=="Canceled_Reversal"){
					$callbackLogAction = 'paymentEcheckReversalCancelation';
				} elseif ($data["payment_status"]=="Reversed"){
					$callbackLogAction = 'paymentEcheckReversal';
				} elseif ($data["payment_status"]=="Processed" || $data["payment_status"]=="Completed"){
					$callbackLogAction = 'paymentEcheckFinished';
				}
			} else {
				if ($data["payment_status"]=="Completed" || $data["payment_status"]=="Processed"){
					if ($data["txn_type"]=="subscr_payment"){
						$callbackLogAction = 'subscriptionPayment';
					} else {
						$callbackLogAction = 'payment';
					}
				} elseif ($data["payment_status"]=="Reversed"){
					if ($data["reason_code"]=="chargeback"){
						$callbackLogAction = 'paymentChargeback';
					} elseif ($data["reason_code"]=="buyer_complaint"){
						$callbackLogAction = 'paymentChargeback';
					} else {
						$callbackLogAction = 'paymentReversal';
					}
				} elseif ($data["payment_status"]=="Canceled_Reversal"){
					if ($data["reason_code"]=="chargeback"){
						$callbackLogAction = 'paymentChargebackCancelation';
					} else {
						$callbackLogAction = 'paymentReversalCancelation';
					}
				} elseif ($data["payment_status"]=="Refunded"){
					$callbackLogAction = 'paymentRefundedGateway';
				}
			}
		} else {
			switch ($data["txn_type"]){
				case "subscr_signup":
					$callbackLogAction = 'subscriptionSignup';
					break;
				case "subscr_cancel":
					$callbackLogAction = 'subscriptionCancelation';
					break;
				case "subscr_failed":
					$callbackLogAction = 'subscriptionFailed';
					break;
				case "new_case":
					$callbackLogAction = 'newDisputeCase';
					$callbackLogActionStatus = 'pending';
					break;
			}
		}
		
		return array($callbackLogAction, $callbackLogActionStatus);
	}

	/**
	 * function to get the client, billing account, config group, and subscription that this ipn matches up to
	 */
	function getIPNMapData($data){
		global $CONF;
		$ret = array(
			"configGroupID" => 0,
			"subscriptionID" => 0,
			"clientID" => 0,
			"billingAccountID" => 0
		);
		
		if ($this->manualClientID>0){
			dbg("returning a manual client id");
			$ret["clientID"] = $this->manualClientID;
			return $ret;
		}
		
		if (!(int)$ret["clientID"] && $data["subscr_id"]){
			$h =& new QueryMaster("subscriptions.*");
			$h->addColumns("clients.*");
			$h->addJoin("clients", "client_id", "subscriptions");
			$h->addAndClause("subscriptions.subscription_external_id", "=", $data["subscr_id"], "s");
			$this->clientInfo = $this->subscriptionInfo = $h->getRow();
			$ret["subscriptionID"] = $this->subscriptionInfo["subscription_id"];
			$ret["clientID"] = $this->clientInfo["client_id"];
			$ret["configGroupID"] = $this->subscriptionInfo["config_group_id"];
			$ret["billingAccountID"] = $this->subscriptionInfo["billing_account_id"];
			dbg("had a subscription id and tried to get data, got: ");
			dbg($ret);
		}
		
		if (!(int)$ret["clientID"] && strtoupper(substr($data["item_number"], 0, 3))=="SCR" && $data["subscr_id"] && !(strtoupper($data["item_number"][0])=="I" && strpos(strtoupper($data["item_number"]), "P"))){
			$h =& new QueryMaster("subscriptions.*");
			$h->addColumns("clients.*");
			$h->addJoin("clients", "client_id", "subscriptions");
			$h->addAndClause("subscriptions.subscription_id", "=", (int)substr($data["item_number"], 3), "i");
			$this->clientInfo = $this->subscriptionInfo = $h->getRow();
			$ret["subscriptionID"] = $this->subscriptionInfo["subscription_id"];
			$ret["clientID"] = $this->clientInfo["client_id"];
			$ret["configGroupID"] = $this->subscriptionInfo["config_group_id"];
			$ret["billingAccountID"] = $this->subscriptionInfo["billing_account_id"];
			dbg("it looks like a subscription, lets check into it: ");
			dbg($ret);
		}
		
		if (!(int)$ret["clientID"] && (int)$data["item_number"] && $data["subscr_id"] && !(strtoupper($data["item_number"][0])=="I" && strpos(strtoupper($data["item_number"]), "P"))){
			$h =& new QueryMaster("subscriptions.*");
			$h->addColumns("clients.*");
			$h->addJoin("clients", "client_id", "subscriptions");
			$h->addAndClause("subscriptions.subscription_id", "=", (int)$data["item_number"], "i");
			$this->clientInfo = $this->subscriptionInfo = $h->getRow();
			$ret["subscriptionID"] = $this->subscriptionInfo["subscription_id"];
			$ret["clientID"] = $this->clientInfo["client_id"];
			$ret["configGroupID"] = $this->subscriptionInfo["config_group_id"];
			$ret["billingAccountID"] = $this->subscriptionInfo["billing_account_id"];
			dbg("it still looks like a subscription, so:");
			dbg($ret);
		}
		
		if (strtoupper($data["item_number"][0])=="I" && strpos(strtoupper($data["item_number"]), "P")){
			$split = explode("P", strtoupper($data["item_number"]));
			$this->invoiceID = (int)substr($split[0], 1);
			$this->configGroupID = $ret["configGroupID"] = (int)$split[1];
			$h =& new QueryMaster("clients.*");
			$h->addColumns("client_contacts.*");
			$h->addTables("packages");
			$h->addTables("invoice_line_items");
			$h->addJoin("packages", "package_id", "invoice_line_items");
			$h->addJoin("clients", "client_id", "packages");
			$h->addAndClause("invoice_line_items.invoice_id", "=", $this->invoiceID, "i");
			$h->addAndClause("client_contacts.contact_primary", "=", 1, "i");
			$h->addGroupByTables("clients", "client_contacts");
			$this->clientInfo = $h->getRow();
			$ret["clientID"] = $this->clientInfo["client_id"];
			dbg("looks like a v5 ipn with a formatted item_number:");
			dbg($ret);
		}
		
		if (!(int)$ret["clientID"] && $data["payer_email"]){
			$h =& new QueryMaster("clients.*");
			$h->addColumns("client_contacts.*");
			$h->addJoin("clients", "client_id", "client_contacts");
			$h->addAndClause("client_contacts.contact_email", "=", $data["payer_email"], "s");
			$h->addOrderByColumns("client_contacts.contact_primary", "DESC");
			$this->clientInfo = $h->getRow();
			$ret["clientID"] = $this->clientInfo["client_id"];
			dbg("trying to track down via payer email:");
			dbg($ret);
		}
		
		if (!(int)$ret["clientID"] && (int)$CONF["paypal"]["enableNameBasedMapping"] && $data["last_name"] && $data["first_name"]){
			$h =& new QueryMaster("clients.*");
			$h->addColumns("client_contacts.*");
			$h->addJoin("clients", "client_id", "client_contacts");
			$h->addAndClause("client_contacts.contact_first_name", "=", $data["first_name"], "s");
			$h->addAndClause("client_contacts.contact_last_name", "=", $data["last_name"], "s");
			$h->addOrderByColumns("client_contacts.contact_primary", "DESC");
			$this->clientInfo = $h->getRow();
			$ret["clientID"] = $this->clientInfo["client_id"];
			dbg("tried to look up based on last name:");
			dbg($ret);
		}
		
		dbg($CONF["paypal"]);
		
		if (((int)$this->createClientOnDemand || (int)$CONF["paypal"]["enableClientAutoCreation"]) && !(int)$ret["clientID"]){
			dbg("creating client on demand");
			$ret["clientID"] = $this->doCreateOnDemandClient($data);
 		}
		
		dbg($data);
		return $ret;
		
	}
	
	function doCreateOnDemandClient($data){
		$clientID = insertFor("clients", array(
			"companyID" => 1,
			"clientCompany" => $data["payer_business_name"],
			"clientStatus" => getAPIStatusID("client", "pending"),
			"applyTax" => 1,
			"themeID" => 1,
			"countriesID" => $countriesID = $this->getCountriesID($data["residence_country"]),
			"clientActive" => 1,
			"clientDateCreated" => time(),
		));
		$contactID = insertFor("contacts", array(
			"clientID" => $clientID,
			"contactUsername" => $data["payer_email"],
			"contactPassword" => getPasswordHash(uniqid("")),
			"contactFirstName" => $data["first_name"],
			"contactLastName" => $data["last_name"],
			"contactEmail" => $data["payer_email"],
			"contactEmailFormat" => 1,
			"contactAddress1" => $data["address_street"],
			"contactCity" => $data["address_city"],
			"contactState" => $data["address_state"],
			"contactZip" => $data["address_zip"],
			"languageID" => 21,
			"countriesID" => $countriesID,
			"contactPrimary" => 1,
			"contactActive" => 1,
			"contactDateCreated" => time(),
		));

		dbg($contactID);
		$q =& new MBQuery("GetEmailTemplateGroups", "");
		$r =& $q->getResult();
		$s =& $r->getSubjectFrom(array("mbapi", "results", "emailTplGroups", "emailTplGroup"));
		$t =& $s->getTable();
		$t->setKey("emailActionID");
		$emailActionIDs = $t->getKeys();
		
		foreach($emailActionIDs as $emailActionID) {
			insertFor("contactEmailActionRels", array(
				"clientContactID" => $contactID,
				"emailActionID" => $emailActionID
			));
		}
		
		return $clientID;
	}
	
	function getCountriesID($iso2){
		$h =& new QueryMaster("countries.countries_id");
		$h->addAndClause("countries_iso_2", "=", $iso2, "s");
		return $h->getOne();
	}
	
	function addPaypalFee($clientID, $invoiceID, $feeAmount){
		if ((float)$feeAmount>0){
			$entries = array();
			$entries[] = array(
				"clientID" => $clientID,
				"invoiceID" => $invoiceID,
				"generalLedgerType" => 1,
				"ledgerAccountID" => getGeneralLedgerAccountID("paypal_fees"),
				"generalLedgerDescription" => TRANS_PAYPAL_FEES,
				"generalLedgerDebitAmount" => doubleval($feeAmount),
				"generalLedgerCreditAmount" => 0,
				"currencyID" => $this->IPNData["mc_currency"]
			);
			$entries[] = array(
				"clientID" => $clientID,
				"invoiceID" => $invoiceID,
				"generalLedgerType" => 1,
				"ledgerAccountID" => getGeneralLedgerAccountID("cash"),
				"generalLedgerDescription" => TRANS_PAYPAL_FEES,
				"generalLedgerDebitAmount" => 0,
				"generalLedgerCreditAmount" => doubleval($feeAmount),
				"currencyID" => $this->IPNData["mc_currency"]
			);
			$gen =& new General();
			$gen->setClientID($clientID);
			$gen->setInvoiceID($invoiceID);
			$gen->setDescription(TRANS_PAYPAL_FEE);
			$gen->setTotalDebits(doubleval($feeAmount));
			$gen->setTotalCredits(doubleval($feeAmount));
			$gen->setCurrencyID($this->IPNData["mc_currency"]);
			$gen->setLedgerEntries($entries);
			$gen->execute();
			dbg("made fee entries");
		}
	}

} // PayPal

