<?php
// $Id: psigate.php 12896 2008-04-25 13:24:54Z btyra $

require_once dirname(__FILE__).'/../../../../lib-tk/shield.php';
require_once mb_path('/lib-mbapi/include/modules/gateway/gateway.php');
require_once mb_path("/lib-mbapi/include/nusoap/core/nusoap.php");

/**
 * This class is an instantiation of the Gateway class, that executes a command pertaining
 * to communicating with a remote registrar API, in this case we communicate directly with Sage Payments XML Web Services interface.
 *
 * @package modernbill
 * @subpackage mbapi
 * @author Blake Tyra <blake@parallels.com>
*/


$className = 'SagePayments';

$GLOBALS['moduleInfo'][$className] = array
(
    'type' => 'gateway',
    'status' => 'TRANS_TESTING',
    'version' => '1.0.0',
    'gatewaytype' => '1',
    'displayName' => $className,
    'author' => 'Parallels, Inc.',
    'capabilities' => array(
    	GATEWAY_CHARGE,
		GATEWAY_AUTHORIZE,
		GATEWAY_PRIOR_AUTH_CHARGE,
		GATEWAY_REFUND, 
		GATEWAY_VOID,
	)
);

class SagePayments extends Gateway {

	/**
	 * This returns the info about this module for display purposes
	 *
	 * @return array An array of arrays containing all the display info for the module.
	 */
	function getModuleInfo() {
		return $GLOBALS['moduleInfo']['SagePayments'];
	}

	/**
	 * Instantiates the gateway module and also the parent Gateway class. Sets capabilities and gateway URLs.
	 *
	 */
	function SagePayments($input) {
		$this->stringRep = 'sagepayments'; // lowercase please
		parent::Gateway($input);

		$moduleInfo = $this->getModuleInfo();

		foreach ($moduleInfo['capabilities'] as $capability) {
			$this->capabilities[$capability] = 1;
		}

		$this->liveURL = 'https://www.sagepayments.net/web_services/vterm_extensions/transaction_processing.asmx';
        $this->testURL = NULL;
	}

	/**
	 * This method returns the array structure to create the config items for an INSTANCE of this
	 * gateway.  Each thing about your gateway that needs to be configured should have its own
	 * config item.
	 * 
	 * @return	An array of arrays containing all the parameters the module uses. 
	 */
	function addDefaultConfigParams() {
		
		/*
		 * [0] = option name
		 * [1] = option type (t = text, p = password, r = radio)
		 * [2] = default value
		 * [3] = option label
		 * [4] = option description
		 */
		$adminOptions = array(
			array('merchantID',			't',		NULL,		'MERCHANTID',			'MERCHANTID_DESC'			),
			array('merchantKey',		't',		NULL,		'MERCHANTKEY',			'MERCHANTKEY_DESC'			),
			array('card_accept_0',		'r', 		'0',		'ACCEPT_VISA',			'ACCEPT_VISA_DESC'			),
			array('card_accept_1',		'r',		'0',		'ACCEPT_MASTERCARD',	'ACCEPT_MASTERCARD_DESC'	),
			array('card_accept_2',		'r',		'0',		'ACCEPT_AMEX',			'ACCEPT_AMEX_DESC'			),
			array('card_accept_3',		'r',		'0'	,		'ACCEPT_DISCOVER',		'ACCEPT_DISCOVER_DESC'		),
			array('card_accept_4',		'r',		'0',		'ACCEPT_JCB',			'ACCEPT_JCB_DESC'			),
			array('card_accept_5',		'r',		'0'	,		'ACCEPT_ENROUTE',		'ACCEPT_ENROUTE_DESC'		),
			array('card_accept_6',		'r',		'0',		'ACCEPT_DINERS',		'ACCEPT_DINERS_DESC'		),
			array('card_accept_7',		'r',		'0',		'ACCEPT_SOLO',			'ACCEPT_SOLO_DESC'			),
			array('card_accept_8',		'r',		'0',		'ACCEPT_MAESTRO',		'ACCEPT_MAESTRO_DESC'		),
			array('card_accept_9',		'r',		'0',		'ACCEPT_DELTA',			'ACCEPT_DELTA_DESC'			),
			array('card_accept_10',		'r',		'0',		'ACCEPT_BANKCARD',		'ACCEPT_BANKCARD_DESC'		),
			array('card_accept_11',		'r',		'0',		'ACCEPT_ELECTRON',		'ACCEPT_ELECTRON_DESC'		)
		);
		
		// this is a a much easier way to generate the proper config options for display
		// you should never have to change the below, just modify the params in the array above
		$i = 1;
		foreach($adminOptions as $adminOption) {
			$defaultConfigParams[] = array(
                NULL,
                NULL,
                $adminOption[0],
                $adminOption[2],
                'TRANS_SAGE_ADMIN',
                'TRANS_SAGE_'.$adminOption[3],
                'TRANS_SAGE_'.$adminOption[4],
                $adminOption[1],
                0,
                $i++,
                0,
                ($adminOption[1] == 'r') ? array(
				    array(NULL,NULL,'1','TRANS_YES',2),
				    array(NULL,NULL,'0','TRANS_NO',1)
			    ) : NULL);
		}
		
		// Grab the configs set at the registrar level
		return $defaultConfigParams;
	}

	/**
	 *
	 * From a child module this function must do the following:
	 *   - Maintain $this->hasExecuted, $this->hasExecutedSuccessfully, $this->timestamp,
	 *     $this->status, $this->accountID
	 *   - Add any queries to the debug list with addDebug()
	 *   - Add any errors to the error list with addError()
	 *   - Return $this->returnData, or the appropriate error constant
	 *
	 * @return  Returns feedback from execution of this Gateway object, or an error constant defined
	 *          at the top of this file.
	 */
	function execute() {
		global $db, $CONF, $gatewayReturnsCache;
		
		// populate $this->params with collapsed array
		// and set the proper URL to hit based on test mode
		$this->params = correctArray($this->params);
		$this->submitURL = $this->liveURL;
		
		switch($this->params['subCommand']) {
			case 'charge':
				$T_code = 'BANKCARD_SALE';
				$T_amt = $this->params['paymentAmount'];
			break;
			case 'auth':
				$T_code = 'BANKCARD_AUTHONLY';
				$T_amt = $this->params['paymentAmount'];
			break;
			case 'finalize':
				$T_code = 'BANKCARD_PRIOR_AUTH_SALE';
                $T_amt = $this->params['paymentAmount'];
				$T_reference = $this->params['transactionCode'];
			break;
			case 'refund':
				$T_code = 'BANKCARD_CREDIT';
				$T_amt = $this->params['paymentAmount'];
				$T_reference = $this->params['transactionCode'];
			break;
			case 'void':
				$T_code = 'BANKCARD_VOID';
                // $T_amt = $this->params['paymentAmount'];
				$T_reference = $this->params['transactionCode'];
			break;
			
			// just in case something funky is going on
			default:
				return $this->createOutputXML(array(
					"status" =>GATEWAY_ERROR,
					"message" => 'Invalid Gateway Action Requested',
					)
				);
		}
		
		$tempPostValues = array(
			// basic authentication that needs to be a part of any hit on PSiGate's API
			'M_ID'		    	=> 		$this->config['merchantID'],
			'M_KEY'     		=>		$this->config['merchantKey'],
		
			// required postfields for all transactions
			'C_CARDNUMBER'		=>		$this->params['billingCardNum'],
			'C_EXP'	        	=>		str_pad($this->params['billingCardExpMonth'],2,'0',STR_PAD_LEFT).substr($this->params['billingCardExpYear'],2,2),
		
			// required postfields for some transaction types
			'T_REFERENCE'	    =>		(isset($T_reference) && trim($T_reference) != '') ? $T_reference : '',
			'T_AMT'		    	=>		(isset($T_amt) && trim($T_amt) != '') ? $T_amt : '',
		
			// optional postvalues that do can magic things like lower merchant rates
			// please note: magic things might also include appearance of unicorns and rainbows
			'C_NAME'			=>		(isset($this->params['contactFirstName']) && isset($this->params['contactLastName']) && trim($this->params['contactFirstName']) != '' && trim($this->params['contactLastName']) != '') ? $this->params['contactFirstName'].' '.$this->params['contactLastName'] : '',
			'C_ADDRESS'			=>		(isset($this->params['contactAddress1']) && trim($this->params['contactAddress1']) != '') ? $this->params['contactAddress1'] : '',
			'C_CITY'			=>		(isset($this->params['contactCity']) && trim($this->params['contactCity']) != '') ? $this->params['contactCity'] : '',
			'C_STATE'			=>		(isset($this->params['contactState']) && trim($this->params['contactState']) != '') ? $this->params['contactState'] : '',
			'C_ZIP'		        =>		(isset($this->params['contactZip']) && trim($this->params['contactZip']) != '' ) ? $this->params['contactZip'] : '',
			'C_COUNTRY'			=>		(isset($this->params["countriesISO2"][0]) && trim($this->params["countriesISO2"][0]) != '') ? $this->params["countriesISO2"][0] : '',
			'C_TELEPHONE'		=>		(isset($this->params['contactPhone1']) && trim($this->params['contactPhone1']) != '') ? $this->params['contactPhone'] : '',
			'C_FAX'				=>		(isset($this->params['contactFax']) && trim($this->params['contactFax']) != '') ? $this->params['contactFax'] : '',
			'C_EMAIL'			=>		(isset($this->params['contactEmail']) && trim($this->params['contactEmail']) != '') ? $this->params['contactEmail'] : '',
			'C_CVV'     		=>		(isset($this->params['cardCVV2']) && trim($this->params['cardCVV2']) != '') ? $this->params['cardCVV2'] : ''
		);
		
		// generate final list of postvalues to go through to SagePayments's API
		$postValues = array();
		foreach($tempPostValues as $postElement => $postValue) {
			if (isset($postValue) && trim($postValue) != '')
				$postValues[$postElement] = str_replace('&','and',$postValue);
		}

        $postValues = '<'.$T_code.' xmlns="https://www.sagepayments.net/web_services/vterm_extensions/TRANSACTION_PROCESSING">'.toXML($postValues).'</'.$T_code.'>';
		
        // instantiate a new SOAP client and make the call to the Sage API
        $soapClient = new nu_soapclient($this->submitURL.'?WSDL',true);
        $soapResult = $soapClient->call(
            $T_code,
            $postValues,
            false,
            'https://www.sagepayments.net/web_services/vterm_extensions/TRANSACTION_PROCESSING/'.$T_code,
            false,
            false,
            'rpc',
            'encoding'
        );
        $soapResult = $soapResult[$T_code.'Result']['diffgram']['NewDataSet']['Table1'];
        
		dbg($soapResult);
		
		
		if (!$soapResult) {
			$this->status = GATEWAY_CONNECTION_FAILURE;
		} else {

			// set more return data based on API response
			switch($soapResult['APPROVAL_INDICATOR']) {
				case 'A':
					$this->status = GATEWAY_SUCCESS;
					$this->message = $soapResult['CODE'].' '.$soapResult['MESSAGE'];
					$this->transactionID = $soapResult['REFERENCE'];
					$this->authorizationCode = $soapResult['CODE'];
				break;
				
				case 'X':
					$this->status = GATEWAY_DECLINED;
					$this->message = $soapResult['CODE'].' '.$soapResult['MESSAGE'];
				break;
				
				case 'E':
					$this->status = GATEWAY_FAILURE;
					$this->message = $soapResult['CODE'].' '.$soapResult['MESSAGE'];
				break;
				
				default:
					$this->status = GATEWAY_FAILURE;
					$this->message = $soapResult['CODE'].' '.$soapResult['MESSAGE'];
				break;
			}

		}
		
		// return necessary information back to ModernBill
		$this->hasExecuted = 1;
		return $this->createOutputXML(array(
			'status' => $this->status,
			'statusName' => getAPIStatusName('transactions', $this->status),
			'statusMessage' => getStatusName('transactions', $this->status),
			'authorizationCode' => $this->authorizationCode,
			'transactionID' => $this->transactionID,
			'message' => $this->message,
			)
		);
	}	
}
?>