<?php
// $Id: actions.php 12972 2008-07-07 10:03:13Z bers $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

/**
 * This file maintains all information having to do with client and administrative actions. It is
 * basically just a big set of action names, keyed on a unique integer. This entire setup exists
 * to maintain higher overall security, and although not intentional, some level of security via
 * ambiguity. Always use the functions located at the bottom of this file to access this array,
 * never access it directly.
 */

/**
 * This is the master regex used to split all actions into like groups. The key is the translated
 * name of the action group. The value is the regex argument used to parse and filter actions to
 * their groups.
 *
 * The interface will use this grouping only for DISPLAY, not for functionality.
 */
$actionGroupRegex = array
(
    "TRANS_ACCOUNTING_ACTIONS" => "accounting|journal|ledger|profit|period|finance",
    "TRANS_ADMIN_ACTIONS" => "admin|profile",
    "TRANS_BOOKMARK_ACTIONS" => "bookmark",
    "TRANS_CERT_ACTIONS" => "cert",
    "TRANS_CLIENT_ACTIONS" => "client|billing|manualpayment",
    "TRANS_COMPANIES_ACTIONS" => "compan",
    "TRANS_CONFIG_ACTIONS" => "config|gateway|registrar",
    "TRANS_COUPON_ACTIONS" => "coupon",
    "TRANS_CYCLE_ACTIONS" => "cycle",
    "TRANS_DASHBOARD_ACTIONS" => "dashboard",
    "TRANS_DOMAIN_ACTIONS" => "domain",
    "TRANS_EMAIL_ACTIONS" => "email",
    "TRANS_EVENT_ACTIONS" => "event",
    "TRANS_FAQ_ACTIONS" => "faq",
    "TRANS_FRAUD_ACTIONS" => "fraud",
    "TRANS_GRAPH_ACTIONS" => "graph",
    "TRANS_HELPDESK_ACTIONS" => "helpdesk",
    "TRANS_INVOICE_ACTIONS" => "invoice",
    "TRANS_LOG_ACTIONS" => "log",
    "TRANS_LOGIN_ACTIONS" => "login|forgot|logout",
    "TRANS_MIGRATION_ACTIONS" => "migrate",
    "TRANS_NEWS_ACTIONS" => "news",
    "TRANS_WIZARD_ACTIONS" => "wizard",
    "TRANS_ORDERFORM_ACTIONS" => "order|cart|question",
    "TRANS_PACKAGE_ACTIONS" => "package",
    "TRANS_PAYMENTGROUP_ACTIONS" => "paymentgroup",
    "TRANS_PAYMENTQUEUE_ACTIONS" => "paymentqueue",
    "TRANS_PPC_ACTIONS" => "ppc",
    "TRANS_PRODUCT_ACTIONS" => "product|usage|price|attribute",
    "TRANS_REPORTS_ACTIONS" => "report",
    "TRANS_SERVER_ACTIONS" => "server|panel|module",
    "TRANS_SYSTEM_ACTIONS" => "system|search",
    "TRANS_TAX_ACTIONS" => "tax",
    "TRANS_THEME_ACTIONS" => "theme",
    "TRANS_TLD_ACTIONS" => "tld",
    "TRANS_TODO_ACTIONS" => "todo",
    "TRANS_TRANSACTION_ACTIONS" => "transaction|check|balance|deposit|credit|fund",
    "TRANS_WEBSITEWIDGETS_ACTIONS" => "websitewidget",
);

/**
 * List all valid client action names. If the action name is not listed here, a client will
 * be denied access.
 *
 * @author Michael Fountain
 */
$CONF["validClientActions"] = array
(
	// client actions from login.php
	"ShowLogin",
	"LoginResponse",
	"LogoutResponse",

	// client actions from dashboard.php
    "ShowDashboard",
    "PayInvoices",
    "PayInvoicesResponse",

	//Helpdesk actions
	"AddHelpdeskTicket",
	"AddHelpdeskTicketResponse",
	"ShowHelpdeskTicketDetails",
	"EditHelpdeskTicketResponse",

);

/**
 * List of all valid actions. If an action is not listed here that is needed, you will
 * be denied access.
 *
 * How to generate this list:
 *		1. Add unique actions to the action dispatch files located in the root/actions
 *		   directory.
 *		2. Run the following file from browser: /lib-tk/scripts/getActions.php
 *		3. Copy the output from the browser and replace the actions array listed below.
 *
 * Note: Adding actions will cause the ID's to change. This is ok because you should
 * NEVER access the actions using the ID alone. Always use the functions located at the
 * bottom of this file to access this array, never access it directly.
 *
 * @author auto-generated by /lib-tk/scripts/getActions.php
 */
$CONF["actions"] = array(
	"APILoginResponse","ActivatePackage","ActivatePackageAddon","ActivatePackageResponse","AddAdmin","AddAdminGroup","AddAdminGroupResponse","AddAdminResponse","AddAttribute","AddAttributeGroup","AddAttributeGroupResponse","AddAttributeMap","AddAttributeMapResponse","AddAttributeMapping","AddAttributeMappingResponse","AddAttributeResponse","AddBillingAccount","AddBillingAccountResponse","AddBookmark","AddBookmarkResponse","AddCartCoupon",
	"AddCartCouponResponse","AddCartDomain","AddCartDomainResponse","AddCartProduct","AddCartProductResponse","AddCert","AddCertResponse","AddCheck","AddCheckResponse","AddClient","AddClientContact","AddClientContactResponse","AddClientCreditResponse","AddClientResponse","AddCompany","AddCompanyResponse","AddCoupon","AddCouponResponse","AddCustomClientAttribute","AddCustomClientAttributeResponse",
	"AddCustomProductAttribute","AddCustomProductAttributeResponse","AddCycle","AddCycleDiscount","AddCycleDiscountGroup","AddCycleDiscountGroupResponse","AddCycleDiscountResponse","AddCycleGroup","AddCycleGroupResponse","AddCycleResponse","AddDeposit","AddDepositResponse","AddDomain","AddDomainResponse","AddEmailTemplate","AddEmailTemplateGroup","AddEmailTemplateGroupResponse","AddEmailTemplateResponse","AddFAQ","AddFAQCategory",
	"AddFAQCategoryResponse","AddFAQResponse","AddFraudCountryThresholdResponse","AddFraudSetResponse","AddGeneralJournalEntry","AddGeneralJournalEntryResponse","AddGenericAttribute","AddGenericAttributeResponse","AddGroupAttribute","AddGroupAttributeResponse","AddHelpdeskBucket","AddHelpdeskBucketResponse","AddHelpdeskDepartment","AddHelpdeskDepartmentResponse","AddHelpdeskResponseTemplate","AddHelpdeskResponseTemplateResponse","AddHelpdeskTicket","AddHelpdeskTicketResponse","AddHistoricalPackage","AddHistoricalPackageMenu",
	"AddHistoricalPackageResponse","AddIPLog","AddIPLogResponse","AddInvalidIpnLog","AddInvalidIpnLogResponse","AddInvoice","AddInvoiceResponse","AddInvoicingGroup","AddInvoicingGroupResponse","AddLedgerAccount","AddMigrationConfig","AddMigrationConfigResponse","AddNews","AddNewsGroup","AddNewsGroupResponse","AddNewsResponse","AddOrderForm","AddOrderFormResponse","AddPackage","AddPackageAddon",
	"AddPackageAddonAttribute","AddPackageAddonAttributeResponse","AddPackageAddonResponse","AddPackageAttribute","AddPackageAttributeResponse","AddPackageResponse","AddPanel","AddPanelResponse","AddPaymentGroup","AddPaymentGroupResponse","AddPrepayDiscount","AddPrepayDiscountResponse","AddPrice","AddProduct","AddProductAddon","AddProductAddonGroup","AddProductAddonGroupResponse","AddProductAddonResponse","AddProductAttributes","AddProductAttributesResponse",
	"AddProductGroup","AddProductGroupResponse","AddProductPricing","AddProductPricingResponse","AddProductRelation","AddProductRelationResponse","AddProductResponse","AddProductValues","AddProductValuesResponse","AddProductVariable","AddProductVariableResponse","AddProductVariableValue","AddProductVariableValueResponse","AddProductVariant","AddProductVariantResponse","AddQuestionValueMap","AddQuestionValueMapResponse","AddQuestionValueMapping","AddQuestionValueMappingResponse","AddRegistrar",
	"AddRegistrarResponse","AddServer","AddServerGroup","AddServerGroupResponse","AddServerResponse","AddSkeletonProduct","AddTLD","AddTLDResponse","AddTaxZone","AddTaxZoneGroup","AddTaxZoneGroupResponse","AddTaxZoneResponse","AddTheme","AddThemeResponse","AddTodo","AddTodoResponse","AddVendor","AddVendorInvoices","AddVendorResponse","AddWebsiteWidget",
	"AddWebsiteWidgetResponse","AddWizardQuestion","AddWizardQuestionResponse","AddWizardStep","AddWizardStepResponse","CancelAuthTransaction","CancelCert","CancelCertResponse","CancelInvoice","CancelInvoiceResponse","CancelPackage","CancelPackageResponse","CancelVendorPackage","CaptureFunds","ChangePackageClient","ChangePackageClientResponse","ChargeVirtualTerminal","ChargeVirtualTerminalResponse","CheckCertStatus","CheckCertStatusResponse",
	"CheckDomainAvailability","CheckDomainTransferAvailability","CheckMultiDomainWhois","CheckRunPaymentQueue","CheckServerLoad","ClearCart","ClearPaymentQueue","CloneAdminGroup","CloneAdminGroupResponse","CloneBookmark","CloneBookmarkResponse","CloneCoupon","CloneCouponResponse","CloneOrderForm","CloneOrderFormResponse","CloneProduct","CloneProductAddon","CloneProductAddonResponse","CloneProductAttribute","CloneProductAttributeResponse",
	"CloneProductResponse","CloneServerGroup","CloneServerGroupResponse","CloneTheme","CloneThemeResponse","ClosePeriod","ClosePeriodConfirm","ClosePeriodResponse","ConfirmCreditNote","ConfirmCreditNoteResponse","CreateMarketplaceSeller","CreatePackage","CreateVendorPackage","CreateVendorUser","CreateVendorUserResponse","DeletePackage","DeletePackageAddon","DeletePackageResponse","DeleteVendorPackage","DisassociateDomain",
	"DowngradePackage","DowngradePackageAddon","DowngradePackageResponse","EditAdmin","EditAdminGroup","EditAdminGroupResponse","EditAdminResponse","EditAppliedValues","EditAppliedValuesResponse","EditAttribute","EditAttributeGroup","EditAttributeGroupResponse","EditAttributeMap","EditAttributeMapResponse","EditAttributeMapping","EditAttributeMappingResponse","EditAttributeResponse","EditBasicClientAttributes","EditBasicClientAttributesResponse","EditBillingAccount",
	"EditBillingAccountResponse","EditBookmark","EditBookmarkResponse","EditCartDomain","EditCartDomainResponse","EditCartProduct","EditCartProductResponse","EditCert","EditCertResponse","EditCheck","EditCheckResponse","EditClient","EditClientArchiveState","EditClientArchiveStateResponse","EditClientContact","EditClientContactResponse","EditClientResponse","EditCompany","EditCompanyResponse","EditConfigParamsResponse",
	"EditCoupon","EditCouponResponse","EditCycle","EditCycleDiscount","EditCycleDiscountGroup","EditCycleDiscountGroupResponse","EditCycleDiscountResponse","EditCycleGroup","EditCycleGroupResponse","EditCycleResponse","EditDbgPageloadsSettings","EditDbgPageloadsSettingsResponse","EditDeposit","EditDepositResponse","EditDomain","EditDomainAccessPassword","EditDomainAccessPasswordResponse","EditDomainContactData","EditDomainContactDataResponse","EditDomainDNS",
	"EditDomainDNSResponse","EditDomainRegistrarLockOptions","EditDomainRegistrarLockOptionsResponse","EditDomainRenewTypeOptions","EditDomainRenewTypeOptionsResponse","EditDomainResponse","EditEmailConfig","EditEmailConfigResponse","EditEmailQueue","EditEmailQueueResponse","EditEmailTemplate","EditEmailTemplateGroup","EditEmailTemplateGroupResponse","EditEmailTemplateResponse","EditEncryptionConfigs","EditEncryptionConfigsConfirm","EditEncryptionConfigsResponse","EditEvent","EditEventResponse","EditFAQ",
	"EditFAQCategory","EditFAQCategoryResponse","EditFAQResponse","EditFraudCountryThreshold","EditFraudCountryThresholdResponse","EditFraudGuardianConfig","EditFraudGuardianConfigResponse","EditFraudSetOrderFormsResponse","EditFraudSetResponse","EditGeneralJournalEntry","EditGeneralJournalEntryResponse","EditGenericAttributeResponse","EditGroupAttribute","EditGroupAttributeResponse","EditHelpdeskBucket","EditHelpdeskBucketResponse","EditHelpdeskDepartment","EditHelpdeskDepartmentResponse","EditHelpdeskResponseTemplate","EditHelpdeskResponseTemplateResponse",
	"EditHelpdeskTicket","EditHelpdeskTicketResponse","EditIPLog","EditIPLogResponse","EditInvalidIpnLog","EditInvalidIpnLogResponse","EditInvoice","EditInvoiceResponse","EditInvoicingGroup","EditInvoicingGroupResponse","EditLedgerAccount","EditLedgerAccountResponse","EditLedgerGroupEntries","EditLedgerGroupEntriesResponse","EditMigrationConfig","EditMigrationConfigResponse","EditMyProfile","EditMyProfileResponse","EditNewOrdersResponse","EditNews",
	"EditNewsGroup","EditNewsGroupResponse","EditNewsResponse","EditOneProductRelations","EditOneProductRelationsResponse","EditOrderForm","EditOrderFormResponse","EditPPCFactoryConfig","EditPPCFactoryConfigResponse","EditPackage","EditPackageAddon","EditPackageAddonAttribute","EditPackageAddonAttributeResponse","EditPackageAddonResponse","EditPackageAddons","EditPackageAddonsResponse","EditPackageAttribute","EditPackageAttributeResponse","EditPackageResponse","EditPanel",
	"EditPanelAttributeResponse","EditPanelConfig","EditPanelConfigResponse","EditPanelResponse","EditPaymentGroup","EditPaymentGroupResponse","EditPaymentQueue","EditPaymentQueueResponse","EditPrepayDiscount","EditPrepayDiscountResponse","EditPrice","EditProduct","EditProductAddon","EditProductAddonAttributes","EditProductAddonAttributesResponse","EditProductAddonGroup","EditProductAddonGroupResponse","EditProductAddonGroups","EditProductAddonGroupsResponse","EditProductAddonResponse",
	"EditProductAddons","EditProductAddonsResponse","EditProductAttribute","EditProductAttributeResponse","EditProductAttributes","EditProductChildren","EditProductChildrenResponse","EditProductCoupons","EditProductCouponsResponse","EditProductCycles","EditProductCyclesResponse","EditProductEmails","EditProductEmailsResponse","EditProductGroup","EditProductGroupResponse","EditProductNotifications","EditProductNotificationsResponse","EditProductOrderForms","EditProductOrderFormsResponse","EditProductPriceResponse",
	"EditProductPrices","EditProductPricesResponse","EditProductRelation","EditProductRelationResponse","EditProductResponse","EditProductServerGroups","EditProductServerGroupsResponse","EditProductTLDs","EditProductTLDsResponse","EditProductValues","EditProductValuesResponse","EditProductVariable","EditProductVariableResponse","EditProductVariableValue","EditProductVariableValueResponse","EditProductVariant","EditProductVariantLedgerEntries","EditProductVariantLedgerEntriesResponse","EditProductVariantPrices","EditProductVariantPricesResponse",
	"EditProductVariantResponse","EditProductsGroup","EditProductsGroupResponse","EditQuestionValueMap","EditQuestionValueMapResponse","EditQuestionValueMapping","EditQuestionValueMappingResponse","EditRegistrarConfig","EditRegistrarConfigResponse","EditRegistrarSettings","EditRegistrarSettingsResponse","EditSSLFactoryConfig","EditSSLFactoryConfigResponse","EditSellerSettings","EditSellerSettingsResponse","EditServer","EditServerGroup","EditServerGroupResponse","EditServerResponse","EditTLD",
	"EditTLDResponse","EditTaxZone","EditTaxZoneGroup","EditTaxZoneGroupResponse","EditTaxZoneResponse","EditTheme","EditThemeResponse","EditTodo","EditTodoResponse","EditUltimateAffiliateConfig","EditUltimateAffiliateConfigResponse","EditUsagePricing","EditUsagePricingResponse","EditVendor","EditVendorResponse","EditWebsiteWidget","EditWebsiteWidgetResponse","EditWizardQuestion","EditWizardQuestionResponse","EditWizardStep",
	"EditWizardStepResponse","EditiDevAffiliateConfig","EditiDevAffiliateConfigResponse","EnablePackage","EnablePackageAddon","EnablePackageResponse","ExportPaymentQueue","ForgottenPasswordResponse","ForwardToClientInterface","GenerateInvoiceEvents","GeneratePaymentQueue","GeneratePaymentQueueResponse","GetAutocompleteClients","GetAutocompleteDomains","GetAutocompleteProducts","GetBillingAccounts","GetClientStatus","GetCompanyInformation","GetEmailTemplate","GetEmailTemplateVariables",
	"GetLogEnabledCommands","GetMBAPIMap","GetOrderForms","GetReportActivePackagesByCycle","GetReportActivePerServer","GetReportActivePerServerGroup","GetReportBilledServerRevenue","GetReportCustomerPaymentsByDate","GetReportDailyInvoices","GetReportDailySignups","GetReportMostPopularPackages","GetReportPackagesByStatus","GetReportPaymentsByPaymentType","GetReportRecurringServerRevenue","GetReportRevenuePerServer","GetVendorInformation","GetVendorProducts","ImportClientFromPanel","ImportClientFromPanelResponse","InstallSystemLicense",
	"InstallSystemLicenseResponse","LockDomainPrices","LockDomainPricesResponse","LockPackageAddonPrices","LockPackageAddonPricesResponse","LockPackagePrices","LockPackagePricesResponse","LockProductAddonPrices","LockProductAddonPricesResponse","LockProductPrices","LockProductPricesResponse","LoginResponse","LogoutResponse","ManageClientActionActivate","ManageClientActionActivateResponse","ManageClientActionArchive","ManageClientActionArchiveResponse","ManageClientActionCancel","ManageClientActionCancelResponse","ManageClientActionDelete",
	"ManageClientActionDeleteResponse","ManageClientActionPurge","ManageClientActionPurgeResponse","ManageClientActionSuspend","ManageClientActionSuspendResponse","ManageClientActionUnsuspend","ManageClientActionUnsuspendResponse","ManageClientNotes","ManageClientNotesResponse","ManageSellerProduct","MergeClient","MergeClientResponse","MergeCompanies","MergeCompaniesResponse","MigrateConfig","MigrateConfigAssociate","MigrateConfigAssociateResponse","MigrateConfigResponse","MigrateDeprocess","MigrateDisabled",
	"MigrateError","MigrateIntro","MigrateManager","MigrateModuleDeprocess","MigrateModuleManager","MigrateModuleProcess","MigrateProcess","MigrateProductDeprocess","MigrateProductManager","MigrateProductProcess","MigrateProductSkip","MigrateSkip","OrderCertCSRRequest","OrderCertCSRRequestResponse","OrderCertCSRVerify","OrderCertConfirmOrder","OrderCertContactRequest","OrderCertContactRequestResponse","OrderCertEnterCSR","OrderCertGetApproverEula",
	"OrderCertGetApproverEulaResponse","OrderCertGetApproverResponse","OrderCertOrderResponse","OrderCertOrderResults","OrderCertOrderSummary","OrderCertReceipt","OrderCertSelectApprover","OrderCertSelectApproverResponse","OrderCertSubmitOrderResponse","OrderCertVerifyCSRResponse","OrderFormProvisioning","PayInvoice","PayInvoiceResponse","PayInvoices","PayInvoicesResponse","PreviewEmailTemplate","ProcessCartCheckout","ProcessCartCheckoutLoader","ProcessCartLogin","ProcessCartLoginResponse",
	"ProcessCartRegister","ProcessManualPayment","ProcessTransactionRefund","ProcessTransactionRefundResponse","PurgeDbgPageloads","PushProduct","ReassignPackageAttributeResponse","ReauthBillingAccount","ReauthBillingAccountResponse","RecordAdminActions","ReencryptBillingAccountsResponse","RegisterDomainConfirm","RegisterDomainConfirmResponse","ReloadEmailTemplateResponse","RemoveAdmin","RemoveAdminGroup","RemoveAdminGroupResponse","RemoveAdminResponse","RemoveAttribute","RemoveAttributeGroup",
	"RemoveAttributeGroupResponse","RemoveAttributeMap","RemoveAttributeMapResponse","RemoveAttributeMapping","RemoveAttributeMappingResponse","RemoveAttributeResponse","RemoveBillingAccount","RemoveBillingAccountResponse","RemoveBookmark","RemoveBookmarkResponse","RemoveCartCoupon","RemoveCartCouponResponse","RemoveCartProduct","RemoveCartProductResponse","RemoveCert","RemoveCertResponse","RemoveClient","RemoveClientContact","RemoveClientContactResponse","RemoveClientNoteResponse",
	"RemoveClientResponse","RemoveCompany","RemoveCompanyResponse","RemoveCoupon","RemoveCouponResponse","RemoveCycle","RemoveCycleDiscount","RemoveCycleDiscountGroup","RemoveCycleDiscountGroupResponse","RemoveCycleDiscountResponse","RemoveCycleGroup","RemoveCycleGroupResponse","RemoveCycleResponse","RemoveDomain","RemoveDomainResponse","RemoveEmail","RemoveEmailQueue","RemoveEmailQueueResponse","RemoveEmailResponse","RemoveEmailTemplate",
	"RemoveEmailTemplateGroup","RemoveEmailTemplateGroupResponse","RemoveEmailTemplateResponse","RemoveEvent","RemoveEventResponse","RemoveFAQ","RemoveFAQCategory","RemoveFAQCategoryResponse","RemoveFAQResponse","RemoveFraudCountryThresholdResponse","RemoveFraudSetResponse","RemoveGateway","RemoveGatewayResponse","RemoveGroupAttribute","RemoveGroupAttributeResponse","RemoveHelpdeskBucket","RemoveHelpdeskBucketResponse","RemoveHelpdeskDepartment","RemoveHelpdeskDepartmentResponse","RemoveHelpdeskResponseTemplate",
	"RemoveHelpdeskResponseTemplateResponse","RemoveHelpdeskTicket","RemoveHelpdeskTicketResponse","RemoveIPLog","RemoveIPLogResponse","RemoveInvalidIpnLog","RemoveInvalidIpnLogResponse","RemoveInvoice","RemoveInvoiceResponse","RemoveInvoiceTransaction","RemoveInvoiceTransactionResponse","RemoveInvoicingGroup","RemoveInvoicingGroupResponse","RemoveLedgerAccount","RemoveLedgerGroup","RemoveLedgerGroupResponse","RemoveMigrationConfig","RemoveMigrationConfigResponse","RemoveNews","RemoveNewsGroup",
	"RemoveNewsGroupResponse","RemoveNewsResponse","RemoveOrderForm","RemoveOrderFormResponse","RemovePackage","RemovePackageAddon","RemovePackageAddonAttribute","RemovePackageAddonAttributeResponse","RemovePackageAddonResponse","RemovePackageAttribute","RemovePackageAttributeResponse","RemovePackageResponse","RemovePanel","RemovePanelResponse","RemovePaymentGroup","RemovePaymentGroupGateway","RemovePaymentGroupGatewayResponse","RemovePaymentGroupResponse","RemovePaymentQueue","RemovePaymentQueueResponse",
	"RemovePrepayDiscount","RemovePrepayDiscountResponse","RemoveProduct","RemoveProductAddon","RemoveProductAddonGroup","RemoveProductAddonGroupResponse","RemoveProductAddonResponse","RemoveProductAttribute","RemoveProductAttributeResponse","RemoveProductGroup","RemoveProductGroupResponse","RemoveProductResponse","RemoveProductValues","RemoveProductValuesResponse","RemoveProductVariable","RemoveProductVariableResponse","RemoveProductVariableValue","RemoveProductVariableValueResponse","RemoveProductVariant","RemoveProductVariantResponse",
	"RemoveQuestion","RemoveQuestionResponse","RemoveQuestionValueMap","RemoveQuestionValueMapResponse","RemoveQuestionValueMapping","RemoveQuestionValueMappingResponse","RemoveServer","RemoveServerGroup","RemoveServerGroupResponse","RemoveServerResponse","RemoveTLD","RemoveTLDResponse","RemoveTaxZone","RemoveTaxZoneGroup","RemoveTaxZoneGroupResponse","RemoveTaxZoneResponse","RemoveTheme","RemoveThemeResponse","RemoveTodo","RemoveTodoResponse",
	"RemoveTransaction","RemoveTransactionResponse","RemoveWebsiteWidget","RemoveWebsiteWidgetResponse","RemoveWizardQuestion","RemoveWizardQuestionResponse","RemoveWizardStep","RemoveWizardStepResponse","RenewDomain","RenewDomainResponse","ReplayCallbackLog","ReplayCallbackLogResponse","ReportClientList","ReportClientListDetailed","ReportClientListGraph","RequestProductSync","RequestVendorInvoiceSync","ResendCertApproverEmail","ResendCertApproverEmailResponse","ResendEmail",
	"ResendEmailResponse","ResendInvoice","ResendInvoiceResponse","ResetContactPassword","ResetContactPasswordResponse","RunDebugEvent","RunEvents","RunPaymentQueue","RunPaymentQueueResponse","SearchDispatchResponse","SendClientEmail","SendClientEmailResponse","SendEmailQueue","SendEmailQueueResponse","SendSellerMessage","ShowAccountingReportsMenu","ShowAccountingStats","ShowAccountsPayableLedger","ShowAccountsReceivableLedger","ShowAdminDetails",
	"ShowAdminGroupDetails","ShowAdminGroups","ShowAdminStats","ShowAdmins","ShowAgingReport","ShowAgingReportContainer","ShowAllTodos","ShowAttributeDetails","ShowAttributeGroups","ShowAttributeMapDetails","ShowAttributeMappingDetails","ShowAttributeMappings","ShowAttributeMaps","ShowAttributes","ShowBalanceSheet","ShowBalanceSheetContainer","ShowBillingAccountDetails","ShowBillingAccounts","ShowBookmarks","ShowCallbackLog",
	"ShowCallbackLogDetails","ShowCart","ShowCartCheckoutConfirmForm","ShowCartCheckoutForm","ShowCartCheckoutSuccess","ShowCartProductAddons","ShowCartProductAddonsStep","ShowCashFlow","ShowCashFlowContainer","ShowCertDetails","ShowCerts","ShowClientARHistory","ShowClientCallbackLog","ShowClientCallbackLogDetails","ShowClientContactDetails","ShowClientContacts","ShowClientCreditHistory","ShowClientDetails","ShowClientGraph","ShowClientStats",
	"ShowClients","ShowCompanies","ShowCompanyDetails","ShowConfigs","ShowCouponDetails","ShowCoupons","ShowCycleDiscountGroups","ShowCycleDiscounts","ShowCycleGroups","ShowCycleGroupsResponse","ShowCycleStats","ShowCycles","ShowCyclesResponse","ShowDashboard","ShowDashboardNewOrders","ShowDashboardTodos","ShowDbgPageloadDetails","ShowDbgPageloads","ShowDomainDetails","ShowDomainEPPCodeStep",
	"ShowDomains","ShowEmailDetails","ShowEmailMenus","ShowEmailQueue","ShowEmailQueueDetails","ShowEmailTemplateDetails","ShowEmailTemplateGroupDetails","ShowEmailTemplateGroups","ShowEmailTemplates","ShowEventInputDetails","ShowEventOutputDetails","ShowEventStatusBar","ShowEvents","ShowEventsStats","ShowFAQs","ShowFinancesMenu","ShowFinancialGraph","ShowForgottenPassword","ShowFraudCountryThresholds","ShowFraudSetDetails",
	"ShowFraudSets","ShowGeneralJournal","ShowGeneralLedger","ShowGenerateInvoiceMenu","ShowGenericBar","ShowGroupAttributes","ShowHelpdesk","ShowHelpdeskBucketDetails","ShowHelpdeskBucketTickets","ShowHelpdeskBuckets","ShowHelpdeskDepartments","ShowHelpdeskGraph","ShowHelpdeskResponseTemplates","ShowHelpdeskTicketDetails","ShowHelpdeskTickets","ShowIPLog","ShowInvalidIpnLogDetails","ShowInvalidIpnLogs","ShowInvoiceDetails","ShowInvoiceSheet",
	"ShowInvoiceSheetContainer","ShowInvoiceStats","ShowInvoices","ShowInvoicingGroups","ShowJournalsMenu","ShowLastFiveClientInvoices","ShowLastFiveClientPackages","ShowLedgerAccountDetails","ShowLedgerAccounts","ShowLedgersMenu","ShowLogDetails","ShowLogMenu","ShowLogin","ShowMBRequirementsTest","ShowMigrationConfigDetails","ShowMigrationConfigs","ShowMultiDomainWhois","ShowMultiDomainWhoisStep","ShowNewOrders","ShowNews",
	"ShowNewsDetails","ShowNewsGroups","ShowOrderFormProductsStep","ShowOrderForms","ShowPDFInvoice","ShowPHPInfo","ShowPPCCodes","ShowPackageAddonAttributeDetails","ShowPackageAddonAttributes","ShowPackageAddonDetails","ShowPackageAddons","ShowPackageAttributeDetails","ShowPackageAttributes","ShowPackageDetails","ShowPackageGraph","ShowPackages","ShowPanelClientLogin","ShowPanelDetails","ShowPanelOptions","ShowPaymentGroups",
	"ShowPaymentQueue","ShowPaymentQueueDetails","ShowPaymentQueueStats","ShowPrepayDiscounts","ShowProductAddonDetails","ShowProductAddonGroups","ShowProductAddons","ShowProductAttributeDetails","ShowProductAttributes","ShowProductDetails","ShowProductGraph","ShowProductGroups","ShowProductRelationDetails","ShowProductRelations","ShowProductStats","ShowProductVariableDetails","ShowProductVariableValueDetails","ShowProductVariableValues","ShowProductVariables","ShowProductVariants",
	"ShowProducts","ShowProductsRevenue","ShowProductsRevenueContainer","ShowProfitAndLoss","ShowProfitAndLossContainer","ShowPurchasesJournal","ShowQuestionValueMapDetails","ShowQuestionValueMappingDetails","ShowQuestionValueMappings","ShowQuestionValueMaps","ShowReceiptsJournal","ShowRegisterDomainForm","ShowRegistrars","ShowReportContainer","ShowReportDetails","ShowReports","ShowResendInvoice","ShowResendInvoiceResponse","ShowSellerStatus","ShowServerDetails",
	"ShowServerGroupDetails","ShowServerGroups","ShowServerGroupsReport","ShowServers","ShowSignupSheet","ShowSignupSheetContainer","ShowSystemLibraryInfo","ShowSystemLicenseInfo","ShowSystemModuleInfo","ShowSystemServerInfo","ShowTLDDetails","ShowTLDLoader","ShowTLDs","ShowTaxPayableLedger","ShowTaxZoneGroups","ShowTaxZones","ShowThemeDetails","ShowThemes","ShowTodoDetails","ShowTodos",
	"ShowTransactionDetails","ShowTransactionLog","ShowTransactions","ShowTransferDomainForm","ShowTrendGraph","ShowUncapturedFunds","ShowVendorDetails","ShowVendors","ShowVirtualTerminals","ShowWebsiteWidget","ShowWebsiteWidgets","ShowWizardQuestions","ShowWizardSteps","StopEvents","SuppressPPCCodeResponse","SuspendPackage","SuspendPackageAddon","SuspendPackageResponse","SuspendVendorPackage","SyncDomainDates",
	"SyncDomainDatesResponse","SyncSellerInventory","SyncVendorAPIURL","SyncVendorDetails","TakeClientPayment","TakeClientPaymentResponse","TestSellerApiURL","TestVendorAPIURL","TransferDomainConfirm","TransferDomainConfirmResponse","UnLockProductAddonPrices","UnLockProductAddonPricesResponse","UnLockProductPrices","UnLockProductPricesResponse","UnlockDomainPrices","UnlockDomainPricesResponse","UnlockPackageAddonPrices","UnlockPackageAddonPricesResponse","UnlockPackagePrices","UnlockPackagePricesResponse",
	"UnsuspendVendorPackage","UpdatePackage","UpdatePackageBillingShares","UpdatePackageBillingSharesResponse","UpdateSellerStatus","UpdateSellerVendor","UpdateSellerVendorResponse","UpdateVendorInformation","UpgradePackage","UpgradePackageAddon","UpgradePackageResponse","UploadReport","UploadReportResponse","VoidInvoice","VoidInvoiceResponse","ShowClientServiceInputForm","ShowClientServiceInputFormResponse","RemoveProductRelation","RemoveProductRelationResponse",
	"ShowBans","ShowBanDetails","EditBan","EditBanResponse","AddBan","AddBanResponse","RemoveBan","RemoveBanResponse","ChangePackageCycle","ChangePackageCycleResponse","AddSingleInvoice","AddSingleInvoiceResponse","EditLedgerGroups","EditProductMiscellaneous","EditProductMiscellaneousResponse","PerformPanelAction","FirstTimeIntroduction","FirstTimeDisplayStep","FirstTimePerformAction","LoginToPanel"
);

