<?php
// $Id: priorities.php 10539 2007-01-21 20:15:08Z mfountain $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2007 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

/**
 * This file maintains all information having to do with priorities, that will ultimately be
 * associated with various objects, scaling their interpreted significance. The order of the
 * $CONF["priorities"] array in this file is very important, do not change it, as the array keys
 * are considered the priority IDs.
 *
 * DO NOT ADD APPLICATION SPECIFC PRIORITIES TO THIS FILE UNLESS THEY ARE
 * SHARED BY ALL MBAPI APPLICATIONS!
 *
 * Application specifc priorities should be located here:
 *
 * /lib-APPLICATION/include/config/internal/priorities.php
 */

$CONF["priorities"] = array();
if(defined("TRANS_LOWEST")) {
	$CONF["priorities"][0] = constant("TRANS_LOWEST");
} else {
	$CONF["priorities"][0] = "Lowest";
}
if(defined("TRANS_LOW")) {
	$CONF["priorities"][1] = constant("TRANS_LOW");
} else {
	$CONF["priorities"][1] = "Low";
}
if(defined("TRANS_NORMAL")) {
	$CONF["priorities"][2] = constant("TRANS_NORMAL");
} else {
	$CONF["priorities"][2] = "Normal";
}
if(defined("TRANS_HIGH")) {
	$CONF["priorities"][3] = constant("TRANS_HIGH");
} else {
	$CONF["priorities"][3] = "High";
}
if(defined("TRANS_HIGHEST")) {
	$CONF["priorities"][4] = constant("TRANS_HIGHEST");
} else {
	$CONF["priorities"][4] = "Highest";
}

/**
 * This function gets the ID of a priority by searching for the given name. If the given name is found
 * in the priority list, this function returns the ID, however if the given name is not found, this
 * function returns false. Note that a priority ID of 0 is possible.
 *
 * @param  name  The priority name to search for, case-insensitive.
 * @return  The priority ID of the given priority name, or false if the given name was not found.
 */
function getPriorityID($name) {
	global $CONF;
	$retval = false;
	$c = count($CONF["priorities"]);
	for($i=0; $i<$c; $i++) {
		if(strcasecmp($CONF["priorities"][$i], $name) === 0) {
			$retval = $i;
			break;
		}
	}
	return $retval;
}

/**
 * This function gets the name of a priority by searching for the given priority ID. If the given ID is
 * found in the priority list, this function returns the name associated with that ID, however if the given
 * ID is not found, this function returns false. Note that a priority ID of 0 is possible.
 *
 * @param  id  The priority ID to search for.
 * @return  The priority name of the given priority ID, or false if the given ID was not found.
 */
function getPriorityName($id) {
	global $CONF;
	$retval = false;
	if($id < count($CONF["priorities"])) {
		$retval = $CONF["priorities"][$id];
	}
	return $retval;
}
?>