<?php
// $Id: regions.php 11305 2007-06-10 17:53:12Z mfountain $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2007 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

/**
 * This file maintains the geographic region associations for countries based on the
 * UN geographical designation. See http://unstats.un.org/unsd/methods/m49/m49regin.htm
 * for list. Note that the array keys of these constants must NEVER change; as the
 * system will break if those array keys change.
 *
 * This file also maintains their helper functions.
 *
 * DO NOT ADD APPLICATION SPECIFC REGIONS TO THIS FILE UNLESS THEY ARE
 * SHARED BY ALL MBAPI APPLICATIONS!
 *
 * Application specifc types should be located here:
 *
 * /lib-APPLICATION/include/config/internal/regions.php
 *
 * @author  Mark Garrett
 * @author 	Michael Fountain
 */

$CONF["regions"]["names"] = array(
"002" => TRANS_AFRICA,
"003" => TRANS_NORTH_AMERICA,
"005" => TRANS_SOUTH_AMERICA,
"009" => TRANS_OCEANIA,
"011" => TRANS_WESTERN_AFRICA,
"013" => TRANS_CENTRAL_AMERICA,
"014" => TRANS_EASTERN_AFRICA,
"015" => TRANS_NORTHERN_AFRICA,
"017" => TRANS_MIDDLE_AFRICA,
"018" => TRANS_SOUTHERN_AFRICA,
"019" => TRANS_AMERICAS,
"021" => TRANS_NORTHERN_AMERICA,
"029" => TRANS_CARIBBEAN,
"030" => TRANS_EASTERN_ASIA,
"035" => TRANS_SOUTHEASTERN_ASIA,
"039" => TRANS_SOUTHERN_EUROPE,
"053" => TRANS_AUSTRALIA_NEW_ZEALAND,
"054" => TRANS_MELANESIA,
"057" => TRANS_MICRONESIA,
"061" => TRANS_POLYNESIA,
"062" => TRANS_SOUTHERN_ASIA,
"142" => TRANS_ASIA,
"143" => TRANS_CENTRAL_ASIA,
"145" => TRANS_WESTERN_ASIA,
"150" => TRANS_EUROPE,
"151" => TRANS_EASTERN_EUROPE,
"154" => TRANS_NORTHERN_EUROPE,
"155" => TRANS_WESTERN_EUROPE,
"419" => TRANS_LATIN_AMERICA_CARIBBEAN,
"QO" => TRANS_OUTLYING_OCEANIA);

$CONF["regions"]["parent"] = array(
"011" => "002",
"014" => "002",
"015" => "002",
"017" => "002",
"018" => "002",
"013" => "003",
"021" => "003",
"029" => "003",
"053" => "009",
"054" => "009",
"057" => "009",
"061" => "009",
"QO"  => "009",
"030" => "142",
"035" => "142",
"062" => "142",
"143" => "142",
"145" => "142",
"039" => "150",
"151" => "150",
"154" => "150",
"155" => "150");

// This is the complete nested hierarchy of regions.
// The list is displayed A-Z.
$CONF["regions"]["nested"] = array
(
    // TRANS_AFRICA
    "002" => array
        (
            "014", // TRANS_EASTERN_AFRICA
            "017", // TRANS_MIDDLE_AFRICA
            "015", // TRANS_NORTHERN_AFRICA
            "018", // TRANS_SOUTHERN_AFRICA
            "011", // TRANS_WESTERN_AFRICA
        ),

    // TRANS_ASIA
    "142" => array
        (
            "143", // TRANS_CENTRAL_ASIA
            "030", // TRANS_EASTERN_ASIA
            "035", // TRANS_SOUTHEASTERN_ASIA
            "062", // TRANS_SOUTHERN_ASIA
            "145", // TRANS_WESTERN_ASIA
        ),

    // TRANS_EUROPE
    "150" => array
        (
            "151", // TRANS_EASTERN_EUROPE
            "154", // TRANS_NORTHERN_EUROPE
            "039", // TRANS_SOUTHERN_EUROPE
            "155", // TRANS_WESTERN_EUROPE
        ),

    // TRANS_NORTH_AMERICA
    "003" => array
        (
            "029", // TRANS_CARIBBEAN
            "013", // TRANS_CENTRAL_AMERICA
            "021", // TRANS_NORTHERN_AMERICA
        ),

    // TRANS_OCEANIA
    "009" => array
        (
            "053", // TRANS_AUSTRALIA_NEW_ZEALAND
            "054", // TRANS_MELANESIA
            "057", // TRANS_MICRONESIA
            "QO" , // TRANS_OUTLYING_OCEANIA
            "061", // TRANS_POLYNESIA
        ),
);

/**
 * Get the integer ID of the region constant associated with the given name constant and type. If the given name
 * is not found, this function returns false.
 *
 * @param  string  The type of region to lookup (i.e. "names", or "parent")
 * @param  string  The name of the region constant to lookup
 * @return  int  The integer ID of the region constant found for the given name, or false if the name was not found.
 */
function getRegionID($type, $name) {
	global $CONF;
	$retval = false;
	$type = strval($type);
	if(is_array($CONF["regions"][$type]) && count($CONF["regions"][$type])) {
		foreach($CONF["regions"][$type] as $key => $value) {
			if(eregi($name,$value)) {
				$retval = $key;
				break;
			}
		}
	}
	return $retval;
}

/**
 * Get the string constant name of the region associated with the given integer ID and type. If the given ID 
 * is not found, then this function returns false.
 *
 * @param  string  The type of region to lookup (i.e. "names")
 * @param  string  The id of the region constant to lookup
 * @return  string  The string constant name associated with the given ID, or false if the ID was not found
 */
function getRegionName($type, $id) {
	global $CONF;
	// the ID is a string b/c it has leading zeros
	$id = strval($id);
	$type = strval($type);
	$retval = false;
	if(is_array($CONF["regions"][$type]) && count($CONF["regions"][$type])) {
		$retval = $CONF["regions"][$type][$id];
	}
	return $retval;
}

/**
 * Get the string parent name of the region associated with the given integer ID. If the given ID
 * is not found, then this function returns false. This can be used in conjunction with getRegionName()
 * to get the string constant of a sub-region's parent.
 *
 * @param  int  The id of the region to lookup
 * @return  string  The string parent name associated with the given ID, or false if the ID was not found
 */
function getRegionParentName($id) {
	global $CONF;
	$id = ((int) $id);
	$retval = false;
	if($CONF["regions"]["parent"][$id]) {
		$retval = $CONF["regions"]["parent"][$id];
	}
	return $retval;
}

/**
 * Returns an array of regions of a certain type.
 *
 * @param  string  The type of regions to return.
 * @return  array  An array of regions of the given type, or literal false if the type was not found.
 */
function getRegions($type) {
	global $CONF;
	$type = strval($type);
	$retval = false;
	if(isset($CONF["regions"][$type])) {
		$retval = $CONF["regions"][$type];
	}
	return $retval;
}
?>
