<?php
// $Id: genericTable.php 10881 2007-03-14 15:03:33Z bsmith $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


class GenericTable {
	/**
	 *	@short width we want to take up
	 *	@var __width
	 *	@a private
	 */
	var $__width = 0;

	/**
	 *	@short height of the table calculated by the object
	 *	@var __height
	 *	@a private
	 */
	var $__height = 0;

	/**
	 *	@short internal map of rows
	 *	@var _rows
	 *	@a protected
	 */
	var $_rows = array();

	/**
	 *	@short bounds record
	 *	@var _colBounds
	 *	@a protected
	 */
	var $_colBounds = array();

	/**
	 *	@short our calculated widths of columns
	 *	@var _calculatedWidths
	 *	@a protected
	 */
	var $_calculatedWidths = array();

	/**
	 *	@short is the table already constructed
	 *	@var __constructed
	 *	@a private
	 */
	var $__constructed = false;





	/**
	 *	@short Constructor
	 *	@a public
	 *	@param int width of the matrix
	 */
	function GenericTable( $x ) {
		$this->setWidth( $x );
	}




	function &__get( $rowID ) {
		$rowID = (int)$rowID;
		if (isset( $this->_rows[ $rowID ] )) {
			return $this->_rows[ $rowID ];
		}

		return $this->_rows[ $rowID ];
	}

	function __isset( $rowID ) {
		return isset( $this->_rows[ (int)$rowID ] );
	}

	function __unset( $rowID ) {
		unset( $this->_rows[ (int)$rowID ] );
	}




	/**
	 *	@short set the width of this table widget
	 *	@a public
	 *	@param int
	 */
	function setWidth( $xx ) {
		if (notZero( $xx ))
			$this->__width = (int)$xx;
	}

	/**
	 *	@short request the width of this widget
	 *	@a public
	 *	@param int column ID to fetch width of
	 *	@return int
	 */
	function getWidth( $colID=NULL ) {
		if (is_numeric( $colID )) {
			if (is_array( $this->_calculatedWidths ) && isset( $this->_calculatedWidths[ $colID ] ))
				return $this->_calculatedWidths[ $colID ];
		}
		return $this->__width;
	}

	/**
	 *	@short set the list of header items
	 *	@a public
	 *	@param
	 *		Just pass the header text in order of display
	 */
	function setHeader() {
		$argv = func_get_args();

		if (count($argv) == 1) {
			if (is_array($argv[0]))
				$argv = $argv[0];
		}


		foreach( (array)$argv as $k => $v ) {
			$this->insert( -1, $k, (string)$v, new FontStyle('Arial', 11, 'B', 'black', 'center') );
		}
	}

	/**
	 *	@short return the number of rows we have to write
	 *	@a public
	 *	@return int
	 */
	function rowCount() {
		return count($this->_rows);
	}

	/**
	 *	@short return a reference to the rows in the table
	 *	@a public
	 *	@return reference
	 */
	function &getRows() {
		return $this->_rows;
	}

	/**
	 *	@short populate the table with an array of data
	 *	@a public
	 *	@param array of data, structure of [rowNum] => array( 'colName' => 'colValue' )
	 *	@param array of data to column map. [columnNum'] => 'colName'
	 *	@param array of styles per column. [colName] => FontStyle
	 */
	function populateWith( $data, $map, $styleMap = '' ) {
		if (!is_array( $data ) || !is_array( $map ))	return -1;

		foreach( (array)$data as $rowData ) {
			$row = count($this->_rows) + 1;

			for( $i=0, $n=count( $map ); $i<=$n; $i++ ) {
				$map[ $i ] = (string)$map[ $i ];

				if (isset( $rowData[ $map[ $i ] ] )) {

					if (isset( $styleMap[ $map[ $i ] ] )) {
						$this->insert( $row, $i, $rowData[ $map[ $i ] ], $styleMap[ $map[ $i ] ] );
					} else {
						$this->insert( $row, $i, $rowData[ $map[ $i ] ] );
					}

				}
			}
		}
	}

	/**
	 *	@short insert data for row
	 *	@a public
	 *	@param int row number
	 *	@param array insert data for row
	 *	@param array of styles per column. [colName] => FontStyle
	 */
	function insertRow( $row, $data, $styleMap = '' ) {
		$i =0;
		foreach( (array)$data as $col => $val ) {
			if (is_array( $styleMap ) && isset( $styleMap[ $col ] ))
				$this->insert( $row, $i++, $val, $styleMap[ $col ] );
			else
				$this->insert( $row, $i++, $val, $styleMap );
		}	unset( $col, $val );
	}

	/**
	 *	@short insert value at location in matrix
	 *	@a public
	 *	@param int row number
	 *	@param int column number
	 *	@param string value to insert
	 *	@param int static width
	 *	@param FontStyle how to display this value
	 */
	function insert( $row, $col, $value, $style='', $staticWidth=0 ) {
		if ($this->__constructed)	return -1;

		if (!isset( $this->_rows[ $row ] ))
			$this->_rows[ $row ] = new MBGenericTableRow();

		$this->_rows[ $row ]->$col = array(
				'value' => (string)$value,
				'style' => $style);

// 		show($this->_rows[ $row ]);
		$len = $this->_rows[ $row ]->$col->getWidth();


		// now check if the colBounds exceeds our bounds
		if (!is_array( $this->_colBounds[ $col ] ))
			$this->_colBounds[ $col ] = array( 'max' => 0, 'static' => 0 );

		if ($len >= $this->_colBounds[ $col ]['max'])
			$this->_colBounds[ $col ]['max'] = $len;

		if (notZero( $staticWidth ) && ($staticWidth >= $this->_colBounds[ $col ]['static']))
			$this->_colBounds[ $col ]['static'] = $staticWidth;

		return;
	}

	/**
	 *	@short remove row or value in column
	 *	@a public
	 *	@param int row number
	 *	@param int column number
	 */
	function remove( $row, $col = -1 ) {
		if ($this->__constructed)
			return -1;

		if (!isset( $this->_rows[ $row ] ))
			return;

		if ($col >= 0) {
			unset( $this->_rows[ $row ]->$col );
			return;
		}

		unset( $this->_rows[ $row ] );
		return;
	}

	function construct() {
		$this->_calculateColumnWidths();
		$this->_calculateRowHeights();
		$this->__constructed = true;
	}



















	function getHeight() {
		if (notZero( $this->__height ))
			return $this->__height;
		else
			$this->construct();

		return $this->__height;
	}

	function _calculateRowHeights() {
		$this->__height = 0;
		foreach( (array)$this->_rows as $k => $v )
			$this->__height += $this->_rows[ $k ]->getHeight();
	}


	function _calculateColumnWidths() {
		$newWidths = array();
		$ratios = array();

		$totalMaxes = 0;
		$totalStatic = 0;

		foreach( (array)$this->_colBounds as $v ) {
			if (notZero( $v['static'] ))
				$totalStatic += $v['static'];
			else
				$totalMaxes += $v['max'];
		}	unset( $v );


		foreach( (array)$this->_colBounds as $k => $v ) {
			if (notZero( $v['static'] ))
				$ratios[ $k ] = 1;
			else
				$ratios[ $k ] = ($v['max'] / $totalMaxes);
		}	unset( $k, $v );

		$widthToSpread = ($this->getWidth() - $totalStatic);
		unset( $totalStatic, $totalMaxes );

		$remainder = 0;
		$this->_calculatedWidths = array();
		foreach( (array)$ratios as $colID => $width ) {
			$width *= $widthToSpread;
			$this->_calculatedWidths[ $colID ] = floor( $width );

			$remainder += $width - $this->_calculatedWidths[ $colID ];

			if ($remainder >= 1) {
				$this->_calculatedWidths[ $colID ]++;
				$remainder--;
			}
		}

		if ($remainder > 0)
			$this->_calculatedWidths[ $colID ] += $remainder;

		unset( $colID, $width, $remainder );
	}

}

























class MBGenericTableRow {
	var $__style = NULL;

	var $_height = 0;

	var $_columns = array();

	function &__get( $colID ) {
		$colID = (int)$colID;

		return $this->_columns[ $colID ];
	}

	function __set( $colID, $value ) {
		$colID = (int)$colID;

		if (!isset( $this->_columns[ $colID ] )) {
			$this->_columns[ $colID ] = new MBGenericTableColumn( $value['value'], $value['style'] );
		}

	}

	function __isset( $colID ) {
		return isset( $this->_columns[ (int)$colID ] );
	}

	function __unset( $colID ) {
		unset( $this->_columns[ (int)$colID ] );
	}

	function getHeight() {
		$this->_height = 0;

		if (is_object( $this->__style )) {
			if (notZero( $this->__style->getHeight() )) {
				$this->_height = $this->__style->getHeight();
			}
		}

		for( $i=0, $n=count( $this->_columns ); $i<$n; $i++ ) {
			if ($this->_columns[ $i ]->getHeight() > $this->_height)
				$this->_height = $this->_columns[ $i ]->getHeight();
		}

		return $this->_height;
	}

	function &getColumns() {
		return $this->_columns;
	}
}


























class MBGenericTableColumn {
	var $__style = NULL;

	var $__value = NULL;

	var $__len = 0;

	function MBGenericTableColumn( $value, $style = '' ) {
		$this->setStyle( $style );
		$this->setValue( $value );
	}

	function setValue( $xx ) {
		if (notNull( $xx )) {
			$this->__value = (string)$xx;
			$this->__len = 0;
			$this->getWidth();
		}

	}

	function getValue() {
		return $this->__value;
	}

	function setStyle( $xx ) {
		if (is_object( $xx )) {
			$this->__style = $xx;
			return;
		}

		$this->__style = new FontStyle();

		if (isset( $xx['family'] ))
			$this->__style->setFamily( $xx['family'] );

		if (isset( $xx['size'] ))
			$this->__style->setSize( $xx['size'] );

		if (isset( $xx['color'] ))
			$this->__style->setFontColor( $xx['color'] );

		if (isset( $xx['bg'] ))
			$this->__style->setFillColor( $xx['bg'] );

	}

	function &getStyle() {
		return $this->__style;
	}

	function getHeight() {
		$numOlines = substr_count( $this->getValue(), "\n" );
		$pad = $this->getStyle();
		$pad = $pad->getPadding();
		return round( ($this->__style->getSize() / 0.7471764) * ( $numOlines ? $numOlines : 1 ) + ($pad*2) );
	}

	function getWidth() {
		if (notZero( $this->__len ))
			return $this->__len;

		$len = 0;
		foreach( explode("\n", $this->getValue()) as $ln ) {
			if ($len < strlen($ln))
				$len = strlen($ln);
		}	unset( $ln );

		// get this style...
		$ts = $this->getStyle();

		if (stripos( $ts->getStyle(), 'B' ) !== false) {
			$len = round($len * (($ts->getSize()/150)*75));
		} else {
			$len = round($len * (($ts->getSize()/150)*72));
		}

		$pad = $ts->getPadding();

		$this->__len = round($len + ($pad*2));
		return $this->__len;
	}
}


if (constant('PHP_4')) {
	overload('GenericTable');
	overload('MBGenericTableColumn');
	overload('MBGenericTableRow');
}
?>