<?php
// $Id: validator.php 11531 2007-07-06 15:46:54Z bsmith $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

mb_add_path(dirname(__FILE__));
if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-tk/include/creditcard/validate.php");

// configure restrictions. Remember that all REs need to match empty except for REQUIRED.
$pcreRestrictions = array(
	"STRING" => "//",
	// match anything
	"REQUIRED" => "/./",
	// 1 or more characters
	"EMPTY" => "/^$/",
	// null
	"BOOL" => "/^[01]?$/",
	// null or 0 or 1
	"INT" => "/^[-]?[0-9]*$/",
	// optional "-", 0 or more of 0-9
	"UNSIGNED_INT" => "/^[0-9]*$/",
	// 0 or more of 0-9
	"POSITIVE_INT" => "/^([0-9]*[0-9]|[1-9])$|^$/",
	// 0 or more 0-9, 1-9
	// null
	"DOUBLE" => "/^([0-9]+\.[0-9]*|[0-9]*\.[0-9]+|[0-9]+)$|^$/",
	// 1 or more 0-9, ".", 0 or more 0-9
	// 0 or more 0-9, ".", 1 or more 0-9
	// 1 or more 0-9
	// null
	"PRICE" => "/^([0-9]{1,10}\.[0-9]{0,2}|[0-9]{0,10}\.[0-9]{1,2}|[0-9]{1,10})$|^$/",
	// 1-10 of 0-9, ".", 0-2 of 0-9
	// 0-10 of 0-9, ".", 1-2 of 0-9
	// 1-10 of 0-9
	// null
	"PERCENT" => "/^[0-9]{1,2}$|^$|^100$|^100\.00$|^100\.0$/",
	// 1-2 of 0-9
	// null
	"PERCENT2" => "/^([0-9]{1,2}\.[0-9]{0,2}|[0-9]{0,2}\.[0-9]{1,2}|[0-9]{1,2})$|^$|^100$|^100\.00$|^100\.0$/",
	// 1-2 of 0-9, ".", 0-2 of 0-9
	// 0-2 of 0-9, ".", 1-2 of 0-9
	// 1-2 of 0-9
	// null
	"MONTH" => "/^(0?[0-9]|1[0-2])$|^$/",
	// optional 0, 0-9
	// 1, 0-2
	// null
	"YEAR" => "/^[0-9]{4}$|^$/",
	// 0-9 of length 4
	// null
	"STUPID_INT" => "/^[0-9]*$|unlimited/",
	// optional "-", 0 or more of 0-9
	// "unlimited"
	"UNIQUE" => "/^$/",
	// null
	"TLD" => "/^[a-z]([a-z.]*[a-z])?$|^$/",
	// a-z, optional a-z plus dots followed by a-z
	// null
);
$methodRestrictions = array(
	"PASSWORD" => "validatePassword",
	"MAXLEN" => "validateMaxLength",
	"MINLEN" => "validateMinLength",
	"MAXVAL" => "validateMaxValue",
	"MINVAL" => "validateMinValue",
	"INSET" => "validateInSet",
	"NOTINSET" => "validateNotInSet",
	"INCOL" => "validateInSourceCol",
	"NOTINCOL" => "validateNotInSourceCol",
	"CARD" => "validateCard",
	"TYPE" => "validateInSet",
	"STATUS" => "validateInSet",
);

// create TRANS_* and [TRANS_]INPUT_RESTRICTION_* constants, and arrange the config for operation
$bit = 0;
$inputRestrictions = array();
$inputRestrictionMesgs = array();
$inputRestrictionNames = array();
foreach ($pcreRestrictions as $const => $pcre) {
	$flag = 1 << $bit++;
	define("INPUT_RESTRICTION_" . $const, $flag);
	$inputRestrictions[] = array($flag, 0, $pcre);
	$inputRestrictionMesgs[$flag] = "TRANS_INPUT_RESTRICTION_" . $const;
	$inputRestrictionNames[$flag] = $const;
}
foreach ($methodRestrictions as $const => $method) {
	$flag = 1 << $bit++;
	define("INPUT_RESTRICTION_" . $const, $flag);
	$inputRestrictions[] = array($flag, 1, $method);
	$inputRestrictionMesgs[$flag] = "TRANS_INPUT_RESTRICTION_" . $const;
	$inputRestrictionNames[$flag] = $const;
}
define("INPUT_RESTRICTION_NONE", 0);
// define("INPUT_RESTRICTION_STRING", 0); // new definition in $pcreRestrictions

/**
 * returns error messages for the specified restriction bitmask
 *
 * @param  int  restriction bitmask
 *
 * @return  string[]  translated error messages.
 */
function errorMesgsFor($restrictions) {
	global $inputRestrictionMesgs;
	$errorMesgs = array();
	foreach($inputRestrictionMesgs as $flag => $mesg) {
		if($restrictions & $flag) {
			$errorMesgs[] = defined($mesg) ? constant($mesg) : $mesg;
		}
	}
	return $errorMesgs;
}

/**
 * input validator class
 *
 * @author  Ben Woolley
 */
class Validator {
	/**
	 * integer bitmask of the restrictions
	 */
	var $restrictions;

	/**
	 * string value to be validated
	 */
	var $value;

	/**
	 * optional mixed arguments to method restrictions
	 */
	var $args;

	/**
	 * integer bitmask of the restrictions which have errored.
	 */
	var $errors;

	/**
	 * array of error messages
	 */
	var $errorMesgs;

	/**
	 * Contructor method
	 *
	 * @param  int  bitmask of the restrictions
	 * @param  string  value to be validated
	 * @param  string  optional comparison value
	 */
	function Validator($restrictions = 0, $value = "", $args = array()) {
		$this->restrictions = $restrictions;
		$this->errors = 0;
		$this->errorMesgs = array();
		$this->value = $value;
		$this->args = $args;
	}


	/**
	 * Validate the value against the restrictions
	 */
	function validate() {
		global $inputRestrictions, $inputRestrictionMesgs;

		// operate all restrictions on the value
		$c = count($inputRestrictions);
		for($i = 0; $i < $c; $i++) {
			$flag = $inputRestrictions[$i][0];
			$restriction = $inputRestrictions[$i][2];

			if(
				$this->restrictions & $flag && (
					$inputRestrictions[$i][1]
					?	!Validator::$restriction($this->value, $this->args)
					:	!preg_match($restriction, $this->value)
				)
			) {

				$this->errors |= $flag;
				$this->errorMesgs[] = defined($inputRestrictionMesgs[$flag]) ? constant($inputRestrictionMesgs[$flag]) : $inputRestrictionMesgs[$flag];
			} elseif (isset($this->args['confirm']) && ($this->value !== $this->args['confirm'])) {
				$this->errors |= $flag;
				$this->errorMesgs[] = defined($inputRestrictionMesgs[$flag]) ? constant($inputRestrictionMesgs[$flag]) : $inputRestrictionMesgs[$flag];
			}



		}
	}

	/**
	 * Get the restrictions bitmask
	 *
	 * @return  int  integer bitmask of the restrictions
	 */
	function getRestrictions() {
		return $this->restrictions;
	}

	/**
	 * Set the restrictions bitmask
	 *
	 * @param  int  integer bitmask of the restrictions
	 */
	function setRestrictions($restrictions) {
		$this->restrictions = $restrictions;
	}

	/**
	 * Get the value to be validated
	 *
	 * @return  string  value to be validated
	 */
	function getValue() {
		return $this->value;
	}

	/**
	 * Set the value to be validated
	 *
	 * @param  string  value to be validated
	 */
	function setValue($value) {
		$this->value = $value;
	}

	/**
	 * Get the restriction function args
	 *
	 * @return  mixed  arguments
	 */
	function gerArgs() {
		return $this->args;
	}

	/**
	 * Set the restriction function args
	 *
	 * @param  mixed  arguments
	 */
	function setArgs($args) {
		$this->args = $args;
	}

	/**
	 * Get the errored restrictions bitmask
	 *
	 * @return  int  integer bitmask of the restrictions which have errored
	 */
	function getErrors() {
		return $this->errors;
	}

	/**
	 * Set the errored restrictions bitmask
	 *
	 * @param  int  integer bitmask of the restrictions which have errored
	 */
	function setErrors($errors) {
		$this->errors = $errors;
	}

	/**
	 * Get the error messages
	 *
	 * @return  string[]  array of translated error messages
	 */
	function getErrorMesgs() {
		return $this->errorMesgs;
	}

	/**
	 * Set the error messages
	 *
	 * @param  string[]  array of translated error messages
	 */
	function setErrorMesgs($errorMesgs) {
		$this->errorMesgs = $errorMesgs;
	}

	/**
	 * password validator
	 *
	 * @param  string  password
	 * @param  array  args with key 'confirm' for confirmation
	 *
	 * @return  boolean  success
	 */
	function validatePassword($password, $args) {
		return ($password === $args['confirm']);
	}

	/**
	 * max length validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'maxLength' with int value
	 *
	 * @return  boolean  success
	 */
	function validateMaxLength($value, $args) {
		return strlen($value) <= $args['maxLength'];
	}

	/**
	 * min length validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'minLength' with int value
	 *
	 * @return  boolean  success
	 */
	function validateMinLength($value, $args) {
		return strlen($value) >= $args['minLength'];
	}

	/**
	 * max value validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'maxValue' with int value
	 *
	 * @return  boolean  success
	 */
	function validateMaxValue($value, $args) {
		return (float) $value <= (float) $args['maxValue'];
	}

	/**
	 * min value validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'minValue' with int value
	 *
	 * @return  boolean  success
	 */
	function validateMinValue($value, $args) {
		return (float) $value >= (float) $args['minValue'];
	}

	/**
	 * set validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'set' with set in keys, _not values_
	 *
	 * @return  boolean  success
	 */
	function validateInSet($value, $args) {
		dbg("checking if ".$value." is a key in:");
		dbg($args['set']);
		return isset($args['set'][$value]);
	}

	/**
	 * not in set validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'set' with set in keys, _not values_
	 *
	 * @return  boolean  success
	 */
	function validateNotInSet($value, $args) {
		dbg("checking if ".$value." is a key in:");
		dbg($args['set']);
		return !isset($args['set'][$value]);
	}

	/**
	 * datasource key validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'source' and 'col'
	 *
	 * @return  boolean  success
	 */
	function validateInSourceCol($value, $args) {
		return (bool) oneOf($args["source"], array($args["col"] => $value));
	}

	/**
	 * datasource key invalidator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'source' and 'col'
	 *
	 * @return  boolean  success
	 */
	function validateNotInSourceCol($value, $args) {
		return !$this->validateInSourceCol($value, $args);
	}
	/**
	 * credit card validator
	 *
	 * @param  string  input
	 * @param  array  args wth key 'allowedCardTypes' with an array of allowed card types
	 *
	 * @return  boolean  validity
	 */
	function validateCard($value, $args) {
		if(isset($args['allowedCardTypes'])) {
			$types = $args['allowedCardTypes'];
		} else {
			$types = array();
		}
		$c =& new CreditCard($value, $types);
		return $c->isValid();

	}

}

/* DEBUG
$v =& new Validator(
	INPUT_RESTRICTION_INT|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD,
	"1234123412341234",
	array(
		'maxLength' => 16,
		'minLength' => 13,
		'allowedCardTypes' => TRANS_VISA,
	)
);
$v->validate();
echo '<pre>int that gets sent back in the URL: ';
print_r($v->getErrors());
echo "\nTranslated error messages: ";
print_r($v->getErrorMesgs());
echo '</pre>';

/* */
?>
