<?php
// $Id: unicode.php 12251 2007-11-16 19:28:24Z bwoolley $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once "entities.php";

// a single character class matching a superset of all of the special characters which may need to be converted into entities; the fewer matched, the fewer callbacks needed
define("UTF8_ASCII_FILTER_PATTERN", "/[^[:alnum:][:space:]\\x80-\\xff._-]/");

// the pattern matching all ASCII and UTF-8 sequences legal or illegal
define("UTF8_SEQUENCE_PATTERN", "/[\\x00-\\x7f]+|[\\xc0-\\xdf][\\x80-\\xbf]|[\\xe0-\\xef][\\x80-\\xbf]{2}|[\\xf0-\\xf7][\\x80-\\xbf]{3}|[\\xf8-\\xfb][\\x80-\\xbf]{4}|[\\xfc-\\xfd][\\x80-\\xbf]{5}/");

// the pattern of illegal sequences within the prior matched UTF-8 sequences (where the syntax is good, but the semantics are bad)
define("UTF8_ILLEGAL_SEQUENCE_PATTERN",
	"/" .
	implode(
		"|",
		array(
			# not allowed anywhere
			"[\\xfe\\xff]",

			# overflow patterns
			"[\\xc0-\\xc1][\\x80-\\xbf]?|\\xe0[\\x80-\\x9f][\\x80-\\xbf]?|\\xf0[\\x80-\\x8f][\\x80-\\xbf]{1,2}|\\xf8[\\x80-\\x87][\\x80-\\xbf]{1,3}|\\xfc[\\x80-\\x83][\\x80-\\xbf]{1,4}",

			# illegal unicode positions
			"\\xef\\xbf\\xbe",
			"\\xef\\xbf\\xbf",

			# UTF-16 surrogates
			"\\xed\\xa0\\x80",
			"\\xed\\xad\\xbf",
			"\\xed\\xae\\x80",
			"\\xed\\xaf\\xbf",
			"\\xed\\xb0\\x80",
			"\\xed\\xbe\\x80",
			"\\xed\\xbf\\xbf",
		)
	)
	. "/"
);

define("UTF8_BAD_STR", utf8_str(UTF8_BAD_NUM));

define("UTF8_ENTITY_NAMES", 0);
define("UTF8_ENTITY_NUMBERS", 1);
define("UTF8_ENTITY_ALL", 2);

function utf8_bad_match_callback($matches) {
	list($match) = $matches;
	return str_repeat(UTF8_BAD_STR, strlen($matches[0]));
	#return "[" . bin2hex($match) . "]";
}

function utf8_int($str) {
	switch(strlen($str)) {
		case 2:
			                                                                                                                                return ((ord($str[0]) & 0x1f) << 6) | (ord($str[1]) & 0x3f);
		case 3:
			                                                                                                return ((ord($str[0]) & 0x0f) << 12) | ((ord($str[1]) & 0x3f) << 6) | (ord($str[2]) & 0x3f);
		case 4:
			                                                                return ((ord($str[0]) & 0x07) << 18) | ((ord($str[1]) & 0x3f) << 12) | ((ord($str[2]) & 0x3f) << 6) | (ord($str[3]) & 0x3f);
		case 5:
			                                return ((ord($str[0]) & 0x03) << 24) | ((ord($str[1]) & 0x3f) << 18) | ((ord($str[2]) & 0x3f) << 12) | ((ord($str[3]) & 0x3f) << 6) | (ord($str[4]) & 0x3f);
		case 6:
			return ((ord($str[0]) & 0x01) << 30) | ((ord($str[1]) & 0x3f) << 24) | ((ord($str[2]) & 0x3f) << 18) | ((ord($str[3]) & 0x3f) << 12) | ((ord($str[4]) & 0x3f) << 6) | (ord($str[5]) & 0x3f);
		case 1:
			return ord($str);
		default:
			return UTF8_BAD_NUM;
	}
}

function utf8_entity($num, $utf8_map_name_func, $quote_set = UTF8_ENTITY_NAMES) {
	switch($quote_set) {
		case UTF8_ENTITY_ALL:
			$val = $utf8_map_name_func($num);
			return is_int($val) ? "&#${val};" : "&${val};";

		case UTF8_ENTITY_NUMBERS:
			return "&#${num};";

		case UTF8_ENTITY_NAMES:
		default:
			$val = $utf8_map_name_func($num);
			return is_int($val) ? utf8_str($val) : "&${val};";
	}	
}

function utf8_xml_entity($num, $quote_set = UTF8_ENTITY_NAMES) {
	return utf8_entity($num, "utf8_xml_map_name", $quote_set);
}

function utf8_html_entity($num, $quote_set = UTF8_ENTITY_NAMES) {
	return utf8_entity($num, "utf8_html_map_name", $quote_set);
}

function utf8_encode_callback($matches, $utf8_entity_func, $quote_set = UTF8_ENTITY_NAMES) {
	return $utf8_entity_func(utf8_int($matches[0]), $quote_set);
}

function utf8_xml_encode_callback_all($matches) {
	return utf8_encode_callback($matches, "utf8_xml_entity", UTF8_ENTITY_ALL);
}

function utf8_xml_encode_callback_names($matches) {
	return utf8_encode_callback($matches, "utf8_xml_entity", UTF8_ENTITY_NAMES);
}

function utf8_xml_encode_callback_numbers($matches) {
	return utf8_encode_callback($matches, "utf8_xml_entity", UTF8_ENTITY_NUMBERS);
}

function utf8_html_encode_callback_all($matches) {
	return utf8_encode_callback($matches, "utf8_html_entity", UTF8_ENTITY_ALL);
}

function utf8_html_encode_callback_names($matches) {
	return utf8_encode_callback($matches, "utf8_html_entity", UTF8_ENTITY_NAMES);
}

function utf8_html_encode_callback_numbers($matches) {
	return utf8_encode_callback($matches, "utf8_html_entity", UTF8_ENTITY_NUMBERS);
}

function utf8_entity_encode($str, $utf8_encode_func, $quote_set = UTF8_ENTITY_NAMES) {
	static $map = array(
		UTF8_ENTITY_NAMES => "names",
		UTF8_ENTITY_NUMBERS => "numbers",
		UTF8_ENTITY_ALL => "all",
	);

	$matches = array();
	preg_match_all(UTF8_SEQUENCE_PATTERN, $str, $matches, PREG_OFFSET_CAPTURE);

	$encoded = "";
	$cursor = 0;
	list($match_list) = $matches;
	//print_r($match_list);
	foreach($match_list as $match_info) {
		list($match, $pos) = $match_info;
		$len = strlen($match);

		if($invalid = substr($str, $cursor, $pos - $cursor)) {
			// invalid sequence
			$encoded .= utf8_bad_match_callback((array) $invalid);
		}

		$callback = $utf8_encode_func . "_" . $map[$quote_set];
		if(ord($match[0]) < 0x80) {
			// ASCII sequence
			$encoded .= preg_replace_callback(UTF8_ASCII_FILTER_PATTERN, $callback, $match);
		} else {
			// UTF-8 sequence

			if(preg_match(UTF8_ILLEGAL_SEQUENCE_PATTERN, $match)) { // using preg_match() instead of $count in preg_replace_callback() for compatibility
				// illegal UTF-8 sequence
				$match = preg_replace_callback(UTF8_ILLEGAL_SEQUENCE_PATTERN, "utf8_bad_match_callback", $match);
				$encoded .= $match;
			} else {
				// good UTF-8 sequence
				$encoded .= $callback((array) $match);
			}
		}

		$cursor = $pos + $len;
	}

	if($invalid = substr($str, $cursor)) {
		// invalid sequence
		$encoded .= utf8_bad_match_callback((array) $match);
	}

	return $encoded;
	
}

function utf8_html_entity_encode($str, $quote_set = UTF8_ENTITY_NAMES) {
	return utf8_entity_encode((string) $str, "utf8_html_encode_callback", $quote_set);
}

function utf8_xml_entity_encode($str, $quote_set = UTF8_ENTITY_NAMES) {
	return utf8_entity_encode((string) $str, "utf8_xml_encode_callback", $quote_set);
}

// based on code from php.net by romans@void.lv
function utf8_str($num) {
	if($num < 0x80)       return chr($num);
	if($num < 0x800)      return                                                                                                                                                 chr(($num >> 6) + 0xc0) . chr(($num & 0x3f) + 0x80);
	if($num < 0x10000)    return                                                                                                             chr(($num >> 12) + 0xe0) . chr((($num >> 6) & 0x3f) + 0x80) . chr(($num & 0x3f) + 0x80);
	if($num < 0x200000)   return                                                                         chr(($num >> 18) + 0xf0) . chr((($num >> 12) & 0x3f) + 0x80) . chr((($num >> 6) & 0x3f) + 0x80) . chr(($num & 0x3f) + 0x80);
	if($num < 0x4000000)  return                                     chr(($num >> 24) + 0xf8) . chr((($num >> 18) & 0x3f) + 0x80) . chr((($num >> 12) & 0x3f) + 0x80) . chr((($num >> 6) & 0x3f) + 0x80) . chr(($num & 0x3f) + 0x80);
	if($num < 0x80000000) return chr(($num >> 30) + 0xfc) . chr((($num >> 24) & 0x3f) + 0x80) . chr((($num >> 18) & 0x3f) + 0x80) . chr((($num >> 12) & 0x3f) + 0x80) . chr((($num >> 6) & 0x3f) + 0x80) . chr(($num & 0x3f) + 0x80);
	return UTF8_BAD_STR;
}

function utf8_literal_str($base, $num) {
	switch($base) {
		case "0":
			return utf8_str(octdec($num));
		case "X":
		case "x":
			return utf8_str(hexdec($num));
		default:
			return utf8_str($num);
	}
}

function utf8_decode_callback($matches, $utf8_map_num_func) {
	list($match, $name, $tag, $type, $base, $num) = $matches;

	switch($type) {
		case "#":
			return utf8_literal_str($base, $num);
		default:
			return utf8_str($utf8_map_num_func($name));
	}
}

function utf8_xml_decode_callback($matches) {
	return utf8_decode_callback($matches, "utf8_xml_map_num");
}

function utf8_html_decode_callback($matches) {
	return utf8_decode_callback($matches, "utf8_html_map_num");
}

function utf8_entity_decode($str, $utf8_decode_func) {
	return preg_replace_callback("/&(((#)(0|[xX])?)?([[:alnum:]]+));/u", $utf8_decode_func, (string) $str);
}

function utf8_html_entity_decode($str) {
	return utf8_entity_decode($str, "utf8_html_decode_callback");
}

function utf8_xml_entity_decode($str) {
	return utf8_entity_decode($str, "utf8_xml_decode_callback");
}

function utf8_xml_entity_decode_array(&$array){
	if (is_array($array)){
		foreach ($array as $k=>$v){
			if (is_array($v)){
				utf8_xml_entity_decode_array($array[$k]);
			} else {
				$array[$k] = utf8_xml_entity_decode($v);
			}
		}
	} else {
		$array =& utf8_xml_entity_decode($array);
	}
}

function utf8_xml_entity_encode_array(&$array){
	if (is_array($array)){
		foreach ($array as $k=>$v){
			if (is_array($v)){
				utf8_xml_entity_encode_array($array[$k]);
			} else {
				$array[$k] = utf8_xml_entity_encode($v);
			}
		}
	} else {
		$array =& utf8_xml_entity_encode($array);
	}
}

function utf8_html_entity_decode_array(&$array){
	if (is_array($array)){
		foreach ($array as $k=>$v){
			if (is_array($v)){
				utf8_html_entity_decode_array($array[$k]);
			} else {
				$array[$k] = utf8_html_entity_decode($v);
			}
		}
	} else {
		$array =& utf8_html_entity_decode($array);
	}
}

function utf8_html_entity_encode_array(&$array){
	if (is_array($array)){
		foreach ($array as $k=>$v){
			if (is_array($v)){
				utf8_html_entity_encode_array($array[$k]);
			} else {
				$array[$k] = utf8_html_entity_encode($v);
			}
		}
	} else {
		$array =& utf8_html_entity_encode($array);
	}
}
